@@ -10,87 +10,14 @@
 import "dart:_internal" show VMLibraryHooks, patch, unsafeCast;
 
 /// These are the additional parts of this patch library:
-// part "deferred_load_patch.dart";
-// part "schedule_microtask_patch.dart";
-// part "timer_patch.dart";
+part "schedule_microtask_patch.dart";
+part "timer_patch.dart";
 
 // Equivalent of calling FATAL from C++ code.
 @pragma("vm:external-name", "DartAsync_fatal")
 external _fatal(msg);
 
-// We need to pass the value as first argument and leave the second and third
-// arguments empty (used for error handling).
-@pragma("vm:recognized", "other")
-dynamic Function(dynamic) _asyncThenWrapperHelper(
-    dynamic Function(dynamic, dynamic) continuation) {
-  @pragma("vm:invisible")
-  dynamic thenWrapper(dynamic arg) => continuation(arg, /*stack_trace=*/ null);
-
-  // Any function that is used as an asynchronous callback must be registered
-  // in the current Zone. Normally, this is done by the future when a
-  // callback is registered (for example with `.then` or `.catchError`). In our
-  // case we want to reuse the same callback multiple times and therefore avoid
-  // the multiple registrations. For our internal futures (`_Future`) we can
-  // use the shortcut-version of `.then`, and skip the registration. However,
-  // that means that the continuation must be registered by us.
-  //
-  // Furthermore, we know that the root-zone doesn't actually do anything and
-  // we can therefore skip the registration call for it.
-  //
-  // Note, that the continuation accepts up to three arguments. If the current
-  // zone is the root zone, we don't wrap the continuation, and a bad
-  // `Future` implementation could potentially invoke the callback with the
-  // wrong number of arguments.
-  final currentZone = Zone._current;
-  if (identical(currentZone, _rootZone) ||
-      identical(currentZone._registerUnaryCallback,
-          _rootZone._registerUnaryCallback)) {
-    return thenWrapper;
-  }
-  return currentZone.registerUnaryCallback<dynamic, dynamic>(thenWrapper);
-}
-
-// We need to pass the exception and stack trace objects as second and third
-// parameter to the continuation.
-dynamic Function(Object, StackTrace) _asyncErrorWrapperHelper(
-    dynamic Function(dynamic, StackTrace) errorCallback) {
-  final currentZone = Zone._current;
-  if (identical(currentZone, _rootZone) ||
-      identical(currentZone._registerBinaryCallback,
-          _rootZone._registerBinaryCallback)) {
-    return errorCallback;
-  }
-  return currentZone
-      .registerBinaryCallback<dynamic, Object, StackTrace>(errorCallback);
-}
-
-/// Registers the [thenCallback] and [errorCallback] on the given [object].
-///
-/// If [object] is not a future, then it is wrapped into one.
-///
-/// Returns the result of registering with `.then`.
-Future _awaitHelper(var object, dynamic Function(dynamic) thenCallback,
-    dynamic Function(Object, StackTrace) errorCallback) {
-  _Future future;
-  if (object is _Future) {
-    future = object;
-  } else if (object is! Future) {
-    future = new _Future().._setValue(object);
-  } else {
-    return object.then(thenCallback, onError: errorCallback);
-  }
-  // `object` is a `_Future`.
-  //
-  // Since the callbacks have been registered in the current zone (see
-  // [_asyncThenWrapperHelper] and [_asyncErrorWrapperHelper]), we can avoid
-  // another registration and directly invoke the no-zone-registration `.then`.
-  //
-  // We can only do this for our internal futures (the default implementation of
-  // all futures that are constructed by the `dart:async` library).
-  return future._thenAwait<dynamic>(thenCallback, errorCallback);
-}
-
-@pragma("vm:entry-point", "call")
+// This function is used when lowering `await for` statements.
 void _asyncStarMoveNextHelper(var stream) {
   if (stream is! _StreamImpl) {
     return;
@@ -111,20 +38,13 @@ class _AsyncStarStreamController<T> {
   @pragma("vm:entry-point")
   StreamController<T> controller;
   @pragma("vm:entry-point")
-  Function? asyncStarBody;
+  void Function(Object?)? asyncStarBody;
   bool isAdding = false;
   bool onListenReceived = false;
   bool isScheduled = false;
   bool isSuspendedAtYield = false;
   _Future? cancellationFuture = null;
 
-  /// Argument passed to the generator when it is resumed after an addStream.
-  ///
-  /// `true` if the generator should exit after `yield*` resumes.
-  /// `false` if the generator should continue after `yield*` resumes.
-  /// `null` otherwies.
-  bool? continuationArgument = null;
-
   Stream<T> get stream {
     final Stream<T> local = controller.stream;
     if (local is _StreamImpl<T>) {
@@ -136,9 +56,7 @@ class _AsyncStarStreamController<T> {
   void runBody() {
     isScheduled = false;
     isSuspendedAtYield = false;
-    final bool? argument = continuationArgument;
-    continuationArgument = null;
-    asyncStarBody!(argument, null);
+    asyncStarBody!(!controller.hasListener);
   }
 
   void scheduleGenerator() {
@@ -149,16 +67,8 @@ class _AsyncStarStreamController<T> {
     scheduleMicrotask(runBody);
   }
 
-  // Adds element to stream, returns true if the caller should terminate
-  // execution of the generator.
-  //
-  // TODO(hausner): Per spec, the generator should be suspended before
-  // exiting when the stream is closed. We could add a getter like this:
-  // get isCancelled => controller.hasListener;
-  // The generator would translate a 'yield e' statement to
-  // controller.add(e);
-  // suspend;
-  // if (controller.isCancelled) return;
+  // Adds element to stream.
+  // Returns true if the caller should terminate execution of the generator.
   @pragma("vm:entry-point", "call")
   bool add(T event) {
     if (!onListenReceived) _fatal("yield before stream is listened to");
@@ -176,40 +86,23 @@ class _AsyncStarStreamController<T> {
   // Adds the elements of stream into this controller's stream.
   // The generator will be scheduled again when all of the
   // elements of the added stream have been consumed.
+  // Returns true if the caller should terminate execution of the generator.
   @pragma("vm:entry-point", "call")
-  void addStream(Stream<T> stream) {
+  bool addStream(Stream<T> stream) {
     if (!onListenReceived) _fatal("yield before stream is listened to");
-
-    if (exitAfterYieldStarIfCancelled()) return;
+    if (!controller.hasListener) {
+      return true;
+    }
 
     isAdding = true;
     final whenDoneAdding = controller.addStream(stream, cancelOnError: false);
     whenDoneAdding.then((_) {
       isAdding = false;
-      if (exitAfterYieldStarIfCancelled()) return;
-      resumeNormallyAfterYieldStar();
-    });
-  }
-
-  /// Schedules the generator to exit after `yield*` if stream was cancelled.
-  ///
-  /// Returns `true` if generator is told to exit and `false` otherwise.
-  bool exitAfterYieldStarIfCancelled() {
-    // If consumer cancelled subscription we should tell async* generator to
-    // finish (i.e. run finally clauses and return).
-    if (!controller.hasListener) {
-      continuationArgument = true;
       scheduleGenerator();
-      return true;
-    }
-    return false;
-  }
+      if (!isScheduled) isSuspendedAtYield = true;
+    });
 
-  /// Schedules the generator to resume normally after `yield*`.
-  void resumeNormallyAfterYieldStar() {
-    continuationArgument = false;
-    scheduleGenerator();
-    if (!isScheduled) isSuspendedAtYield = true;
+    return false;
   }
 
   void addError(Object error, StackTrace stackTrace) {
@@ -241,8 +134,7 @@ class _AsyncStarStreamController<T> {
     controller.close();
   }
 
-  _AsyncStarStreamController(this.asyncStarBody)
-      : controller = new StreamController(sync: true) {
+  _AsyncStarStreamController() : controller = new StreamController(sync: true) {
     controller.onListen = this.onListen;
     controller.onResume = this.onResume;
     controller.onCancel = this.onCancel;
@@ -284,73 +176,30 @@ class _StreamImpl<T> {
   Function? _generator;
 }
 
-@pragma("vm:entry-point", "call")
-void _completeOnAsyncReturn(_Future _future, Object? value, bool is_sync) {
-  // The first awaited expression is invoked sync. so complete is async. to
-  // allow then and error handlers to be attached.
-  // async_jump_var=0 is prior to first await, =1 is first await.
-  if (!is_sync || value is Future) {
-    _future._asyncCompleteUnchecked(value);
-  } else {
-    _future._completeWithValue(value);
-  }
-}
-
-@pragma("vm:entry-point", "call")
-void _completeWithNoFutureOnAsyncReturn(
-    _Future _future, Object? value, bool is_sync) {
-  // The first awaited expression is invoked sync. so complete is async. to
-  // allow then and error handlers to be attached.
-  // async_jump_var=0 is prior to first await, =1 is first await.
-  if (!is_sync) {
-    _future._asyncCompleteUncheckedNoFuture(value);
-  } else {
-    _future._completeWithValue(value);
-  }
-}
-
-@pragma("vm:entry-point", "call")
-void _completeOnAsyncError(
-    _Future _future, Object e, StackTrace st, bool is_sync) {
-  if (!is_sync) {
-    _future._asyncCompleteError(e, st);
-  } else {
-    _future._completeError(e, st);
-  }
-}
-
 @pragma("vm:external-name", "AsyncStarMoveNext_debuggerStepCheck")
 external void _moveNextDebuggerStepCheck(Function async_op);
 
 @pragma("vm:entry-point")
 class _SuspendState {
-  static const bool _trace = false;
-
   @pragma("vm:entry-point", "call")
   @pragma("vm:invisible")
   static Object? _initAsync<T>() {
-    if (_trace) print('_initAsync<$T>');
     return _Future<T>();
   }
 
   @pragma("vm:invisible")
-  @pragma("vm:recognized", "other")
   void _createAsyncCallbacks() {
-    if (_trace) print('_createAsyncCallbacks');
+    @pragma('vm:awaiter-link')
+    final suspendState = this;
 
     @pragma("vm:invisible")
     thenCallback(value) {
-      if (_trace) print('thenCallback (this=$this, value=$value)');
-      _resume(value, null, null);
+      suspendState._resume(value, null, null);
     }
 
     @pragma("vm:invisible")
     errorCallback(Object exception, StackTrace stackTrace) {
-      if (_trace) {
-        print('errorCallback (this=$this, '
-            'exception=$exception, stackTrace=$stackTrace)');
-      }
-      _resume(null, exception, stackTrace);
+      suspendState._resume(null, exception, stackTrace);
     }
 
     final currentZone = Zone._current;
@@ -442,7 +291,6 @@ class _SuspendState {
   @pragma("vm:entry-point", "call")
   @pragma("vm:invisible")
   Object? _await(Object? object) {
-    if (_trace) print('_await (object=$object)');
     if (_thenCallback == null) {
       _createAsyncCallbacks();
     }
@@ -464,11 +312,34 @@ class _SuspendState {
 
   @pragma("vm:entry-point", "call")
   @pragma("vm:invisible")
-  static Future _returnAsync(Object suspendState, Object? returnValue) {
-    if (_trace) {
-      print('_returnAsync (suspendState=$suspendState, '
-          'returnValue=$returnValue)');
+  Object? _awaitWithTypeCheck<T>(Object? object) {
+    if (_thenCallback == null) {
+      _createAsyncCallbacks();
     }
+    // Declare a new variable to avoid type promotion of 'object' to
+    // 'Future<T>', as it would disable further type promotion to '_Future'.
+    final obj = object;
+    if (obj is Future<T>) {
+      if (object is _Future) {
+        if (object._isComplete) {
+          _awaitCompletedFuture(object);
+        } else {
+          object._thenAwait<dynamic>(
+              unsafeCast<dynamic Function(dynamic)>(_thenCallback),
+              unsafeCast<dynamic Function(Object, StackTrace)>(_errorCallback));
+        }
+      } else {
+        _awaitUserDefinedFuture(obj);
+      }
+    } else {
+      _awaitNotFuture(object);
+    }
+    return _functionData;
+  }
+
+  @pragma("vm:entry-point", "call")
+  @pragma("vm:invisible")
+  static Future _returnAsync(Object suspendState, Object? returnValue) {
     _Future future;
     if (suspendState is _SuspendState) {
       future = unsafeCast<_Future>(suspendState._functionData);
@@ -487,10 +358,6 @@ class _SuspendState {
   @pragma("vm:invisible")
   static Future _returnAsyncNotFuture(
       Object suspendState, Object? returnValue) {
-    if (_trace) {
-      print('_returnAsyncNotFuture (suspendState=$suspendState, '
-          'returnValue=$returnValue)');
-    }
     _Future future;
     if (suspendState is _SuspendState) {
       future = unsafeCast<_Future>(suspendState._functionData);
@@ -504,16 +371,16 @@ class _SuspendState {
   @pragma("vm:entry-point", "call")
   @pragma("vm:invisible")
   static Object? _initAsyncStar<T>() {
-    if (_trace) print('_initAsyncStar<$T>');
-    return _AsyncStarStreamController<T>(null);
+    return _AsyncStarStreamController<T>();
   }
 
   @pragma("vm:invisible")
-  @pragma("vm:recognized", "other")
   _createAsyncStarCallback(_AsyncStarStreamController controller) {
-    controller.asyncStarBody = (value, _) {
-      if (_trace) print('asyncStarBody callback (value=$value)');
-      _resume(value, null, null);
+    @pragma('vm:awaiter-link')
+    final suspendState = this;
+
+    controller.asyncStarBody = (value) {
+      suspendState._resume(value, null, null);
     };
   }
 
@@ -531,10 +398,6 @@ class _SuspendState {
   @pragma("vm:entry-point", "call")
   @pragma("vm:invisible")
   static void _returnAsyncStar(Object suspendState, Object? returnValue) {
-    if (_trace) {
-      print('_returnAsyncStar (suspendState=$suspendState, '
-          'returnValue=$returnValue)');
-    }
     final controller = unsafeCast<_AsyncStarStreamController>(
         unsafeCast<_SuspendState>(suspendState)._functionData);
     controller.close();
@@ -544,10 +407,6 @@ class _SuspendState {
   @pragma("vm:invisible")
   static Object? _handleException(
       Object suspendState, Object exception, StackTrace stackTrace) {
-    if (_trace) {
-      print('_handleException (suspendState=$suspendState, '
-          'exception=$exception, stackTrace=$stackTrace)');
-    }
     Object? functionData;
     bool isSync = true;
     if (suspendState is _SuspendState) {
@@ -558,7 +417,11 @@ class _SuspendState {
     }
     if (functionData is _Future) {
       // async function.
-      _completeOnAsyncError(functionData, exception, stackTrace, isSync);
+      if (!isSync) {
+        functionData._asyncCompleteError(exception, stackTrace);
+      } else {
+        functionData._completeError(exception, stackTrace);
+      }
     } else if (functionData is _AsyncStarStreamController) {
       // async* function.
       functionData.addError(exception, stackTrace);
@@ -572,30 +435,15 @@ class _SuspendState {
   @pragma("vm:entry-point", "call")
   @pragma("vm:invisible")
   static Object? _initSyncStar<T>() {
-    if (_trace) print('_initSyncStar<$T>');
     return _SyncStarIterable<T>();
   }
 
   @pragma("vm:entry-point", "call")
   @pragma("vm:invisible")
-  Object? _yieldSyncStar(Object? object) {
-    if (_trace) print('_yieldSyncStar($object)');
+  Object? _suspendSyncStarAtStart(Object? object) {
     final data = _functionData;
-    if (data is _SyncStarIterable) {
-      data._stateAtStart = this;
-      return data;
-    } else {
-      // Update state in the iterator in case SuspendState was reallocated.
-      unsafeCast<_SyncStarIterator>(data)._state = this;
-    }
-    return true;
-  }
-
-  @pragma("vm:entry-point", "call")
-  @pragma("vm:invisible")
-  static bool _returnSyncStar(Object suspendState, Object? returnValue) {
-    if (_trace) print('_returnSyncStar');
-    return false;
+    unsafeCast<_SyncStarIterable>(data)._stateAtStart = this;
+    return data;
   }
 
   @pragma("vm:recognized", "other")
@@ -706,8 +554,8 @@ class _SyncStarIterator<T> implements Iterator<T> {
 
       try {
         // Resume current sync* method in order to move to the next value.
-        final bool hasMore =
-            _state!._resume(null, pendingException, pendingStackTrace) as bool;
+        final bool hasMore = unsafeCast<bool>(unsafeCast<_SuspendState>(_state)
+            ._resume(null, pendingException, pendingStackTrace));
         pendingException = null;
         pendingStackTrace = null;
         if (!hasMore) {
@@ -728,6 +576,8 @@ class _SyncStarIterator<T> implements Iterator<T> {
       // Case: yield* some_iterator.
       final iterable = _yieldStarIterable;
       if (iterable != null) {
+        _yieldStarIterable = null;
+        _current = null;
         if (iterable is _SyncStarIterable) {
           // We got a recursive yield* of sync* function. Instead of creating
           // a new iterator we replace our current _state (remembering the
@@ -739,11 +589,14 @@ class _SyncStarIterator<T> implements Iterator<T> {
           nestedState._functionData = this;
           _state = nestedState;
         } else {
-          _yieldStarIterator = iterable.iterator;
+          try {
+            _yieldStarIterator = iterable.iterator;
+          } catch (exception, stackTrace) {
+            pendingException = exception;
+            pendingStackTrace = stackTrace;
+          }
         }
-        _yieldStarIterable = null;
-        _current = null;
-        // Fetch the next item.
+        // Fetch the next item or continue with exception.
         continue;
       }
 
