@@ -2,7 +2,7 @@
 // for details. All rights reserved. Use of this source code is governed by a
 // BSD-style license that can be found in the LICENSE file.
 
-// part of "core_patch.dart";
+part of "core_patch.dart";
 
 @patch
 class Error {
@@ -59,10 +59,10 @@ class _AssertionError extends Error implements AssertionError {
       return condition;
     }
     if (condition is _Closure) {
-      return (condition as dynamic)();
+      return (condition as dynamic Function())();
     }
     if (condition is Function) {
-      condition = condition();
+      condition = (condition as dynamic Function())();
     }
     return condition;
   }
@@ -95,7 +95,7 @@ class _AssertionError extends Error implements AssertionError {
   final Object? message;
 }
 
-class _TypeError extends Error implements TypeError, CastError {
+class _TypeError extends Error implements TypeError {
   @pragma("vm:entry-point")
   _TypeError._create(this._url, this._line, this._column, this._message);
 
@@ -106,43 +106,10 @@ class _TypeError extends Error implements TypeError, CastError {
 
   String toString() => _message;
 
-  final String _url;
-  final int _line;
-  final int _column;
-  final String _message;
-}
-
-class _CastError extends Error implements CastError, TypeError {
-  @pragma("vm:entry-point")
-  _CastError._create(this._url, this._line, this._column, this._errorMsg);
-
-  // A CastError is allocated by TypeError._throwNew() when dstName equals
-  // Symbols::InTypeCast().
-
-  String toString() => _errorMsg;
-
-  // Fields _url, _line, and _column are only used for debugging purposes.
   final String? _url;
   final int? _line;
   final int? _column;
-  final String _errorMsg;
-}
-
-@patch
-class FallThroughError {
-  @patch
-  @pragma("vm:entry-point")
-  FallThroughError._create(this._url, this._line);
-
-  @patch
-  String toString() {
-    return "'$_url': Switch case fall-through at line $_line.";
-  }
-
-  // These new fields cannot be declared final, because a constructor exists
-  // in the original version of this patched class.
-  String? _url;
-  int _line = 0;
+  final String _message;
 }
 
 class _InternalError {
@@ -161,28 +128,36 @@ class UnsupportedError {
 }
 
 @patch
-class CyclicInitializationError {
-  static _throwNew(String variableName) {
-    throw new CyclicInitializationError(variableName);
+@pragma("vm:entry-point")
+class StateError {
+  @pragma("vm:entry-point")
+  static _throwNew(String msg) {
+    throw new StateError(msg);
   }
 }
 
-@patch
-class AbstractClassInstantiationError {
+/// Error thrown when a lazily initialized variable cannot be initialized.
+///
+/// Cyclic dependencies are no longer detected at runtime in null safe code.
+/// Such code will fail in other ways instead,
+/// possibly with a [StackOverflowError].
+///
+/// Will be removed when support for non-null-safe code is discontinued.
+@Deprecated("Remove when no longer supporting non-null-safe code.")
+class _CyclicInitializationError extends Error {
+  final String? variableName;
   @pragma("vm:entry-point")
-  AbstractClassInstantiationError._create(
-      this._className, this._url, this._line);
-
-  @patch
+  _CyclicInitializationError([this.variableName]);
   String toString() {
-    return "Cannot instantiate abstract class $_className: "
-        "_url '$_url' line $_line";
+    var variableName = this.variableName;
+    return variableName == null
+        ? "Reading static variable during its initialization"
+        : "Reading static variable '$variableName' during its initialization";
   }
 
-  // These new fields cannot be declared final, because a constructor exists
-  // in the original version of this patched class.
-  String? _url;
-  int _line = 0;
+  static _throwNew(String variableName) {
+    throw new _CyclicInitializationError(variableName);
+  }
 }
 
 @patch
@@ -217,18 +192,6 @@ class NoSuchMethodError {
         typeArgumentsLength, typeArguments, arguments, argumentNames);
   }
 
-  // Deprecated constructor.
-  @patch
-  NoSuchMethodError(this._receiver, Symbol memberName,
-      List? positionalArguments, Map<Symbol, dynamic>? namedArguments,
-      [List? existingArgumentNames = null]) // existingArgumentNames ignored.
-      : this._invocation = new _InvocationMirror._withType(
-            memberName,
-            _InvocationMirror._UNINITIALIZED,
-            null, // Type arguments not supported in deprecated constructor.
-            positionalArguments,
-            namedArguments);
-
   // Helper to build a map of named arguments.
   static Map<Symbol, dynamic> _NamedArgumentsMap(
       List arguments, List argumentNames) {
@@ -508,3 +471,33 @@ class _DuplicatedFieldInitializerError extends Error {
 
   toString() => "Error: field '$_name' is already initialized.";
 }
+
+// Implementations needed to implement the `_stackTrace` member added
+// in the @patch class of [Error].
+
+@patch
+class OutOfMemoryError {
+  StackTrace? get _stackTrace =>
+      throw UnsupportedError('OutOfMemoryError._stackTrace');
+  void set _stackTrace(StackTrace? _) {
+    throw UnsupportedError('OutOfMemoryError._stackTrace');
+  }
+}
+
+@patch
+class StackOverflowError {
+  StackTrace? get _stackTrace =>
+      throw UnsupportedError('StackOverflowError._stackTrace');
+  void set _stackTrace(StackTrace? _) {
+    throw UnsupportedError('StackOverflowError._stackTrace');
+  }
+}
+
+@patch
+class IntegerDivisionByZeroException {
+  StackTrace? get _stackTrace =>
+      throw UnsupportedError('IntegerDivisionByZeroException._stackTrace');
+  void set _stackTrace(StackTrace? _) {
+    throw UnsupportedError('IntegerDivisionByZeroException._stackTrace');
+  }
+}
