@@ -8,6 +8,7 @@ part of dart.typed_data;
 ///
 /// It is a compile-time error for a class to attempt to extend or implement
 /// UnmodifiableByteBufferView.
+// TODO(53785): @Deprecated('No replacement')
 abstract final class UnmodifiableByteBufferView implements ByteBuffer {
   external factory UnmodifiableByteBufferView(ByteBuffer data);
 }
@@ -16,6 +17,7 @@ abstract final class UnmodifiableByteBufferView implements ByteBuffer {
 ///
 /// It is a compile-time error for a class to attempt to extend or implement
 /// UnmodifiableByteDataView.
+// TODO(53785): @Deprecated('Use ByteData.asUnmodifiableView() instead')
 abstract final class UnmodifiableByteDataView implements ByteData {
   external factory UnmodifiableByteDataView(ByteData data);
 }
@@ -24,6 +26,7 @@ abstract final class UnmodifiableByteDataView implements ByteData {
 ///
 /// It is a compile-time error for a class to attempt to extend or implement
 /// UnmodifiableUint8ListView.
+// TODO(53785): @Deprecated('Use Uint8List.asUnmodifiableView() instead')
 abstract final class UnmodifiableUint8ListView implements Uint8List {
   external factory UnmodifiableUint8ListView(Uint8List list);
 }
@@ -32,6 +35,7 @@ abstract final class UnmodifiableUint8ListView implements Uint8List {
 ///
 /// It is a compile-time error for a class to attempt to extend or implement
 /// UnmodifiableInt8ListView.
+// TODO(53785): @Deprecated('Use Int8List.asUnmodifiableView() instead')
 abstract final class UnmodifiableInt8ListView implements Int8List {
   external factory UnmodifiableInt8ListView(Int8List list);
 }
@@ -40,7 +44,9 @@ abstract final class UnmodifiableInt8ListView implements Int8List {
 ///
 /// It is a compile-time error for a class to attempt to extend or implement
 /// UnmodifiableUint8ClampedListView.
-abstract final class UnmodifiableUint8ClampedListView implements Uint8ClampedList {
+// TODO(53785): @Deprecated('Use Uint8ClampedList.asUnmodifiableView() instead')
+abstract final class UnmodifiableUint8ClampedListView
+    implements Uint8ClampedList {
   external factory UnmodifiableUint8ClampedListView(Uint8ClampedList list);
 }
 
@@ -48,6 +54,7 @@ abstract final class UnmodifiableUint8ClampedListView implements Uint8ClampedLis
 ///
 /// It is a compile-time error for a class to attempt to extend or implement
 /// UnmodifiableUint16ListView.
+// TODO(53785): @Deprecated('Use Uint16List.asUnmodifiableView() instead')
 abstract final class UnmodifiableUint16ListView implements Uint16List {
   external factory UnmodifiableUint16ListView(Uint16List list);
 }
@@ -56,6 +63,7 @@ abstract final class UnmodifiableUint16ListView implements Uint16List {
 ///
 /// It is a compile-time error for a class to attempt to extend or implement
 /// UnmodifiableInt16ListView.
+// TODO(53785): @Deprecated('Use Int16List.asUnmodifiableView() instead')
 abstract final class UnmodifiableInt16ListView implements Int16List {
   external factory UnmodifiableInt16ListView(Int16List list);
 }
@@ -64,6 +72,7 @@ abstract final class UnmodifiableInt16ListView implements Int16List {
 ///
 /// It is a compile-time error for a class to attempt to extend or implement
 /// UnmodifiableUint32ListView.
+// TODO(53785): @Deprecated('Use Uint32List.asUnmodifiableView() instead')
 abstract final class UnmodifiableUint32ListView implements Uint32List {
   external factory UnmodifiableUint32ListView(Uint32List list);
 }
@@ -72,6 +81,7 @@ abstract final class UnmodifiableUint32ListView implements Uint32List {
 ///
 /// It is a compile-time error for a class to attempt to extend or implement
 /// UnmodifiableInt32ListView.
+// TODO(53785): @Deprecated('Use Int32List.asUnmodifiableView() instead')
 abstract final class UnmodifiableInt32ListView implements Int32List {
   external factory UnmodifiableInt32ListView(Int32List list);
 }
@@ -80,6 +90,7 @@ abstract final class UnmodifiableInt32ListView implements Int32List {
 ///
 /// It is a compile-time error for a class to attempt to extend or implement
 /// UnmodifiableUint64ListView.
+// TODO(53785): @Deprecated('Use Uint64List.asUnmodifiableView() instead')
 abstract final class UnmodifiableUint64ListView implements Uint64List {
   external factory UnmodifiableUint64ListView(Uint64List list);
 }
@@ -88,6 +99,7 @@ abstract final class UnmodifiableUint64ListView implements Uint64List {
 ///
 /// It is a compile-time error for a class to attempt to extend or implement
 /// UnmodifiableInt64ListView.
+// TODO(53785): @Deprecated('Use Int64List.asUnmodifiableView() instead')
 abstract final class UnmodifiableInt64ListView implements Int64List {
   external factory UnmodifiableInt64ListView(Int64List list);
 }
@@ -96,6 +108,7 @@ abstract final class UnmodifiableInt64ListView implements Int64List {
 ///
 /// It is a compile-time error for a class to attempt to extend or implement
 /// UnmodifiableInt32x4ListView.
+// TODO(53785): @Deprecated('Use Int32x4List.asUnmodifiableView() instead')
 abstract final class UnmodifiableInt32x4ListView implements Int32x4List {
   external factory UnmodifiableInt32x4ListView(Int32x4List list);
 }
@@ -104,6 +117,7 @@ abstract final class UnmodifiableInt32x4ListView implements Int32x4List {
 ///
 /// It is a compile-time error for a class to attempt to extend or implement
 /// UnmodifiableFloat32x4ListView.
+// TODO(53785): @Deprecated('Use Float32x4List.asUnmodifiableView() instead')
 abstract final class UnmodifiableFloat32x4ListView implements Float32x4List {
   external factory UnmodifiableFloat32x4ListView(Float32x4List list);
 }
@@ -112,6 +126,7 @@ abstract final class UnmodifiableFloat32x4ListView implements Float32x4List {
 ///
 /// It is a compile-time error for a class to attempt to extend or implement
 /// UnmodifiableFloat64x2ListView.
+// TODO(53785): @Deprecated('Use Float64x2List.asUnmodifiableView() instead')
 abstract final class UnmodifiableFloat64x2ListView implements Float64x2List {
   external factory UnmodifiableFloat64x2ListView(Float64x2List list);
 }
@@ -120,6 +135,7 @@ abstract final class UnmodifiableFloat64x2ListView implements Float64x2List {
 ///
 /// It is a compile-time error for a class to attempt to extend or implement
 /// UnmodifiableFloat32ListView.
+// TODO(53785): @Deprecated('Use Float32List.asUnmodifiableView() instead')
 abstract final class UnmodifiableFloat32ListView implements Float32List {
   external factory UnmodifiableFloat32ListView(Float32List list);
 }
@@ -128,6 +144,7 @@ abstract final class UnmodifiableFloat32ListView implements Float32List {
 ///
 /// It is a compile-time error for a class to attempt to extend or implement
 /// UnmodifiableFloat64ListView.
+// TODO(53785): @Deprecated('Use Float64List.asUnmodifiableView() instead')
 abstract final class UnmodifiableFloat64ListView implements Float64List {
   external factory UnmodifiableFloat64ListView(Float64List list);
 }
