@@ -62,10 +62,11 @@ part 'engine/canvaskit/picture.dart';
 part 'engine/canvaskit/picture_recorder.dart';
 part 'engine/canvaskit/raster_cache.dart';
 part 'engine/canvaskit/rasterizer.dart';
+part 'engine/canvaskit/render_canvas.dart';
+part 'engine/canvaskit/render_canvas_factory.dart';
 part 'engine/canvaskit/renderer.dart';
 part 'engine/canvaskit/shader.dart';
 part 'engine/canvaskit/surface.dart';
-part 'engine/canvaskit/surface_factory.dart';
 part 'engine/canvaskit/text.dart';
 part 'engine/canvaskit/text_fragmenter.dart';
 part 'engine/canvaskit/util.dart';
@@ -73,6 +74,7 @@ part 'engine/canvaskit/vertices.dart';
 part 'engine/clipboard.dart';
 part 'engine/color_filter.dart';
 part 'engine/configuration.dart';
+part 'engine/display.dart';
 part 'engine/dom.dart';
 part 'engine/embedder.dart';
 part 'engine/engine_canvas.dart';
@@ -81,7 +83,6 @@ part 'engine/font_fallback_data.dart';
 part 'engine/font_fallbacks.dart';
 part 'engine/fonts.dart';
 part 'engine/frame_reference.dart';
-part 'engine/global_styles.dart';
 part 'engine/html/backdrop_filter.dart';
 part 'engine/html/bitmap_canvas.dart';
 part 'engine/html/canvas.dart';
@@ -123,15 +124,19 @@ part 'engine/html/transform.dart';
 part 'engine/html_image_codec.dart';
 part 'engine/image_decoder.dart';
 part 'engine/initialization.dart';
-part 'engine/interval_tree.dart';
+part 'engine/js_interop/js_app.dart';
 part 'engine/js_interop/js_loader.dart';
 part 'engine/js_interop/js_promise.dart';
 part 'engine/js_interop/js_typed_data.dart';
 part 'engine/key_map.g.dart';
 part 'engine/keyboard_binding.dart';
-part 'engine/mouse_cursor.dart';
+part 'engine/layers.dart';
+part 'engine/mouse/context_menu.dart';
+part 'engine/mouse/cursor.dart';
+part 'engine/mouse/prevent_default.dart';
 part 'engine/navigation/history.dart';
 part 'engine/noto_font.dart';
+part 'engine/noto_font_encoding.dart';
 part 'engine/onscreen_logging.dart';
 part 'engine/picture.dart';
 part 'engine/platform_dispatcher.dart';
@@ -148,7 +153,9 @@ part 'engine/raw_keyboard.dart';
 part 'engine/renderer.dart';
 part 'engine/rrect_renderer.dart';
 part 'engine/safe_browser_api.dart';
+part 'engine/scene_builder.dart';
 part 'engine/scene_painting.dart';
+part 'engine/scene_view.dart';
 part 'engine/semantics/accessibility.dart';
 part 'engine/semantics/checkable.dart';
 part 'engine/semantics/dialog.dart';
@@ -156,7 +163,9 @@ part 'engine/semantics/focusable.dart';
 part 'engine/semantics/image.dart';
 part 'engine/semantics/incrementable.dart';
 part 'engine/semantics/label_and_value.dart';
+part 'engine/semantics/link.dart';
 part 'engine/semantics/live_region.dart';
+part 'engine/semantics/platform_view.dart';
 part 'engine/semantics/scrollable.dart';
 part 'engine/semantics/semantics.dart';
 part 'engine/semantics/semantics_helper.dart';
@@ -197,8 +206,11 @@ part 'engine/vector_math.dart';
 part 'engine/view_embedder/dimensions_provider/custom_element_dimensions_provider.dart';
 part 'engine/view_embedder/dimensions_provider/dimensions_provider.dart';
 part 'engine/view_embedder/dimensions_provider/full_page_dimensions_provider.dart';
+part 'engine/view_embedder/dom_manager.dart';
 part 'engine/view_embedder/embedding_strategy/custom_element_embedding_strategy.dart';
 part 'engine/view_embedder/embedding_strategy/embedding_strategy.dart';
 part 'engine/view_embedder/embedding_strategy/full_page_embedding_strategy.dart';
+part 'engine/view_embedder/flutter_view_manager.dart';
 part 'engine/view_embedder/hot_restart_cache_handler.dart';
+part 'engine/view_embedder/style_manager.dart';
 part 'engine/window.dart';
