@@ -119,19 +119,19 @@ extension CanvasKitExtension on CanvasKit {
   // Text decoration enum is embedded in the CanvasKit object itself.
   @JS('NoDecoration')
   external JSNumber get _NoDecoration;
-  double get NoDecoration => _NoDecoration.toDart;
+  double get NoDecoration => _NoDecoration.toDartDouble;
 
   @JS('UnderlineDecoration')
   external JSNumber get _UnderlineDecoration;
-  double get UnderlineDecoration => _UnderlineDecoration.toDart;
+  double get UnderlineDecoration => _UnderlineDecoration.toDartDouble;
 
   @JS('OverlineDecoration')
   external JSNumber get _OverlineDecoration;
-  double get OverlineDecoration => _OverlineDecoration.toDart;
+  double get OverlineDecoration => _OverlineDecoration.toDartDouble;
 
   @JS('LineThroughDecoration')
   external JSNumber get _LineThroughDecoration;
-  double get LineThroughDecoration => _LineThroughDecoration.toDart;
+  double get LineThroughDecoration => _LineThroughDecoration.toDartDouble;
   // End of text decoration enum.
 
   external SkTextDecorationStyleEnum get DecorationStyle;
@@ -148,7 +148,14 @@ extension CanvasKitExtension on CanvasKit {
       DomCanvasElement canvas, SkWebGLContextOptions options);
   double GetWebGLContext(
       DomCanvasElement canvas, SkWebGLContextOptions options) =>
-        _GetWebGLContext(canvas, options).toDart;
+        _GetWebGLContext(canvas, options).toDartDouble;
+
+  @JS('GetWebGLContext')
+  external JSNumber _GetOffscreenWebGLContext(
+      DomOffscreenCanvas canvas, SkWebGLContextOptions options);
+  double GetOffscreenWebGLContext(
+          DomOffscreenCanvas canvas, SkWebGLContextOptions options) =>
+      _GetOffscreenWebGLContext(canvas, options).toDartDouble;
 
   @JS('MakeGrContext')
   external SkGrContext _MakeGrContext(JSNumber glContext);
@@ -188,6 +195,9 @@ extension CanvasKitExtension on CanvasKit {
 
   external SkSurface MakeSWCanvasSurface(DomCanvasElement canvas);
 
+  @JS('MakeSWCanvasSurface')
+  external SkSurface MakeOffscreenSWCanvasSurface(DomOffscreenCanvas canvas);
+
   /// Creates an image from decoded pixels represented as a list of bytes.
   ///
   /// The pixel data must be encoded according to the image info in [info].
@@ -208,14 +218,31 @@ extension CanvasKitExtension on CanvasKit {
   ) => _MakeImage(info, pixels.toJS, bytesPerRow.toJS);
 
   @JS('MakeLazyImageFromTextureSource')
-  external SkImage? _MakeLazyImageFromTextureSource(
+  external SkImage? _MakeLazyImageFromTextureSource2(
     JSAny src,
     SkPartialImageInfo info,
   );
-  SkImage? MakeLazyImageFromTextureSource(
+
+  @JS('MakeLazyImageFromTextureSource')
+  external SkImage? _MakeLazyImageFromTextureSource3(
+    JSAny src,
+    JSNumber zeroSecondArgument,
+    JSBoolean srcIsPremultiplied,
+  );
+
+  SkImage? MakeLazyImageFromTextureSourceWithInfo(
     Object src,
     SkPartialImageInfo info,
-  ) => _MakeLazyImageFromTextureSource(src.toJSAnyShallow, info);
+  ) => _MakeLazyImageFromTextureSource2(src.toJSAnyShallow, info);
+
+  SkImage? MakeLazyImageFromImageBitmap(
+    DomImageBitmap imageBitmap,
+    bool hasPremultipliedAlpha,
+  ) => _MakeLazyImageFromTextureSource3(
+    imageBitmap as JSObject,
+    0.toJS,
+    hasPremultipliedAlpha.toJS,
+  );
 }
 
 @JS('window.CanvasKitInit')
@@ -275,11 +302,11 @@ extension SkSurfaceExtension on SkSurface {
 
   @JS('width')
   external JSNumber _width();
-  double width() => _width().toDart;
+  double width() => _width().toDartDouble;
 
   @JS('height')
   external JSNumber _height();
-  double height() => _height().toDart;
+  double height() => _height().toDartDouble;
 
   external JSVoid dispose();
   external SkImage makeImageSnapshot();
@@ -316,7 +343,7 @@ class SkFontSlant {}
 extension SkFontSlantExtension on SkFontSlant {
   @JS('value')
   external JSNumber get _value;
-  double get value => _value.toDart;
+  double get value => _value.toDartDouble;
 }
 
 final List<SkFontSlant> _skFontSlants = <SkFontSlant>[
@@ -352,7 +379,7 @@ class SkFontWeight {}
 extension SkFontWeightExtension on SkFontWeight {
   @JS('value')
   external JSNumber get _value;
-  double get value => _value.toDart;
+  double get value => _value.toDartDouble;
 }
 
 final List<SkFontWeight> _skFontWeights = <SkFontWeight>[
@@ -387,7 +414,7 @@ class SkAffinity {}
 extension SkAffinityExtension on SkAffinity {
   @JS('value')
   external JSNumber get _value;
-  double get value => _value.toDart;
+  double get value => _value.toDartDouble;
 }
 
 final List<SkAffinity> _skAffinitys = <SkAffinity>[
@@ -415,7 +442,7 @@ class SkTextDirection {}
 extension SkTextDirectionExtension on SkTextDirection {
   @JS('value')
   external JSNumber get _value;
-  double get value => _value.toDart;
+  double get value => _value.toDartDouble;
 }
 
 // Flutter enumerates text directions as RTL, LTR, while CanvasKit
@@ -449,7 +476,7 @@ class SkTextAlign {}
 extension SkTextAlignExtension on SkTextAlign {
   @JS('value')
   external JSNumber get _value;
-  double get value => _value.toDart;
+  double get value => _value.toDartDouble;
 }
 
 final List<SkTextAlign> _skTextAligns = <SkTextAlign>[
@@ -483,7 +510,7 @@ class SkTextHeightBehavior {}
 extension SkTextHeightBehaviorExtension on SkTextHeightBehavior {
   @JS('value')
   external JSNumber get _value;
-  double get value => _value.toDart;
+  double get value => _value.toDartDouble;
 }
 
 final List<SkTextHeightBehavior> _skTextHeightBehaviors =
@@ -520,7 +547,7 @@ class SkRectHeightStyle {}
 extension SkRectHeightStyleExtension on SkRectHeightStyle {
   @JS('value')
   external JSNumber get _value;
-  double get value => _value.toDart;
+  double get value => _value.toDartDouble;
 }
 
 final List<SkRectHeightStyle> _skRectHeightStyles = <SkRectHeightStyle>[
@@ -552,7 +579,7 @@ class SkRectWidthStyle {}
 extension SkRectWidthStyleExtension on SkRectWidthStyle {
   @JS('value')
   external JSNumber get _value;
-  double get value => _value.toDart;
+  double get value => _value.toDartDouble;
 }
 
 final List<SkRectWidthStyle> _skRectWidthStyles = <SkRectWidthStyle>[
@@ -582,7 +609,7 @@ class SkVertexMode {}
 extension SkVertexModeExtension on SkVertexMode {
   @JS('value')
   external JSNumber get _value;
-  double get value => _value.toDart;
+  double get value => _value.toDartDouble;
 }
 
 final List<SkVertexMode> _skVertexModes = <SkVertexMode>[
@@ -612,7 +639,7 @@ class SkPointMode {}
 extension SkPointModeExtension on SkPointMode {
   @JS('value')
   external JSNumber get _value;
-  double get value => _value.toDart;
+  double get value => _value.toDartDouble;
 }
 
 final List<SkPointMode> _skPointModes = <SkPointMode>[
@@ -641,7 +668,7 @@ class SkClipOp {}
 extension SkClipOpExtension on SkClipOp {
   @JS('value')
   external JSNumber get _value;
-  double get value => _value.toDart;
+  double get value => _value.toDartDouble;
 }
 
 final List<SkClipOp> _skClipOps = <SkClipOp>[
@@ -669,7 +696,7 @@ class SkFillType {}
 extension SkFillTypeExtension on SkFillType {
   @JS('value')
   external JSNumber get _value;
-  double get value => _value.toDart;
+  double get value => _value.toDartDouble;
 }
 
 final List<SkFillType> _skFillTypes = <SkFillType>[
@@ -700,7 +727,7 @@ class SkPathOp {}
 extension SkPathOpExtension on SkPathOp {
   @JS('value')
   external JSNumber get _value;
-  double get value => _value.toDart;
+  double get value => _value.toDartDouble;
 }
 
 final List<SkPathOp> _skPathOps = <SkPathOp>[
@@ -733,7 +760,7 @@ class SkBlurStyle {}
 extension SkBlurStyleExtension on SkBlurStyle {
   @JS('value')
   external JSNumber get _value;
-  double get value => _value.toDart;
+  double get value => _value.toDartDouble;
 }
 
 final List<SkBlurStyle> _skBlurStyles = <SkBlurStyle>[
@@ -764,7 +791,7 @@ class SkStrokeCap {}
 extension SkStrokeCapExtension on SkStrokeCap {
   @JS('value')
   external JSNumber get _value;
-  double get value => _value.toDart;
+  double get value => _value.toDartDouble;
 }
 
 final List<SkStrokeCap> _skStrokeCaps = <SkStrokeCap>[
@@ -793,7 +820,7 @@ class SkPaintStyle {}
 extension SkPaintStyleExtension on SkPaintStyle {
   @JS('value')
   external JSNumber get _value;
-  double get value => _value.toDart;
+  double get value => _value.toDartDouble;
 }
 
 final List<SkPaintStyle> _skPaintStyles = <SkPaintStyle>[
@@ -848,7 +875,7 @@ class SkBlendMode {}
 extension SkBlendModeExtension on SkBlendMode {
   @JS('value')
   external JSNumber get _value;
-  double get value => _value.toDart;
+  double get value => _value.toDartDouble;
 }
 
 final List<SkBlendMode> _skBlendModes = <SkBlendMode>[
@@ -904,7 +931,7 @@ class SkStrokeJoin {}
 extension SkStrokeJoinExtension on SkStrokeJoin {
   @JS('value')
   external JSNumber get _value;
-  double get value => _value.toDart;
+  double get value => _value.toDartDouble;
 }
 
 final List<SkStrokeJoin> _skStrokeJoins = <SkStrokeJoin>[
@@ -935,7 +962,7 @@ class SkTileMode {}
 extension SkTileModeExtension on SkTileMode {
   @JS('value')
   external JSNumber get _value;
-  double get value => _value.toDart;
+  double get value => _value.toDartDouble;
 }
 
 final List<SkTileMode> _skTileModes = <SkTileMode>[
@@ -965,7 +992,7 @@ class SkFilterMode {}
 extension SkFilterModeExtension on SkFilterMode {
   @JS('value')
   external JSNumber get _value;
-  double get value => _value.toDart;
+  double get value => _value.toDartDouble;
 }
 
 SkFilterMode toSkFilterMode(ui.FilterQuality filterQuality) {
@@ -991,7 +1018,7 @@ class SkMipmapMode {}
 extension SkMipmapModeExtension on SkMipmapMode {
   @JS('value')
   external JSNumber get _value;
-  double get value => _value.toDart;
+  double get value => _value.toDartDouble;
 }
 
 SkMipmapMode toSkMipmapMode(ui.FilterQuality filterQuality) {
@@ -1017,7 +1044,7 @@ class SkAlphaType {}
 extension SkAlphaTypeExtension on SkAlphaType {
   @JS('value')
   external JSNumber get _value;
-  double get value => _value.toDart;
+  double get value => _value.toDartDouble;
 }
 
 @JS()
@@ -1045,7 +1072,7 @@ class SkColorType {}
 extension SkColorTypeExtension on SkColorType {
   @JS('value')
   external JSNumber get _value;
-  double get value => _value.toDart;
+  double get value => _value.toDartDouble;
 }
 
 @JS()
@@ -1056,31 +1083,31 @@ class SkAnimatedImage {}
 extension SkAnimatedImageExtension on SkAnimatedImage {
   @JS('getFrameCount')
   external JSNumber _getFrameCount();
-  double getFrameCount() => _getFrameCount().toDart;
+  double getFrameCount() => _getFrameCount().toDartDouble;
 
   @JS('getRepetitionCount')
   external JSNumber _getRepetitionCount();
-  double getRepetitionCount() => _getRepetitionCount().toDart;
+  double getRepetitionCount() => _getRepetitionCount().toDartDouble;
 
   /// Returns duration in milliseconds.
   @JS('currentFrameDuration')
   external JSNumber _currentFrameDuration();
-  double currentFrameDuration() => _currentFrameDuration().toDart;
+  double currentFrameDuration() => _currentFrameDuration().toDartDouble;
 
   /// Advances to the next frame and returns its duration in milliseconds.
   @JS('decodeNextFrame')
   external JSNumber _decodeNextFrame();
-  double decodeNextFrame() => _decodeNextFrame().toDart;
+  double decodeNextFrame() => _decodeNextFrame().toDartDouble;
 
   external SkImage makeImageAtCurrentFrame();
 
   @JS('width')
   external JSNumber _width();
-  double width() => _width().toDart;
+  double width() => _width().toDartDouble;
 
   @JS('height')
   external JSNumber _height();
-  double height() => _height().toDart;
+  double height() => _height().toDartDouble;
 
   /// Deletes the C++ object.
   ///
@@ -1102,11 +1129,11 @@ extension SkImageExtension on SkImage {
 
   @JS('width')
   external JSNumber _width();
-  double width() => _width().toDart;
+  double width() => _width().toDartDouble;
 
   @JS('height')
   external JSNumber _height();
-  double height() => _height().toDart;
+  double height() => _height().toDartDouble;
 
   @JS('makeShaderCubic')
   external SkShader _makeShaderCubic(
@@ -1461,6 +1488,12 @@ class SkImageFilter {}
 
 extension SkImageFilterExtension on SkImageFilter {
   external JSVoid delete();
+
+
+  @JS('getOutputBounds')
+  external JSInt32Array _getOutputBounds(JSFloat32Array bounds);
+  Int32List getOutputBounds(Float32List bounds) =>
+      _getOutputBounds(bounds.toJS).toDart;
 }
 
 @JS()
@@ -1613,7 +1646,7 @@ extension SkFloat32ListExtension on SkFloat32List {
   /// The number of objects this pointer refers to.
   @JS('length')
   external JSNumber get _length;
-  double get length => _length.toDart;
+  double get length => _length.toDartDouble;
 
   @JS('length')
   external set _length(JSNumber length);
@@ -1644,7 +1677,7 @@ extension SkUint32ListExtension on SkUint32List {
   /// The number of objects this pointer refers to.
   @JS('length')
   external JSNumber get _length;
-  double get length => _length.toDart;
+  double get length => _length.toDartDouble;
 
   @JS('length')
   external set _length(JSNumber length);
@@ -2073,7 +2106,7 @@ extension SkContourMeasureExtension on SkContourMeasure {
 
   @JS('length')
   external JSNumber _length();
-  double length() => _length().toDart;
+  double length() => _length().toDartDouble;
 
   external JSVoid delete();
 }
@@ -2184,8 +2217,10 @@ class SkPictureRecorder {
 
 extension SkPictureRecorderExtension on SkPictureRecorder {
   @JS('beginRecording')
-  external SkCanvas _beginRecording(JSFloat32Array bounds);
-  SkCanvas beginRecording(Float32List bounds) => _beginRecording(bounds.toJS);
+  external SkCanvas _beginRecording(
+      JSFloat32Array bounds, JSBoolean computeBounds);
+  SkCanvas beginRecording(Float32List bounds) =>
+      _beginRecording(bounds.toJS, true.toJS);
 
   external SkPicture finishRecordingAsPicture();
   external JSVoid delete();
@@ -2502,11 +2537,11 @@ extension SkCanvasExtension on SkCanvas {
 
   @JS('save')
   external JSNumber _save();
-  double save() => _save().toDart;
+  double save() => _save().toDartDouble;
 
   @JS('getSaveCount')
   external JSNumber _getSaveCount();
-  double getSaveCount() => _getSaveCount().toDart;
+  double getSaveCount() => _getSaveCount().toDartDouble;
 
   @JS('saveLayer')
   external JSVoid _saveLayer(
@@ -2583,6 +2618,14 @@ class SkPicture {}
 
 extension SkPictureExtension on SkPicture {
   external JSVoid delete();
+
+  @JS('cullRect')
+  external JSFloat32Array _cullRect();
+  Float32List cullRect() => _cullRect().toDart;
+
+  @JS('approximateBytesUsed')
+  external JSNumber _approximateBytesUsed();
+  int approximateBytesUsed() => _approximateBytesUsed().toDartInt;
 }
 
 @JS()
@@ -2711,6 +2754,8 @@ extension SkParagraphStylePropertiesExtension on SkParagraphStyleProperties {
   @JS('replaceTabCharacters')
   external set _replaceTabCharacters(JSBoolean? bool);
   set replaceTabCharacters(bool? bool) => _replaceTabCharacters = bool?.toJS;
+
+  external set applyRoundingHack(bool applyRoundingHack);
 }
 
 @JS()
@@ -2736,7 +2781,7 @@ class SkTextDecorationStyle {}
 extension SkTextDecorationStyleExtension on SkTextDecorationStyle {
   @JS('value')
   external JSNumber get _value;
-  double get value => _value.toDart;
+  double get value => _value.toDartDouble;
 }
 
 final List<SkTextDecorationStyle> _skTextDecorationStyles =
@@ -2768,7 +2813,7 @@ class SkTextBaseline {}
 extension SkTextBaselineExtension on SkTextBaseline {
   @JS('value')
   external JSNumber get _value;
-  double get value => _value.toDart;
+  double get value => _value.toDartDouble;
 }
 
 final List<SkTextBaseline> _skTextBaselines = <SkTextBaseline>[
@@ -2800,7 +2845,7 @@ class SkPlaceholderAlignment {}
 extension SkPlaceholderAlignmentExtension on SkPlaceholderAlignment {
   @JS('value')
   external JSNumber get _value;
-  double get value => _value.toDart;
+  double get value => _value.toDartDouble;
 }
 
 final List<SkPlaceholderAlignment> _skPlaceholderAlignments =
@@ -2886,7 +2931,7 @@ extension SkTextStylePropertiesExtension on SkTextStyleProperties {
   external set fontStyle(SkFontStyle? value);
 
   @JS('shadows')
-  external set _shadows(JSArray? value);
+  external set _shadows(JSArray<JSAny?>? value);
   set shadows(List<SkTextShadow>? value) =>
       // TODO(joshualitt): remove this cast when we reify JS types on JS
       // backends.
@@ -2894,7 +2939,7 @@ extension SkTextStylePropertiesExtension on SkTextStyleProperties {
       _shadows = (value as List<JSAny>?)?.toJS;
 
   @JS('fontFeatures')
-  external set _fontFeatures(JSArray? value);
+  external set _fontFeatures(JSArray<JSAny?>? value);
   set fontFeatures(List<SkFontFeature>? value) =>
       // TODO(joshualitt): remove this cast when we reify JS types on JS
       // backends.
@@ -2902,7 +2947,7 @@ extension SkTextStylePropertiesExtension on SkTextStyleProperties {
       _fontFeatures = (value as List<JSAny>?)?.toJS;
 
   @JS('fontVariations')
-  external set _fontVariations(JSArray? value);
+  external set _fontVariations(JSArray<JSAny?>? value);
   set fontVariations(List<SkFontVariation>? value) =>
       // TODO(joshualitt): remove this cast when we reify JS types on JS
       // backends.
@@ -3090,19 +3135,19 @@ class SkLineMetrics {}
 extension SkLineMetricsExtension on SkLineMetrics {
   @JS('startIndex')
   external JSNumber get _startIndex;
-  double get startIndex => _startIndex.toDart;
+  double get startIndex => _startIndex.toDartDouble;
 
   @JS('endIndex')
   external JSNumber get _endIndex;
-  double get endIndex => _endIndex.toDart;
+  double get endIndex => _endIndex.toDartDouble;
 
   @JS('endExcludingWhitespaces')
   external JSNumber get _endExcludingWhitespaces;
-  double get endExcludingWhitespaces => _endExcludingWhitespaces.toDart;
+  double get endExcludingWhitespaces => _endExcludingWhitespaces.toDartDouble;
 
   @JS('endIncludingNewline')
   external JSNumber get _endIncludingNewline;
-  double get endIncludingNewline => _endIncludingNewline.toDart;
+  double get endIncludingNewline => _endIncludingNewline.toDartDouble;
 
   @JS('isHardBreak')
   external JSBoolean get _isHardBreak;
@@ -3110,31 +3155,54 @@ extension SkLineMetricsExtension on SkLineMetrics {
 
   @JS('ascent')
   external JSNumber get _ascent;
-  double get ascent => _ascent.toDart;
+  double get ascent => _ascent.toDartDouble;
 
   @JS('descent')
   external JSNumber get _descent;
-  double get descent => _descent.toDart;
+  double get descent => _descent.toDartDouble;
 
   @JS('height')
   external JSNumber get _height;
-  double get height => _height.toDart;
+  double get height => _height.toDartDouble;
 
   @JS('width')
   external JSNumber get _width;
-  double get width => _width.toDart;
+  double get width => _width.toDartDouble;
 
   @JS('left')
   external JSNumber get _left;
-  double get left => _left.toDart;
+  double get left => _left.toDartDouble;
 
   @JS('baseline')
   external JSNumber get _baseline;
-  double get baseline => _baseline.toDart;
+  double get baseline => _baseline.toDartDouble;
 
   @JS('lineNumber')
   external JSNumber get _lineNumber;
-  double get lineNumber => _lineNumber.toDart;
+  double get lineNumber => _lineNumber.toDartDouble;
+}
+
+@JS()
+@anonymous
+@staticInterop
+class SkGlyphClusterInfo {}
+
+extension SkGlyphClusterInfoExtension on SkGlyphClusterInfo {
+  @JS('graphemeLayoutBounds')
+  external JSArray<JSAny?> get _bounds;
+
+  @JS('dir')
+  external SkTextDirection get _direction;
+
+  @JS('graphemeClusterTextRange')
+  external SkTextRange get _textRange;
+
+  ui.GlyphInfo get _glyphInfo {
+    final List<JSNumber> list = _bounds.toDart.cast<JSNumber>();
+    final ui.Rect bounds = ui.Rect.fromLTRB(list[0].toDartDouble, list[1].toDartDouble, list[2].toDartDouble, list[3].toDartDouble);
+    final ui.TextRange textRange = ui.TextRange(start: _textRange.start.toInt(), end: _textRange.end.toInt());
+    return ui.GlyphInfo(bounds, textRange, ui.TextDirection.values[_direction.value.toInt()]);
+  }
 }
 
 @JS()
@@ -3162,7 +3230,7 @@ class SkParagraph {}
 extension SkParagraphExtension on SkParagraph {
   @JS('getAlphabeticBaseline')
   external JSNumber _getAlphabeticBaseline();
-  double getAlphabeticBaseline() => _getAlphabeticBaseline().toDart;
+  double getAlphabeticBaseline() => _getAlphabeticBaseline().toDartDouble;
 
   @JS('didExceedMaxLines')
   external JSBoolean _didExceedMaxLines();
@@ -3170,35 +3238,47 @@ extension SkParagraphExtension on SkParagraph {
 
   @JS('getHeight')
   external JSNumber _getHeight();
-  double getHeight() => _getHeight().toDart;
+  double getHeight() => _getHeight().toDartDouble;
 
   @JS('getIdeographicBaseline')
   external JSNumber _getIdeographicBaseline();
-  double getIdeographicBaseline() => _getIdeographicBaseline().toDart;
+  double getIdeographicBaseline() => _getIdeographicBaseline().toDartDouble;
 
   @JS('getLineMetrics')
-  external JSArray _getLineMetrics();
+  external JSArray<JSAny?> _getLineMetrics();
   List<SkLineMetrics> getLineMetrics() =>
       _getLineMetrics().toDart.cast<SkLineMetrics>();
 
+  @JS('getLineMetricsAt')
+  external SkLineMetrics? _getLineMetricsAt(JSNumber index);
+  SkLineMetrics? getLineMetricsAt(double index) => _getLineMetricsAt(index.toJS);
+
+  @JS('getNumberOfLines')
+  external JSNumber _getNumberOfLines();
+  double getNumberOfLines() => _getNumberOfLines().toDartDouble;
+
+  @JS('getLineNumberAt')
+  external JSNumber _getLineNumberAt(JSNumber index);
+  double getLineNumberAt(double index) => _getLineNumberAt(index.toJS).toDartDouble;
+
   @JS('getLongestLine')
   external JSNumber _getLongestLine();
-  double getLongestLine() => _getLongestLine().toDart;
+  double getLongestLine() => _getLongestLine().toDartDouble;
 
   @JS('getMaxIntrinsicWidth')
   external JSNumber _getMaxIntrinsicWidth();
-  double getMaxIntrinsicWidth() => _getMaxIntrinsicWidth().toDart;
+  double getMaxIntrinsicWidth() => _getMaxIntrinsicWidth().toDartDouble;
 
   @JS('getMinIntrinsicWidth')
   external JSNumber _getMinIntrinsicWidth();
-  double getMinIntrinsicWidth() => _getMinIntrinsicWidth().toDart;
+  double getMinIntrinsicWidth() => _getMinIntrinsicWidth().toDartDouble;
 
   @JS('getMaxWidth')
   external JSNumber _getMaxWidth();
-  double getMaxWidth() => _getMaxWidth().toDart;
+  double getMaxWidth() => _getMaxWidth().toDartDouble;
 
   @JS('getRectsForRange')
-  external JSArray _getRectsForRange(
+  external JSArray<JSAny?> _getRectsForRange(
     JSNumber start,
     JSNumber end,
     SkRectHeightStyle heightStyle,
@@ -3213,7 +3293,7 @@ extension SkParagraphExtension on SkParagraph {
                          widthStyle).toDart.cast<SkRectWithDirection>();
 
   @JS('getRectsForPlaceholders')
-  external JSArray _getRectsForPlaceholders();
+  external JSArray<JSAny?> _getRectsForPlaceholders();
   List<SkRectWithDirection> getRectsForPlaceholders() =>
       _getRectsForPlaceholders().toDart.cast<SkRectWithDirection>();
 
@@ -3227,6 +3307,14 @@ extension SkParagraphExtension on SkParagraph {
     double y,
   ) => _getGlyphPositionAtCoordinate(x.toJS, y.toJS);
 
+  @JS('getGlyphInfoAt')
+  external SkGlyphClusterInfo? _getGlyphInfoAt(JSNumber position);
+  ui.GlyphInfo? getGlyphInfoAt(double position) => _getGlyphInfoAt(position.toJS)?._glyphInfo;
+
+  @JS('getClosestGlyphInfoAtCoordinate')
+  external SkGlyphClusterInfo? _getClosestGlyphInfoAtCoordinate(JSNumber x, JSNumber y);
+  ui.GlyphInfo? getClosestGlyphInfoAt(double x, double y) => _getClosestGlyphInfoAtCoordinate(x.toJS, y.toJS)?._glyphInfo;
+
   @JS('getWordBoundary')
   external SkTextRange _getWordBoundary(JSNumber position);
   SkTextRange getWordBoundary(double position) =>
@@ -3248,7 +3336,7 @@ extension SkTextPositionExtnsion on SkTextPosition {
 
   @JS('pos')
   external JSNumber get _pos;
-  double get pos => _pos.toDart;
+  double get pos => _pos.toDartDouble;
 }
 
 @JS()
@@ -3258,11 +3346,11 @@ class SkTextRange {}
 extension SkTextRangeExtension on SkTextRange {
   @JS('start')
   external JSNumber get _start;
-  double get start => _start.toDart;
+  double get start => _start.toDartDouble;
 
   @JS('end')
   external JSNumber get _end;
-  double get end => _end.toDart;
+  double get end => _end.toDartDouble;
 }
 
 @JS()
@@ -3380,7 +3468,7 @@ class SkData {}
 extension SkDataExtension on SkData {
   @JS('size')
   external JSNumber _size();
-  double size() => _size().toDart;
+  double size() => _size().toDartDouble;
 
   @JS('isEmpty')
   external JSBoolean _isEmpty();
@@ -3424,7 +3512,7 @@ extension SkImageInfoExtension on SkImageInfo {
 
   @JS('height')
   external JSNumber get _height;
-  double get height => _height.toDart;
+  double get height => _height.toDartDouble;
 
   @JS('isEmpty')
   external JSBoolean get _isEmpty;
@@ -3440,7 +3528,7 @@ extension SkImageInfoExtension on SkImageInfo {
 
   @JS('width')
   external JSNumber get _width;
-  double get width => _width.toDart;
+  double get width => _width.toDartDouble;
 
   external SkImageInfo makeAlphaType(SkAlphaType alphaType);
   external SkImageInfo makeColorSpace(ColorSpace colorSpace);
@@ -3483,11 +3571,11 @@ extension SkPartialImageInfoExtension on SkPartialImageInfo {
 
   @JS('height')
   external JSNumber get _height;
-  double get height => _height.toDart;
+  double get height => _height.toDartDouble;
 
   @JS('width')
   external JSNumber get _width;
-  double get width => _width.toDart;
+  double get width => _width.toDartDouble;
 }
 
 @JS('window.flutterCanvasKit.RuntimeEffect')
