@@ -91,8 +91,8 @@ class SurfaceSceneBuilder implements ui.SceneBuilder {
       // scene to devicepixelratio. Use identity instead since CSS uses
       // logical device pixels.
       if (!ui_web.debugEmulateFlutterTesterEnvironment) {
-        assert(matrix4[0] == window.devicePixelRatio &&
-            matrix4[5] == window.devicePixelRatio);
+        assert(matrix4[0] == EngineFlutterDisplay.instance.devicePixelRatio &&
+            matrix4[5] == EngineFlutterDisplay.instance.devicePixelRatio);
       }
       matrix = Matrix4.identity().storage;
     } else {
@@ -307,7 +307,7 @@ class SurfaceSceneBuilder implements ui.SceneBuilder {
   /// overlay or not.
   ///
   /// We use this to avoid spamming the console with redundant warning messages.
-  static bool _webOnlyDidWarnAboutPerformanceOverlay = false;
+  static bool _didWarnAboutPerformanceOverlay = false;
 
   void _addPerformanceOverlay(
     int enabledOptions,
@@ -316,8 +316,8 @@ class SurfaceSceneBuilder implements ui.SceneBuilder {
     double top,
     double bottom,
   ) {
-    if (!_webOnlyDidWarnAboutPerformanceOverlay) {
-      _webOnlyDidWarnAboutPerformanceOverlay = true;
+    if (!_didWarnAboutPerformanceOverlay) {
+      _didWarnAboutPerformanceOverlay = true;
       printWarning("The performance overlay isn't supported on the web");
     }
   }
