@@ -15,6 +15,9 @@ ui.VoidCallback? scheduleFrameCallback;
 typedef HighContrastListener = void Function(bool enabled);
 typedef _KeyDataResponseCallback = void Function(bool handled);
 
+const StandardMethodCodec standardCodec = StandardMethodCodec();
+const JSONMethodCodec jsonCodec = JSONMethodCodec();
+
 /// Determines if high contrast is enabled using media query 'forced-colors: active' for Windows
 class HighContrastSupport {
   static HighContrastSupport instance = HighContrastSupport();
@@ -23,7 +26,8 @@ class HighContrastSupport {
   final List<HighContrastListener> _listeners = <HighContrastListener>[];
 
   /// Reference to css media query that indicates whether high contrast is on.
-  final DomMediaQueryList _highContrastMediaQuery = domWindow.matchMedia(_highContrastMediaQueryString);
+  final DomMediaQueryList _highContrastMediaQuery =
+      domWindow.matchMedia(_highContrastMediaQueryString);
   late final DomEventListener _onHighContrastChangeListener =
       createDomEventListener(_onHighContrastChange);
 
@@ -54,24 +58,6 @@ class HighContrastSupport {
   }
 }
 
-class EngineFlutterDisplay extends ui.Display {
-  EngineFlutterDisplay({
-    required this.id,
-    required this.devicePixelRatio,
-    required this.size,
-    required this.refreshRate,
-  });
-
-  @override
-  final int id;
-  @override
-  final double devicePixelRatio;
-  @override
-  final ui.Size size;
-  @override
-  final double refreshRate;
-}
-
 /// Platform event dispatcher.
 ///
 /// This is the central entry point for platform messages and configuration
@@ -86,6 +72,12 @@ class EnginePlatformDispatcher extends ui.PlatformDispatcher {
     _addLocaleChangedListener();
     registerHotRestartListener(dispose);
     _setAppLifecycleState(ui.AppLifecycleState.resumed);
+    viewManager.onViewDisposed.listen((_) {
+      // Send a metrics changed event to the framework when a view is disposed.
+      // View creation/resize is handled by the `_didResize` handler in the
+      // EngineFlutterView itself.
+      invokeOnMetricsChanged();
+    });
   }
 
   /// The [EnginePlatformDispatcher] singleton.
@@ -113,6 +105,7 @@ class EnginePlatformDispatcher extends ui.PlatformDispatcher {
     _disconnectFontSizeObserver();
     _removeLocaleChangedListener();
     HighContrastSupport.instance.removeListener(_updateHighContrast);
+    viewManager.dispose();
   }
 
   /// Receives all events related to platform configuration changes.
@@ -135,32 +128,20 @@ class EnginePlatformDispatcher extends ui.PlatformDispatcher {
   }
 
   @override
-  Iterable<ui.Display> get displays => <ui.Display>[
-    EngineFlutterDisplay(
-      id: 0,
-      size: ui.Size(domWindow.screen?.width ?? 0, domWindow.screen?.height ?? 0),
-      devicePixelRatio: domWindow.devicePixelRatio,
-      refreshRate: 60,
-    )
+  Iterable<EngineFlutterDisplay> displays = <EngineFlutterDisplay>[
+    EngineFlutterDisplay.instance,
   ];
 
+  late final FlutterViewManager viewManager = FlutterViewManager(this);
+
   /// The current list of windows.
   @override
-  Iterable<ui.FlutterView> get views => viewData.values;
-  final Map<int, ui.FlutterView> viewData = <int, ui.FlutterView>{};
+  Iterable<EngineFlutterView> get views => viewManager.views;
 
-  /// Returns the [FlutterView] with the provided ID if one exists, or null
+  /// Returns the [EngineFlutterView] with the provided ID if one exists, or null
   /// otherwise.
   @override
-  ui.FlutterView? view({required int id}) => viewData[id];
-
-  /// A map of opaque platform window identifiers to window configurations.
-  ///
-  /// This should be considered a protected member, only to be used by
-  /// [PlatformDispatcher] subclasses.
-  Map<Object, ViewConfiguration> get windowConfigurations => _windowConfigurations;
-  final Map<Object, ViewConfiguration> _windowConfigurations =
-      <Object, ViewConfiguration>{};
+  EngineFlutterView? view({required int id}) => viewManager[id];
 
   /// The [FlutterView] provided by the engine if the platform is unable to
   /// create windows, or, for backwards compatibility.
@@ -188,7 +169,8 @@ class EnginePlatformDispatcher extends ui.PlatformDispatcher {
   /// * [PlatformDisptacher.views] for a list of all [FlutterView]s provided
   ///   by the platform.
   @override
-  ui.FlutterView? get implicitView => viewData[kImplicitViewId];
+  EngineFlutterWindow? get implicitView =>
+      viewManager[kImplicitViewId] as EngineFlutterWindow?;
 
   /// A callback that is invoked whenever the platform's [devicePixelRatio],
   /// [physicalSize], [padding], [viewInsets], or [systemGestureInsets]
@@ -225,12 +207,9 @@ class EnginePlatformDispatcher extends ui.PlatformDispatcher {
     }
   }
 
-  /// Returns device pixel ratio returned by browser.
-  static double get browserDevicePixelRatio {
-    final double ratio = domWindow.devicePixelRatio;
-    // Guard against WebOS returning 0.
-    return (ratio == 0.0) ? 1.0 : ratio;
-  }
+  /// A set of views which have rendered in the current `onBeginFrame` or
+  /// `onDrawFrame` scope.
+  Set<ui.FlutterView>? _viewsRenderedInCurrentFrame;
 
   /// A callback invoked when any window begins a frame.
   ///
@@ -254,7 +233,9 @@ class EnginePlatformDispatcher extends ui.PlatformDispatcher {
   /// Engine code should use this method instead of the callback directly.
   /// Otherwise zones won't work properly.
   void invokeOnBeginFrame(Duration duration) {
+    _viewsRenderedInCurrentFrame = <ui.FlutterView>{};
     invoke1<Duration>(_onBeginFrame, _onBeginFrameZone, duration);
+    _viewsRenderedInCurrentFrame = null;
   }
 
   /// A callback that is invoked for each frame after [onBeginFrame] has
@@ -275,7 +256,9 @@ class EnginePlatformDispatcher extends ui.PlatformDispatcher {
   /// Engine code should use this method instead of the callback directly.
   /// Otherwise zones won't work properly.
   void invokeOnDrawFrame() {
+    _viewsRenderedInCurrentFrame = <ui.FlutterView>{};
     invoke(_onDrawFrame, _onDrawFrameZone);
+    _viewsRenderedInCurrentFrame = null;
   }
 
   /// A callback that is invoked when pointer data is available.
@@ -456,8 +439,6 @@ class EnginePlatformDispatcher extends ui.PlatformDispatcher {
     };
   }
 
-  PlatformViewMessageHandler? _platformViewMessageHandler;
-
   void _sendPlatformMessage(
     String name,
     ByteData? data,
@@ -493,11 +474,9 @@ class EnginePlatformDispatcher extends ui.PlatformDispatcher {
     }
 
     switch (name) {
-
       /// This should be in sync with shell/common/shell.cc
       case 'flutter/skia':
-        const MethodCodec codec = JSONMethodCodec();
-        final MethodCall decoded = codec.decodeMethodCall(data);
+        final MethodCall decoded = jsonCodec.decodeMethodCall(data);
         switch (decoded.method) {
           case 'Skia.setResourceCacheMaxBytes':
             if (renderer is CanvasKitRenderer) {
@@ -506,13 +485,14 @@ class EnginePlatformDispatcher extends ui.PlatformDispatcher {
                 'Argument to Skia.setResourceCacheMaxBytes must be an int, but was ${decoded.arguments.runtimeType}',
               );
               final int cacheSizeInBytes = decoded.arguments as int;
-              CanvasKitRenderer.instance.resourceCacheMaxBytes = cacheSizeInBytes;
+              CanvasKitRenderer.instance.resourceCacheMaxBytes =
+                  cacheSizeInBytes;
             }
 
             // Also respond in HTML mode. Otherwise, apps would have to detect
             // CanvasKit vs HTML before invoking this method.
             replyToPlatformMessage(
-                callback, codec.encodeSuccessEnvelope(<bool>[true]));
+                callback, jsonCodec.encodeSuccessEnvelope(<bool>[true]));
         }
         return;
 
@@ -522,51 +502,69 @@ class EnginePlatformDispatcher extends ui.PlatformDispatcher {
         return;
 
       case 'flutter/platform':
-        const MethodCodec codec = JSONMethodCodec();
-        final MethodCall decoded = codec.decodeMethodCall(data);
+        final MethodCall decoded = jsonCodec.decodeMethodCall(data);
         switch (decoded.method) {
           case 'SystemNavigator.pop':
             // TODO(a-wallen): As multi-window support expands, the pop call
             // will need to include the view ID. Right now only one view is
             // supported.
-            (viewData[kImplicitViewId]! as EngineFlutterWindow)
-                .browserHistory
-                .exit()
-                .then((_) {
+            //
+            // TODO(mdebbar): What should we do in multi-view mode?
+            //                https://github.com/flutter/flutter/issues/139174
+            if (implicitView != null) {
+              implicitView!.browserHistory.exit().then((_) {
+                replyToPlatformMessage(
+                  callback,
+                  jsonCodec.encodeSuccessEnvelope(true),
+                );
+              });
+            } else {
               replyToPlatformMessage(
-                  callback, codec.encodeSuccessEnvelope(true));
-            });
+                callback,
+                jsonCodec.encodeSuccessEnvelope(true),
+              );
+            }
             return;
           case 'HapticFeedback.vibrate':
             final String? type = decoded.arguments as String?;
             vibrate(_getHapticFeedbackDuration(type));
-            replyToPlatformMessage(callback, codec.encodeSuccessEnvelope(true));
+            replyToPlatformMessage(
+                callback, jsonCodec.encodeSuccessEnvelope(true));
             return;
           case 'SystemChrome.setApplicationSwitcherDescription':
-            final Map<String, Object?> arguments = decoded.arguments as Map<String, Object?>;
+            final Map<String, Object?> arguments =
+                decoded.arguments as Map<String, Object?>;
             final String label = arguments['label'] as String? ?? '';
             // TODO(web): Stop setting the color from here, https://github.com/flutter/flutter/issues/123365
-            final int primaryColor = arguments['primaryColor'] as int? ?? 0xFF000000;
+            final int primaryColor =
+                arguments['primaryColor'] as int? ?? 0xFF000000;
             domDocument.title = label;
             setThemeColor(ui.Color(primaryColor));
-            replyToPlatformMessage(callback, codec.encodeSuccessEnvelope(true));
+            replyToPlatformMessage(
+                callback, jsonCodec.encodeSuccessEnvelope(true));
             return;
           case 'SystemChrome.setSystemUIOverlayStyle':
-            final Map<String, Object?> arguments = decoded.arguments as Map<String, Object?>;
+            final Map<String, Object?> arguments =
+                decoded.arguments as Map<String, Object?>;
             final int? statusBarColor = arguments['statusBarColor'] as int?;
-            setThemeColor(statusBarColor == null ? null : ui.Color(statusBarColor));
-            replyToPlatformMessage(callback, codec.encodeSuccessEnvelope(true));
+            setThemeColor(
+                statusBarColor == null ? null : ui.Color(statusBarColor));
+            replyToPlatformMessage(
+                callback, jsonCodec.encodeSuccessEnvelope(true));
             return;
           case 'SystemChrome.setPreferredOrientations':
             final List<dynamic> arguments = decoded.arguments as List<dynamic>;
-            flutterViewEmbedder.setPreferredOrientation(arguments).then((bool success) {
+            ScreenOrientation.instance
+                .setPreferredOrientation(arguments)
+                .then((bool success) {
               replyToPlatformMessage(
-                  callback, codec.encodeSuccessEnvelope(success));
+                  callback, jsonCodec.encodeSuccessEnvelope(success));
             });
             return;
           case 'SystemSound.play':
             // There are no default system sounds on web.
-            replyToPlatformMessage(callback, codec.encodeSuccessEnvelope(true));
+            replyToPlatformMessage(
+                callback, jsonCodec.encodeSuccessEnvelope(true));
             return;
           case 'Clipboard.setData':
             ClipboardMessageHandler().setDataMethodCall(decoded, callback);
@@ -574,6 +572,9 @@ class EnginePlatformDispatcher extends ui.PlatformDispatcher {
           case 'Clipboard.getData':
             ClipboardMessageHandler().getDataMethodCall(callback);
             return;
+          case 'Clipboard.hasStrings':
+            ClipboardMessageHandler().hasStringsMethodCall(callback);
+            return;
         }
 
       // Dispatched by the bindings to delay service worker initialization.
@@ -586,52 +587,57 @@ class EnginePlatformDispatcher extends ui.PlatformDispatcher {
         return;
 
       case 'flutter/contextmenu':
-        const MethodCodec codec = JSONMethodCodec();
-        final MethodCall decoded = codec.decodeMethodCall(data);
+        final MethodCall decoded = jsonCodec.decodeMethodCall(data);
         switch (decoded.method) {
           case 'enableContextMenu':
-            flutterViewEmbedder.enableContextMenu();
-            replyToPlatformMessage(callback, codec.encodeSuccessEnvelope(true));
+            implicitView!.contextMenu.enable();
+            replyToPlatformMessage(
+                callback, jsonCodec.encodeSuccessEnvelope(true));
             return;
           case 'disableContextMenu':
-            flutterViewEmbedder.disableContextMenu();
-            replyToPlatformMessage(callback, codec.encodeSuccessEnvelope(true));
+            implicitView!.contextMenu.disable();
+            replyToPlatformMessage(
+                callback, jsonCodec.encodeSuccessEnvelope(true));
             return;
         }
         return;
 
       case 'flutter/mousecursor':
-        const MethodCodec codec = StandardMethodCodec();
-        final MethodCall decoded = codec.decodeMethodCall(data);
-        final Map<dynamic, dynamic> arguments = decoded.arguments as Map<dynamic, dynamic>;
+        final MethodCall decoded = standardCodec.decodeMethodCall(data);
+        final Map<dynamic, dynamic> arguments =
+            decoded.arguments as Map<dynamic, dynamic>;
         switch (decoded.method) {
           case 'activateSystemCursor':
-            MouseCursor.instance!.activateSystemCursor(arguments.tryString('kind'));
+            // TODO(mdebbar): Once the framework starts sending us a viewId, we
+            //                should use it to grab the correct view.
+            //                https://github.com/flutter/flutter/issues/140226
+            views.firstOrNull?.mouseCursor
+                .activateSystemCursor(arguments.tryString('kind'));
         }
         return;
 
       case 'flutter/web_test_e2e':
-        const MethodCodec codec = JSONMethodCodec();
         replyToPlatformMessage(
             callback,
-            codec.encodeSuccessEnvelope(
-                _handleWebTestEnd2EndMessage(codec, data)));
+            jsonCodec.encodeSuccessEnvelope(
+                _handleWebTestEnd2EndMessage(jsonCodec, data)));
         return;
 
-      case 'flutter/platform_views':
-        _platformViewMessageHandler ??= PlatformViewMessageHandler(
-          contentManager: platformViewManager,
-          contentHandler: (DomElement content) {
-            flutterViewEmbedder.glassPaneElement.append(content);
-          },
-        );
-        _platformViewMessageHandler!.handlePlatformViewCall(data, callback!);
+      case PlatformViewMessageHandler.channelName:
+        // `arguments` can be a Map<String, Object> for `create`,
+        // but an `int` for `dispose`, hence why `dynamic` everywhere.
+        final MethodCall(:String method, :dynamic arguments) =
+            standardCodec.decodeMethodCall(data);
+        PlatformViewMessageHandler.instance
+            .handlePlatformViewCall(method, arguments, callback!);
         return;
 
       case 'flutter/accessibility':
         // In widget tests we want to bypass processing of platform messages.
         const StandardMessageCodec codec = StandardMessageCodec();
-        flutterViewEmbedder.accessibilityAnnouncements.handleMessage(codec, data);
+        // TODO(yjbanov): Dispatch the announcement to the correct view?
+        //                https://github.com/flutter/flutter/issues/137445
+        implicitView?.accessibilityAnnouncements.handleMessage(codec, data);
         replyToPlatformMessage(callback, codec.encodeMessage(true));
         return;
 
@@ -639,16 +645,23 @@ class EnginePlatformDispatcher extends ui.PlatformDispatcher {
         // TODO(a-wallen): As multi-window support expands, the navigation call
         // will need to include the view ID. Right now only one view is
         // supported.
-        (viewData[kImplicitViewId]! as EngineFlutterWindow)
-            .handleNavigationMessage(data)
-            .then((bool handled) {
-          if (handled) {
-            const MethodCodec codec = JSONMethodCodec();
-            replyToPlatformMessage(callback, codec.encodeSuccessEnvelope(true));
-          } else {
-            callback?.call(null);
-          }
-        });
+        //
+        // TODO(mdebbar): What should we do in multi-view mode?
+        //                https://github.com/flutter/flutter/issues/139174
+        if (implicitView != null) {
+          implicitView!.handleNavigationMessage(data).then((bool handled) {
+            if (handled) {
+              replyToPlatformMessage(
+                callback,
+                jsonCodec.encodeSuccessEnvelope(true),
+              );
+            } else {
+              callback?.call(null);
+            }
+          });
+        } else {
+          callback?.call(null);
+        }
 
         // As soon as Flutter starts taking control of the app navigation, we
         // should reset _defaultRouteName to "/" so it doesn't have any
@@ -668,9 +681,11 @@ class EnginePlatformDispatcher extends ui.PlatformDispatcher {
     replyToPlatformMessage(callback, null);
   }
 
-  Future<void> _handleFlutterAssetsMessage(String url, ui.PlatformMessageResponseCallback? callback) async {
+  Future<void> _handleFlutterAssetsMessage(
+      String url, ui.PlatformMessageResponseCallback? callback) async {
     try {
-      final HttpFetchResponse response = await ui_web.assetManager.loadAsset(url) as HttpFetchResponse;
+      final HttpFetchResponse response =
+          await ui_web.assetManager.loadAsset(url) as HttpFetchResponse;
       final ByteBuffer assetData = await response.asByteBuffer();
       replyToPlatformMessage(callback, assetData.asByteData());
     } catch (error) {
@@ -740,8 +755,27 @@ class EnginePlatformDispatcher extends ui.PlatformDispatcher {
   ///  * [RendererBinding], the Flutter framework class which manages layout and
   ///    painting.
   @override
-  void render(ui.Scene scene, [ui.FlutterView? view]) {
-    renderer.renderScene(scene);
+  Future<void> render(ui.Scene scene, [ui.FlutterView? view]) async {
+    assert(view != null || implicitView != null,
+        'Calling render without a FlutterView');
+    if (view == null && implicitView == null) {
+      // If there is no view to render into, then this is a no-op.
+      return;
+    }
+    final ui.FlutterView viewToRender = view ?? implicitView!;
+
+    // Only render in an `onDrawFrame` or `onBeginFrame` scope. This is checked
+    // by checking if the `_viewsRenderedInCurrentFrame` is non-null and this
+    // view hasn't been rendered already in this scope.
+    final bool shouldRender =
+        _viewsRenderedInCurrentFrame?.add(viewToRender) ?? false;
+    // TODO(harryterkelsen): HTML renderer needs to violate the render rule in
+    // order to perform golden tests in Flutter framework because on the HTML
+    // renderer, golden tests render to DOM and then take a browser screenshot,
+    // https://github.com/flutter/flutter/issues/137073.
+    if (shouldRender || renderer.rendererTag == 'html') {
+      await renderer.renderScene(scene, viewToRender);
+    }
   }
 
   /// Additional accessibility features that may be enabled by the platform.
@@ -788,7 +822,7 @@ class EnginePlatformDispatcher extends ui.PlatformDispatcher {
     call `updateSemantics`.
   ''')
   void updateSemantics(ui.SemanticsUpdate update) {
-    EngineSemanticsOwner.instance.updateSemantics(update);
+    implicitView?.semantics.updateSemantics(update);
   }
 
   /// This is equivalent to `locales.first`, except that it will provide an
@@ -834,11 +868,11 @@ class EnginePlatformDispatcher extends ui.PlatformDispatcher {
     }
     updateLocales(); // First time, for good measure.
     _onLocaleChangedSubscription =
-      DomSubscription(domWindow, 'languagechange', (DomEvent _) {
-        // Update internal config, then propagate the changes.
-        updateLocales();
-        invokeOnLocaleChanged();
-      });
+        DomSubscription(domWindow, 'languagechange', (DomEvent _) {
+      // Update internal config, then propagate the changes.
+      updateLocales();
+      invokeOnLocaleChanged();
+    });
   }
 
   /// Removes the [_onLocaleChangedSubscription].
@@ -969,7 +1003,7 @@ class EnginePlatformDispatcher extends ui.PlatformDispatcher {
     const String styleAttribute = 'style';
 
     _fontSizeObserver = createDomMutationObserver(
-        (JSArray mutations, DomMutationObserver _) {
+        (JSArray<JSAny?> mutations, DomMutationObserver _) {
       for (final JSAny? mutation in mutations.toDart) {
         final DomMutationRecord record = mutation! as DomMutationRecord;
         if (record.type == 'attributes' &&
@@ -995,7 +1029,7 @@ class EnginePlatformDispatcher extends ui.PlatformDispatcher {
   void _setAppLifecycleState(ui.AppLifecycleState state) {
     sendPlatformMessage(
       'flutter/lifecycle',
-      Uint8List.fromList(utf8.encode(state.toString())).buffer.asByteData(),
+      ByteData.sublistView(utf8.encode(state.toString())),
       null,
     );
   }
@@ -1027,7 +1061,8 @@ class EnginePlatformDispatcher extends ui.PlatformDispatcher {
 
   void updateSemanticsEnabled(bool semanticsEnabled) {
     if (semanticsEnabled != this.semanticsEnabled) {
-      configuration = configuration.copyWith(semanticsEnabled: semanticsEnabled);
+      configuration =
+          configuration.copyWith(semanticsEnabled: semanticsEnabled);
       if (_onSemanticsEnabledChanged != null) {
         invokeOnSemanticsEnabledChanged();
       }
@@ -1081,8 +1116,7 @@ class EnginePlatformDispatcher extends ui.PlatformDispatcher {
         : ui.Brightness.light);
 
     _brightnessMediaQueryListener = createDomEventListener((DomEvent event) {
-      final DomMediaQueryListEvent mqEvent =
-          event as DomMediaQueryListEvent;
+      final DomMediaQueryListEvent mqEvent = event as DomMediaQueryListEvent;
       _updatePlatformBrightness(
           mqEvent.matches! ? ui.Brightness.dark : ui.Brightness.light);
     });
@@ -1131,8 +1165,7 @@ class EnginePlatformDispatcher extends ui.PlatformDispatcher {
   ///  * [WidgetsBindingObserver], for a mechanism at the widgets layer to
   ///    observe when this callback is invoked.
   @override
-  ui.VoidCallback? get onSystemFontFamilyChanged =>
-      _onSystemFontFamilyChanged;
+  ui.VoidCallback? get onSystemFontFamilyChanged => _onSystemFontFamilyChanged;
   ui.VoidCallback? _onSystemFontFamilyChanged;
   Zone? _onSystemFontFamilyChangedZone;
   @override
@@ -1184,7 +1217,8 @@ class EnginePlatformDispatcher extends ui.PlatformDispatcher {
   /// The framework invokes this callback in the same zone in which the
   /// callback was set.
   @override
-  ui.SemanticsActionEventCallback? get onSemanticsActionEvent => _onSemanticsActionEvent;
+  ui.SemanticsActionEventCallback? get onSemanticsActionEvent =>
+      _onSemanticsActionEvent;
   ui.SemanticsActionEventCallback? _onSemanticsActionEvent;
   Zone _onSemanticsActionEventZone = Zone.root;
   @override
@@ -1198,12 +1232,14 @@ class EnginePlatformDispatcher extends ui.PlatformDispatcher {
   void invokeOnSemanticsAction(
       int nodeId, ui.SemanticsAction action, ByteData? args) {
     invoke1<ui.SemanticsActionEvent>(
-        _onSemanticsActionEvent, _onSemanticsActionEventZone, ui.SemanticsActionEvent(
-          type: action,
-          nodeId: nodeId,
-          viewId: 0, // TODO(goderbauer): Wire up the real view ID.
-          arguments: args,
-        ),
+      _onSemanticsActionEvent,
+      _onSemanticsActionEventZone,
+      ui.SemanticsActionEvent(
+        type: action,
+        nodeId: nodeId,
+        viewId: 0, // TODO(goderbauer): Wire up the real view ID.
+        arguments: args,
+      ),
     );
   }
 
@@ -1239,7 +1275,7 @@ class EnginePlatformDispatcher extends ui.PlatformDispatcher {
   /// ## iOS
   ///
   /// On iOS, calling
-  /// [`FlutterViewController.setInitialRoute`](/objcdoc/Classes/FlutterViewController.html#/c:objc%28cs%29FlutterViewController%28im%29setInitialRoute:)
+  /// [`FlutterViewController.setInitialRoute`](/ios-embedder/interface_flutter_view_controller.html#a7f269c2da73312f856d42611cc12a33f)
   /// will set this value. The value must be set sufficiently early, i.e. before
   /// the [runApp] call is executed in Dart, for this to have any effect on the
   /// framework. The `application:didFinishLaunchingWithOptions:` method is a
@@ -1252,8 +1288,10 @@ class EnginePlatformDispatcher extends ui.PlatformDispatcher {
   ///    requests from the embedder.
   @override
   String get defaultRouteName {
+    // TODO(mdebbar): What should we do in multi-view mode?
+    //                https://github.com/flutter/flutter/issues/139174
     return _defaultRouteName ??=
-        (viewData[kImplicitViewId]! as EngineFlutterWindow).browserHistory.currentPath;
+        implicitView?.browserHistory.currentPath ?? '/';
   }
 
   /// Lazily initialized when the `defaultRouteName` getter is invoked.
@@ -1278,6 +1316,10 @@ class EnginePlatformDispatcher extends ui.PlatformDispatcher {
 
   @override
   ui.FrameData get frameData => const ui.FrameData.webOnly();
+
+  @override
+  double scaleFontSize(double unscaledFontSize) =>
+      unscaledFontSize * textScaleFactor;
 }
 
 bool _handleWebTestEnd2EndMessage(MethodCodec codec, ByteData? data) {
@@ -1285,7 +1327,7 @@ bool _handleWebTestEnd2EndMessage(MethodCodec codec, ByteData? data) {
   final double ratio = double.parse(decoded.arguments as String);
   switch (decoded.method) {
     case 'setDevicePixelRatio':
-      window.debugOverrideDevicePixelRatio(ratio);
+      EngineFlutterDisplay.instance.debugOverrideDevicePixelRatio(ratio);
       EnginePlatformDispatcher.instance.onMetricsChanged!();
       return true;
   }
@@ -1363,7 +1405,8 @@ const double _defaultRootFontSize = 16.0;
 /// Finds the text scale factor of the browser by looking at the computed style
 /// of the browser's <html> element.
 double findBrowserTextScaleFactor() {
-  final num fontSize = parseFontSize(domDocument.documentElement!) ?? _defaultRootFontSize;
+  final num fontSize =
+      parseFontSize(domDocument.documentElement!) ?? _defaultRootFontSize;
   return fontSize / _defaultRootFontSize;
 }
 
@@ -1371,7 +1414,6 @@ class ViewConfiguration {
   const ViewConfiguration({
     this.view,
     this.devicePixelRatio = 1.0,
-    this.geometry = ui.Rect.zero,
     this.visible = false,
     this.viewInsets = ui.ViewPadding.zero as ViewPadding,
     this.viewPadding = ui.ViewPadding.zero as ViewPadding,
@@ -1382,9 +1424,8 @@ class ViewConfiguration {
   });
 
   ViewConfiguration copyWith({
-    ui.FlutterView? view,
+    EngineFlutterView? view,
     double? devicePixelRatio,
-    ui.Rect? geometry,
     bool? visible,
     ViewPadding? viewInsets,
     ViewPadding? viewPadding,
@@ -1396,7 +1437,6 @@ class ViewConfiguration {
     return ViewConfiguration(
       view: view ?? this.view,
       devicePixelRatio: devicePixelRatio ?? this.devicePixelRatio,
-      geometry: geometry ?? this.geometry,
       visible: visible ?? this.visible,
       viewInsets: viewInsets ?? this.viewInsets,
       viewPadding: viewPadding ?? this.viewPadding,
@@ -1407,9 +1447,8 @@ class ViewConfiguration {
     );
   }
 
-  final ui.FlutterView? view;
+  final EngineFlutterView? view;
   final double devicePixelRatio;
-  final ui.Rect geometry;
   final bool visible;
   final ViewPadding viewInsets;
   final ViewPadding viewPadding;
@@ -1420,7 +1459,7 @@ class ViewConfiguration {
 
   @override
   String toString() {
-    return '$runtimeType[view: $view, geometry: $geometry]';
+    return '$runtimeType[view: $view]';
   }
 }
 
@@ -1447,8 +1486,10 @@ class PlatformConfiguration {
     String? systemFontFamily,
   }) {
     return PlatformConfiguration(
-      accessibilityFeatures: accessibilityFeatures ?? this.accessibilityFeatures,
-      alwaysUse24HourFormat: alwaysUse24HourFormat ?? this.alwaysUse24HourFormat,
+      accessibilityFeatures:
+          accessibilityFeatures ?? this.accessibilityFeatures,
+      alwaysUse24HourFormat:
+          alwaysUse24HourFormat ?? this.alwaysUse24HourFormat,
       semanticsEnabled: semanticsEnabled ?? this.semanticsEnabled,
       platformBrightness: platformBrightness ?? this.platformBrightness,
       textScaleFactor: textScaleFactor ?? this.textScaleFactor,
