@@ -146,8 +146,6 @@ class SemanticsTextEditingStrategy extends DefaultTextEditingStrategy {
       {OnChangeCallback? onChange, OnActionCallback? onAction}) {
     isEnabled = true;
     inputConfiguration = inputConfig;
-    onChange = onChange;
-    onAction = onAction;
     applyConfiguration(inputConfig);
   }
 
@@ -220,6 +218,16 @@ class TextField extends PrimaryRoleManager {
     return editableElement!;
   }
 
+  @override
+  bool focusAsRouteDefault() {
+    final DomHTMLElement? editableElement = this.editableElement;
+    if (editableElement == null) {
+      return false;
+    }
+    editableElement.focus();
+    return true;
+  }
+
   /// Timer that times when to set the location of the input text.
   ///
   /// This is only used for iOS. In iOS, virtual keyboard shifts the screen.
@@ -269,7 +277,7 @@ class TextField extends PrimaryRoleManager {
       ..left = '0'
       ..width = '${semanticsObject.rect!.width}px'
       ..height = '${semanticsObject.rect!.height}px';
-    semanticsObject.element.append(activeEditableElement);
+    append(activeEditableElement);
   }
 
   void _setupDomElement() {
@@ -290,12 +298,21 @@ class TextField extends PrimaryRoleManager {
     _initializeEditableElement();
     activeEditableElement.addEventListener('focus',
         createDomEventListener((DomEvent event) {
-          if (semanticsObject.owner.gestureMode != GestureMode.browserGestures) {
+          if (EngineSemantics.instance.gestureMode != GestureMode.browserGestures) {
             return;
           }
 
           EnginePlatformDispatcher.instance.invokeOnSemanticsAction(
-              semanticsObject.id, ui.SemanticsAction.tap, null);
+              semanticsObject.id, ui.SemanticsAction.didGainAccessibilityFocus, null);
+        }));
+    activeEditableElement.addEventListener('blur',
+        createDomEventListener((DomEvent event) {
+          if (EngineSemantics.instance.gestureMode != GestureMode.browserGestures) {
+            return;
+          }
+
+          EnginePlatformDispatcher.instance.invokeOnSemanticsAction(
+              semanticsObject.id, ui.SemanticsAction.didLoseAccessibilityFocus, null);
         }));
   }
 
@@ -321,22 +338,21 @@ class TextField extends PrimaryRoleManager {
       return;
     }
 
-    semanticsObject.element
-      ..setAttribute('role', 'textbox')
-      ..setAttribute('contenteditable', 'false')
-      ..setAttribute('tabindex', '0');
+    setAttribute('role', 'textbox');
+    setAttribute('contenteditable', 'false');
+    setAttribute('tabindex', '0');
 
     num? lastPointerDownOffsetX;
     num? lastPointerDownOffsetY;
 
-    semanticsObject.element.addEventListener('pointerdown',
+    addEventListener('pointerdown',
         createDomEventListener((DomEvent event) {
           final DomPointerEvent pointerEvent = event as DomPointerEvent;
           lastPointerDownOffsetX = pointerEvent.clientX;
           lastPointerDownOffsetY = pointerEvent.clientY;
         }), true);
 
-    semanticsObject.element.addEventListener('pointerup',
+    addEventListener('pointerup',
         createDomEventListener((DomEvent event) {
       final DomPointerEvent pointerEvent = event as DomPointerEvent;
 
@@ -384,17 +400,17 @@ class TextField extends PrimaryRoleManager {
     // represent the same text field. It will confuse VoiceOver, so `role` needs to
     // be assigned and removed, based on whether or not editableElement exists.
     activeEditableElement.focus();
-    semanticsObject.element.removeAttribute('role');
+    removeAttribute('role');
 
     activeEditableElement.addEventListener('blur',
         createDomEventListener((DomEvent event) {
-      semanticsObject.element.setAttribute('role', 'textbox');
+      setAttribute('role', 'textbox');
       activeEditableElement.remove();
       SemanticsTextEditingStrategy._instance?.deactivate(this);
 
       // Focus on semantics element before removing the editable element, so that
       // the user can continue navigating the page with the assistive technology.
-      semanticsObject.element.focus();
+      element.focus();
       editableElement = null;
     }));
   }
@@ -432,7 +448,7 @@ class TextField extends PrimaryRoleManager {
       }
     }
 
-    final DomElement element = editableElement ?? semanticsObject.element;
+    final DomElement element = editableElement ?? this.element;
     if (semanticsObject.hasLabel) {
       element.setAttribute(
         'aria-label',
