@@ -31,7 +31,7 @@ class CanvasParagraph implements ui.Paragraph {
   final EngineParagraphStyle paragraphStyle;
 
   /// The full textual content of the paragraph.
-  late String plainText;
+  final String plainText;
 
   /// Whether this paragraph can be drawn on a bitmap canvas.
   ///
@@ -85,9 +85,9 @@ class CanvasParagraph implements ui.Paragraph {
     // `30.5` then the TextPainter in the framework will ceil the `30.5` width
     // which will result in a width of `40.0` that's higher than the constraint
     // width.
-    constraints = ui.ParagraphConstraints(
-      width: constraints.width.floorToDouble(),
-    );
+    if (!ui.ParagraphBuilder.shouldDisableRoundingHack) {
+      constraints = ui.ParagraphConstraints(width: constraints.width.floorToDouble());
+    }
 
     if (constraints == _lastUsedConstraints) {
       return;
@@ -196,6 +196,35 @@ class CanvasParagraph implements ui.Paragraph {
     return _layoutService.getPositionForOffset(offset);
   }
 
+  @override
+  ui.GlyphInfo? getClosestGlyphInfoForOffset(ui.Offset offset) => _layoutService.getClosestGlyphInfo(offset);
+
+  @override
+  ui.GlyphInfo? getGlyphInfoAt(int codeUnitOffset) {
+    final int? lineNumber = _findLine(codeUnitOffset, 0, numberOfLines);
+    if (lineNumber == null) {
+      return null;
+    }
+    final ParagraphLine line = lines[lineNumber];
+    final ui.TextRange? range = line.getCharacterRangeAt(codeUnitOffset);
+    if (range == null) {
+      return null;
+    }
+    assert(line.overlapsWith(range.start, range.end));
+    for (final LayoutFragment fragment in line.fragments) {
+      if (fragment.overlapsWith(range.start, range.end)) {
+        // If the grapheme cluster is split into multiple fragments (which really
+        // shouldn't happen but currently if they are in different TextSpans they
+        // don't combine), use the layout box of the first base character as its
+        // layout box has a better chance to be not that far-off.
+        final ui.TextBox textBox = fragment.toTextBox(start: range.start, end: range.end);
+        return ui.GlyphInfo(textBox.toRect(), range, textBox.direction);
+      }
+    }
+    assert(false, 'This should not be reachable.');
+    return null;
+  }
+
   @override
   ui.TextRange getWordBoundary(ui.TextPosition position) {
     final int characterPosition;
@@ -212,17 +241,12 @@ class CanvasParagraph implements ui.Paragraph {
 
   @override
   ui.TextRange getLineBoundary(ui.TextPosition position) {
-    final int index = position.offset;
-
-    int i;
-    for (i = 0; i < lines.length - 1; i++) {
-      final ParagraphLine line = lines[i];
-      if (index >= line.startIndex && index < line.endIndex) {
-        break;
-      }
+    if (lines.isEmpty) {
+      return ui.TextRange.empty;
     }
-
-    final ParagraphLine line = lines[i];
+    final int? lineNumber = getLineNumberAt(position.offset);
+    // Fallback to the last line for backward compatibility.
+    final ParagraphLine line = lineNumber != null ? lines[lineNumber] : lines.last;
     return ui.TextRange(start: line.startIndex, end: line.endIndex - line.trailingNewlines);
   }
 
@@ -231,6 +255,39 @@ class CanvasParagraph implements ui.Paragraph {
     return lines.map((ParagraphLine line) => line.lineMetrics).toList();
   }
 
+  @override
+  EngineLineMetrics? getLineMetricsAt(int lineNumber) {
+    return 0 <= lineNumber && lineNumber < lines.length
+      ? lines[lineNumber].lineMetrics
+      : null;
+  }
+
+  @override
+  int get numberOfLines => lines.length;
+
+  @override
+  int? getLineNumberAt(int codeUnitOffset) => _findLine(codeUnitOffset, 0, lines.length);
+
+  int? _findLine(int codeUnitOffset, int startLine, int endLine) {
+    assert(endLine <= lines.length);
+    final bool isOutOfBounds = endLine <= startLine
+                            || codeUnitOffset < lines[startLine].startIndex
+                            || (endLine < numberOfLines && lines[endLine].startIndex <= codeUnitOffset);
+    if (isOutOfBounds) {
+      return null;
+    }
+
+    if (endLine == startLine + 1) {
+      assert(lines[startLine].startIndex <= codeUnitOffset);
+      assert(endLine == numberOfLines || codeUnitOffset < lines[endLine].startIndex);
+      return codeUnitOffset >= lines[startLine].visibleEndIndex ? null : startLine;
+    }
+    // endLine >= startLine + 2 thus we have
+    // startLine + 1 <= midIndex <= endLine - 1
+    final int midIndex = (startLine + endLine) ~/ 2;
+    return _findLine(codeUnitOffset, midIndex, endLine) ?? _findLine(codeUnitOffset, startLine, midIndex);
+  }
+
   bool _disposed = false;
 
   @override
@@ -365,6 +422,7 @@ abstract class StyleNode {
         letterSpacing: _letterSpacing,
         wordSpacing: _wordSpacing,
         height: _height,
+        leadingDistribution: _leadingDistribution,
         locale: _locale,
         background: _background,
         foreground: _foreground,
@@ -390,6 +448,7 @@ abstract class StyleNode {
   double? get _letterSpacing;
   double? get _wordSpacing;
   double? get _height;
+  ui.TextLeadingDistribution? get _leadingDistribution;
   ui.Locale? get _locale;
   ui.Paint? get _background;
   ui.Paint? get _foreground;
@@ -455,6 +514,9 @@ class ChildStyleNode extends StyleNode {
   @override
   double? get _height => style.height ?? parent._height;
 
+  @override
+  ui.TextLeadingDistribution? get _leadingDistribution => style.leadingDistribution ?? parent._leadingDistribution;
+
   @override
   ui.Locale? get _locale => style.locale ?? parent._locale;
 
@@ -509,7 +571,7 @@ class RootStyleNode extends StyleNode {
   ui.TextBaseline? get _textBaseline => null;
 
   @override
-  String get _fontFamily => paragraphStyle.fontFamily ?? FlutterViewEmbedder.defaultFontFamily;
+  String get _fontFamily => paragraphStyle.fontFamily ?? StyleManager.defaultFontFamily;
 
   @override
   List<String>? get _fontFamilyFallback => null;
@@ -521,7 +583,7 @@ class RootStyleNode extends StyleNode {
   List<ui.FontVariation>? get _fontVariations => null;
 
   @override
-  double get _fontSize => paragraphStyle.fontSize ?? FlutterViewEmbedder.defaultFontSize;
+  double get _fontSize => paragraphStyle.fontSize ?? StyleManager.defaultFontSize;
 
   @override
   double? get _letterSpacing => null;
@@ -532,6 +594,9 @@ class RootStyleNode extends StyleNode {
   @override
   double? get _height => paragraphStyle.height;
 
+  @override
+  ui.TextLeadingDistribution? get _leadingDistribution => null;
+
   @override
   ui.Locale? get _locale => paragraphStyle.locale;
 
@@ -639,6 +704,12 @@ class CanvasParagraphBuilder implements ui.ParagraphBuilder {
       return;
     }
 
+    final double? letterSpacing = style.letterSpacing;
+    if (letterSpacing != null && letterSpacing != 0.0) {
+      _canDrawOnCanvas = false;
+      return;
+    }
+
     final ui.TextDecoration? decoration = style.decoration;
     if (decoration != null && decoration != ui.TextDecoration.none) {
       _canDrawOnCanvas = false;
