@@ -81,8 +81,9 @@ class EngineLineMetrics implements ui.LineMetrics {
 
   @override
   String toString() {
-    if (assertionsEnabled) {
-      return 'LineMetrics(hardBreak: $hardBreak, '
+    String result = super.toString();
+    assert(() {
+      result = 'LineMetrics(hardBreak: $hardBreak, '
           'ascent: $ascent, '
           'descent: $descent, '
           'unscaledAscent: $unscaledAscent, '
@@ -91,9 +92,9 @@ class EngineLineMetrics implements ui.LineMetrics {
           'left: $left, '
           'baseline: $baseline, '
           'lineNumber: $lineNumber)';
-    } else {
-      return super.toString();
-    }
+      return true;
+    }());
+    return result;
   }
 }
 
@@ -107,16 +108,18 @@ class ParagraphLine {
     required double left,
     required double baseline,
     required int lineNumber,
-    required this.ellipsis,
     required this.startIndex,
     required this.endIndex,
-    required this.endIndexWithoutNewlines,
+    required this.trailingNewlines,
+    required this.trailingSpaces,
+    required this.spaceCount,
     required this.widthWithTrailingSpaces,
-    required this.boxes,
-    required this.spaceBoxCount,
-    required this.trailingSpaceBoxCount,
+    required this.fragments,
+    required this.textDirection,
+    required this.paragraph,
     this.displayText,
-  }) : lineMetrics = EngineLineMetrics(
+  }) : assert(trailingNewlines <= endIndex - startIndex),
+       lineMetrics = EngineLineMetrics(
           hardBreak: hardBreak,
           ascent: ascent,
           descent: descent,
@@ -131,12 +134,6 @@ class ParagraphLine {
   /// Metrics for this line of the paragraph.
   final EngineLineMetrics lineMetrics;
 
-  /// The string to be displayed as an overflow indicator.
-  ///
-  /// When the value is non-null, it means this line is overflowing and the
-  /// [ellipsis] needs to be displayed at the end of it.
-  final String? ellipsis;
-
   /// The index (inclusive) in the text where this line begins.
   final int startIndex;
 
@@ -146,9 +143,25 @@ class ParagraphLine {
   /// the text and doesn't stop at the overflow cutoff.
   final int endIndex;
 
-  /// The index (exclusive) in the text where this line ends, ignoring newline
-  /// characters.
-  final int endIndexWithoutNewlines;
+  /// The largest visible index (exclusive) in this line.
+  ///
+  /// When the line contains an overflow, or is ellipsized at the end, this is
+  /// the largest index that remains visible in this line. If the entire line is
+  /// ellipsized, this returns [startIndex];
+  late final int visibleEndIndex = switch (fragments) {
+    [] => startIndex,
+    [...final List<LayoutFragment> rest, EllipsisFragment()]
+    || final List<LayoutFragment> rest => rest.last.end,
+  };
+
+  /// The number of new line characters at the end of the line.
+  final int trailingNewlines;
+
+  /// The number of spaces at the end of the line.
+  final int trailingSpaces;
+
+  /// The number of space characters in the entire line.
+  final int spaceCount;
 
   /// The full width of the line including all trailing space but not new lines.
   ///
@@ -162,21 +175,24 @@ class ParagraphLine {
   /// spaces so [widthWithTrailingSpaces] is more suitable.
   final double widthWithTrailingSpaces;
 
-  /// The list of boxes representing the entire line, possibly across multiple
-  /// spans.
-  final List<RangeBox> boxes;
+  /// The fragments that make up this line.
+  ///
+  /// The fragments in the [List] are sorted by their logical order in within the
+  /// line. In other words, a [LayoutFragment] in the [List] will have larger
+  /// start and end indices than all [LayoutFragment]s that appear before it.
+  final List<LayoutFragment> fragments;
 
-  /// The number of boxes that are space-only.
-  final int spaceBoxCount;
-
-  /// The number of trailing boxes that are space-only.
-  final int trailingSpaceBoxCount;
+  /// The text direction of this line, which is the same as the paragraph's.
+  final ui.TextDirection textDirection;
 
   /// The text to be rendered on the screen representing this line.
   final String? displayText;
 
-  /// The number of space-only boxes excluding trailing spaces.
-  int get nonTrailingSpaceBoxCount => spaceBoxCount - trailingSpaceBoxCount;
+  /// The [CanvasParagraph] this line is part of.
+  final CanvasParagraph paragraph;
+
+  /// The number of space characters in the line excluding trailing spaces.
+  int get nonTrailingSpaces => spaceCount - trailingSpaces;
 
   // Convenient getters for line metrics properties.
 
@@ -194,17 +210,184 @@ class ParagraphLine {
     return startIndex < this.endIndex && this.startIndex < endIndex;
   }
 
+  String getText(CanvasParagraph paragraph) {
+    final StringBuffer buffer = StringBuffer();
+    for (final LayoutFragment fragment in fragments) {
+      buffer.write(fragment.getText(paragraph));
+    }
+    return buffer.toString();
+  }
+
+  // This is the fallback graphme breaker that is only used if Intl.Segmenter()
+  // is not supported so _fromDomSegmenter can't be called. This implementation
+  // breaks the text into UTF-16 codepoints instead of graphme clusters.
+  List<int> _fallbackGraphemeStartIterable(String lineText) {
+    final List<int> graphemeStarts = <int>[];
+    bool precededByHighSurrogate = false;
+    for (int i = 0; i < lineText.length; i++) {
+      final int maskedCodeUnit = lineText.codeUnitAt(i) & 0xFC00;
+      // Only skip `i` if it points to a low surrogate in a valid surrogate pair.
+      if (maskedCodeUnit != 0xDC00 || !precededByHighSurrogate) {
+        graphemeStarts.add(startIndex + i);
+      }
+      precededByHighSurrogate = maskedCodeUnit == 0xD800;
+    }
+    return graphemeStarts;
+  }
+
+  // This will be called at most once to lazily populate _graphemeStarts.
+  List<int> _fromDomSegmenter(String fragmentText) {
+    final DomSegmenter domSegmenter = createIntlSegmenter(granularity: 'grapheme');
+    final List<int> graphemeStarts = <int>[];
+    final Iterator<DomSegment> segments = domSegmenter.segment(fragmentText).iterator();
+    while (segments.moveNext()) {
+      graphemeStarts.add(segments.current.index + startIndex);
+    }
+    assert(graphemeStarts.isEmpty || graphemeStarts.first == startIndex);
+    return graphemeStarts;
+  }
+
+  List<int> _breakTextIntoGraphemes(String text) {
+    final List<int> graphemeStarts = domIntl.Segmenter == null ? _fallbackGraphemeStartIterable(text) : _fromDomSegmenter(text);
+    // Add the end index of the fragment to the list if the text is not empty.
+    if (graphemeStarts.isNotEmpty) {
+      graphemeStarts.add(visibleEndIndex);
+    }
+    return graphemeStarts;
+  }
+
+  /// This List contains an ascending sequence of UTF16 offsets that points to
+  /// grapheme starts within the line. Each UTF16 offset is relative to the
+  /// start of the paragraph, instead of the start of the line.
+  ///
+  /// For example, `graphemeStarts[n]` gives the UTF16 offset of the `n`-th
+  /// grapheme in the line.
+  late final List<int> graphemeStarts = visibleEndIndex == startIndex
+    ? const <int>[]
+    : _breakTextIntoGraphemes(paragraph.plainText.substring(startIndex, visibleEndIndex));
+
+  /// Translate a UTF16 code unit in the paragaph (`offset`), to a grapheme
+  /// offset with in the current line.
+  ///
+  /// The `start` and `end` parameters are both grapheme offsets within the
+  /// current line. They are used to limit the search range (so the return value
+  /// that corresponds to the code unit `offset` must be with in [start, end)).
+  int graphemeStartIndexBefore(int offset, int start, int end) {
+    int low = start;
+    int high = end;
+    assert(0 <= low);
+    assert(low < high);
+
+    final List<int> lineGraphemeBreaks = graphemeStarts;
+    assert(offset >= lineGraphemeBreaks[start]);
+    assert(offset < lineGraphemeBreaks.last, '$offset, $lineGraphemeBreaks');
+    assert(end == lineGraphemeBreaks.length || offset < lineGraphemeBreaks[end]);
+    while (low + 2 <= high) {
+      // high >= low + 2, so low + 1 <= mid <= high - 1
+      final int mid = (low + high) ~/ 2;
+      switch (lineGraphemeBreaks[mid] - offset) {
+        case > 0: high = mid;
+        case < 0: low = mid;
+        case == 0: return mid;
+      }
+    }
+
+    assert(lineGraphemeBreaks[low] <= offset);
+    assert(high == lineGraphemeBreaks.length || offset < lineGraphemeBreaks[high]);
+    return low;
+  }
+
+  /// Returns the UTF-16 range of the character that encloses the code unit at
+  /// the given offset.
+  ui.TextRange? getCharacterRangeAt(int codeUnitOffset) {
+    assert(codeUnitOffset >= this.startIndex);
+    if (codeUnitOffset >= visibleEndIndex || graphemeStarts.isEmpty) {
+      return null;
+    }
+
+    final int startIndex = graphemeStartIndexBefore(codeUnitOffset, 0, graphemeStarts.length);
+    assert(startIndex < graphemeStarts.length - 1);
+    return ui.TextRange(start: graphemeStarts[startIndex], end: graphemeStarts[startIndex + 1]);
+  }
+
+  LayoutFragment? closestFragmentTo(LayoutFragment targetFragment, bool searchLeft) {
+    ({LayoutFragment fragment, double distance})? closestFragment;
+    for (final LayoutFragment fragment in fragments) {
+      assert(fragment is! EllipsisFragment);
+      if (fragment.start >= visibleEndIndex) {
+        break;
+      }
+      if (fragment.graphemeStartIndexRange == null) {
+        continue;
+      }
+      final double distance = searchLeft
+        ? targetFragment.left - fragment.right
+        : fragment.left - targetFragment.right;
+      final double? minDistance = closestFragment?.distance;
+      switch (distance) {
+        case > 0.0 when minDistance == null || minDistance > distance:
+          closestFragment = (fragment: fragment, distance: distance);
+        case == 0.0: return fragment;
+        case _: continue;
+      }
+    }
+    return closestFragment?.fragment;
+  }
+
+  /// Finds the closest [LayoutFragment] to the given horizontal offset `dx` in
+  /// this line, that is not an [EllipsisFragment] and contains at least one
+  /// grapheme start.
+  LayoutFragment? closestFragmentAtOffset(double dx) {
+    if (graphemeStarts.isEmpty) {
+      return null;
+    }
+    assert(graphemeStarts.length >= 2);
+    int graphemeIndex = 0;
+    ({LayoutFragment fragment, double distance})? closestFragment;
+    for (final LayoutFragment fragment in fragments) {
+      assert(fragment is! EllipsisFragment);
+      if (fragment.start >= visibleEndIndex) {
+        break;
+      }
+      if (fragment.length == 0) {
+        continue;
+      }
+      while (fragment.start > graphemeStarts[graphemeIndex]) {
+        graphemeIndex += 1;
+      }
+      final int firstGraphemeStartInFragment = graphemeStarts[graphemeIndex];
+      if (firstGraphemeStartInFragment >= fragment.end) {
+        continue;
+      }
+      final double distance;
+      if (dx < fragment.left) {
+        distance = fragment.left - dx;
+      } else if (dx > fragment.right) {
+        distance = dx - fragment.right;
+      } else {
+        return fragment;
+      }
+      assert(distance > 0);
+
+      final double? minDistance = closestFragment?.distance;
+      if (minDistance == null || minDistance > distance) {
+        closestFragment = (fragment: fragment, distance: distance);
+      }
+    }
+    return closestFragment?.fragment;
+  }
+
   @override
   int get hashCode => Object.hash(
         lineMetrics,
-        ellipsis,
         startIndex,
         endIndex,
-        endIndexWithoutNewlines,
+        trailingNewlines,
+        trailingSpaces,
+        spaceCount,
         widthWithTrailingSpaces,
-        boxes,
-        spaceBoxCount,
-        trailingSpaceBoxCount,
+        fragments,
+        textDirection,
         displayText,
       );
 
@@ -218,16 +401,21 @@ class ParagraphLine {
     }
     return other is ParagraphLine &&
         other.lineMetrics == lineMetrics &&
-        other.ellipsis == ellipsis &&
         other.startIndex == startIndex &&
         other.endIndex == endIndex &&
-        other.endIndexWithoutNewlines == endIndexWithoutNewlines &&
+        other.trailingNewlines == trailingNewlines &&
+        other.trailingSpaces == trailingSpaces &&
+        other.spaceCount == spaceCount &&
         other.widthWithTrailingSpaces == widthWithTrailingSpaces &&
-        other.boxes == boxes &&
-        other.spaceBoxCount == spaceBoxCount &&
-        other.trailingSpaceBoxCount == trailingSpaceBoxCount &&
+        other.fragments == fragments &&
+        other.textDirection == textDirection &&
         other.displayText == displayText;
   }
+
+  @override
+  String toString() {
+    return '$ParagraphLine($startIndex, $endIndex, $lineMetrics)';
+  }
 }
 
 /// The web implementation of [ui.ParagraphStyle].
@@ -277,7 +465,7 @@ class EngineParagraphStyle implements ui.ParagraphStyle {
       // When there's no strut height, always use paragraph style height.
       return height;
     }
-    if (strutStyle._forceStrutHeight == true) {
+    if (strutStyle._forceStrutHeight ?? false) {
       // When strut height is forced, ignore paragraph style height.
       return strutHeight;
     }
@@ -326,10 +514,11 @@ class EngineParagraphStyle implements ui.ParagraphStyle {
 
   @override
   String toString() {
-    if (assertionsEnabled) {
+    String result = super.toString();
+    assert(() {
       final double? fontSize = this.fontSize;
       final double? height = this.height;
-      return 'ParagraphStyle('
+      result = 'ParagraphStyle('
           'textAlign: ${textAlign ?? "unspecified"}, '
           'textDirection: ${textDirection ?? "unspecified"}, '
           'fontWeight: ${fontWeight ?? "unspecified"}, '
@@ -339,12 +528,12 @@ class EngineParagraphStyle implements ui.ParagraphStyle {
           'fontFamily: ${fontFamily ?? "unspecified"}, '
           'fontSize: ${fontSize != null ? fontSize.toStringAsFixed(1) : "unspecified"}, '
           'height: ${height != null ? "${height.toStringAsFixed(1)}x" : "unspecified"}, '
-          'ellipsis: ${ellipsis != null ? "\"$ellipsis\"" : "unspecified"}, '
+          'ellipsis: ${ellipsis != null ? '"$ellipsis"' : "unspecified"}, '
           'locale: ${locale ?? "unspecified"}'
           ')';
-    } else {
-      return super.toString();
-    }
+      return true;
+    }());
+    return result;
   }
 }
 
@@ -370,6 +559,7 @@ class EngineTextStyle implements ui.TextStyle {
     required double? letterSpacing,
     required double? wordSpacing,
     required double? height,
+    required ui.TextLeadingDistribution? leadingDistribution,
     required ui.Locale? locale,
     required ui.Paint? background,
     required ui.Paint? foreground,
@@ -397,6 +587,7 @@ class EngineTextStyle implements ui.TextStyle {
     this.letterSpacing,
     this.wordSpacing,
     this.height,
+    this.leadingDistribution,
     this.locale,
     this.background,
     this.foreground,
@@ -442,23 +633,25 @@ class EngineTextStyle implements ui.TextStyle {
   final double? letterSpacing;
   final double? wordSpacing;
   final double? height;
+  final ui.TextLeadingDistribution? leadingDistribution;
   final ui.Locale? locale;
   final ui.Paint? background;
   final ui.Paint? foreground;
   final List<ui.Shadow>? shadows;
 
+  static final List<String> _testFonts = <String>['FlutterTest', 'Ahem'];
   String get effectiveFontFamily {
-    if (assertionsEnabled) {
-      // In the flutter tester environment, we use a predictable-size font
-      // "Ahem". This makes widget tests predictable and less flaky.
-      if (ui.debugEmulateFlutterTesterEnvironment) {
-        return 'Ahem';
+    final String fontFamily = this.fontFamily.isEmpty ? StyleManager.defaultFontFamily : this.fontFamily;
+    // In the flutter tester environment, we use predictable-size test fonts.
+    // This makes widget tests predictable and less flaky.
+    String result = fontFamily;
+    assert(() {
+      if (ui_web.debugEmulateFlutterTesterEnvironment && !_testFonts.contains(fontFamily)) {
+        result = _testFonts.first;
       }
-    }
-    if (fontFamily.isEmpty) {
-      return FlutterViewEmbedder.defaultFontFamily;
-    }
-    return fontFamily;
+      return true;
+    }());
+    return result;
   }
 
   String? _cssFontString;
@@ -480,7 +673,7 @@ class EngineTextStyle implements ui.TextStyle {
   TextHeightStyle _createHeightStyle() {
     return TextHeightStyle(
       fontFamily: effectiveFontFamily,
-      fontSize: fontSize ?? FlutterViewEmbedder.defaultFontSize,
+      fontSize: fontSize ?? StyleManager.defaultFontSize,
       height: height,
       // TODO(mdebbar): Pass the actual value when font features become supported
       //                https://github.com/flutter/flutter/issues/64595
@@ -494,58 +687,72 @@ class EngineTextStyle implements ui.TextStyle {
     if (identical(this, other)) {
       return true;
     }
-    if (other.runtimeType != runtimeType) {
-      return false;
-    }
-    return other is EngineTextStyle &&
-        other.color == color &&
-        other.decoration == decoration &&
-        other.decorationColor == decorationColor &&
-        other.decorationStyle == decorationStyle &&
-        other.fontWeight == fontWeight &&
-        other.fontStyle == fontStyle &&
-        other.textBaseline == textBaseline &&
-        other.fontFamily == fontFamily &&
-        other.fontSize == fontSize &&
-        other.letterSpacing == letterSpacing &&
-        other.wordSpacing == wordSpacing &&
-        other.height == height &&
-        other.locale == locale &&
-        other.background == background &&
-        other.foreground == foreground &&
-        listEquals<ui.Shadow>(other.shadows, shadows) &&
-        listEquals<String>(other.fontFamilyFallback, fontFamilyFallback);
+    return other is EngineTextStyle
+        && other.color == color
+        && other.decoration == decoration
+        && other.decorationColor == decorationColor
+        && other.decorationStyle == decorationStyle
+        && other.fontWeight == fontWeight
+        && other.fontStyle == fontStyle
+        && other.textBaseline == textBaseline
+        && other.leadingDistribution == leadingDistribution
+        && other.fontFamily == fontFamily
+        && other.fontSize == fontSize
+        && other.letterSpacing == letterSpacing
+        && other.wordSpacing == wordSpacing
+        && other.height == height
+        && other.decorationThickness == decorationThickness
+        && other.locale == locale
+        && other.background == background
+        && other.foreground == foreground
+        && listEquals<ui.Shadow>(other.shadows, shadows)
+        && listEquals<String>(other.fontFamilyFallback, fontFamilyFallback)
+        && listEquals<ui.FontFeature>(other.fontFeatures, fontFeatures)
+        && listEquals<ui.FontVariation>(other.fontVariations, fontVariations);
   }
 
   @override
-  int get hashCode => Object.hash(
-        color,
-        decoration,
-        decorationColor,
-        decorationStyle,
-        decorationThickness,
-        fontWeight,
-        fontStyle,
-        textBaseline,
-        fontFamily,
-        fontFamilyFallback,
-        fontSize,
-        letterSpacing,
-        wordSpacing,
-        height,
-        locale,
-        background,
-        foreground,
-        shadows,
-      );
+  int get hashCode {
+    final List<ui.Shadow>? shadows = this.shadows;
+    final List<ui.FontFeature>? fontFeatures = this.fontFeatures;
+    final List<ui.FontVariation>? fontVariations = this.fontVariations;
+    final List<String>? fontFamilyFallback = this.fontFamilyFallback;
+    return Object.hash(
+      color,
+      decoration,
+      decorationColor,
+      decorationStyle,
+      fontWeight,
+      fontStyle,
+      textBaseline,
+      leadingDistribution,
+      fontFamily,
+      fontFamilyFallback == null ? null : Object.hashAll(fontFamilyFallback),
+      fontSize,
+      letterSpacing,
+      wordSpacing,
+      height,
+      locale,
+      background,
+      foreground,
+      shadows == null ? null : Object.hashAll(shadows),
+      decorationThickness,
+      // Object.hash goes up to 20 arguments, but we have 21
+      Object.hash(
+        fontFeatures == null ? null : Object.hashAll(fontFeatures),
+        fontVariations == null ? null : Object.hashAll(fontVariations),
+      )
+    );
+  }
 
   @override
   String toString() {
-    if (assertionsEnabled) {
+    String result = super.toString();
+    assert(() {
       final List<String>? fontFamilyFallback = this.fontFamilyFallback;
       final double? fontSize = this.fontSize;
       final double? height = this.height;
-      return 'TextStyle('
+      result = 'TextStyle('
           'color: ${color ?? "unspecified"}, '
           'decoration: ${decoration ?? "unspecified"}, '
           'decorationColor: ${decorationColor ?? "unspecified"}, '
@@ -567,9 +774,9 @@ class EngineTextStyle implements ui.TextStyle {
           'fontFeatures: ${fontFeatures ?? "unspecified"}, '
           'fontVariations: ${fontVariations ?? "unspecified"}'
           ')';
-    } else {
-      return super.toString();
-    }
+      return true;
+    }());
+    return result;
   }
 }
 
@@ -670,12 +877,18 @@ class ParagraphPlaceholder {
   final ui.TextBaseline baseline;
 }
 
-/// Converts [fontWeight] to its CSS equivalent value.
-String? fontWeightToCss(ui.FontWeight? fontWeight) {
-  if (fontWeight == null) {
-    return null;
+extension FontStyleExtension on ui.FontStyle {
+  /// Converts a [ui.FontStyle] value to its CSS equivalent.
+  String toCssString() {
+    return this == ui.FontStyle.normal ? 'normal' : 'italic';
+  }
+}
+
+extension FontWeightExtension on ui.FontWeight {
+  /// Converts a [ui.FontWeight] value to its CSS equivalent.
+  String toCssString() {
+    return fontWeightIndexToCss(fontWeightIndex: index);
   }
-  return fontWeightIndexToCss(fontWeightIndex: fontWeight.index);
 }
 
 String fontWeightIndexToCss({int fontWeightIndex = 3}) {
@@ -711,16 +924,10 @@ String fontWeightIndexToCss({int fontWeightIndex = 3}) {
 
 /// Applies a text [style] to an [element], translating the properties to their
 /// corresponding CSS equivalents.
-///
-/// If [isSpan] is true, the text element is a span within richtext and
-/// should not assign effectiveFontFamily if fontFamily was not specified.
 void applyTextStyleToElement({
-  required DomHTMLElement element,
+  required DomElement element,
   required EngineTextStyle style,
-  bool isSpan = false,
 }) {
-  assert(element != null); // ignore: unnecessary_null_comparison
-  assert(style != null); // ignore: unnecessary_null_comparison
   bool updateDecoration = false;
   final DomCSSStyleDeclaration cssStyle = element.style;
 
@@ -738,31 +945,30 @@ void applyTextStyleToElement({
     final double adaptedWidth = strokeWidth != null && strokeWidth > 0
         ? strokeWidth
         : 1.0 / ui.window.devicePixelRatio;
-    cssStyle.textStroke = '${adaptedWidth}px ${colorToCssString(color)}';
+    cssStyle.textStroke = '${adaptedWidth}px ${color?.toCssString()}';
   } else if (color != null) {
-    cssStyle.color = colorToCssString(color)!;
+    cssStyle.color = color.toCssString();
   }
   final ui.Color? background = style.background?.color;
   if (background != null) {
-    cssStyle.backgroundColor = colorToCssString(background)!;
+    cssStyle.backgroundColor = background.toCssString();
   }
   final double? fontSize = style.fontSize;
   if (fontSize != null) {
     cssStyle.fontSize = '${fontSize.floor()}px';
   }
   if (style.fontWeight != null) {
-    cssStyle.fontWeight = fontWeightToCss(style.fontWeight)!;
+    cssStyle.fontWeight = style.fontWeight!.toCssString();
   }
   if (style.fontStyle != null) {
-    cssStyle.fontStyle =
-        style.fontStyle == ui.FontStyle.normal ? 'normal' : 'italic';
+    cssStyle.fontStyle = style.fontStyle!.toCssString();
   }
   // For test environment use effectiveFontFamily since we need to
-  // consistently use Ahem font.
-  if (isSpan && !ui.debugEmulateFlutterTesterEnvironment) {
-    cssStyle.fontFamily = canonicalizeFontFamily(style.fontFamily)!;
-  } else {
+  // consistently use the correct test font.
+  if (ui_web.debugEmulateFlutterTesterEnvironment) {
     cssStyle.fontFamily = canonicalizeFontFamily(style.effectiveFontFamily)!;
+  } else {
+    cssStyle.fontFamily = canonicalizeFontFamily(style.fontFamily)!;
   }
   if (style.letterSpacing != null) {
     cssStyle.letterSpacing = '${style.letterSpacing}px';
@@ -790,7 +996,7 @@ void applyTextStyleToElement({
         }
         final ui.Color? decorationColor = style.decorationColor;
         if (decorationColor != null) {
-          cssStyle.textDecorationColor = colorToCssString(decorationColor)!;
+          cssStyle.textDecorationColor = decorationColor.toCssString();
         }
       }
     }
@@ -825,7 +1031,7 @@ String _shadowListToCss(List<ui.Shadow> shadows) {
     }
     final ui.Shadow shadow = shadows[i];
     sb.write('${shadow.offset.dx}px ${shadow.offset.dy}px '
-        '${shadow.blurRadius}px ${colorToCssString(shadow.color)}');
+        '${shadow.blurRadius}px ${shadow.color.toCssString()}');
   }
   return sb.toString();
 }
