@@ -7,6 +7,7 @@ part of dart._engine;
 
 
 
+
 /// Generic callback signature, used by [_futurize].
 typedef Callback<T> = void Function(T result);
 
@@ -43,10 +44,10 @@ Future<T> futurize<T>(Callbacker<T> callbacker) {
   // If the callback synchronously throws an error, then synchronously
   // rethrow that error instead of adding it to the completer. This
   // prevents the Zone from receiving an uncaught exception.
-  bool sync = true;
+  bool isSync = true;
   final String? error = callbacker((T? t) {
     if (t == null) {
-      if (sync) {
+      if (isSync) {
         throw Exception('operation failed');
       } else {
         completer.completeError(Exception('operation failed'));
@@ -55,7 +56,7 @@ Future<T> futurize<T>(Callbacker<T> callbacker) {
       completer.complete(t);
     }
   });
-  sync = false;
+  isSync = false;
   if (error != null) {
     throw Exception(error);
   }
@@ -208,12 +209,6 @@ String float64ListToCssTransform3d(List<double> matrix) {
   }
 }
 
-bool get assertionsEnabled {
-  bool k = false;
-  assert(k = true);
-  return k;
-}
-
 final Float32List _tempRectData = Float32List(4);
 
 /// Transforms a [ui.Rect] given the effective [transform].
@@ -221,7 +216,7 @@ final Float32List _tempRectData = Float32List(4);
 /// The resulting rect is aligned to the pixel grid, i.e. two of
 /// its sides are vertical and two are horizontal. In the presence of rotations
 /// the rectangle is inflated such that it fits the rotated rectangle.
-ui.Rect transformRect(Matrix4 transform, ui.Rect rect) {
+ui.Rect transformRectWithMatrix(Matrix4 transform, ui.Rect rect) {
   _tempRectData[0] = rect.left;
   _tempRectData[1] = rect.top;
   _tempRectData[2] = rect.right;
@@ -330,12 +325,18 @@ bool rectContainsOther(ui.Rect rect, ui.Rect other) {
       rect.bottom >= other.bottom;
 }
 
-/// Converts color to a css compatible attribute value.
-String? colorToCssString(ui.Color? color) {
-  if (color == null) {
-    return null;
+extension CssColor on ui.Color {
+  /// Converts color to a css compatible attribute value.
+  String toCssString() {
+    return colorValueToCssString(value);
+  }
+}
+
+// Converts a color value (as an int) into a CSS-compatible value.
+String colorValueToCssString(int value) {
+  if (value == 0xFF000000) {
+    return '#000000';
   }
-  final int value = color.value;
   if ((0xff000000 & value) == 0xff000000) {
     final String hexValue = (value & 0xFFFFFF).toRadixString(16);
     final int hexValueLength = hexValue.length;
@@ -544,17 +545,6 @@ String blurSigmasToCssString(double sigmaX, double sigmaY) {
   return 'blur(${(sigmaX + sigmaY) * 0.5}px)';
 }
 
-/// Checks if the dynamic [object] is equal to null.
-bool unsafeIsNull(dynamic object) {
-  return object == null;
-}
-
-/// A typed variant of [domWindow.fetch].
-Future<DomResponse> httpFetch(String url) async {
-  final Object? result = await domWindow.fetch(url);
-  return result! as DomResponse;
-}
-
 /// Extensions to [Map] that make it easier to treat it as a JSON object. The
 /// keys are `dynamic` because when JSON is deserialized from method channels
 /// it arrives as `Map<dynamic, dynamic>`.
@@ -613,22 +603,43 @@ extension JsonExtensions on Map<dynamic, dynamic> {
   }
 
   int readInt(String propertyName) {
-    return this[propertyName] as int;
+    return (this[propertyName] as num).toInt();
   }
 
   int? tryInt(String propertyName) {
-    return this[propertyName] as int?;
+    return (this[propertyName] as num?)?.toInt();
   }
 
   double readDouble(String propertyName) {
-    return this[propertyName] as double;
+    return (this[propertyName] as num).toDouble();
   }
 
   double? tryDouble(String propertyName) {
-    return this[propertyName] as double?;
+    return (this[propertyName] as num?)?.toDouble();
   }
 }
 
+/// Extracts view ID from the [MethodCall.arguments] map.
+///
+/// Throws if the view ID is not present or if [arguments] is not a map.
+int readViewId(Object? arguments) {
+  final int? viewId = tryViewId(arguments);
+  if (viewId == null) {
+    throw Exception('Could not find a `viewId` in the arguments: $arguments');
+  }
+  return viewId;
+}
+
+/// Extracts view ID from the [MethodCall.arguments] map.
+///
+/// Returns null if the view ID is not present or if [arguments] is not a map.
+int? tryViewId(Object? arguments) {
+  if (arguments is Map) {
+    return arguments.tryInt('viewId');
+  }
+  return null;
+}
+
 /// Prints a list of bytes in hex format.
 ///
 /// Bytes are separated by one space and are padded on the left to always show
@@ -639,7 +650,7 @@ extension JsonExtensions on Map<dynamic, dynamic> {
 ///     Input: [0, 1, 2, 3]
 ///     Output: 0x00 0x01 0x02 0x03
 String bytesToHexString(List<int> data) {
-  return data.map((int byte) => '0x' + byte.toRadixString(16).padLeft(2, '0')).join(' ');
+  return data.map((int byte) => '0x${byte.toRadixString(16).padLeft(2, '0')}').join(' ');
 }
 
 /// Sets a style property on [element].
@@ -670,16 +681,21 @@ void setClipPath(DomElement element, String? value) {
   }
 }
 
-void setThemeColor(ui.Color color) {
+void setThemeColor(ui.Color? color) {
   DomHTMLMetaElement? theme =
       domDocument.querySelector('#flutterweb-theme') as DomHTMLMetaElement?;
-  if (theme == null) {
-    theme = createDomHTMLMetaElement()
-      ..id = 'flutterweb-theme'
-      ..name = 'theme-color';
-    domDocument.head!.append(theme);
+
+  if (color != null) {
+    if (theme == null) {
+      theme = createDomHTMLMetaElement()
+        ..id = 'flutterweb-theme'
+        ..name = 'theme-color';
+      domDocument.head!.append(theme);
+    }
+    theme.content = color.toCssString();
+  } else {
+    theme?.remove();
   }
-  theme.content = colorToCssString(color)!;
 }
 
 bool? _ellipseFeatureDetected;
@@ -730,3 +746,102 @@ extension FirstWhereOrNull<T> on Iterable<T> {
     return null;
   }
 }
+
+typedef _LruCacheEntry<K extends Object, V extends Object> = ({K key, V value});
+
+/// Caches up to a [maximumSize] key-value pairs.
+///
+/// Call [cache] to cache a key-value pair.
+class LruCache<K extends Object, V extends Object> {
+  LruCache(this.maximumSize);
+
+  /// The maximum number of key/value pairs this cache can contain.
+  ///
+  /// To avoid exceeding this limit the cache remove least recently used items.
+  final int maximumSize;
+
+  /// A doubly linked list of the objects in the cache.
+  ///
+  /// This makes it fast to move a recently used object to the front.
+  final DoubleLinkedQueue<_LruCacheEntry<K, V>> _itemQueue = DoubleLinkedQueue<_LruCacheEntry<K, V>>();
+
+  
+  DoubleLinkedQueue<_LruCacheEntry<K, V>> get debugItemQueue => _itemQueue;
+
+  /// A map of objects to their associated node in the [_itemQueue].
+  ///
+  /// This makes it fast to find the node in the queue when we need to
+  /// move the object to the front of the queue.
+  final Map<K, DoubleLinkedQueueEntry<_LruCacheEntry<K, V>>> _itemMap = <K, DoubleLinkedQueueEntry<_LruCacheEntry<K, V>>>{};
+
+  
+  Map<K, DoubleLinkedQueueEntry<_LruCacheEntry<K, V>>> get itemMap => _itemMap;
+
+  /// The number of objects in the cache.
+  int get length => _itemQueue.length;
+
+  /// Whether or not [object] is in the cache.
+  ///
+  /// This is only for testing.
+  
+  bool debugContainsValue(V object) {
+    return _itemMap.containsValue(object);
+  }
+
+  
+  bool debugContainsKey(K key) {
+    return _itemMap.containsKey(key);
+  }
+
+  /// Returns the cached value associated with the [key].
+  ///
+  /// If the value is not in the cache, returns null.
+  V? operator[](K key) {
+    return _itemMap[key]?.element.value;
+  }
+
+  /// Caches the given [key]/[value] pair in this cache.
+  ///
+  /// If the pair is not already in the cache, adds it to the cache as the most
+  /// recently used pair.
+  ///
+  /// If the [key] is already in the cache, moves it to the most recently used
+  /// position. If the [value] corresponding to the [key] is different from
+  /// what's in the cache, updates the value.
+  void cache(K key, V value) {
+    final DoubleLinkedQueueEntry<_LruCacheEntry<K, V>>? item = _itemMap[key];
+    if (item == null) {
+      // New key-value pair, just add.
+      _add(key, value);
+    } else if (item.element.value != value) {
+      // Key already in the cache, but value is new. Re-add.
+      item.remove();
+      _add(key, value);
+    } else {
+      // Key-value pair already in the cache, move to most recently used.
+      item.remove();
+      _itemQueue.addFirst(item.element);
+      _itemMap[key] = _itemQueue.firstEntry()!;
+    }
+  }
+
+  void clear() {
+    _itemQueue.clear();
+    _itemMap.clear();
+  }
+
+  void _add(K key, V value) {
+    _itemQueue.addFirst((key: key, value: value));
+    _itemMap[key] = _itemQueue.firstEntry()!;
+
+    if (_itemQueue.length > maximumSize) {
+      _removeLeastRecentlyUsedValue();
+    }
+  }
+
+  void _removeLeastRecentlyUsedValue() {
+    final bool didRemove = _itemMap.remove(_itemQueue.last.key) != null;
+    assert(didRemove);
+    _itemQueue.removeLast();
+  }
+}
