@@ -27,7 +27,7 @@ typedef DisposeFunction<T extends NativeType> = void Function(Pointer<T>);
 class SkwasmFinalizationRegistry<T extends NativeType> {
   SkwasmFinalizationRegistry(this.dispose)
     : registry = createDomFinalizationRegistry(((JSNumber address) =>
-      dispose(Pointer<T>.fromAddress(address.toDart.toInt()))
+      dispose(Pointer<T>.fromAddress(address.toDartDouble.toInt()))
     ).toJS);
 
   final DomFinalizationRegistry registry;
