@@ -14,6 +14,7 @@ import 'dart:math' as math;
 import 'dart:typed_data';
 
 import 'dart:_engine' as engine;
+import 'dart:ui_web' as ui_web;
 
 part 'annotations.dart';
 part 'canvas.dart';
@@ -24,6 +25,7 @@ part 'hash_codes.dart';
 part 'initialization.dart';
 part 'key.dart';
 part 'lerp.dart';
+part 'math.dart';
 part 'natives.dart';
 part 'painting.dart';
 part 'path.dart';
