@@ -283,6 +283,8 @@ class SemanticsFlag {
   static const int _kIsSliderIndex = 1 << 23;
   static const int _kIsKeyboardKeyIndex = 1 << 24;
   static const int _kIsCheckStateMixedIndex = 1 << 25;
+  static const int _kHasExpandedStateIndex = 1 << 26;
+  static const int _kIsExpandedIndex = 1 << 27;
   // READ THIS: if you add a flag here, you MUST update the numSemanticsFlags
   // value in testing/dart/semantics_test.dart, or tests will fail. Also,
   // please update the Flag enum in
@@ -527,6 +529,27 @@ class SemanticsFlag {
   /// navigate to the next page when reaching the end of the current one.
   static const SemanticsFlag hasImplicitScrolling = SemanticsFlag._(_kHasImplicitScrollingIndex, 'hasImplicitScrolling');
 
+  /// The semantics node has the quality of either being "expanded" or "collapsed".
+  ///
+  /// For example, a [SubmenuButton] widget has expanded state.
+  ///
+  /// See also:
+  ///
+  ///   * [SemanticsFlag.isExpanded], which controls whether the node is "expanded" or "collapsed".
+  static const SemanticsFlag hasExpandedState = SemanticsFlag._(_kHasExpandedStateIndex, 'hasExpandedState');
+
+  /// Whether a semantics node is expanded.
+  ///
+  /// If true, the semantics node is "expanded". If false, the semantics node is
+  /// "collapsed".
+  ///
+  /// For example, if a [SubmenuButton] shows its children, [isExpanded] is true.
+  ///
+  /// See also:
+  ///
+  ///   * [SemanticsFlag.hasExpandedState], which enables an expanded/collapsed state.
+  static const SemanticsFlag isExpanded = SemanticsFlag._(_kIsExpandedIndex, 'isExpanded');
+
   /// The possible semantics flags.
   ///
   /// The map's key is the [index] of the flag and the value is the flag itself.
@@ -557,6 +580,8 @@ class SemanticsFlag {
     _kIsSliderIndex: isSlider,
     _kIsKeyboardKeyIndex: isKeyboardKey,
     _kIsCheckStateMixedIndex: isCheckStateMixed,
+    _kHasExpandedStateIndex: hasExpandedState,
+    _kIsExpandedIndex: isExpanded,
   };
 
   static List<SemanticsFlag> get values => _kFlagById.values.toList(growable: false);
@@ -705,6 +730,10 @@ abstract class SemanticsUpdateBuilder {
   /// [PlatformDispatcher.onSemanticsActionEvent] callback might be called with
   /// an action that is no longer possible.
   ///
+  /// The `identifier` is a string that describes the node for UI automation
+  /// tools that work by querying the accessibility hierarchy, such as Android
+  /// UI Automator, iOS XCUITest, or Appium. It's not exposed to users.
+  ///
   /// The `label` is a string that describes this node. The `value` property
   /// describes the current value of the node as a string. The `increasedValue`
   /// string will become the `value` string after a [SemanticsAction.increase]
@@ -713,8 +742,8 @@ abstract class SemanticsUpdateBuilder {
   /// string describes what result an action performed on this node has. The
   /// reading direction of all these strings is given by `textDirection`.
   ///
-  /// The `labelAttirbutes`, `valueAttirbutes`, `hintAttributes`,
-  /// `increasedValueAttirbutes`, and `decreasedValueAttributes` are the lists of
+  /// The `labelAttributes`, `valueAttributes`, `hintAttributes`,
+  /// `increasedValueAttributes`, and `decreasedValueAttributes` are the lists of
   /// [StringAttribute] carried by the `label`, `value`, `hint`, `increasedValue`,
   /// and `decreasedValue` respectively. Their contents must not be changed during
   /// the semantics update.
@@ -778,6 +807,7 @@ abstract class SemanticsUpdateBuilder {
     required double elevation,
     required double thickness,
     required Rect rect,
+    required String identifier,
     required String label,
     required List<StringAttribute> labelAttributes,
     required String value,
@@ -788,8 +818,8 @@ abstract class SemanticsUpdateBuilder {
     required List<StringAttribute> decreasedValueAttributes,
     required String hint,
     required List<StringAttribute> hintAttributes,
-    String? tooltip,
-    TextDirection? textDirection,
+    required String tooltip,
+    required TextDirection? textDirection,
     required Float64List transform,
     required Int32List childrenInTraversalOrder,
     required Int32List childrenInHitTestOrder,
@@ -823,9 +853,7 @@ abstract class SemanticsUpdateBuilder {
   SemanticsUpdate build();
 }
 
-@pragma('vm:entry-point')
 base class _NativeSemanticsUpdateBuilder extends NativeFieldWrapperClass1 implements SemanticsUpdateBuilder {
-  @pragma('vm:entry-point')
   _NativeSemanticsUpdateBuilder() { _constructor(); }
 
   @Native<Void Function(Handle)>(symbol: 'SemanticsUpdateBuilder::Create')
@@ -849,6 +877,7 @@ base class _NativeSemanticsUpdateBuilder extends NativeFieldWrapperClass1 implem
     required double elevation,
     required double thickness,
     required Rect rect,
+    required String identifier,
     required String label,
     required List<StringAttribute> labelAttributes,
     required String value,
@@ -859,8 +888,8 @@ base class _NativeSemanticsUpdateBuilder extends NativeFieldWrapperClass1 implem
     required List<StringAttribute> decreasedValueAttributes,
     required String hint,
     required List<StringAttribute> hintAttributes,
-    String? tooltip,
-    TextDirection? textDirection,
+    required String tooltip,
+    required TextDirection? textDirection,
     required Float64List transform,
     required Int32List childrenInTraversalOrder,
     required Int32List childrenInHitTestOrder,
@@ -887,6 +916,7 @@ base class _NativeSemanticsUpdateBuilder extends NativeFieldWrapperClass1 implem
       rect.bottom,
       elevation,
       thickness,
+      identifier,
       label,
       labelAttributes,
       value,
@@ -897,7 +927,7 @@ base class _NativeSemanticsUpdateBuilder extends NativeFieldWrapperClass1 implem
       decreasedValueAttributes,
       hint,
       hintAttributes,
-      tooltip ?? '',
+      tooltip,
       textDirection != null ? textDirection.index + 1 : 0,
       transform,
       childrenInTraversalOrder,
@@ -938,6 +968,7 @@ base class _NativeSemanticsUpdateBuilder extends NativeFieldWrapperClass1 implem
           Handle,
           Handle,
           Handle,
+          Handle,
           Int32,
           Handle,
           Handle,
@@ -963,6 +994,7 @@ base class _NativeSemanticsUpdateBuilder extends NativeFieldWrapperClass1 implem
       double bottom,
       double elevation,
       double thickness,
+      String? identifier,
       String label,
       List<StringAttribute> labelAttributes,
       String value,
@@ -1014,13 +1046,11 @@ abstract class SemanticsUpdate {
   void dispose();
 }
 
-@pragma('vm:entry-point')
 base class _NativeSemanticsUpdate extends NativeFieldWrapperClass1 implements SemanticsUpdate {
   /// This class is created by the engine, and should not be instantiated
   /// or extended directly.
   ///
   /// To create a SemanticsUpdate object, use a [SemanticsUpdateBuilder].
-  @pragma('vm:entry-point')
   _NativeSemanticsUpdate._();
 
   @override
