@@ -87,7 +87,8 @@ Future<void> _testBuildIosFramework(Directory projectDir, { bool isModule = fals
   const String outputDirectoryName = 'flutter-frameworks';
 
   await inDirectory(projectDir, () async {
-    await flutter(
+    final StringBuffer outputError = StringBuffer();
+    await evalFlutter(
       'build',
       options: <String>[
         'ios-framework',
@@ -96,33 +97,23 @@ Future<void> _testBuildIosFramework(Directory projectDir, { bool isModule = fals
         '--obfuscate',
         '--split-debug-info=symbols',
       ],
+      stderr: outputError,
     );
+    if (!outputError.toString().contains('Bitcode support has been deprecated.')) {
+      throw TaskResult.failure('Missing bitcode deprecation warning');
+    }
   });
 
   final String outputPath = path.join(projectDir.path, outputDirectoryName);
 
-  // TODO(jmagman): Remove ios-arm64_armv7 checks when armv7 engine artifacts are removed.
-  final String arm64FlutterFramework = path.join(
+  checkFileExists(path.join(
     outputPath,
     'Debug',
     'Flutter.xcframework',
     'ios-arm64',
     'Flutter.framework',
-  );
-
-  final String armv7FlutterFramework = path.join(
-    outputPath,
-    'Debug',
-    'Flutter.xcframework',
-    'ios-arm64_armv7',
-    'Flutter.framework',
-  );
-
-  final bool arm64FlutterBinaryExists = exists(File(path.join(arm64FlutterFramework, 'Flutter')));
-  final bool armv7FlutterBinaryExists = exists(File(path.join(armv7FlutterFramework, 'Flutter')));
-  if (!arm64FlutterBinaryExists && !armv7FlutterBinaryExists) {
-    throw TaskResult.failure('Expected debug Flutter engine artifact binary to exist');
-  }
+    'Flutter',
+  ));
 
   final String debugAppFrameworkPath = path.join(
     outputPath,
@@ -185,7 +176,6 @@ Future<void> _testBuildIosFramework(Directory projectDir, { bool isModule = fals
     );
 
     await _checkDylib(appFrameworkPath);
-    await _checkBitcode(appFrameworkPath, mode);
 
     final String aotSymbols = await _dylibSymbols(appFrameworkPath);
 
@@ -203,6 +193,23 @@ Future<void> _testBuildIosFramework(Directory projectDir, { bool isModule = fals
       'vm_snapshot_data',
     ));
 
+    final String appFrameworkDsymPath = path.join(
+      outputPath,
+      mode,
+      'App.xcframework',
+      'ios-arm64',
+      'dSYMs',
+      'App.framework.dSYM'
+    );
+    checkDirectoryExists(appFrameworkDsymPath);
+    await _checkDsym(path.join(
+      appFrameworkDsymPath,
+      'Contents',
+      'Resources',
+      'DWARF',
+      'App',
+    ));
+
     checkFileExists(path.join(
       outputPath,
       mode,
@@ -225,32 +232,14 @@ Future<void> _testBuildIosFramework(Directory projectDir, { bool isModule = fals
   section("Check all modes' engine dylib");
 
   for (final String mode in <String>['Debug', 'Profile', 'Release']) {
-    // TODO(jmagman): Remove ios-arm64_armv7 checks when armv7 engine artifacts are removed.
-    final String arm64EngineBinary = path.join(
+    checkFileExists(path.join(
       outputPath,
       mode,
       'Flutter.xcframework',
       'ios-arm64',
       'Flutter.framework',
       'Flutter',
-    );
-
-    final String arm64Armv7EngineBinary = path.join(
-      outputPath,
-      mode,
-      'Flutter.xcframework',
-      'ios-arm64_armv7',
-      'Flutter.framework',
-      'Flutter',
-    );
-
-    if (exists(File(arm64EngineBinary))) {
-      await _checkBitcode(arm64EngineBinary, mode);
-    } else if (exists(File(arm64Armv7EngineBinary))) {
-      await _checkBitcode(arm64Armv7EngineBinary, mode);
-    } else {
-      throw TaskResult.failure('Expected Flutter $mode engine artifact binary to exist');
-    }
+    ));
 
     checkFileExists(path.join(
       outputPath,
@@ -285,12 +274,21 @@ Future<void> _testBuildIosFramework(Directory projectDir, { bool isModule = fals
     );
 
     await _checkDylib(pluginFrameworkPath);
-    await _checkBitcode(pluginFrameworkPath, mode);
     if (!await _linksOnFlutter(pluginFrameworkPath)) {
       throw TaskResult.failure('$pluginFrameworkPath does not link on Flutter');
     }
 
+    // TODO(jmagman): Remove ios-arm64_armv7 checks when CI is updated to Xcode 14.
     final String transitiveDependencyFrameworkPath = path.join(
+      outputPath,
+      mode,
+      'Reachability.xcframework',
+      'ios-arm64',
+      'Reachability.framework',
+      'Reachability',
+    );
+
+    final String armv7TransitiveDependencyFrameworkPath = path.join(
       outputPath,
       mode,
       'Reachability.xcframework',
@@ -298,8 +296,17 @@ Future<void> _testBuildIosFramework(Directory projectDir, { bool isModule = fals
       'Reachability.framework',
       'Reachability',
     );
-    if (await _linksOnFlutter(transitiveDependencyFrameworkPath)) {
-      throw TaskResult.failure('Transitive dependency $transitiveDependencyFrameworkPath unexpectedly links on Flutter');
+
+    final bool transitiveDependencyExists = exists(File(transitiveDependencyFrameworkPath));
+    final bool armv7TransitiveDependencyExists = exists(File(armv7TransitiveDependencyFrameworkPath));
+    if (!transitiveDependencyExists && !armv7TransitiveDependencyExists) {
+      throw TaskResult.failure('Expected debug Flutter engine artifact binary to exist');
+    }
+
+    if ((transitiveDependencyExists && await _linksOnFlutter(transitiveDependencyFrameworkPath)) ||
+        (armv7TransitiveDependencyExists && await _linksOnFlutter(armv7TransitiveDependencyFrameworkPath))) {
+      throw TaskResult.failure(
+          'Transitive dependency $transitiveDependencyFrameworkPath unexpectedly links on Flutter');
     }
 
     checkFileExists(path.join(
@@ -346,14 +353,6 @@ Future<void> _testBuildIosFramework(Directory projectDir, { bool isModule = fals
     checkFileExists(simulatorFrameworkHeaderPath);
   }
 
-  checkDirectoryExists(path.join(
-    outputPath,
-    'Release',
-    'connectivity.xcframework',
-    'ios-arm64',
-    'BCSymbolMaps',
-  ));
-
   section('Check all modes have generated plugin registrant');
 
   for (final String mode in <String>['Debug', 'Profile', 'Release']) {
@@ -369,7 +368,6 @@ Future<void> _testBuildIosFramework(Directory projectDir, { bool isModule = fals
       'FlutterPluginRegistrant',
     );
     await _checkStatic(registrantFrameworkPath);
-    await _checkBitcode(registrantFrameworkPath, mode);
 
     checkFileExists(path.join(
       outputPath,
@@ -426,6 +424,25 @@ Future<void> _testBuildIosFramework(Directory projectDir, { bool isModule = fals
       'App',
     ));
 
+    if (mode != 'Debug') {
+      final String appFrameworkDsymPath = path.join(
+        cocoapodsOutputPath,
+        mode,
+        'App.xcframework',
+        'ios-arm64',
+        'dSYMs',
+        'App.framework.dSYM'
+      );
+      checkDirectoryExists(appFrameworkDsymPath);
+      await _checkDsym(path.join(
+        appFrameworkDsymPath,
+        'Contents',
+        'Resources',
+        'DWARF',
+        'App',
+      ));
+    }
+
     if (Directory(path.join(
           cocoapodsOutputPath,
           mode,
@@ -476,6 +493,12 @@ Future<void> _testBuildIosFramework(Directory projectDir, { bool isModule = fals
       isModule) {
     throw TaskResult.failure('Unexpected GeneratedPluginRegistrant.m.');
   }
+
+  section('Build frameworks without plugins');
+  await _testBuildFrameworksWithoutPlugins(projectDir, platform: 'ios');
+
+  section('check --static cannot be used with the --no-plugins flag');
+  await _testStaticAndNoPlugins(projectDir);
 }
 
 
@@ -604,6 +627,23 @@ Future<void> _testBuildMacOSFramework(Directory projectDir) async {
       'Resources',
       'Info.plist',
     ));
+
+    final String appFrameworkDsymPath = path.join(
+      outputPath,
+      mode,
+      'App.xcframework',
+      'macos-arm64_x86_64',
+      'dSYMs',
+      'App.framework.dSYM'
+    );
+    checkDirectoryExists(appFrameworkDsymPath);
+    await _checkDsym(path.join(
+      appFrameworkDsymPath,
+      'Contents',
+      'Resources',
+      'DWARF',
+      'App',
+    ));
   }
 
   section("Check all modes' engine dylib");
@@ -734,6 +774,25 @@ Future<void> _testBuildMacOSFramework(Directory projectDir) async {
       'App',
     ));
 
+    if (mode != 'Debug') {
+      final String appFrameworkDsymPath = path.join(
+        cocoapodsOutputPath,
+        mode,
+        'App.xcframework',
+        'macos-arm64_x86_64',
+        'dSYMs',
+        'App.framework.dSYM'
+      );
+      checkDirectoryExists(appFrameworkDsymPath);
+      await _checkDsym(path.join(
+        appFrameworkDsymPath,
+        'Contents',
+        'Resources',
+        'DWARF',
+        'App',
+      ));
+    }
+
     await _checkStatic(path.join(
       cocoapodsOutputPath,
       mode,
@@ -763,6 +822,83 @@ Future<void> _testBuildMacOSFramework(Directory projectDir) async {
     outputPath,
     'GeneratedPluginRegistrant.swift',
   ));
+
+  section('Build frameworks without plugins');
+  await _testBuildFrameworksWithoutPlugins(projectDir, platform: 'macos');
+}
+
+Future<void> _testBuildFrameworksWithoutPlugins(Directory projectDir, { required String platform}) async {
+  const String noPluginsOutputDir = 'flutter-frameworks-no-plugins';
+
+  await inDirectory(projectDir, () async {
+    await flutter(
+      'build',
+      options: <String>[
+        '$platform-framework',
+        '--cocoapods',
+        '--force', // Allow podspec creation on master.
+        '--output=$noPluginsOutputDir',
+        '--no-plugins',
+      ],
+    );
+  });
+
+  final String noPluginsOutputPath = path.join(projectDir.path, noPluginsOutputDir);
+  for (final String mode in <String>['Debug', 'Profile', 'Release']) {
+    checkFileExists(path.join(
+      noPluginsOutputPath,
+      mode,
+      'Flutter${platform == 'macos' ? 'MacOS' : ''}.podspec',
+    ));
+    checkDirectoryExists(path.join(
+      noPluginsOutputPath,
+      mode,
+      'App.xcframework',
+    ));
+
+    checkDirectoryNotExists(path.join(
+      noPluginsOutputPath,
+      mode,
+      'package_info.xcframework',
+    ));
+
+    checkDirectoryNotExists(path.join(
+      noPluginsOutputPath,
+      mode,
+      'connectivity.xcframework',
+    ));
+
+    checkDirectoryNotExists(path.join(
+      noPluginsOutputPath,
+      mode,
+      'Reachability.xcframework',
+    ));
+  }
+}
+
+Future<void> _testStaticAndNoPlugins(Directory projectDir) async {
+  const String noPluginsOutputDir = 'flutter-frameworks-no-plugins-static';
+  final ProcessResult result = await inDirectory(projectDir, () async {
+    return executeFlutter(
+        'build',
+        options: <String>[
+          'ios-framework',
+          '--cocoapods',
+          '--force', // Allow podspec creation on master.
+          '--output=$noPluginsOutputDir',
+          '--no-plugins',
+          '--static'
+        ],
+        canFail: true
+    );
+  });
+  if (result.exitCode == 0) {
+    throw TaskResult.failure('Build framework command did not exit with error as expected');
+  }
+  final String output = '${result.stdout}\n${result.stderr}';
+  if (!output.contains('--static cannot be used with the --no-plugins flag')) {
+    throw TaskResult.failure(output);
+  }
 }
 
 Future<void> _checkDylib(String pathToLibrary) async {
@@ -772,6 +908,13 @@ Future<void> _checkDylib(String pathToLibrary) async {
   }
 }
 
+Future<void> _checkDsym(String pathToSymbolFile) async {
+  final String binaryFileType = await fileType(pathToSymbolFile);
+  if (!binaryFileType.contains('dSYM companion file')) {
+    throw TaskResult.failure('$pathToSymbolFile is not a dSYM, found: $binaryFileType');
+  }
+}
+
 Future<void> _checkStatic(String pathToLibrary) async {
   final String binaryFileType = await fileType(pathToLibrary);
   if (!binaryFileType.contains('current ar archive random library')) {
@@ -779,15 +922,6 @@ Future<void> _checkStatic(String pathToLibrary) async {
   }
 }
 
-Future<void> _checkBitcode(String frameworkPath, String mode) async {
-  checkFileExists(frameworkPath);
-
-  // Bitcode only needed in Release mode for archiving.
-  if (mode == 'Release' && !await containsBitcode(frameworkPath)) {
-    throw TaskResult.failure('$frameworkPath does not contain bitcode');
-  }
-}
-
 Future<String> _dylibSymbols(String pathToDylib) {
   return eval('nm', <String>[
     '-g',
