@@ -27,8 +27,8 @@ Future<void> main() async {
       },
     );
 
-    // this variable cannot be `late`, as we reference it in the `finally` block
-    // which may execute before this field has been initialized
+    // This variable cannot be `late`, as we reference it in the `finally` block
+    // which may execute before this field has been initialized.
     String? simulatorDeviceId;
     section('Create Flutter module project');
 
@@ -51,7 +51,7 @@ Future<void> main() async {
       final Directory flutterModuleLibSource = Directory(path.join(flutterDirectory.path, 'dev', 'integration_tests', 'ios_host_app', 'flutterapp', 'lib'));
       final Directory flutterModuleLibDestination = Directory(path.join(projectDir.path, 'lib'));
 
-      // These test files don't have a .dart prefix so the analyzer will ignore them. They aren't in a
+      // These test files don't have a .dart extension so the analyzer will ignore them. They aren't in a
       // package and don't work on their own outside of the test module just created.
       final File main = File(path.join(flutterModuleLibSource.path, 'main'));
       main.copySync(path.join(flutterModuleLibDestination.path, 'main.dart'));
@@ -59,6 +59,36 @@ Future<void> main() async {
       final File marquee = File(path.join(flutterModuleLibSource.path, 'marquee'));
       marquee.copySync(path.join(flutterModuleLibDestination.path, 'marquee.dart'));
 
+      section('Create package with native assets');
+
+      await flutter(
+        'config',
+        options: <String>['--enable-native-assets'],
+      );
+
+      const String ffiPackageName = 'ffi_package';
+      await createFfiPackage(ffiPackageName, tempDir);
+
+      section('Add FFI package');
+
+      final File pubspec = File(path.join(projectDir.path, 'pubspec.yaml'));
+      String content = await pubspec.readAsString();
+      content = content.replaceFirst(
+        'dependencies:\n',
+        '''
+dependencies:
+  $ffiPackageName:
+    path: ../$ffiPackageName
+''',
+      );
+      await pubspec.writeAsString(content, flush: true);
+      await inDirectory(projectDir, () async {
+        await flutter(
+          'packages',
+          options: <String>['get'],
+        );
+      });
+
       section('Build ephemeral host app in release mode without CocoaPods');
 
       await inDirectory(projectDir, () async {
@@ -162,15 +192,13 @@ Future<void> main() async {
 
       section('Add plugins');
 
-      final File pubspec = File(path.join(projectDir.path, 'pubspec.yaml'));
-      String content = await pubspec.readAsString();
       content = content.replaceFirst(
-        '\ndependencies:\n',
+        'dependencies:\n',
         // One framework, one Dart-only, one that does not support iOS, and one with a resource bundle.
         '''
 dependencies:
   url_launcher: 6.0.20
-  android_alarm_manager: 0.4.5+11
+  android_alarm_manager: 2.0.2
   google_sign_in_ios: 5.5.0
   $dartPluginName:
     path: ../$dartPluginName
@@ -221,6 +249,11 @@ dependencies:
       // Dart-only, no embedded framework.
       checkDirectoryNotExists(path.join(ephemeralIOSHostApp.path, 'Frameworks', '$dartPluginName.framework'));
 
+      // Native assets embedded, no embedded framework.
+      const String libFfiPackageDylib = 'lib$ffiPackageName.dylib';
+      checkFileExists(path.join(ephemeralIOSHostApp.path, 'Frameworks', libFfiPackageDylib));
+      checkDirectoryNotExists(path.join(ephemeralIOSHostApp.path, 'Frameworks', '$ffiPackageName.framework'));
+
       section('Clean and pub get module');
 
       await inDirectory(projectDir, () async {
@@ -350,6 +383,11 @@ end
         'isolate_snapshot_data',
       ));
 
+      checkFileExists(path.join(
+        hostFrameworksDirectory,
+        libFfiPackageDylib,
+      ));
+
       section('Check the NOTICE file is correct');
 
       final String licenseFilePath = path.join(
@@ -437,7 +475,6 @@ end
         if ((await fileType(builtFlutterBinary)).contains('armv7')) {
           throw TaskResult.failure('Unexpected armv7 architecture slice in $builtFlutterBinary');
         }
-        await checkContainsBitcode(builtFlutterBinary);
 
         final String builtAppBinary = path.join(
           archivedAppPath,
@@ -449,7 +486,13 @@ end
         if ((await fileType(builtAppBinary)).contains('armv7')) {
           throw TaskResult.failure('Unexpected armv7 architecture slice in $builtAppBinary');
         }
-        await checkContainsBitcode(builtAppBinary);
+
+        // Check native assets are bundled.
+        checkFileExists(path.join(
+          archivedAppPath,
+          'Frameworks',
+          libFfiPackageDylib,
+        ));
 
         // The host app example builds plugins statically, url_launcher_ios.framework
         // should not exist.
@@ -458,6 +501,16 @@ end
           'Frameworks',
           'url_launcher_ios.framework',
         ));
+
+        checkFileExists(path.join(
+          '${objectiveCBuildArchiveDirectory.path}.xcarchive',
+          'dSYMs',
+          'App.framework.dSYM',
+          'Contents',
+          'Resources',
+          'DWARF',
+          'App'
+        ));
       });
 
       section('Run platform unit tests');
@@ -503,6 +556,7 @@ end
                 <String>[
                   '-r',
                   '-9',
+                  '-q',
                   zipPath,
                   'result.xcresult',
                 ],
@@ -539,8 +593,7 @@ end
       );
 
       if (!xcodebuildOutput.contains('flutter --verbose --local-engine-src-path=bogus assemble') || // Verbose output
-          !xcodebuildOutput.contains('Unable to detect a Flutter engine build directory in bogus') ||
-          !xcodebuildOutput.contains('Command PhaseScriptExecution failed with a nonzero exit code')) {
+          !xcodebuildOutput.contains('Unable to detect a Flutter engine build directory in bogus')) {
         return TaskResult.failure('Host Objective-C app build succeeded though flutter script failed');
       }
 
@@ -610,7 +663,7 @@ end
     } catch (e) {
       return TaskResult.failure(e.toString());
     } finally {
-      unawaited(removeIOSimulator(simulatorDeviceId));
+      unawaited(removeIOSSimulator(simulatorDeviceId));
       rmTree(tempDir);
     }
   });
