@@ -8,6 +8,9 @@ import 'package:flutter_devicelab/tasks/web_benchmarks.dart';
 /// Runs all Web benchmarks using the CanvasKit rendering backend.
 Future<void> main() async {
   await task(() async {
-    return runWebBenchmark(useCanvasKit: true);
+    return runWebBenchmark((
+      webRenderer: 'canvaskit',
+      useWasm: false
+    ));
   });
 }
