@@ -112,36 +112,41 @@ class TextFormFieldDemoState extends State<TextFormFieldDemo> {
 
   String? _validateName(String? value) {
     _formWasEdited = true;
-    if (value!.isEmpty)
+    if (value!.isEmpty) {
       return 'Name is required.';
+    }
     final RegExp nameExp = RegExp(r'^[A-Za-z ]+$');
-    if (!nameExp.hasMatch(value))
+    if (!nameExp.hasMatch(value)) {
       return 'Please enter only alphabetical characters.';
+    }
     return null;
   }
 
   String? _validatePhoneNumber(String? value) {
     _formWasEdited = true;
     final RegExp phoneExp = RegExp(r'^\(\d\d\d\) \d\d\d\-\d\d\d\d$');
-    if (!phoneExp.hasMatch(value!))
+    if (!phoneExp.hasMatch(value!)) {
       return '(###) ###-#### - Enter a US phone number.';
+    }
     return null;
   }
 
   String? _validatePassword(String? value) {
     _formWasEdited = true;
     final FormFieldState<String> passwordField = _passwordFieldKey.currentState!;
-    if (passwordField.value == null || passwordField.value!.isEmpty)
+    if (passwordField.value == null || passwordField.value!.isEmpty) {
       return 'Please enter a password.';
-    if (passwordField.value != value)
+    }
+    if (passwordField.value != value) {
       return "The passwords don't match";
+    }
     return null;
   }
 
-  Future<bool> _warnUserAboutInvalidData() async {
-    final FormState? form = _formKey.currentState;
-    if (form == null || !_formWasEdited || form.validate())
-      return true;
+  Future<void> _handlePopInvoked(bool didPop) async {
+    if (didPop) {
+      return;
+    }
 
     final bool? result = await showDialog<bool>(
       context: context,
@@ -162,7 +167,14 @@ class TextFormFieldDemoState extends State<TextFormFieldDemo> {
         );
       },
     );
-    return result!;
+
+    if (result ?? false) {
+      // Since this is the root route, quit the app where possible by invoking
+      // the SystemNavigator. If this wasn't the root route, then
+      // Navigator.maybePop could be used instead.
+      // See https://github.com/flutter/flutter/issues/11490
+      SystemNavigator.pop();
+    }
   }
 
   @override
@@ -179,7 +191,8 @@ class TextFormFieldDemoState extends State<TextFormFieldDemo> {
         child: Form(
           key: _formKey,
           autovalidateMode: _autovalidateMode,
-          onWillPop: _warnUserAboutInvalidData,
+          canPop: _formKey.currentState == null || !_formWasEdited || _formKey.currentState!.validate(),
+          onPopInvoked: _handlePopInvoked,
           child: Scrollbar(
             child: SingleChildScrollView(
               primary: true,
@@ -287,7 +300,7 @@ class TextFormFieldDemoState extends State<TextFormFieldDemo> {
                   const SizedBox(height: 24.0),
                   Text(
                     '* indicates required field',
-                    style: Theme.of(context).textTheme.caption,
+                    style: Theme.of(context).textTheme.bodySmall,
                   ),
                   const SizedBox(height: 24.0),
                 ],
@@ -313,30 +326,35 @@ class _UsNumberTextInputFormatter extends TextInputFormatter {
     final StringBuffer newText = StringBuffer();
     if (newTextLength >= 1) {
       newText.write('(');
-      if (newValue.selection.end >= 1)
+      if (newValue.selection.end >= 1) {
         selectionIndex++;
+      }
     }
     if (newTextLength >= 4) {
       final String value = newValue.text.substring(0, usedSubstringIndex = 3);
       newText.write('$value) ');
-      if (newValue.selection.end >= 3)
+      if (newValue.selection.end >= 3) {
         selectionIndex += 2;
+      }
     }
     if (newTextLength >= 7) {
       final String value = newValue.text.substring(3, usedSubstringIndex = 6);
       newText.write('$value-');
-      if (newValue.selection.end >= 6)
+      if (newValue.selection.end >= 6) {
         selectionIndex++;
+      }
     }
     if (newTextLength >= 11) {
       final String value = newValue.text.substring(6, usedSubstringIndex = 10);
       newText.write('$value ');
-      if (newValue.selection.end >= 10)
+      if (newValue.selection.end >= 10) {
         selectionIndex++;
+      }
     }
     // Dump the rest.
-    if (newTextLength >= usedSubstringIndex)
+    if (newTextLength >= usedSubstringIndex) {
       newText.write(newValue.text.substring(usedSubstringIndex));
+    }
     return TextEditingValue(
       text: newText.toString(),
       selection: TextSelection.collapsed(offset: selectionIndex),
