@@ -12,7 +12,7 @@ class ButtonTemplate extends TokenTemplate {
   final String tokenGroup;
 
   String _backgroundColor() {
-    if (tokens.containsKey('$tokenGroup.container.color')) {
+    if (tokenAvailable('$tokenGroup.container.color')) {
       return '''
 
     MaterialStateProperty.resolveWith((Set<MaterialState> states) {
@@ -24,32 +24,40 @@ class ButtonTemplate extends TokenTemplate {
     }
     return '''
 
-    ButtonStyleButton.allOrNull<Color>(Colors.transparent)''';
+    const MaterialStatePropertyAll<Color>(Colors.transparent)''';
   }
 
   String _elevation() {
-    if (tokens.containsKey('$tokenGroup.container.elevation')) {
+    if (tokenAvailable('$tokenGroup.container.elevation')) {
       return '''
 
     MaterialStateProperty.resolveWith((Set<MaterialState> states) {
       if (states.contains(MaterialState.disabled)) {
         return ${elevation("$tokenGroup.disabled.container")};
       }
+      if (states.contains(MaterialState.pressed)) {
+        return ${elevation("$tokenGroup.pressed.container")};
+      }
       if (states.contains(MaterialState.hovered)) {
         return ${elevation("$tokenGroup.hover.container")};
       }
       if (states.contains(MaterialState.focused)) {
         return ${elevation("$tokenGroup.focus.container")};
       }
-      if (states.contains(MaterialState.pressed)) {
-        return ${elevation("$tokenGroup.pressed.container")};
-      }
       return ${elevation("$tokenGroup.container")};
     })''';
     }
     return '''
 
-    ButtonStyleButton.allOrNull<double>(0.0)''';
+    const MaterialStatePropertyAll<double>(0.0)''';
+  }
+
+  String _elevationColor(String token) {
+    if (tokenAvailable(token)) {
+      return 'MaterialStatePropertyAll<Color>(${color(token)})';
+    } else {
+      return 'const MaterialStatePropertyAll<Color>(Colors.transparent)';
+    }
   }
 
   @override
@@ -84,61 +92,60 @@ class _${blockName}DefaultsM3 extends ButtonStyle {
   @override
   MaterialStateProperty<Color?>? get overlayColor =>
     MaterialStateProperty.resolveWith((Set<MaterialState> states) {
+      if (states.contains(MaterialState.pressed)) {
+        return ${componentColor('$tokenGroup.pressed.state-layer')};
+      }
       if (states.contains(MaterialState.hovered)) {
         return ${componentColor('$tokenGroup.hover.state-layer')};
       }
       if (states.contains(MaterialState.focused)) {
         return ${componentColor('$tokenGroup.focus.state-layer')};
       }
-      if (states.contains(MaterialState.pressed)) {
-        return ${componentColor('$tokenGroup.pressed.state-layer')};
-      }
       return null;
     });
 
-${tokens.containsKey("$tokenGroup.container.shadow-color") ? '''
   @override
   MaterialStateProperty<Color>? get shadowColor =>
-    ButtonStyleButton.allOrNull<Color>(${color("$tokenGroup.container.shadow-color")});''' : '''
-  // No default shadow color'''}
+    ${_elevationColor("$tokenGroup.container.shadow-color")};
 
-${tokens.containsKey("$tokenGroup.container.surface-tint-layer.color") ? '''
   @override
   MaterialStateProperty<Color>? get surfaceTintColor =>
-    ButtonStyleButton.allOrNull<Color>(${color("$tokenGroup.container.surface-tint-layer.color")});''' : '''
-  // No default surface tint color'''}
+    ${_elevationColor("$tokenGroup.container.surface-tint-layer.color")};
 
   @override
   MaterialStateProperty<double>? get elevation =>${_elevation()};
 
   @override
   MaterialStateProperty<EdgeInsetsGeometry>? get padding =>
-    ButtonStyleButton.allOrNull<EdgeInsetsGeometry>(_scaledPadding(context));
+    MaterialStatePropertyAll<EdgeInsetsGeometry>(_scaledPadding(context));
 
   @override
   MaterialStateProperty<Size>? get minimumSize =>
-    ButtonStyleButton.allOrNull<Size>(const Size(64.0, ${tokens["$tokenGroup.container.height"]}));
+    const MaterialStatePropertyAll<Size>(Size(64.0, ${getToken("$tokenGroup.container.height")}));
 
   // No default fixedSize
 
   @override
   MaterialStateProperty<Size>? get maximumSize =>
-    ButtonStyleButton.allOrNull<Size>(Size.infinite);
+    const MaterialStatePropertyAll<Size>(Size.infinite);
 
-${tokens.containsKey("$tokenGroup.outline.color") ? '''
+${tokenAvailable("$tokenGroup.outline.color") ? '''
   @override
   MaterialStateProperty<BorderSide>? get side =>
     MaterialStateProperty.resolveWith((Set<MaterialState> states) {
     if (states.contains(MaterialState.disabled)) {
       return ${border("$tokenGroup.disabled.outline")};
     }
+    if (states.contains(MaterialState.focused)) {
+      return ${border('$tokenGroup.focus.outline')};
+    }
     return ${border("$tokenGroup.outline")};
   });''' : '''
   // No default side'''}
 
   @override
   MaterialStateProperty<OutlinedBorder>? get shape =>
-    ButtonStyleButton.allOrNull<OutlinedBorder>(${shape("$tokenGroup.container")});
+    const MaterialStatePropertyAll<OutlinedBorder>(${shape("$tokenGroup.container", '')});
 
   @override
   MaterialStateProperty<MouseCursor?>? get mouseCursor =>
