@@ -152,19 +152,13 @@ class PathCommandAnimation {
   }
 
   String toDart() {
-    String dartCommandClass;
-    switch (type) {
-      case 'M':
-        dartCommandClass = '_PathMoveTo';
-      case 'C':
-        dartCommandClass = '_PathCubicTo';
-      case 'L':
-        dartCommandClass = '_PathLineTo';
-      case 'Z':
-        dartCommandClass = '_PathClose';
-      default:
-        throw Exception('unsupported path command: $type');
-    }
+    final String dartCommandClass = switch (type) {
+      'M' => '_PathMoveTo',
+      'C' => '_PathCubicTo',
+      'L' => '_PathLineTo',
+      'Z' => '_PathClose',
+      _ => throw Exception('unsupported path command: $type'),
+    };
     final StringBuffer sb = StringBuffer();
     sb.write('${kIndent * 4}const $dartCommandClass(\n');
     for (final List<Point<double>> pointFrames in points) {
