@@ -2,28 +2,24 @@
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
-// Flutter code sample for MaterialStateOutlinedBorder
-
 import 'package:flutter/material.dart';
 
-void main() => runApp(const MyApp());
+/// Flutter code sample for [MaterialStateOutlinedBorder].
 
-class MyApp extends StatelessWidget {
-  const MyApp({super.key});
+void main() => runApp(const MaterialStateOutlinedBorderExampleApp());
 
-  static const String _title = 'Flutter Code Sample';
+class MaterialStateOutlinedBorderExampleApp extends StatelessWidget {
+  const MaterialStateOutlinedBorderExampleApp({super.key});
 
   @override
   Widget build(BuildContext context) {
     return const MaterialApp(
-      title: _title,
-      home: MyStatefulWidget(),
+      home: MaterialStateOutlinedBorderExample(),
     );
   }
 }
 
-class SelectedBorder extends RoundedRectangleBorder
-    implements MaterialStateOutlinedBorder {
+class SelectedBorder extends RoundedRectangleBorder implements MaterialStateOutlinedBorder {
   const SelectedBorder();
 
   @override
@@ -35,14 +31,14 @@ class SelectedBorder extends RoundedRectangleBorder
   }
 }
 
-class MyStatefulWidget extends StatefulWidget {
-  const MyStatefulWidget({super.key});
+class MaterialStateOutlinedBorderExample extends StatefulWidget {
+  const MaterialStateOutlinedBorderExample({super.key});
 
   @override
-  State<MyStatefulWidget> createState() => _MyStatefulWidgetState();
+  State<MaterialStateOutlinedBorderExample> createState() => _MaterialStateOutlinedBorderExampleState();
 }
 
-class _MyStatefulWidgetState extends State<MyStatefulWidget> {
+class _MaterialStateOutlinedBorderExampleState extends State<MaterialStateOutlinedBorderExample> {
   bool isSelected = true;
 
   @override
