@@ -2,33 +2,30 @@
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
-// Flutter code sample for OutlinedButton
-
 import 'package:flutter/material.dart';
 
-void main() => runApp(const MyApp());
+/// Flutter code sample for [OutlinedButton].
 
-class MyApp extends StatelessWidget {
-  const MyApp({super.key});
+void main() => runApp(const OutlinedButtonExampleApp());
 
-  static const String _title = 'Flutter Code Sample';
+class OutlinedButtonExampleApp extends StatelessWidget {
+  const OutlinedButtonExampleApp({super.key});
 
   @override
   Widget build(BuildContext context) {
     return MaterialApp(
-      title: _title,
       home: Scaffold(
-        appBar: AppBar(title: const Text(_title)),
+        appBar: AppBar(title: const Text('OutlinedButton Sample')),
         body: const Center(
-          child: MyStatelessWidget(),
+          child: OutlinedButtonExample(),
         ),
       ),
     );
   }
 }
 
-class MyStatelessWidget extends StatelessWidget {
-  const MyStatelessWidget({super.key});
+class OutlinedButtonExample extends StatelessWidget {
+  const OutlinedButtonExample({super.key});
 
   @override
   Widget build(BuildContext context) {
