@@ -2,15 +2,14 @@
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
-// Flutter code sample for FontFeature.FontFeature.contextualAlternates
-import 'dart:ui';
-
 import 'package:flutter/widgets.dart';
 
+/// Flutter code sample for [FontFeature.FontFeature.contextualAlternates].
+
 void main() => runApp(const ExampleApp());
 
 class ExampleApp extends StatelessWidget {
-  const ExampleApp({Key? key}) : super(key: key);
+  const ExampleApp({super.key});
 
   @override
   Widget build(BuildContext context) {
@@ -22,11 +21,12 @@ class ExampleApp extends StatelessWidget {
 }
 
 class ExampleWidget extends StatelessWidget {
-  const ExampleWidget({Key? key}) : super(key: key);
+  const ExampleWidget({super.key});
 
   @override
   Widget build(BuildContext context) {
-    // The Barriecito font can be downloaded from Google Fonts (https://www.google.com/fonts).
+    // The Barriecito font can be downloaded from Google Fonts
+    // (https://www.google.com/fonts).
     return const Text(
       "Ooohh, we weren't going to tell him that.",
       style: TextStyle(
