@@ -9,11 +9,16 @@ import 'package:flutter_test/flutter_test.dart';
 void main() {
   testWidgets('Finds CircularProgressIndicator', (WidgetTester tester) async {
     await tester.pumpWidget(
-      const example.MyApp(),
+      const example.ProgressIndicatorApp(),
     );
+
     expect(
       find.bySemanticsLabel('Circular progress indicator'),
       findsOneWidget,
     );
+
+    // Test if CircularProgressIndicator is animating.
+    await tester.pump(const Duration(seconds: 2));
+    expect(tester.hasRunningAnimations, isTrue);
   });
 }
