@@ -19,7 +19,7 @@ import '../camera.dart';
 // TODO(stuartmorgan): Fix this naming the next time there's a breaking change
 // to this package.
 // ignore: camel_case_types
-typedef onLatestImageAvailable = Function(CameraImage image);
+typedef onLatestImageAvailable = void Function(CameraImage image);
 
 /// Completes with a list of available cameras.
 ///
@@ -265,7 +265,10 @@ class CameraController extends ValueNotifier<CameraValue> {
 
   bool _isDisposed = false;
   StreamSubscription<CameraImageData>? _imageStreamSubscription;
-  FutureOr<bool>? _initCalled;
+  // A Future awaiting an attempt to initialize (e.g. after `initialize` was
+  // just called). If the controller has not been initialized at least once,
+  // this value is null.
+  Future<void>? _initializeFuture;
   StreamSubscription<DeviceOrientationChangedEvent>?
       _deviceOrientationSubscription;
 
@@ -294,11 +297,15 @@ class CameraController extends ValueNotifier<CameraValue> {
         'initialize was called on a disposed CameraController',
       );
     }
+
+    final Completer<void> initializeCompleter = Completer<void>();
+    _initializeFuture = initializeCompleter.future;
+
     try {
       final Completer<CameraInitializedEvent> initializeCompleter =
           Completer<CameraInitializedEvent>();
 
-      _deviceOrientationSubscription = CameraPlatform.instance
+      _deviceOrientationSubscription ??= CameraPlatform.instance
           .onDeviceOrientationChanged()
           .listen((DeviceOrientationChangedEvent event) {
         value = value.copyWith(
@@ -343,9 +350,9 @@ class CameraController extends ValueNotifier<CameraValue> {
       );
     } on PlatformException catch (e) {
       throw CameraException(e.code, e.message);
+    } finally {
+      initializeCompleter.complete();
     }
-
-    _initCalled = true;
   }
 
   /// Prepare the capture session for video recording.
@@ -402,6 +409,11 @@ class CameraController extends ValueNotifier<CameraValue> {
       await CameraPlatform.instance.setDescriptionWhileRecording(description);
       value = value.copyWith(description: description);
     } else {
+      if (_initializeFuture != null) {
+        await _initializeFuture;
+        await CameraPlatform.instance.dispose(_cameraId);
+      }
+
       await _initializeWithDescription(description);
     }
   }
@@ -518,7 +530,7 @@ class CameraController extends ValueNotifier<CameraValue> {
       );
     }
 
-    Function(CameraImageData image)? streamCallback;
+    void Function(CameraImageData image)? streamCallback;
     if (onAvailable != null) {
       streamCallback = (CameraImageData imageData) {
         onAvailable(CameraImage.fromPlatformInterface(imageData));
@@ -841,8 +853,8 @@ class CameraController extends ValueNotifier<CameraValue> {
     _unawaited(_deviceOrientationSubscription?.cancel());
     _isDisposed = true;
     super.dispose();
-    if (_initCalled != null) {
-      await _initCalled;
+    if (_initializeFuture != null) {
+      await _initializeFuture;
       await CameraPlatform.instance.dispose(_cameraId);
     }
   }
