@@ -12,12 +12,14 @@ import 'package:analyzer/source/line_info.dart';
 import 'package:analyzer/src/dart/analysis/driver.dart';
 import 'package:analyzer/src/dart/analysis/file_state.dart';
 import 'package:analyzer/src/dart/analysis/index.dart';
+import 'package:analyzer/src/dart/analysis/results.dart';
 import 'package:analyzer/src/dart/ast/extensions.dart';
 import 'package:analyzer/src/dart/ast/utilities.dart';
 import 'package:analyzer/src/dart/element/element.dart';
 import 'package:analyzer/src/summary/idl.dart';
 import 'package:analyzer/src/util/performance/operation_performance.dart';
 import 'package:analyzer/src/utilities/cancellation.dart';
+import 'package:analyzer/src/utilities/fuzzy_matcher.dart';
 import 'package:collection/collection.dart';
 
 Element _getEnclosingElement(CompilationUnitElement unitElement, int offset) {
@@ -37,6 +39,10 @@ DeclarationKind? _getSearchElementKind(Element element) {
     return DeclarationKind.ENUM;
   }
 
+  if (element is ExtensionTypeElement) {
+    return DeclarationKind.EXTENSION_TYPE;
+  }
+
   if (element is MixinElement) {
     return DeclarationKind.MIXIN;
   }
@@ -123,6 +129,7 @@ enum DeclarationKind {
   ENUM,
   ENUM_CONSTANT,
   EXTENSION,
+  EXTENSION_TYPE,
   FIELD,
   FUNCTION,
   FUNCTION_TYPE_ALIAS,
@@ -142,47 +149,44 @@ class FindDeclarations {
   final List<AnalysisDriver> drivers;
   final WorkspaceSymbols result;
   final int? maxResults;
-  final bool Function(String) isMatch;
+  final String pattern;
+  final FuzzyMatcher matcher;
   final String? onlyForFile;
   final bool onlyAnalyzed;
+  final OwnedFiles ownedFiles;
   final OperationPerformanceImpl performance;
 
   FindDeclarations(
     this.drivers,
     this.result,
-    this.isMatch,
+    this.pattern,
     this.maxResults, {
     this.onlyForFile,
     this.onlyAnalyzed = false,
+    required this.ownedFiles,
     required this.performance,
-  });
+  }) : matcher = FuzzyMatcher(pattern);
 
   Future<void> compute([CancellationToken? cancellationToken]) async {
-    var searchedFiles = SearchedFiles();
-    // Add analyzed files first, so priority is given to drivers that analyze
-    // files over those that just reference them.
-    for (var driver in drivers) {
-      searchedFiles.ownAnalyzed(driver.search);
-    }
     if (!onlyAnalyzed) {
       await performance.runAsync('discoverAvailableFiles', (performance) async {
         await Future.wait(
           drivers.map((driver) => driver.discoverAvailableFiles()),
         );
       });
-
-      performance.run('ownKnown', (performance) {
-        for (var driver in drivers) {
-          searchedFiles.ownKnown(driver.search);
-        }
-      });
     }
 
+    final entries = [
+      ...ownedFiles.addedFiles.entries,
+      if (!onlyAnalyzed) ...ownedFiles.knownFiles.entries,
+    ];
+
     await performance.runAsync('findDeclarations', (performance) async {
       await _FindDeclarations(
-        searchedFiles,
+        entries,
         result,
-        isMatch,
+        pattern,
+        matcher,
         maxResults,
         onlyForFile: onlyForFile,
         performance: performance,
@@ -307,6 +311,7 @@ class Search {
         if (unitResult is UnitElementResult) {
           unitResult.element.classes.forEach(addElements);
           unitResult.element.enums.forEach(addElements);
+          unitResult.element.extensionTypes.forEach(addElements);
           unitResult.element.mixins.forEach(addElements);
         }
       }
@@ -463,6 +468,7 @@ class Search {
         unitElement.classes.forEach(addElement);
         unitElement.enums.forEach(addElement);
         unitElement.extensions.forEach(addElement);
+        unitElement.extensionTypes.forEach(addElement);
         unitElement.functions.forEach(addElement);
         unitElement.mixins.forEach(addElement);
         unitElement.topLevelVariables.forEach(addElement);
@@ -1005,7 +1011,7 @@ class _FindCompilationUnitDeclarations {
   final LineInfo lineInfo;
   final WorkspaceSymbols result;
   final int? maxResults;
-  final bool Function(String) isMatch;
+  final FuzzyMatcher matcher;
   final void Function(Declaration) collect;
 
   _FindCompilationUnitDeclarations(
@@ -1013,7 +1019,7 @@ class _FindCompilationUnitDeclarations {
     this.filePath,
     this.result,
     this.maxResults,
-    this.isMatch,
+    this.matcher,
     this.collect,
   ) : lineInfo = unit.lineInfo;
 
@@ -1027,6 +1033,7 @@ class _FindCompilationUnitDeclarations {
     _addClasses(unit.enums);
     _addClasses(unit.mixins);
     _addExtensions(unit.extensions);
+    _addClasses(unit.extensionTypes);
     _addFunctions(unit.functions);
     _addTypeAliases(unit.typeAliases);
     _addVariables(unit.topLevelVariables);
@@ -1066,7 +1073,7 @@ class _FindCompilationUnitDeclarations {
       throw const _MaxNumberOfDeclarationsError();
     }
 
-    if (!isMatch(name)) {
+    if (matcher.score(name) < 0) {
       return;
     }
 
@@ -1173,17 +1180,19 @@ class _FindCompilationUnitDeclarations {
 
 /// Searches through [files] for declarations.
 class _FindDeclarations {
-  final SearchedFiles files;
+  final List<MapEntry<Uri, AnalysisDriver>> fileEntries;
   final WorkspaceSymbols result;
   final int? maxResults;
-  final bool Function(String) isMatch;
+  final String pattern;
+  final FuzzyMatcher matcher;
   final String? onlyForFile;
   final OperationPerformanceImpl performance;
 
   _FindDeclarations(
-    this.files,
+    this.fileEntries,
     this.result,
-    this.isMatch,
+    this.pattern,
+    this.matcher,
     this.maxResults, {
     this.onlyForFile,
     required this.performance,
@@ -1203,30 +1212,42 @@ class _FindDeclarations {
 
     var filesProcessed = 0;
     try {
-      for (var entry in files.uriOwners.entries) {
+      for (var entry in fileEntries) {
         var uri = entry.key;
-        var search = entry.value;
-        var elementResult = await performance.runAsync(
+        var analysisDriver = entry.value;
+
+        final libraryElement = await performance.runAsync(
           'getLibraryByUri',
           (performance) async {
-            return await search._driver.getLibraryByUri(uri.toString());
+            final result = await analysisDriver.getLibraryByUri('$uri');
+            if (result is LibraryElementResultImpl) {
+              return result.element as LibraryElementImpl;
+            }
+            return null;
           },
         );
-        if (elementResult is LibraryElementResult) {
-          var units = elementResult.element.units;
+
+        if (libraryElement != null) {
+          // Check if there is any name that could match the pattern.
+          var match = libraryElement.nameUnion.contains(pattern);
+          if (!match) {
+            continue;
+          }
+
+          var units = libraryElement.units;
           for (var i = 0; i < units.length; i++) {
             var unit = units[i];
             var filePath = unit.source.fullName;
             if (onlyForFile != null && filePath != onlyForFile) {
               continue;
             }
-            performance.run('finder', (performance) {
+            performance.run('unitDeclarations', (performance) {
               var finder = _FindCompilationUnitDeclarations(
                 unit,
                 filePath,
                 result,
                 maxResults,
-                isMatch,
+                matcher,
                 result.declarations.add,
               );
               finder.compute(cancellationToken);
