@@ -57,11 +57,6 @@ class TypeSystemImpl implements TypeSystem {
   /// The provider of types for the system.
   final TypeProviderImpl typeProvider;
 
-  /// False if implicit casts should always be disallowed.
-  ///
-  /// This affects the behavior of [isAssignableTo].
-  bool implicitCasts;
-
   /// True if "strict casts" should be enforced.
   ///
   /// This affects the behavior of [isAssignableTo].
@@ -91,7 +86,6 @@ class TypeSystemImpl implements TypeSystem {
   late final SubtypeHelper _subtypeHelper;
 
   TypeSystemImpl({
-    required this.implicitCasts,
     required this.isNonNullableByDefault,
     required this.strictCasts,
     required this.strictInference,
@@ -602,19 +596,6 @@ class TypeSystemImpl implements TypeSystem {
     return parameters;
   }
 
-  /// Computes the greatest lower bound of [T1] and [T2].
-  DartType getGreatestLowerBound(DartType T1, DartType T2) {
-    return _greatestLowerBoundHelper.getGreatestLowerBound(T1, T2);
-  }
-
-  /// Compute the least upper bound of two types.
-  ///
-  /// https://github.com/dart-lang/language
-  /// See `resources/type-system/upper-lower-bounds.md`
-  DartType getLeastUpperBound(DartType T1, DartType T2) {
-    return _leastUpperBoundHelper.getLeastUpperBound(T1, T2);
-  }
-
   /// Returns the greatest closure of [type] with respect to [typeParameters].
   ///
   /// https://github.com/dart-lang/language
@@ -677,6 +658,11 @@ class TypeSystemImpl implements TypeSystem {
     }
   }
 
+  @override
+  DartType greatestLowerBound(DartType T1, DartType T2) {
+    return _greatestLowerBoundHelper.getGreatestLowerBound(T1, T2);
+  }
+
   /// Given a generic function type `F<T0, T1, ... Tn>` and a context type C,
   /// infer an instantiation of F, such that `F<S0, S1, ..., Sn>` <: C.
   ///
@@ -932,10 +918,9 @@ class TypeSystemImpl implements TypeSystem {
       }
     }
 
-    // First make sure that the static analysis options, `implicit-casts: false`
-    // and `strict-casts: true` disable all downcasts, including casts from
-    // `dynamic`.
-    if (!implicitCasts || strictCasts) {
+    // First make sure that the static analysis option, `strict-casts: true`
+    // disables all downcasts, including casts from `dynamic`.
+    if (strictCasts) {
       return false;
     }
 
@@ -1303,8 +1288,13 @@ class TypeSystemImpl implements TypeSystem {
       return isNonNullable(type.promotedBound!);
     } else if (type.nullabilitySuffix == NullabilitySuffix.question) {
       return false;
-    } else if (type is InterfaceType && type.isDartAsyncFutureOr) {
-      return isNonNullable(type.typeArguments[0]);
+    } else if (type is InterfaceTypeImpl) {
+      if (type.isDartAsyncFutureOr) {
+        return isNonNullable(type.typeArguments[0]);
+      }
+      if (type.representationType case final representationType?) {
+        return isNonNullable(representationType);
+      }
     } else if (type is TypeParameterType) {
       var bound = type.element.bound;
       return bound != null && isNonNullable(bound);
@@ -1348,8 +1338,10 @@ class TypeSystemImpl implements TypeSystem {
       return isNullable(type.promotedBound!);
     } else if (type.nullabilitySuffix == NullabilitySuffix.question) {
       return true;
-    } else if (type.isDartAsyncFutureOr) {
-      return isNullable((type as InterfaceType).typeArguments[0]);
+    } else if (type is InterfaceTypeImpl) {
+      if (type.isDartAsyncFutureOr) {
+        return isNullable(type.typeArguments[0]);
+      }
     }
     return false;
   }
@@ -1392,8 +1384,13 @@ class TypeSystemImpl implements TypeSystem {
       return false;
     } else if (type.nullabilitySuffix != NullabilitySuffix.none) {
       return false;
-    } else if (type is InterfaceType && type.isDartAsyncFutureOr) {
-      return isStrictlyNonNullable(type.typeArguments[0]);
+    } else if (type is InterfaceTypeImpl) {
+      if (type.isDartAsyncFutureOr) {
+        return isStrictlyNonNullable(type.typeArguments[0]);
+      }
+      if (type.representationType case final representationType?) {
+        return isStrictlyNonNullable(representationType);
+      }
     } else if (type is TypeParameterType) {
       return isStrictlyNonNullable(type.bound);
     }
@@ -1450,6 +1447,26 @@ class TypeSystemImpl implements TypeSystem {
     return false;
   }
 
+  /// Whether [type] is a valid superinterface for an extension type.
+  bool isValidExtensionTypeSuperinterface(DartType type) {
+    if (type is! InterfaceType) {
+      return false;
+    }
+
+    if (type.nullabilitySuffix == NullabilitySuffix.question) {
+      return false;
+    }
+
+    if (type.isDartAsyncFutureOr ||
+        type.isDartCoreFunction ||
+        type.isDartCoreNull ||
+        type.isDartCoreRecord) {
+      return false;
+    }
+
+    return true;
+  }
+
   /// See `15.2 Super-bounded types` in the language specification.
   TypeBoundedResult isWellBounded(
     DartType type, {
@@ -1523,8 +1540,8 @@ class TypeSystemImpl implements TypeSystem {
   }
 
   @override
-  DartType leastUpperBound(DartType leftType, DartType rightType) {
-    return getLeastUpperBound(leftType, rightType);
+  DartType leastUpperBound(DartType T1, DartType T2) {
+    return _leastUpperBoundHelper.getLeastUpperBound(T1, T2);
   }
 
   /// Returns a nullable version of [type].  The result would be equivalent to
@@ -1911,11 +1928,9 @@ class TypeSystemImpl implements TypeSystem {
   }
 
   void updateOptions({
-    required bool implicitCasts,
     required bool strictCasts,
     required bool strictInference,
   }) {
-    this.implicitCasts = implicitCasts;
     this.strictCasts = strictCasts;
     this.strictInference = strictInference;
   }
