@@ -2,7 +2,6 @@
 // for details. All rights reserved. Use of this source code is governed by a
 // BSD-style license that can be found in the LICENSE file.
 
-import 'package:analyzer/dart/ast/ast.dart';
 import 'package:analyzer/dart/ast/syntactic_entity.dart';
 import 'package:analyzer/dart/element/element.dart';
 import 'package:analyzer/dart/element/nullability_suffix.dart';
@@ -828,7 +827,7 @@ class FunctionReferenceResolver {
   }) {
     // TODO(srawlins): set the static element of [typeName].
     // This involves a fair amount of resolution, as [name] may be a prefixed
-    // identifier, etc. [TypeName]s should be resolved in [ResolutionVisitor],
+    // identifier, etc. [NamedType]s should be resolved in [ResolutionVisitor],
     // and this could be done for nodes like this via [AstRewriter].
     var typeName = name.toNamedType(
       typeArguments: node.typeArguments,
