@@ -8,7 +8,7 @@ import 'webgl1.dart';
 
 @JS('EXT_sRGB')
 @staticInterop
-class EXT_sRGB implements JSObject {
+class EXT_sRGB {
   external static GLenum get SRGB_EXT;
   external static GLenum get SRGB_ALPHA_EXT;
   external static GLenum get SRGB8_ALPHA8_EXT;
