@@ -4,17 +4,9 @@
 
 import 'dart:js_interop';
 
-@JS('NavigatorStorage')
-@staticInterop
-class NavigatorStorage implements JSObject {}
-
-extension NavigatorStorageExtension on NavigatorStorage {
-  external StorageManager get storage;
-}
-
 @JS('StorageManager')
 @staticInterop
-class StorageManager implements JSObject {}
+class StorageManager {}
 
 extension StorageManagerExtension on StorageManager {
   external JSPromise getDirectory();
@@ -26,7 +18,7 @@ extension StorageManagerExtension on StorageManager {
 @JS()
 @staticInterop
 @anonymous
-class StorageEstimate implements JSObject {
+class StorageEstimate {
   external factory StorageEstimate({
     int usage,
     int quota,
