@@ -8,7 +8,7 @@ import 'webgl1.dart';
 
 @JS('WEBGL_compressed_texture_etc')
 @staticInterop
-class WEBGL_compressed_texture_etc implements JSObject {
+class WEBGL_compressed_texture_etc {
   external static GLenum get COMPRESSED_R11_EAC;
   external static GLenum get COMPRESSED_SIGNED_R11_EAC;
   external static GLenum get COMPRESSED_RG11_EAC;
