@@ -69,6 +69,9 @@ class Engine {
   /// The same seed will shuffle the tests in the same way every time.
   int? testRandomizeOrderingSeed;
 
+  /// Whether to stop running tests after a failure.
+  bool _stopOnFirstFailure;
+
   /// A pool that limits the number of test suites running concurrently.
   final Pool _runPool;
 
@@ -97,7 +100,7 @@ class Engine {
   }
 
   /// A group of futures for each test suite.
-  final _group = FutureGroup();
+  final _group = FutureGroup<void>();
 
   /// All of the engine's stream subscriptions.
   final _subscriptions = <StreamSubscription>{};
@@ -202,8 +205,16 @@ class Engine {
   /// Omitting this argument or passing `0` disables shuffling.
   ///
   /// [coverage] specifies a directory to output coverage information.
-  Engine({int? concurrency, String? coverage, this.testRandomizeOrderingSeed})
-      : _runPool = Pool(concurrency ?? 1),
+  ///
+  /// If [stopOnFirstFailure] then a single failing test will cause the engine
+  /// to [close] and stop ruunning further tests.
+  Engine({
+    int? concurrency,
+    String? coverage,
+    this.testRandomizeOrderingSeed,
+    bool stopOnFirstFailure = false,
+  })  : _runPool = Pool(concurrency ?? 1),
+        _stopOnFirstFailure = stopOnFirstFailure,
         _coverage = coverage {
     _group.future.then((_) {
       _onTestStartedGroup.close();
@@ -222,8 +233,12 @@ class Engine {
   /// [concurrency] controls how many suites are run at once. If [runSkipped] is
   /// `true`, skipped tests will be run as though they weren't skipped.
   factory Engine.withSuites(List<RunnerSuite> suites,
-      {int? concurrency, String? coverage}) {
-    var engine = Engine(concurrency: concurrency, coverage: coverage);
+      {int? concurrency, String? coverage, bool stopOnFirstFailure = false}) {
+    var engine = Engine(
+      concurrency: concurrency,
+      coverage: coverage,
+      stopOnFirstFailure: stopOnFirstFailure,
+    );
     for (var suite in suites) {
       engine.suiteSink.add(suite);
     }
@@ -371,7 +386,10 @@ class Engine {
     // loop pump to avoid starving non-microtask events.
     await Future(() {});
 
-    if (!_restarted.contains(liveTest)) return;
+    if (!_restarted.contains(liveTest)) {
+      if (_stopOnFirstFailure && liveTest.state.result.isFailing) close();
+      return;
+    }
     await _runLiveTest(suiteController, liveTest.copy(),
         countSuccess: countSuccess);
     _restarted.remove(liveTest);
