@@ -8,6 +8,7 @@ import 'dart:io';
 
 import 'package:dwds/data/debug_event.dart';
 import 'package:dwds/data/register_event.dart';
+import 'package:dwds/src/config/tool_configuration.dart';
 import 'package:dwds/src/connections/app_connection.dart';
 import 'package:dwds/src/debugging/debugger.dart';
 import 'package:dwds/src/debugging/execution_context.dart';
@@ -18,7 +19,6 @@ import 'package:dwds/src/debugging/modules.dart';
 import 'package:dwds/src/debugging/remote_debugger.dart';
 import 'package:dwds/src/debugging/skip_list.dart';
 import 'package:dwds/src/events.dart';
-import 'package:dwds/src/loaders/strategy.dart';
 import 'package:dwds/src/readers/asset_reader.dart';
 import 'package:dwds/src/services/batched_expression_evaluator.dart';
 import 'package:dwds/src/services/expression_compiler.dart';
@@ -27,7 +27,8 @@ import 'package:dwds/src/utilities/dart_uri.dart';
 import 'package:dwds/src/utilities/shared.dart';
 import 'package:logging/logging.dart' hide LogRecord;
 import 'package:pub_semver/pub_semver.dart' as semver;
-import 'package:vm_service/vm_service.dart';
+import 'package:vm_service/vm_service.dart' hide vmServiceVersion;
+import 'package:vm_service_interface/vm_service_interface.dart';
 import 'package:webkit_inspection_protocol/webkit_inspection_protocol.dart';
 
 /// A proxy from the chrome debug protocol to the dart vm service protocol.
@@ -173,21 +174,30 @@ class ChromeProxyService implements VmServiceInterface {
   }
 
   Future<void> _updateCompilerDependencies(String entrypoint) async {
-    final metadataProvider = globalLoadStrategy.metadataProviderFor(entrypoint);
-    final moduleFormat = globalLoadStrategy.moduleFormat;
+    final loadStrategy = globalToolConfiguration.loadStrategy;
+    final moduleFormat = loadStrategy.moduleFormat;
+    final canaryFeatures = loadStrategy.buildSettings.canaryFeatures;
+    final experiments = loadStrategy.buildSettings.experiments;
+
+    // TODO(annagrin): Read null safety setting from the build settings.
+    final metadataProvider = loadStrategy.metadataProviderFor(entrypoint);
     final soundNullSafety = await metadataProvider.soundNullSafety;
 
     _logger.info('Initializing expression compiler for $entrypoint '
         'with sound null safety: $soundNullSafety');
 
+    final compilerOptions = CompilerOptions(
+      moduleFormat: moduleFormat,
+      soundNullSafety: soundNullSafety,
+      canaryFeatures: canaryFeatures,
+      experiments: experiments,
+    );
+
     final compiler = _compiler;
     if (compiler != null) {
-      await compiler.initialize(
-        moduleFormat: moduleFormat,
-        soundNullSafety: soundNullSafety,
-      );
+      await compiler.initialize(compilerOptions);
       final dependencies =
-          await globalLoadStrategy.moduleInfoForEntrypoint(entrypoint);
+          await loadStrategy.moduleInfoForEntrypoint(entrypoint);
       await captureElapsedTime(
         () async {
           final result = await compiler.updateDependencies(dependencies);
@@ -382,7 +392,7 @@ class ChromeProxyService implements VmServiceInterface {
     String scriptId,
     int line, {
     int? column,
-  }) async {
+  }) {
     return wrapInErrorHandlerAsync(
       'addBreakpoint',
       () => _addBreakpoint(isolateId, scriptId, line),
@@ -485,7 +495,7 @@ class ChromeProxyService implements VmServiceInterface {
       ),
     );
     final expression = '''
-${globalLoadStrategy.loadModuleSnippet}("dart_sdk").developer.invokeExtension(
+${globalToolConfiguration.loadStrategy.loadModuleSnippet}("dart_sdk").developer.invokeExtension(
     "$method", JSON.stringify(${jsonEncode(stringArgs)}));
 ''';
     final result = await inspector.jsEvaluate(expression, awaitPromise: true);
@@ -608,7 +618,35 @@ ${globalLoadStrategy.loadModuleSnippet}("dart_sdk").developer.invokeExtension(
           await isCompilerInitialized;
           _checkIsolate('evaluate', isolateId);
 
-          final library = await inspector.getLibrary(targetId);
+          late Obj object;
+          try {
+            object = await inspector.getObject(targetId);
+          } catch (_) {
+            return ErrorRef(
+              kind: 'error',
+              message: 'Evaluate is called on an unsupported target:'
+                  '$targetId',
+              id: createId(),
+            );
+          }
+
+          final library =
+              object is Library ? object : inspector.isolate.rootLib;
+
+          if (object is Instance) {
+            // Evaluate is called on a target - convert this to a dart
+            // expression and scope by adding a target variable to the
+            // expression and the scope, for example:
+            //
+            // Library: 'package:hello_world/main.dart'
+            // Expression: 'hashCode' => 'x.hashCode'
+            // Scope: {} => { 'x' : targetId }
+
+            final target = _newVariableForScope(scope);
+            expression = '$target.$expression';
+            scope = (scope ?? {})..addAll({target: targetId});
+          }
+
           return await _getEvaluationResult(
             isolateId,
             () => evaluator.evaluateExpression(
@@ -621,7 +659,7 @@ ${globalLoadStrategy.loadModuleSnippet}("dart_sdk").developer.invokeExtension(
           );
         }
         throw RPCError(
-          'evaluateInFrame',
+          'evaluate',
           RPCErrorKind.kInvalidRequest.code,
           'Expression evaluation is not supported for this configuration.',
         );
@@ -630,6 +668,15 @@ ${globalLoadStrategy.loadModuleSnippet}("dart_sdk").developer.invokeExtension(
     );
   }
 
+  String _newVariableForScope(Map<String, String>? scope) {
+    // Find a new variable not in scope.
+    var candidate = 'x';
+    while (scope?.containsKey(candidate) ?? false) {
+      candidate += '\$1';
+    }
+    return candidate;
+  }
+
   @override
   Future<Response> evaluateInFrame(
     String isolateId,
@@ -803,6 +850,9 @@ ${globalLoadStrategy.loadModuleSnippet}("dart_sdk").developer.invokeExtension(
     bool? forceCompile,
     bool? reportLines,
     List<String>? libraryFilters,
+    // Note: Ignore the optional librariesAlreadyCompiled parameter. It is here
+    // to match the VM service interface.
+    List<String>? librariesAlreadyCompiled,
   }) =>
       wrapInErrorHandlerAsync(
         'getSourceReport',
@@ -987,8 +1037,8 @@ ${globalLoadStrategy.loadModuleSnippet}("dart_sdk").developer.invokeExtension(
     return (await debuggerFuture).pause();
   }
 
-  // Note: Ignore the optional local parameter, it is there to keep the method
-  // signature consistent with the VM service interface.
+  // Note: Ignore the optional local parameter, when it is set to `true` the
+  // request is intercepted and handled by DDS.
   @override
   Future<UriList> lookupResolvedPackageUris(
     String isolateId,
@@ -1348,7 +1398,8 @@ ${globalLoadStrategy.loadModuleSnippet}("dart_sdk").developer.invokeExtension(
       final isolate = inspector.isolate;
       if (isolateRef.id != isolate.id) return;
 
-      final firstArgValue = event.args[0].value as String;
+      final args = event.args;
+      final firstArgValue = (args.isNotEmpty ? args[0].value : null) as String?;
       // TODO(nshahan) - Migrate 'inspect' and 'log' events to the injected
       // client communication approach as well?
       switch (firstArgValue) {
