@@ -22,11 +22,11 @@
 library rendering;
 
 export 'package:flutter/foundation.dart' show
-  VoidCallback,
+  DiagnosticLevel,
   ValueChanged,
   ValueGetter,
   ValueSetter,
-  DiagnosticLevel;
+  VoidCallback;
 export 'package:flutter/semantics.dart';
 export 'package:vector_math/vector_math_64.dart' show Matrix4;
 
@@ -37,6 +37,7 @@ export 'src/rendering/custom_layout.dart';
 export 'src/rendering/custom_paint.dart';
 export 'src/rendering/debug.dart';
 export 'src/rendering/debug_overflow_indicator.dart';
+export 'src/rendering/decorated_sliver.dart';
 export 'src/rendering/editable.dart';
 export 'src/rendering/error.dart';
 export 'src/rendering/flex.dart';
@@ -55,11 +56,13 @@ export 'src/rendering/proxy_box.dart';
 export 'src/rendering/proxy_sliver.dart';
 export 'src/rendering/rotated_box.dart';
 export 'src/rendering/selection.dart';
+export 'src/rendering/service_extensions.dart';
 export 'src/rendering/shifted_box.dart';
 export 'src/rendering/sliver.dart';
 export 'src/rendering/sliver_fill.dart';
 export 'src/rendering/sliver_fixed_extent_list.dart';
 export 'src/rendering/sliver_grid.dart';
+export 'src/rendering/sliver_group.dart';
 export 'src/rendering/sliver_list.dart';
 export 'src/rendering/sliver_multi_box_adaptor.dart';
 export 'src/rendering/sliver_padding.dart';
