@@ -49,12 +49,11 @@ const double _kOverAndUnderCenterOpacity = 0.447;
 ///
 ///  * [ListWheelScrollView], the generic widget backing this picker without
 ///    the iOS design specific chrome.
-///  * <https://developer.apple.com/ios/human-interface-guidelines/controls/pickers/>
+///  * <https://developer.apple.com/design/human-interface-guidelines/pickers/>
 class CupertinoPicker extends StatefulWidget {
   /// Creates a picker from a concrete list of children.
   ///
-  /// The [diameterRatio] and [itemExtent] arguments must not be null. The
-  /// [itemExtent] must be greater than zero.
+  /// The [itemExtent] must be greater than zero.
   ///
   /// The [backgroundColor] defaults to null, which disables background painting entirely.
   /// (i.e. the picker is going to have a completely transparent background), to match
@@ -99,11 +98,11 @@ class CupertinoPicker extends StatefulWidget {
   /// normally the builder is only called once for each index (except when
   /// rebuilding - the cache is cleared).
   ///
-  /// The [itemBuilder] argument must not be null. The [childCount] argument
-  /// reflects the number of children that will be provided by the [itemBuilder].
+  /// The [childCount] argument reflects the number of children that will be
+  /// provided by the [itemBuilder].
   /// {@macro flutter.widgets.ListWheelChildBuilderDelegate.childCount}
   ///
-  /// The [itemExtent] argument must be non-null and positive.
+  /// The [itemExtent] argument must be positive.
   ///
   /// The [backgroundColor] defaults to null, which disables background painting entirely.
   /// (i.e. the picker is going to have a completely transparent background), to match
@@ -134,7 +133,7 @@ class CupertinoPicker extends StatefulWidget {
   ///
   /// For more details, see [ListWheelScrollView.diameterRatio].
   ///
-  /// Must not be null and defaults to `1.1` to visually mimic iOS.
+  /// Defaults to 1.1 to visually mimic iOS.
   final double diameterRatio;
 
   /// Background color behind the children.
@@ -219,6 +218,7 @@ class _CupertinoPickerState extends State<CupertinoPicker> {
   void didUpdateWidget(CupertinoPicker oldWidget) {
     super.didUpdateWidget(oldWidget);
     if (widget.scrollController != null && oldWidget.scrollController == null) {
+      _controller?.dispose();
       _controller = null;
     } else if (widget.scrollController == null && oldWidget.scrollController != null) {
       assert(_controller == null);
@@ -334,13 +334,12 @@ class CupertinoPickerDefaultSelectionOverlay extends StatelessWidget {
   /// area (or the currently selected item, depending on how you described it
   /// elsewhere) of a [CupertinoPicker].
   ///
-  /// The [background] argument default value is [CupertinoColors.tertiarySystemFill].
-  /// It must be non-null.
+  /// The [background] argument default value is
+  /// [CupertinoColors.tertiarySystemFill].
   ///
   /// The [capStartEdge] and [capEndEdge] arguments decide whether to add a
   /// default margin and use rounded corners on the left and right side of the
-  /// rectangular overlay.
-  /// Default to true and must not be null.
+  /// rectangular overlay, and they both default to true.
   const CupertinoPickerDefaultSelectionOverlay({
     super.key,
     this.background = CupertinoColors.tertiarySystemFill,
