@@ -47,7 +47,7 @@ class CupertinoThumbPainter {
   const CupertinoThumbPainter({
     this.color = CupertinoColors.white,
     this.shadows = _kSliderBoxShadows,
-  }) : assert(shadows != null);
+  });
 
   /// Creates an object that paints an iOS-style switch thumb.
   const CupertinoThumbPainter.switchThumb({
@@ -59,8 +59,6 @@ class CupertinoThumbPainter {
   final Color color;
 
   /// The list of [BoxShadow] to paint below the thumb.
-  ///
-  /// Must not be null.
   final List<BoxShadow> shadows;
 
   /// Half the default diameter of the thumb.
