@@ -32,30 +32,58 @@ class BottomAppBarTheme with Diagnosticable {
     this.color,
     this.elevation,
     this.shape,
+    this.height,
+    this.surfaceTintColor,
+    this.shadowColor,
+    this.padding,
   });
 
-  /// Default value for [BottomAppBar.color].
+  /// Overrides the default value for [BottomAppBar.color].
   ///
   /// If null, [BottomAppBar] uses [ThemeData.bottomAppBarColor].
   final Color? color;
 
-  /// Default value for [BottomAppBar.elevation].
+  /// Overrides the default value for [BottomAppBar.elevation].
   final double? elevation;
 
-  /// Default value for [BottomAppBar.shape].
+  /// Overrides the default value for [BottomAppBar.shape].
   final NotchedShape? shape;
 
+  /// Overrides the default value for [BottomAppBar.height].
+  final double? height;
+
+  /// Overrides the default value for [BottomAppBar.surfaceTintColor].
+  ///
+  /// If null, [BottomAppBar] will not display an overlay color.
+  ///
+  /// See [Material.surfaceTintColor] for more details.
+  final Color? surfaceTintColor;
+
+  /// Overrides the default value for [BottomAppBar.shadowColor].
+  final Color? shadowColor;
+
+  /// Overrides the default value for [BottomAppBar.padding].
+  final EdgeInsetsGeometry? padding;
+
   /// Creates a copy of this object but with the given fields replaced with the
   /// new values.
   BottomAppBarTheme copyWith({
     Color? color,
     double? elevation,
     NotchedShape? shape,
+    double? height,
+    Color? surfaceTintColor,
+    Color? shadowColor,
+    EdgeInsetsGeometry? padding,
   }) {
     return BottomAppBarTheme(
       color: color ?? this.color,
       elevation: elevation ?? this.elevation,
       shape: shape ?? this.shape,
+      height: height ?? this.height,
+      surfaceTintColor: surfaceTintColor ?? this.surfaceTintColor,
+      shadowColor: shadowColor ?? this.shadowColor,
+      padding: padding ?? this.padding,
     );
   }
 
@@ -66,15 +94,19 @@ class BottomAppBarTheme with Diagnosticable {
 
   /// Linearly interpolate between two BAB themes.
   ///
-  /// The argument `t` must not be null.
-  ///
   /// {@macro dart.ui.shadow.lerp}
   static BottomAppBarTheme lerp(BottomAppBarTheme? a, BottomAppBarTheme? b, double t) {
-    assert(t != null);
+    if (identical(a, b) && a != null) {
+      return a;
+    }
     return BottomAppBarTheme(
       color: Color.lerp(a?.color, b?.color, t),
       elevation: lerpDouble(a?.elevation, b?.elevation, t),
       shape: t < 0.5 ? a?.shape : b?.shape,
+      height: lerpDouble(a?.height, b?.height, t),
+      surfaceTintColor: Color.lerp(a?.surfaceTintColor, b?.surfaceTintColor, t),
+      shadowColor: Color.lerp(a?.shadowColor, b?.shadowColor, t),
+      padding: EdgeInsetsGeometry.lerp(a?.padding, b?.padding, t),
     );
   }
 
@@ -83,6 +115,10 @@ class BottomAppBarTheme with Diagnosticable {
     color,
     elevation,
     shape,
+    height,
+    surfaceTintColor,
+    shadowColor,
+    padding,
   );
 
   @override
@@ -96,7 +132,11 @@ class BottomAppBarTheme with Diagnosticable {
     return other is BottomAppBarTheme
         && other.color == color
         && other.elevation == elevation
-        && other.shape == shape;
+        && other.shape == shape
+        && other.height == height
+        && other.surfaceTintColor == surfaceTintColor
+        && other.shadowColor == shadowColor
+        && other.padding == padding;
   }
 
   @override
@@ -105,5 +145,9 @@ class BottomAppBarTheme with Diagnosticable {
     properties.add(ColorProperty('color', color, defaultValue: null));
     properties.add(DiagnosticsProperty<double>('elevation', elevation, defaultValue: null));
     properties.add(DiagnosticsProperty<NotchedShape>('shape', shape, defaultValue: null));
+    properties.add(DiagnosticsProperty<double>('height', height, defaultValue: null));
+    properties.add(ColorProperty('surfaceTintColor', surfaceTintColor, defaultValue: null));
+    properties.add(ColorProperty('shadowColor', shadowColor, defaultValue: null));
+    properties.add(DiagnosticsProperty<EdgeInsetsGeometry>('padding', padding, defaultValue: null));
   }
 }
