@@ -173,8 +173,6 @@ class IconButton extends StatelessWidget {
   /// Requires one of its ancestors to be a [Material] widget. This requirement
   /// no longer exists if [ThemeData.useMaterial3] is set to true.
   ///
-  /// [autofocus] argument must not be null (though it has default value).
-  ///
   /// The [icon] argument must be specified, and is typically either an [Icon]
   /// or an [ImageIcon].
   const IconButton({
@@ -312,6 +310,9 @@ class IconButton extends StatelessWidget {
   /// [Icon.size] instead, then the [IconButton] would default to 24.0 and then
   /// the [Icon] itself would likely get clipped.
   ///
+  /// This property is only used when [icon] is or contains an [Icon] widget. It will be
+  /// ignored if other widgets are used, such as an [Image].
+  ///
   /// If [ThemeData.useMaterial3] is set to true and this is null, the size of the
   /// [IconButton] would default to 24.0. The size given here is passed down to the
   /// [ButtonStyle.iconSize] property.
@@ -362,8 +363,6 @@ class IconButton extends StatelessWidget {
   /// [IconTheme] and therefore should not be explicitly given in the icon
   /// widget.
   ///
-  /// This property must not be null.
-  ///
   /// See [Icon], [ImageIcon].
   final Widget icon;
 
@@ -880,6 +879,12 @@ class _SelectableIconButtonState extends State<_SelectableIconButton> {
       ),
     );
   }
+
+  @override
+  void dispose() {
+    statesController.dispose();
+    super.dispose();
+  }
 }
 
 class _IconButtonM3 extends ButtonStyleButton {
@@ -963,9 +968,9 @@ class _IconButtonM3 extends ButtonStyleButton {
 
     bool isIconThemeDefault(Color? color) {
       if (isDark) {
-        return color == kDefaultIconLightColor;
+        return identical(color, kDefaultIconLightColor);
       }
-      return color == kDefaultIconDarkColor;
+      return identical(color, kDefaultIconDarkColor);
     }
     final bool isDefaultColor = isIconThemeDefault(iconTheme.color);
     final bool isDefaultSize = iconTheme.size == const IconThemeData.fallback().size;
