@@ -2,12 +2,9 @@
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
-import 'dart:async';
 import 'dart:math' as math;
-import 'dart:typed_data';
 import 'dart:ui' as ui;
 
-import 'package:flutter/services.dart';
 import 'package:flutter/widgets.dart';
 import 'package:vector_math/vector_math_64.dart';
 
@@ -17,11 +14,8 @@ import 'material.dart';
 /// Begin a Material 3 ink sparkle ripple, centered at the tap or click position
 /// relative to the [referenceBox].
 ///
-/// This effect relies on a shader, and therefore hardware acceleration.
-/// Currently, this is only supported by certain C++ engine platforms. The
-/// platforms that are currently supported are Android, iOS, MacOS, Windows,
-/// and Linux. Support for CanvasKit web can be tracked here:
-///  - https://github.com/flutter/flutter/issues/85238
+/// This effect relies on a shader and therefore is unsupported on the Flutter
+/// Web HTML backend.
 ///
 /// To use this effect, pass an instance of [splashFactory] to the
 /// `splashFactory` parameter of either the Material [ThemeData] or any
@@ -101,15 +95,15 @@ class InkSparkle extends InteractiveInkFeature {
   /// identified in the shader as "noise", and the sparkles are derived from
   /// pseudorandom triangular noise.
   InkSparkle({
-    required MaterialInkController controller,
-    required RenderBox referenceBox,
+    required super.controller,
+    required super.referenceBox,
     required super.color,
     required Offset position,
     required TextDirection textDirection,
     bool containedInkWell = true,
     RectCallback? rectCallback,
     BorderRadius? borderRadius,
-    ShapeBorder? customBorder,
+    super.customBorder,
     double? radius,
     super.onRemoved,
     double? turbulenceSeed,
@@ -117,20 +111,19 @@ class InkSparkle extends InteractiveInkFeature {
        _color = color,
        _position = position,
        _borderRadius = borderRadius ?? BorderRadius.zero,
-       _customBorder = customBorder,
        _textDirection = textDirection,
-       _targetRadius = (radius ?? _getTargetRadius(referenceBox, containedInkWell, rectCallback, position)) * _targetRadiusMultiplier,
-       _clipCallback = _getClipCallback(referenceBox, containedInkWell, rectCallback),
-       super(controller: controller, referenceBox: referenceBox) {
+       _targetRadius = (radius ?? _getTargetRadius(
+                                    referenceBox,
+                                    containedInkWell,
+                                    rectCallback,
+                                    position,
+                                  )
+                       ) * _targetRadiusMultiplier,
+       _clipCallback = _getClipCallback(referenceBox, containedInkWell, rectCallback) {
     // InkSparkle will not be painted until the async compilation completes.
-    _InkSparkleFactory.compileShaderIfNeccessary();
+    _InkSparkleFactory.initializeShader();
     controller.addInkFeature(this);
 
-    // All animation values are derived from Android 12 source code. See:
-    // - https://cs.android.com/android/platform/superproject/+/master:frameworks/base/graphics/java/android/graphics/drawable/RippleShader.java
-    // - https://cs.android.com/android/platform/superproject/+/master:frameworks/base/graphics/java/android/graphics/drawable/RippleDrawable.java
-    // - https://cs.android.com/android/platform/superproject/+/master:frameworks/base/graphics/java/android/graphics/drawable/RippleAnimationSession.java
-
     // Immediately begin animating the ink.
     _animationController = AnimationController(
       duration: _animationDuration,
@@ -206,8 +199,15 @@ class InkSparkle extends InteractiveInkFeature {
       ],
     ).animate(_animationController);
 
-    // Creates an element of randomness so that ink eminating from the same
+    // Creates an element of randomness so that ink emanating from the same
     // pixel have slightly different rings and sparkles.
+    assert((){
+      // In tests, randomness can cause flakes. So if a seed has not
+      // already been specified (i.e. for the purpose of the test), set it to
+      // the constant turbulence seed.
+      turbulenceSeed ??= _InkSparkleFactory.constantSeed;
+      return true;
+    }());
     _turbulenceSeed = turbulenceSeed ?? math.Random().nextDouble() * 1000.0;
   }
 
@@ -239,11 +239,13 @@ class InkSparkle extends InteractiveInkFeature {
   final Color _color;
   final Offset _position;
   final BorderRadius _borderRadius;
-  final ShapeBorder? _customBorder;
   final double _targetRadius;
   final RectCallback? _clipCallback;
   final TextDirection _textDirection;
 
+  late final ui.FragmentShader _fragmentShader;
+  bool _fragmentShaderInitialized = false;
+
   /// Used to specify this type of ink splash for an [InkWell], [InkResponse],
   /// material [Theme], or [ButtonStyle].
   ///
@@ -262,6 +264,9 @@ class InkSparkle extends InteractiveInkFeature {
   void dispose() {
     _animationController.stop();
     _animationController.dispose();
+    if (_fragmentShaderInitialized) {
+      _fragmentShader.dispose();
+    }
     super.dispose();
   }
 
@@ -270,28 +275,35 @@ class InkSparkle extends InteractiveInkFeature {
     assert(_animationController.isAnimating);
 
     // InkSparkle can only paint if its shader has been compiled.
-    if (_InkSparkleFactory._shaderManager == null) {
+    if (_InkSparkleFactory._program == null) {
       // Skipping paintFeature because the shader it relies on is not ready to
-      // be used. InkSparkleFactory.compileShaderIfNeccessary must complete
+      // be used. InkSparkleFactory.initializeShader must complete
       // before InkSparkle can paint.
       return;
     }
 
+    if (!_fragmentShaderInitialized) {
+      _fragmentShader = _InkSparkleFactory._program!.fragmentShader();
+      _fragmentShaderInitialized = true;
+    }
+
     canvas.save();
     _transformCanvas(canvas: canvas, transform: transform);
     if (_clipCallback != null) {
       _clipCanvas(
         canvas: canvas,
-        clipCallback: _clipCallback!,
+        clipCallback: _clipCallback,
         textDirection: _textDirection,
-        customBorder: _customBorder,
+        customBorder: customBorder,
         borderRadius: _borderRadius,
       );
     }
 
-    final Paint paint = Paint()..shader = _createRippleShader();
+    _updateFragmentShader();
+
+    final Paint paint = Paint()..shader = _fragmentShader;
     if (_clipCallback != null) {
-      canvas.drawRect(_clipCallback!(), paint);
+      canvas.drawRect(_clipCallback(), paint);
     } else {
       canvas.drawPaint(paint);
     }
@@ -301,12 +313,13 @@ class InkSparkle extends InteractiveInkFeature {
   double get _width => referenceBox.size.width;
   double get _height => referenceBox.size.height;
 
+
   /// All double values for uniforms come from the Android 12 ripple
   /// implementation from the following files:
   /// - https://cs.android.com/android/platform/superproject/+/master:frameworks/base/graphics/java/android/graphics/drawable/RippleShader.java
   /// - https://cs.android.com/android/platform/superproject/+/master:frameworks/base/graphics/java/android/graphics/drawable/RippleDrawable.java
   /// - https://cs.android.com/android/platform/superproject/+/master:frameworks/base/graphics/java/android/graphics/drawable/RippleAnimationSession.java
-  Shader _createRippleShader() {
+  void _updateFragmentShader() {
     const double turbulenceScale = 1.5;
     final double turbulencePhase = _turbulenceSeed + _radiusScale.value;
     final double noisePhase = turbulencePhase;
@@ -314,55 +327,48 @@ class InkSparkle extends InteractiveInkFeature {
     final double rotation2 = turbulencePhase * _rotateLeft + 2.0 * math.pi;
     final double rotation3 = turbulencePhase * _rotateRight + 2.75 * math.pi;
 
-    return _InkSparkleFactory._shaderManager!.shader(
-      // The following uniforms are the same throughout the animation.
-      uColor: _colorToVector4(_color),
-      uSparkleColor: Vector4(1.0, 1.0, 1.0, 1.0),
-      uBlur: 1.0,
-      uMaxRadius: _targetRadius,
-      uResolutionScale: Vector2(1.0 / _width, 1.0 / _height),
-      uNoiseScale: Vector2(_noiseDensity / _width, _noiseDensity / _height),
-
-      // The following uniforms update each frame of the animation.
-      uCenter: _center.value,
-      uRadiusScale: _radiusScale.value,
-      uAlpha: _alpha.value,
-      uSparkleAlpha: _sparkleAlpha.value,
-
-      // The following uniforms are driven by the turbulence phase and change
-      // each frame of the animation. These uniforms are uses to modify the
-      // default (if these fields are unset or 0) circular outer ring to a
-      // non-uniform shape that is more like an actual ink splash. In addition
-      // to the positional based triangular noise created in the shader, these
-      // uniforms also vary the appearance of the sparkles even when the same
-      // location is tapped.
-      uTurbulencePhase: turbulencePhase,
-      uNoisePhase: noisePhase / 1000.0,
-      uCircle1: Vector2(
-        turbulenceScale * 0.5 + (turbulencePhase * 0.01 * math.cos(turbulenceScale * 0.55)),
-        turbulenceScale * 0.5 + (turbulencePhase * 0.01 * math.sin(turbulenceScale * 0.55)),
-      ),
-      uCircle2: Vector2(
-        turbulenceScale * 0.2 + (turbulencePhase * -0.0066 * math.cos(turbulenceScale * 0.45)),
-        turbulenceScale * 0.2 + (turbulencePhase * -0.0066 * math.sin(turbulenceScale * 0.45)),
-      ),
-      uCircle3: Vector2(
-        turbulenceScale + (turbulencePhase * -0.0066 * math.cos(turbulenceScale * 0.35)),
-        turbulenceScale + (turbulencePhase * -0.0066 * math.sin(turbulenceScale * 0.35)),
-      ),
-      uRotation1: Vector2(math.cos(rotation1), math.sin(rotation1)),
-      uRotation2: Vector2(math.cos(rotation2), math.sin(rotation2)),
-      uRotation3: Vector2(math.cos(rotation3), math.sin(rotation3)),
-    );
-  }
-
-  Vector4 _colorToVector4(Color color) {
-    return Vector4(
-        color.red / 255.0,
-        color.blue / 255.0,
-        color.green / 255.0,
-        color.alpha / 255.0,
-      );
+    _fragmentShader
+      // uColor
+      ..setFloat(0, _color.red / 255.0)
+      ..setFloat(1, _color.green / 255.0)
+      ..setFloat(2, _color.blue / 255.0)
+      ..setFloat(3, _color.alpha / 255.0)
+      // Composite 1 (u_alpha, u_sparkle_alpha, u_blur, u_radius_scale)
+      ..setFloat(4, _alpha.value)
+      ..setFloat(5, _sparkleAlpha.value)
+      ..setFloat(6, 1.0)
+      ..setFloat(7, _radiusScale.value)
+      // uCenter
+      ..setFloat(8, _center.value.x)
+      ..setFloat(9, _center.value.y)
+      // uMaxRadius
+      ..setFloat(10, _targetRadius)
+      // uResolutionScale
+      ..setFloat(11, 1.0 / _width)
+      ..setFloat(12, 1.0 / _height)
+      // uNoiseScale
+      ..setFloat(13, _noiseDensity / _width)
+      ..setFloat(14, _noiseDensity / _height)
+      // uNoisePhase
+      ..setFloat(15, noisePhase / 1000.0)
+      // uCircle1
+      ..setFloat(16, turbulenceScale * 0.5 + (turbulencePhase * 0.01 * math.cos(turbulenceScale * 0.55)))
+      ..setFloat(17, turbulenceScale * 0.5 + (turbulencePhase * 0.01 * math.sin(turbulenceScale * 0.55)))
+      // uCircle2
+      ..setFloat(18, turbulenceScale * 0.2 + (turbulencePhase * -0.0066 * math.cos(turbulenceScale * 0.45)))
+      ..setFloat(19, turbulenceScale * 0.2 + (turbulencePhase * -0.0066 * math.sin(turbulenceScale * 0.45)))
+      // uCircle3
+      ..setFloat(20, turbulenceScale + (turbulencePhase * -0.0066 * math.cos(turbulenceScale * 0.35)))
+      ..setFloat(21, turbulenceScale + (turbulencePhase * -0.0066 * math.sin(turbulenceScale * 0.35)))
+      // uRotation1
+      ..setFloat(22, math.cos(rotation1))
+      ..setFloat(23, math.sin(rotation1))
+      // uRotation2
+      ..setFloat(24, math.cos(rotation2))
+      ..setFloat(25, math.sin(rotation2))
+      // uRotation3
+      ..setFloat(26, math.cos(rotation3))
+      ..setFloat(27, math.sin(rotation3));
   }
 
   /// Transforms the canvas for an ink feature to be painted on the [canvas].
@@ -430,19 +436,23 @@ class InkSparkle extends InteractiveInkFeature {
 class _InkSparkleFactory extends InteractiveInkFeatureFactory {
   const _InkSparkleFactory() : turbulenceSeed = null;
 
-  const _InkSparkleFactory.constantTurbulenceSeed() : turbulenceSeed = 1337.0;
+  const _InkSparkleFactory.constantTurbulenceSeed() : turbulenceSeed = _InkSparkleFactory.constantSeed;
 
-  // TODO(clocksmith): Update this once shaders are precompiled.
-  static void compileShaderIfNeccessary() {
+  static const double constantSeed = 1337.0;
+
+  static void initializeShader() {
     if (!_initCalled) {
-      FragmentShaderManager.inkSparkle().then((FragmentShaderManager manager) {
-        _shaderManager = manager;
-      });
+      ui.FragmentProgram.fromAsset('shaders/ink_sparkle.frag').then(
+        (ui.FragmentProgram program) {
+          _program = program;
+        },
+      );
       _initCalled = true;
     }
   }
+
   static bool _initCalled = false;
-  static FragmentShaderManager? _shaderManager;
+  static ui.FragmentProgram? _program;
 
   final double? turbulenceSeed;
 
@@ -503,78 +513,3 @@ double _getTargetRadius(
   final double d2 = (size.topRight(Offset.zero) - size.bottomLeft(Offset.zero)).distance;
   return math.max(d1, d2) / 2.0;
 }
-
-// The code below is generated by the package: fragment_shader_manager.
-// It is hand modified to update for flutter/flutter style and asset loading.
-
-/// A generated class for managing [FragmentProgram] that includes a
-/// pre-transpiled shader program into SPIR-V.
-///
-/// See:
-/// - https://github.com/material-components/material-components-flutter-experimental/tree/fragment-shader-manager/fragment_shader_manager
-///
-/// GLSL source for this shader is under `shaders/ink_sparkle.frag`.
-class FragmentShaderManager {
-  FragmentShaderManager._();
-
-  static late ui.FragmentProgram _program;
-
-  /// Creates an [FragmentShaderManager] with an [InkSparkle] effect.
-  static Future<FragmentShaderManager> inkSparkle() async {
-    final FragmentShaderManager manager = FragmentShaderManager._();
-    await manager._compile();
-    return manager;
-  }
-
-  /// Compiles the spir-v bytecode into a shader program.
-  Future<void> _compile() async {
-    final ByteData data = await rootBundle.load('shaders/ink_sparkle.frag');
-    _program = await ui.FragmentProgram.compile(spirv: data.buffer);
-  }
-
-  /// Creates a shader with the original program and optional uniforms.
-  ///
-  /// A new shader must be made whenever the uniforms are updated.
-  Shader shader({
-    Vector4? uColor,
-    double? uAlpha,
-    Vector4? uSparkleColor,
-    double? uSparkleAlpha,
-    double? uBlur,
-    Vector2? uCenter,
-    double? uRadiusScale,
-    double? uMaxRadius,
-    Vector2? uResolutionScale,
-    Vector2? uNoiseScale,
-    double? uNoisePhase,
-    double? uTurbulencePhase,
-    Vector2? uCircle1,
-    Vector2? uCircle2,
-    Vector2? uCircle3,
-    Vector2? uRotation1,
-    Vector2? uRotation2,
-    Vector2? uRotation3,
-  }) {
-    return _program.shader(
-      floatUniforms: Float32List.fromList(<double>[
-        ...uColor != null ? uColor.storage : <double>[0, 0, 0, 0],
-        ...uAlpha != null ? <double>[uAlpha] : <double>[0],
-        ...uSparkleColor != null ? uSparkleColor.storage : <double>[0, 0, 0, 0],
-        ...uSparkleAlpha != null ? <double>[uSparkleAlpha] : <double>[0],
-        ...uBlur != null ? <double>[uBlur] : <double>[0],
-        ...uCenter != null ? uCenter.storage : <double>[0, 0],
-        ...uRadiusScale != null ? <double>[uRadiusScale] : <double>[0],
-        ...uMaxRadius != null ? <double>[uMaxRadius] : <double>[0],
-        ...uResolutionScale != null ? uResolutionScale.storage : <double>[0, 0],
-        ...uNoiseScale != null ? uNoiseScale.storage : <double>[0, 0],
-        ...uNoisePhase != null ? <double>[uNoisePhase] : <double>[0],
-        ...uCircle1 != null ? uCircle1.storage : <double>[0, 0],
-        ...uCircle2 != null ? uCircle2.storage : <double>[0, 0],
-        ...uCircle3 != null ? uCircle3.storage : <double>[0, 0],
-        ...uRotation1 != null ? uRotation1.storage : <double>[0, 0],
-        ...uRotation2 != null ? uRotation2.storage : <double>[0, 0],
-        ...uRotation3 != null ? uRotation3.storage : <double>[0, 0],
-      ]),
-    );
-  }
-}
