@@ -18,6 +18,10 @@ import 'theme.dart';
 import 'theme_data.dart';
 import 'toggle_buttons_theme.dart';
 
+// Examples can assume:
+// List<bool> isSelected = <bool>[];
+// void setState(dynamic arg) { }
+
 /// A set of toggle buttons.
 ///
 /// The list of [children] are laid out along [direction]. The state of each button
@@ -26,8 +30,35 @@ import 'toggle_buttons_theme.dart';
 /// correlated by their index in the list. The length of [isSelected] has to
 /// match the length of the [children] list.
 ///
+/// There is a Material 3 version of this component, [SegmentedButton],
+/// that's preferred for applications that are configured for Material 3
+/// (see [ThemeData.useMaterial3]).
+///
 /// {@youtube 560 315 https://www.youtube.com/watch?v=kVEguaQWGAY}
 ///
+/// ## Updating to [SegmentedButton]
+///
+/// There is a Material 3 version of this component, [SegmentedButton],
+/// that's preferred for applications that are configured for Material 3
+/// (see [ThemeData.useMaterial3]). The [SegmentedButton] widget's visuals
+/// are a little bit different, see the Material 3 spec at
+/// <https://m3.material.io/components/segmented-buttons/overview> for
+/// more details. The [SegmentedButton] widget's API is also slightly different.
+/// While the [ToggleButtons] widget can have list of widgets, the
+/// [SegmentedButton] widget has a list of [ButtonSegment]s with
+/// a type value. While the [ToggleButtons] uses a list of boolean values
+/// to determine the selection state of each button, the [SegmentedButton]
+/// uses a set of type values to determine the selection state of each segment.
+/// The [SegmentedButton.style] is a [ButtonStyle] style field, which can be
+/// used to customize the entire segmented button and the individual segments.
+///
+/// {@tool dartpad}
+/// This sample shows how to migrate [ToggleButtons] that allows multiple
+/// or no selection to [SegmentedButton] that allows multiple or no selection.
+///
+/// ** See code in examples/api/lib/material/toggle_buttons/toggle_buttons.1.dart **
+/// {@end-tool}
+///
 /// {@tool dartpad}
 /// This example showcase [ToggleButtons] in various configurations.
 ///
@@ -43,34 +74,32 @@ import 'toggle_buttons_theme.dart';
 /// Here is an implementation that allows for multiple buttons to be
 /// simultaneously selected, while requiring none of the buttons to be
 /// selected.
+///
 /// ```dart
 /// ToggleButtons(
-///   children: <Widget>[
-///     Icon(Icons.ac_unit),
-///     Icon(Icons.call),
-///     Icon(Icons.cake),
-///   ],
+///   isSelected: isSelected,
 ///   onPressed: (int index) {
 ///     setState(() {
 ///       isSelected[index] = !isSelected[index];
 ///     });
 ///   },
-///   isSelected: isSelected,
+///   children: const <Widget>[
+///     Icon(Icons.ac_unit),
+///     Icon(Icons.call),
+///     Icon(Icons.cake),
+///   ],
 /// ),
 /// ```
 ///
 /// {@animation 700 150 https://flutter.github.io/assets-for-api-docs/assets/material/toggle_buttons_required_mutually_exclusive.mp4}
 ///
-/// Here is an implementation that requires mutually exclusive selection
-/// while requiring at least one selection. Note that this assumes that
-/// [isSelected] was properly initialized with one selection.
+/// Here is an implementation that requires mutually exclusive selection while
+/// requiring at least one selection. This assumes that [isSelected] was
+/// properly initialized with one selection.
+///
 /// ```dart
 /// ToggleButtons(
-///   children: <Widget>[
-///     Icon(Icons.ac_unit),
-///     Icon(Icons.call),
-///     Icon(Icons.cake),
-///   ],
+///   isSelected: isSelected,
 ///   onPressed: (int index) {
 ///     setState(() {
 ///       for (int buttonIndex = 0; buttonIndex < isSelected.length; buttonIndex++) {
@@ -82,7 +111,11 @@ import 'toggle_buttons_theme.dart';
 ///       }
 ///     });
 ///   },
-///   isSelected: isSelected,
+///   children: const <Widget>[
+///     Icon(Icons.ac_unit),
+///     Icon(Icons.call),
+///     Icon(Icons.cake),
+///   ],
 /// ),
 /// ```
 ///
@@ -90,13 +123,10 @@ import 'toggle_buttons_theme.dart';
 ///
 /// Here is an implementation that requires mutually exclusive selection,
 /// but allows for none of the buttons to be selected.
+///
 /// ```dart
 /// ToggleButtons(
-///   children: <Widget>[
-///     Icon(Icons.ac_unit),
-///     Icon(Icons.call),
-///     Icon(Icons.cake),
-///   ],
+///   isSelected: isSelected,
 ///   onPressed: (int index) {
 ///     setState(() {
 ///       for (int buttonIndex = 0; buttonIndex < isSelected.length; buttonIndex++) {
@@ -108,37 +138,42 @@ import 'toggle_buttons_theme.dart';
 ///       }
 ///     });
 ///   },
-///   isSelected: isSelected,
+///   children: const <Widget>[
+///     Icon(Icons.ac_unit),
+///     Icon(Icons.call),
+///     Icon(Icons.cake),
+///   ],
 /// ),
 /// ```
 ///
 /// {@animation 700 150 https://flutter.github.io/assets-for-api-docs/assets/material/toggle_buttons_required.mp4}
 ///
 /// Here is an implementation that allows for multiple buttons to be
-/// simultaneously selected, while requiring at least one selection. Note
-/// that this assumes that [isSelected] was properly initialized with one
-/// selection.
+/// simultaneously selected, while requiring at least one selection. This
+/// assumes that [isSelected] was properly initialized with one selection.
+///
 /// ```dart
 /// ToggleButtons(
-///   children: <Widget>[
-///     Icon(Icons.ac_unit),
-///     Icon(Icons.call),
-///     Icon(Icons.cake),
-///   ],
+///   isSelected: isSelected,
 ///   onPressed: (int index) {
 ///     int count = 0;
-///     isSelected.forEach((bool val) {
-///       if (val) count++;
-///     });
-///
-///     if (isSelected[index] && count < 2)
+///     for (final bool value in isSelected) {
+///       if (value) {
+///         count += 1;
+///       }
+///     }
+///     if (isSelected[index] && count < 2) {
 ///       return;
-///
+///     }
 ///     setState(() {
 ///       isSelected[index] = !isSelected[index];
 ///     });
 ///   },
-///   isSelected: isSelected,
+///   children: const <Widget>[
+///     Icon(Icons.ac_unit),
+///     Icon(Icons.call),
+///     Icon(Icons.cake),
+///   ],
 /// ),
 /// ```
 ///
@@ -170,9 +205,8 @@ class ToggleButtons extends StatelessWidget {
   ///
   /// Both [children] and [isSelected] properties arguments are required.
   ///
-  /// [isSelected] values must be non-null. [focusNodes] must be null or a
-  /// list of non-null nodes. [renderBorder] and [direction] must not be null.
-  /// If [direction] is [Axis.vertical], [verticalDirection] must not be null.
+  /// The [focusNodes] argument must be null or a list of nodes. If [direction]
+  /// is [Axis.vertical], [verticalDirection] must not be null.
   const ToggleButtons({
     super.key,
     required this.children,
@@ -199,12 +233,7 @@ class ToggleButtons extends StatelessWidget {
     this.borderWidth,
     this.direction = Axis.horizontal,
     this.verticalDirection = VerticalDirection.down,
-  }) :
-    assert(children != null),
-    assert(isSelected != null),
-    assert(children.length == isSelected.length),
-    assert(direction != null),
-    assert(direction == Axis.horizontal || verticalDirection != null);
+  }) : assert(children.length == isSelected.length);
 
   static const double _defaultBorderWidth = 1.0;
 
@@ -644,18 +673,6 @@ class ToggleButtons extends StatelessWidget {
 
   @override
   Widget build(BuildContext context) {
-    assert(
-      !isSelected.any((bool val) => val == null),
-      'All elements of isSelected must be non-null.\n'
-      'The current list of isSelected values is as follows:\n'
-      '$isSelected',
-    );
-    assert(
-      focusNodes == null || !focusNodes!.any((FocusNode val) => val == null),
-      'All elements of focusNodes must be non-null.\n'
-      'The current list of focus node values is as follows:\n'
-      '$focusNodes',
-    );
     assert(
       () {
         if (focusNodes != null) {
@@ -702,7 +719,7 @@ class ToggleButtons extends StatelessWidget {
       }
       final TextStyle currentTextStyle = textStyle
         ?? toggleButtonsTheme.textStyle
-        ?? theme.textTheme.bodyText2!;
+        ?? theme.textTheme.bodyMedium!;
       final BoxConstraints? currentConstraints = constraints
         ?? toggleButtonsTheme.constraints;
       final Size minimumSize = currentConstraints == null
@@ -727,10 +744,8 @@ class ToggleButtons extends StatelessWidget {
           }
           assert(minPaddingSize.width >= 0.0);
           assert(minPaddingSize.height >= 0.0);
-          break;
         case MaterialTapTargetSize.shrinkWrap:
           minPaddingSize = Size.zero;
-          break;
       }
 
       Widget button = _SelectToggleButton(
@@ -790,7 +805,7 @@ class ToggleButtons extends StatelessWidget {
       return MergeSemantics(
         child: Semantics(
           container: true,
-          toggled: isSelected[index],
+          checked: isSelected[index],
           enabled: onPressed != null,
           child: _InputPadding(
             minSize: minPaddingSize,
@@ -903,20 +918,24 @@ class _ToggleButtonDefaultOverlay extends MaterialStateProperty<Color?> {
   @override
   Color? resolve(Set<MaterialState> states) {
     if (selected) {
-      if (states.contains(MaterialState.hovered)) {
-        return hoverColor ?? colorScheme?.primary.withOpacity(0.04);
-      } else if (states.contains(MaterialState.focused)) {
-        return focusColor ?? colorScheme?.primary.withOpacity(0.12);
-      } else if (states.contains(MaterialState.pressed)) {
+      if (states.contains(MaterialState.pressed)) {
         return splashColor ?? colorScheme?.primary.withOpacity(0.16);
       }
+      if (states.contains(MaterialState.hovered)) {
+        return hoverColor ?? colorScheme?.primary.withOpacity(0.04);
+      }
+      if (states.contains(MaterialState.focused)) {
+        return focusColor ?? colorScheme?.primary.withOpacity(0.12);
+      }
     } else if (unselected) {
+      if (states.contains(MaterialState.pressed)) {
+        return splashColor ?? highlightColor ?? colorScheme?.onSurface.withOpacity(0.16);
+      }
       if (states.contains(MaterialState.hovered)) {
         return hoverColor ?? colorScheme?.onSurface.withOpacity(0.04);
-      } else if (states.contains(MaterialState.focused)) {
+      }
+      if (states.contains(MaterialState.focused)) {
         return focusColor ?? colorScheme?.onSurface.withOpacity(0.12);
-      } else if (states.contains(MaterialState.pressed)) {
-        return splashColor ?? highlightColor ?? colorScheme?.onSurface.withOpacity(0.16);
       }
     }
     return null;
@@ -1197,19 +1216,15 @@ class _SelectToggleButtonRenderObject extends RenderShiftedBox {
       switch (textDirection) {
         case TextDirection.ltr:
           childParentData.offset = Offset(leadingBorderSide.width, borderSide.width);
-          break;
         case TextDirection.rtl:
           childParentData.offset = Offset(trailingBorderSide.width, borderSide.width);
-          break;
       }
     } else {
       switch (verticalDirection) {
         case VerticalDirection.down:
           childParentData.offset = Offset(borderSide.width, leadingBorderSide.width);
-          break;
         case VerticalDirection.up:
           childParentData.offset = Offset(borderSide.width, trailingBorderSide.width);
-          break;
       }
     }
   }
@@ -1363,7 +1378,6 @@ class _SelectToggleButtonRenderObject extends RenderShiftedBox {
               ..lineTo(outer.right - rrect.trRadiusX, rrect.bottom);
             context.canvas.drawPath(horizontalPaths, horizontalPaint);
           }
-          break;
         case TextDirection.rtl:
           if (isLastButton) {
             final Path leadingPath = Path();
@@ -1403,7 +1417,6 @@ class _SelectToggleButtonRenderObject extends RenderShiftedBox {
               ..lineTo(outer.left - rrect.tlRadiusX, rrect.bottom);
             context.canvas.drawPath(horizontalPaths, horizontalPaint);
           }
-          break;
       }
     } else {
       switch (verticalDirection) {
@@ -1446,7 +1459,6 @@ class _SelectToggleButtonRenderObject extends RenderShiftedBox {
               ..lineTo(rrect.right, outer.bottom);
             context.canvas.drawPath(paths, paint);
           }
-          break;
         case VerticalDirection.up:
           if (isLastButton) {
             final Path bottomPath = Path();
@@ -1486,7 +1498,6 @@ class _SelectToggleButtonRenderObject extends RenderShiftedBox {
               ..lineTo(rrect.right, outer.bottom - leadingBorderSide.width);
             context.canvas.drawPath(paths, paint);
           }
-          break;
       }
     }
   }
