@@ -12,8 +12,11 @@ export 'dart:ui' show
   Color,
   ColorFilter,
   FilterQuality,
+  FontFeature,
   FontStyle,
+  FontVariation,
   FontWeight,
+  GlyphInfo,
   ImageShader,
   Locale,
   MaskFilter,
@@ -23,11 +26,12 @@ export 'dart:ui' show
   Path,
   PathFillType,
   PathOperation,
-  Radius,
   RRect,
   RSTransform,
+  Radius,
   Rect,
   Shader,
+  Shadow,
   Size,
   StrokeCap,
   StrokeJoin,
@@ -38,13 +42,15 @@ export 'dart:ui' show
   TextDecoration,
   TextDecorationStyle,
   TextDirection,
+  TextHeightBehavior,
+  TextLeadingDistribution,
   TextPosition,
   TileMode,
   VertexMode,
   // TODO(werainkhatri): remove these after their deprecation period in engine
   // https://github.com/flutter/flutter/pull/99505
-  hashValues, // ignore: deprecated_member_use
-  hashList; // ignore: deprecated_member_use
+  hashList, // ignore: deprecated_member_use
+  hashValues; // ignore: deprecated_member_use
 
 export 'package:flutter/foundation.dart' show VoidCallback;
 
@@ -111,7 +117,7 @@ enum RenderComparison {
 /// See also:
 ///
 ///  * [AxisDirection], which is a directional version of this enum (with values
-///    light left and right, rather than just horizontal).
+///    like left and right, rather than just horizontal).
 ///  * [TextDirection], which disambiguates between left-to-right horizontal
 ///    content and right-to-left horizontal content.
 enum Axis {
@@ -136,7 +142,6 @@ enum Axis {
 ///
 ///  * [flipAxisDirection], which does the same thing for [AxisDirection] values.
 Axis flipAxis(Axis direction) {
-  assert(direction != null);
   switch (direction) {
     case Axis.horizontal:
       return Axis.vertical;
@@ -168,33 +173,102 @@ enum VerticalDirection {
   down,
 }
 
-/// A direction along either the horizontal or vertical [Axis].
+/// A direction along either the horizontal or vertical [Axis] in which the
+/// origin, or zero position, is determined.
+///
+/// This value relates to the direction in which the scroll offset increases
+/// from the origin. This value does not represent the direction of user input
+/// that may be modifying the scroll offset, such as from a drag. For the
+/// active scrolling direction, see [ScrollDirection].
+///
+/// {@tool dartpad}
+/// This sample shows a [CustomScrollView], with [Radio] buttons in the
+/// [AppBar.bottom] that change the [AxisDirection] to illustrate different
+/// configurations.
+///
+/// ** See code in examples/api/lib/painting/axis_direction/axis_direction.0.dart **
+/// {@end-tool}
+///
+/// See also:
+///
+///   * [ScrollDirection], the direction of active scrolling, relative to the positive
+///     scroll offset axis given by an [AxisDirection] and a [GrowthDirection].
+///   * [GrowthDirection], the direction in which slivers and their content are
+///     ordered, relative to the scroll offset axis as specified by
+///     [AxisDirection].
+///   * [CustomScrollView.anchor], the relative position of the zero scroll
+///     offset in a viewport and inflection point for [AxisDirection]s of the
+///     same cardinal [Axis].
+///   * [axisDirectionIsReversed], which returns whether traveling along the
+///     given axis direction visits coordinates along that axis in numerically
+///     decreasing order.
 enum AxisDirection {
-  /// Zero is at the bottom and positive values are above it: `⇈`
+  /// A direction in the [Axis.vertical] where zero is at the bottom and
+  /// positive values are above it: `⇈`
   ///
   /// Alphabetical content with a [GrowthDirection.forward] would have the A at
-  /// the bottom and the Z at the top. This is an unusual configuration.
+  /// the bottom and the Z at the top.
+  ///
+  /// For example, the behavior of a [ListView] with [ListView.reverse] set to
+  /// true would have this axis direction.
+  ///
+  /// See also:
+  ///
+  ///   * [axisDirectionIsReversed], which returns whether traveling along the
+  ///     given axis direction visits coordinates along that axis in numerically
+  ///     decreasing order.
   up,
 
-  /// Zero is on the left and positive values are to the right of it: `⇉`
+  /// A direction in the [Axis.horizontal] where zero is on the left and
+  /// positive values are to the right of it: `⇉`
   ///
   /// Alphabetical content with a [GrowthDirection.forward] would have the A on
   /// the left and the Z on the right. This is the ordinary reading order for a
   /// horizontal set of tabs in an English application, for example.
+  ///
+  /// For example, the behavior of a [ListView] with [ListView.scrollDirection]
+  /// set to [Axis.horizontal] would have this axis direction.
+  ///
+  /// See also:
+  ///
+  ///   * [axisDirectionIsReversed], which returns whether traveling along the
+  ///     given axis direction visits coordinates along that axis in numerically
+  ///     decreasing order.
   right,
 
-  /// Zero is at the top and positive values are below it: `⇊`
+  /// A direction in the [Axis.vertical] where zero is at the top and positive
+  /// values are below it: `⇊`
   ///
   /// Alphabetical content with a [GrowthDirection.forward] would have the A at
   /// the top and the Z at the bottom. This is the ordinary reading order for a
   /// vertical list.
+  ///
+  /// For example, the default behavior of a [ListView] would have this axis
+  /// direction.
+  ///
+  /// See also:
+  ///
+  ///   * [axisDirectionIsReversed], which returns whether traveling along the
+  ///     given axis direction visits coordinates along that axis in numerically
+  ///     decreasing order.
   down,
 
-  /// Zero is to the right and positive values are to the left of it: `⇇`
+  /// A direction in the [Axis.horizontal] where zero is to the right and
+  /// positive values are to the left of it: `⇇`
   ///
   /// Alphabetical content with a [GrowthDirection.forward] would have the A at
   /// the right and the Z at the left. This is the ordinary reading order for a
   /// horizontal set of tabs in a Hebrew application, for example.
+  ///
+  /// For example, the behavior of a [ListView] with [ListView.scrollDirection]
+  /// set to [Axis.horizontal] and [ListView.reverse] set to true would have
+  /// this axis direction.
+  ///
+  /// See also:
+  ///
+  ///   * [axisDirectionIsReversed], which returns whether traveling along the
+  ///     given axis direction visits coordinates along that axis in numerically
+  ///     decreasing order.
   left,
 }
 
@@ -204,7 +278,6 @@ enum AxisDirection {
 /// [AxisDirection.down] and returns [Axis.horizontal] for [AxisDirection.left]
 /// and [AxisDirection.right].
 Axis axisDirectionToAxis(AxisDirection axisDirection) {
-  assert(axisDirection != null);
   switch (axisDirection) {
     case AxisDirection.up:
     case AxisDirection.down:
@@ -220,7 +293,6 @@ Axis axisDirectionToAxis(AxisDirection axisDirection) {
 /// Specifically, returns [AxisDirection.left] for [TextDirection.rtl] and
 /// [AxisDirection.right] for [TextDirection.ltr].
 AxisDirection textDirectionToAxisDirection(TextDirection textDirection) {
-  assert(textDirection != null);
   switch (textDirection) {
     case TextDirection.rtl:
       return AxisDirection.left;
@@ -239,7 +311,6 @@ AxisDirection textDirectionToAxisDirection(TextDirection textDirection) {
 ///
 ///  * [flipAxis], which does the same thing for [Axis] values.
 AxisDirection flipAxisDirection(AxisDirection axisDirection) {
-  assert(axisDirection != null);
   switch (axisDirection) {
     case AxisDirection.up:
       return AxisDirection.down;
@@ -258,7 +329,6 @@ AxisDirection flipAxisDirection(AxisDirection axisDirection) {
 /// Specifically, returns true for [AxisDirection.up] and [AxisDirection.left]
 /// and false for [AxisDirection.down] and [AxisDirection.right].
 bool axisDirectionIsReversed(AxisDirection axisDirection) {
-  assert(axisDirection != null);
   switch (axisDirection) {
     case AxisDirection.up:
     case AxisDirection.left:
