@@ -5,6 +5,7 @@
 import 'dart:math' as math;
 
 import 'package:flutter/animation.dart';
+import 'package:flutter/foundation.dart';
 import 'package:vector_math/vector_math_64.dart' show Matrix4;
 
 import 'box.dart';
@@ -146,7 +147,7 @@ class RenderListWheelViewport
     implements RenderAbstractViewport {
   /// Creates a [RenderListWheelViewport] which renders children on a wheel.
   ///
-  /// All arguments must not be null. Optional arguments have reasonable defaults.
+  /// Optional arguments have reasonable defaults.
   RenderListWheelViewport({
     required this.childManager,
     required ViewportOffset offset,
@@ -220,8 +221,6 @@ class RenderListWheelViewport
   /// viewport uses to select which part of its content to display. As the user
   /// scrolls the viewport, this value changes, which changes the content that
   /// is displayed.
-  ///
-  /// Must not be null.
   ViewportOffset get offset => _offset;
   ViewportOffset _offset;
   set offset(ViewportOffset value) {
@@ -264,7 +263,7 @@ class RenderListWheelViewport
   ///
   /// Defaults to an arbitrary but aesthetically reasonable number of 2.0.
   ///
-  /// Must not be null and must be positive.
+  /// Must be a positive number.
   /// {@endtemplate}
   double get diameterRatio => _diameterRatio;
   double _diameterRatio;
@@ -293,7 +292,7 @@ class RenderListWheelViewport
   /// A larger number brings the vanishing point closer and a smaller number
   /// pushes the vanishing point further.
   ///
-  /// Must not be null and must be positive.
+  /// Must be a positive number.
   /// {@endtemplate}
   double get perspective => _perspective;
   double _perspective;
@@ -402,7 +401,7 @@ class RenderListWheelViewport
   /// The size of the children along the main axis. Children [RenderBox]es will
   /// be given the [BoxConstraints] of this exact size.
   ///
-  /// Must not be null and must be positive.
+  /// Must be a positive number.
   /// {@endtemplate}
   double get itemExtent => _itemExtent;
   double _itemExtent;
@@ -432,7 +431,7 @@ class RenderListWheelViewport
   /// Changing this value will change the number of children built and shown
   /// inside the wheel.
   ///
-  /// Must not be null and must be positive.
+  /// Must be a positive number.
   /// {@endtemplate}
   ///
   /// Defaults to 1.
@@ -454,9 +453,9 @@ class RenderListWheelViewport
   /// If false, every child will be painted. However the [Scrollable] is still
   /// the size of the viewport and detects gestures inside only.
   ///
-  /// Defaults to false. Must not be null. Cannot be true if [clipBehavior]
-  /// is not [Clip.none] since children outside the viewport will be clipped, and
-  /// therefore cannot render children outside the viewport.
+  /// Defaults to false. Cannot be true if [clipBehavior] is not [Clip.none]
+  /// since children outside the viewport will be clipped, and therefore cannot
+  /// render children outside the viewport.
   /// {@endtemplate}
   bool get renderChildrenOutsideViewport => _renderChildrenOutsideViewport;
   bool _renderChildrenOutsideViewport;
@@ -475,7 +474,7 @@ class RenderListWheelViewport
 
   /// {@macro flutter.material.Material.clipBehavior}
   ///
-  /// Defaults to [Clip.hardEdge], and must not be null.
+  /// Defaults to [Clip.hardEdge].
   Clip get clipBehavior => _clipBehavior;
   Clip _clipBehavior = Clip.hardEdge;
   set clipBehavior(Clip value) {
@@ -617,7 +616,8 @@ class RenderListWheelViewport
   bool get sizedByParent => true;
 
   @override
-  Size computeDryLayout(BoxConstraints constraints) {
+  @protected
+  Size computeDryLayout(covariant BoxConstraints constraints) {
     return constraints.biggest;
   }
 
@@ -763,8 +763,9 @@ class RenderListWheelViewport
 
     // Relayout all active children.
     RenderBox? child = firstChild;
+    int index = currentFirstIndex;
     while (child != null) {
-      child.layout(childConstraints, parentUsesSize: true);
+      _layoutChild(child, childConstraints, index++);
       child = childAfter(child);
     }
 
@@ -823,27 +824,47 @@ class RenderListWheelViewport
   @override
   void dispose() {
     _clipRectLayer.layer = null;
+    _childOpacityLayerHandler.layer = null;
     super.dispose();
   }
-
+  final LayerHandle<OpacityLayer> _childOpacityLayerHandler = LayerHandle<OpacityLayer>();
   /// Paints all children visible in the current viewport.
   void _paintVisibleChildren(PaintingContext context, Offset offset) {
+    // The magnifier cannot be turned off if the opacity is less than 1.0.
+    if (overAndUnderCenterOpacity >= 1) {
+      _paintAllChildren(context, offset);
+      return;
+    }
+
+    // In order to reduce the number of opacity layers, we first paint all
+    // partially opaque children, then finally paint the fully opaque children.
+    _childOpacityLayerHandler.layer = context.pushOpacity(offset, (overAndUnderCenterOpacity * 255).round(), (PaintingContext context, Offset offset) {
+      _paintAllChildren(context, offset, center: false);
+    });
+    _paintAllChildren(context, offset, center: true);
+  }
+
+  void _paintAllChildren(PaintingContext context, Offset offset, { bool? center }) {
     RenderBox? childToPaint = firstChild;
     while (childToPaint != null) {
       final ListWheelParentData childParentData = childToPaint.parentData! as ListWheelParentData;
-      _paintTransformedChild(childToPaint, context, offset, childParentData.offset);
+      _paintTransformedChild(childToPaint, context, offset, childParentData.offset, center: center);
       childToPaint = childAfter(childToPaint);
     }
   }
 
-  /// Takes in a child with a **scrollable layout offset** and paints it in the
-  /// **transformed cylindrical space viewport painting coordinates**.
+  // Takes in a child with a **scrollable layout offset** and paints it in the
+  // **transformed cylindrical space viewport painting coordinates**.
+  //
+  // The value of `center` is passed through to _paintChildWithMagnifier only
+  // if the magnifier is enabled and/or opacity is < 1.0.
   void _paintTransformedChild(
     RenderBox child,
     PaintingContext context,
     Offset offset,
-    Offset layoutOffset,
-  ) {
+    Offset layoutOffset, {
+    required bool? center,
+  }) {
     final Offset untransformedPaintingCoordinates = offset
         + Offset(
             layoutOffset.dx,
@@ -876,22 +897,35 @@ class RenderListWheelViewport
 
     final bool shouldApplyOffCenterDim = overAndUnderCenterOpacity < 1;
     if (useMagnifier || shouldApplyOffCenterDim) {
-      _paintChildWithMagnifier(context, offset, child, transform, offsetToCenter, untransformedPaintingCoordinates);
+      _paintChildWithMagnifier(context, offset, child, transform, offsetToCenter, untransformedPaintingCoordinates, center: center);
     } else {
+      assert(center == null);
       _paintChildCylindrically(context, offset, child, transform, offsetToCenter);
     }
   }
 
-  /// Paint child with the magnifier active - the child will be rendered
-  /// differently if it intersects with the magnifier.
+  // Paint child with the magnifier active - the child will be rendered
+  // differently if it intersects with the magnifier.
+  //
+  // `center` controls how items that partially intersect the center magnifier
+  // are rendered. If `center` is false, items are only painted cynlindrically.
+  // If `center` is true, only the clipped magnifier items are painted.
+  // If `center` is null, partially intersecting items are painted both as the
+  // magnifier and cynlidrical item, while non-intersecting items are painted
+  // only cylindrically.
+  //
+  // This property is used to lift the opacity that would be applied to each
+  // cylindrical item into a single layer, reducing the rendering cost of the
+  // pickers which use this viewport.
   void _paintChildWithMagnifier(
     PaintingContext context,
     Offset offset,
     RenderBox child,
     Matrix4 cylindricalTransform,
     Offset offsetToCenter,
-    Offset untransformedPaintingCoordinates,
-  ) {
+    Offset untransformedPaintingCoordinates, {
+    required bool? center,
+  }) {
     final double magnifierTopLinePosition =
         size.height / 2 - _itemExtent * _magnification / 2;
     final double magnifierBottomLinePosition =
@@ -902,27 +936,28 @@ class RenderListWheelViewport
     final bool isBeforeMagnifierBottomLine = untransformedPaintingCoordinates.dy
         <= magnifierBottomLinePosition;
 
+    final Rect centerRect = Rect.fromLTWH(
+      0.0,
+      magnifierTopLinePosition,
+      size.width,
+      _itemExtent * _magnification,
+    );
+    final Rect topHalfRect = Rect.fromLTWH(
+      0.0,
+      0.0,
+      size.width,
+      magnifierTopLinePosition,
+    );
+    final Rect bottomHalfRect = Rect.fromLTWH(
+      0.0,
+      magnifierBottomLinePosition,
+      size.width,
+      magnifierTopLinePosition,
+    );
     // Some part of the child is in the center magnifier.
-    if (isAfterMagnifierTopLine && isBeforeMagnifierBottomLine) {
-      final Rect centerRect = Rect.fromLTWH(
-        0.0,
-        magnifierTopLinePosition,
-        size.width,
-        _itemExtent * _magnification,
-      );
-      final Rect topHalfRect = Rect.fromLTWH(
-        0.0,
-        0.0,
-        size.width,
-        magnifierTopLinePosition,
-      );
-      final Rect bottomHalfRect = Rect.fromLTWH(
-        0.0,
-        magnifierBottomLinePosition,
-        size.width,
-        magnifierTopLinePosition,
-      );
+    final bool inCenter = isAfterMagnifierTopLine && isBeforeMagnifierBottomLine;
 
+    if ((center == null || center) && inCenter) {
       // Clipping the part in the center.
       context.pushClipRect(
         needsCompositing,
@@ -939,8 +974,10 @@ class RenderListWheelViewport
           );
         },
       );
+    }
 
-      // Clipping the part in either the top-half or bottom-half of the wheel.
+    // Clipping the part in either the top-half or bottom-half of the wheel.
+    if ((center == null || !center) && inCenter) {
       context.pushClipRect(
         needsCompositing,
         offset,
@@ -948,16 +985,18 @@ class RenderListWheelViewport
           ? topHalfRect
           : bottomHalfRect,
         (PaintingContext context, Offset offset) {
-          _paintChildCylindrically(
-            context,
-            offset,
-            child,
-            cylindricalTransform,
-            offsetToCenter,
-          );
+            _paintChildCylindrically(
+              context,
+              offset,
+              child,
+              cylindricalTransform,
+              offsetToCenter,
+            );
         },
       );
-    } else {
+    }
+
+    if ((center == null || !center) && !inCenter) {
       _paintChildCylindrically(
         context,
         offset,
@@ -987,17 +1026,12 @@ class RenderListWheelViewport
       );
     }
 
-    // Paint child cylindrically, with [overAndUnderCenterOpacity].
-    void opacityPainter(PaintingContext context, Offset offset) {
-      context.pushOpacity(offset, (overAndUnderCenterOpacity * 255).round(), painter);
-    }
-
     context.pushTransform(
       needsCompositing,
       offset,
       _centerOriginTransform(cylindricalTransform),
       // Pre-transform painting function.
-      overAndUnderCenterOpacity == 1 ? painter : opacityPainter,
+      painter,
     );
 
     final ListWheelParentData childParentData = child.parentData! as ListWheelParentData;
@@ -1090,17 +1124,21 @@ class RenderListWheelViewport
   }
 
   @override
-  RevealedOffset getOffsetToReveal(RenderObject target, double alignment, { Rect? rect }) {
+  RevealedOffset getOffsetToReveal(
+    RenderObject target,
+    double alignment, {
+    Rect? rect,
+    Axis? axis, // Unused, only Axis.vertical supported by this viewport.
+  }) {
     // `target` is only fully revealed when in the selected/center position. Therefore,
     // this method always returns the offset that shows `target` in the center position,
     // which is the same offset for all `alignment` values.
-
     rect ??= target.paintBounds;
 
     // `child` will be the last RenderObject before the viewport when walking up from `target`.
     RenderObject child = target;
     while (child.parent != this) {
-      child = child.parent! as RenderObject;
+      child = child.parent!;
     }
 
     final ListWheelParentData parentData = child.parentData! as ListWheelParentData;
