@@ -3,11 +3,11 @@
 // found in the LICENSE file.
 
 import 'dart:io';
+
 import 'package:flutter/foundation.dart';
 
 import 'binding.dart';
 import 'hardware_keyboard.dart';
-import 'keyboard_key.g.dart';
 import 'raw_keyboard_android.dart';
 import 'raw_keyboard_fuchsia.dart';
 import 'raw_keyboard_ios.dart';
@@ -25,10 +25,18 @@ export 'keyboard_key.g.dart' show LogicalKeyboardKey, PhysicalKeyboardKey;
 /// discrimination between which key is pressed (e.g. the left or right SHIFT
 /// key).
 ///
+/// This enum is deprecated and will be removed. There is no direct substitute
+/// planned, since this enum will no longer be necessary once [RawKeyEvent] and
+/// associated APIs are removed.
+///
 /// See also:
 ///
 ///  * [RawKeyEventData.isModifierPressed], which accepts this enum as an
 ///    argument.
+@Deprecated(
+  'No longer supported. '
+  'This feature was deprecated after v3.18.0-2.0.pre.',
+)
 enum KeyboardSide {
   /// Matches if either the left, right or both versions of the key are pressed.
   any,
@@ -45,10 +53,18 @@ enum KeyboardSide {
 
 /// An enum describing the type of modifier key that is being pressed.
 ///
+/// This enum is deprecated and will be removed. There is no direct substitute
+/// planned, since this enum will no longer be necessary once [RawKeyEvent] and
+/// associated APIs are removed.
+///
 /// See also:
 ///
 ///  * [RawKeyEventData.isModifierPressed], which accepts this enum as an
 ///    argument.
+@Deprecated(
+  'No longer supported. '
+  'This feature was deprecated after v3.18.0-2.0.pre.',
+)
 enum ModifierKey {
   /// The CTRL modifier key.
   ///
@@ -88,7 +104,7 @@ enum ModifierKey {
 
   /// The SCROLL LOCK modifier key.
   ///
-  /// Typically, there is one of these.  Only shown as "pressed" when the scroll
+  /// Typically, there is one of these. Only shown as "pressed" when the scroll
   /// lock is on, so on a key up when the mode is turned on, on each key press
   /// when it's enabled, and on a key down when it is turned off.
   scrollLockModifier,
@@ -106,6 +122,9 @@ enum ModifierKey {
 
 /// Base class for platform-specific key event data.
 ///
+/// This class is deprecated, and will be removed. Platform specific key event
+/// data will no be longer available. See [KeyEvent] for what is available.
+///
 /// This base class exists to have a common type to use for each of the
 /// target platform's key event data structures.
 ///
@@ -116,56 +135,112 @@ enum ModifierKey {
 ///  * [RawKeyDownEvent] and [RawKeyUpEvent], the classes that hold the
 ///    reference to [RawKeyEventData] subclasses.
 ///  * [RawKeyboard], which uses these interfaces to expose key data.
+@Deprecated(
+  'Platform specific key event data is no longer available. See KeyEvent for what is available. '
+  'This feature was deprecated after v3.18.0-2.0.pre.',
+)
 @immutable
 abstract class RawKeyEventData with Diagnosticable {
   /// Abstract const constructor. This constructor enables subclasses to provide
   /// const constructors so that they can be used in const expressions.
+  @Deprecated(
+    'Platform specific key event data is no longer available. See KeyEvent for what is available. '
+    'This feature was deprecated after v3.18.0-2.0.pre.',
+  )
   const RawKeyEventData();
 
   /// Returns true if the given [ModifierKey] was pressed at the time of this
   /// event.
   ///
+  /// This method is deprecated and will be removed. For equivalent information,
+  /// inspect [HardwareKeyboard.logicalKeysPressed] instead.
+  ///
   /// If [side] is specified, then this restricts its check to the specified
   /// side of the keyboard. Defaults to checking for the key being down on
   /// either side of the keyboard. If there is only one instance of the key on
   /// the keyboard, then [side] is ignored.
+  @Deprecated(
+    'No longer available. Inspect HardwareKeyboard.instance.logicalKeysPressed instead. '
+    'This feature was deprecated after v3.18.0-2.0.pre.',
+  )
   bool isModifierPressed(ModifierKey key, { KeyboardSide side = KeyboardSide.any });
 
   /// Returns a [KeyboardSide] enum value that describes which side or sides of
   /// the given keyboard modifier key were pressed at the time of this event.
   ///
+  /// This method is deprecated and will be removed.
+  ///
   /// If the modifier key wasn't pressed at the time of this event, returns
   /// null. If the given key only appears in one place on the keyboard, returns
   /// [KeyboardSide.all] if pressed. If the given platform does not specify
   /// the side, return [KeyboardSide.any].
+  @Deprecated(
+    'No longer available. '
+    'This feature was deprecated after v3.18.0-2.0.pre.',
+  )
   KeyboardSide? getModifierSide(ModifierKey key);
 
   /// Returns true if a CTRL modifier key was pressed at the time of this event,
   /// regardless of which side of the keyboard it is on.
   ///
+  /// This method is deprecated and will be removed. Use
+  /// [HardwareKeyboard.isControlPressed] instead.
+  ///
   /// Use [isModifierPressed] if you need to know which control key was pressed.
+  @Deprecated(
+    'Use HardwareKeyboard.instance.isControlPressed instead. '
+    'This feature was deprecated after v3.18.0-2.0.pre.',
+  )
   bool get isControlPressed => isModifierPressed(ModifierKey.controlModifier);
 
   /// Returns true if a SHIFT modifier key was pressed at the time of this
   /// event, regardless of which side of the keyboard it is on.
   ///
+  /// This method is deprecated and will be removed. Use
+  /// [HardwareKeyboard.isShiftPressed] instead.
+  ///
   /// Use [isModifierPressed] if you need to know which shift key was pressed.
+  @Deprecated(
+    'Use HardwareKeyboard.instance.isShiftPressed instead. '
+    'This feature was deprecated after v3.18.0-2.0.pre.',
+  )
   bool get isShiftPressed => isModifierPressed(ModifierKey.shiftModifier);
 
   /// Returns true if a ALT modifier key was pressed at the time of this event,
   /// regardless of which side of the keyboard it is on.
   ///
+  /// This method is deprecated and will be removed. Use
+  /// [HardwareKeyboard.isAltPressed] instead.
+  ///
   /// Use [isModifierPressed] if you need to know which alt key was pressed.
+  @Deprecated(
+    'Use HardwareKeyboard.instance.isAltPressed instead. '
+    'This feature was deprecated after v3.18.0-2.0.pre.',
+  )
   bool get isAltPressed => isModifierPressed(ModifierKey.altModifier);
 
   /// Returns true if a META modifier key was pressed at the time of this event,
   /// regardless of which side of the keyboard it is on.
   ///
+  /// This method is deprecated and will be removed. Use
+  /// [HardwareKeyboard.isMetaPressed] instead.
+  ///
   /// Use [isModifierPressed] if you need to know which meta key was pressed.
+  @Deprecated(
+    'Use HardwareKeyboard.instance.isMetaPressed instead. '
+    'This feature was deprecated after v3.18.0-2.0.pre.',
+  )
   bool get isMetaPressed => isModifierPressed(ModifierKey.metaModifier);
 
   /// Returns a map of modifier keys that were pressed at the time of this
   /// event, and the keyboard side or sides that the key was on.
+  ///
+  /// This method is deprecated and will be removed. For equivalent information,
+  /// inspect [HardwareKeyboard.logicalKeysPressed] instead.
+  @Deprecated(
+    'No longer available. Inspect HardwareKeyboard.instance.logicalKeysPressed instead. '
+    'This feature was deprecated after v3.18.0-2.0.pre.',
+  )
   Map<ModifierKey, KeyboardSide> get modifiersPressed {
     final Map<ModifierKey, KeyboardSide> result = <ModifierKey, KeyboardSide>{};
     for (final ModifierKey key in ModifierKey.values) {
@@ -254,6 +329,11 @@ abstract class RawKeyEventData with Diagnosticable {
 
 /// Defines the interface for raw key events.
 ///
+/// This type of event has been deprecated, and will be removed at a future
+/// date. Instead of using this, use the [KeyEvent] class instead, which means
+/// you will use [Focus.onKeyEvent], [HardwareKeyboard] and [KeyboardListener]
+/// to get these events instead of the raw key event equivalents.
+///
 /// Raw key events pass through as much information as possible from the
 /// underlying platform's key events, which allows them to provide a high level
 /// of fidelity but a low level of portability.
@@ -265,20 +345,28 @@ abstract class RawKeyEventData with Diagnosticable {
 ///
 /// See also:
 ///
-///  * [LogicalKeyboardKey], an object that describes the logical meaning of a
-///    key.
-///  * [PhysicalKeyboardKey], an object that describes the physical location of
-///    a key.
-///  * [RawKeyDownEvent], a specialization for events representing the user
-///    pressing a key.
-///  * [RawKeyUpEvent], a specialization for events representing the user
-///    releasing a key.
-///  * [RawKeyboard], which uses this interface to expose key data.
-///  * [RawKeyboardListener], a widget that listens for raw key events.
+/// * [LogicalKeyboardKey], an object that describes the logical meaning of a
+///   key.
+/// * [PhysicalKeyboardKey], an object that describes the physical location of a
+///   key.
+/// * [RawKeyDownEvent], a specialization for events representing the user
+///   pressing a key.
+/// * [RawKeyUpEvent], a specialization for events representing the user
+///   releasing a key.
+/// * [RawKeyboard], which uses this interface to expose key data.
+/// * [RawKeyboardListener], a widget that listens for raw key events.
+@Deprecated(
+  'Use KeyEvent instead. '
+  'This feature was deprecated after v3.18.0-2.0.pre.',
+)
 @immutable
 abstract class RawKeyEvent with Diagnosticable {
   /// Abstract const constructor. This constructor enables subclasses to provide
   /// const constructors so that they can be used in const expressions.
+  @Deprecated(
+    'Use KeyEvent instead. '
+    'This feature was deprecated after v3.18.0-2.0.pre.',
+  )
   const RawKeyEvent({
     required this.data,
     this.character,
@@ -290,6 +378,10 @@ abstract class RawKeyEvent with Diagnosticable {
   ///
   /// [RawKeyEvent.repeat] will be derived from the current keyboard state,
   /// instead of using the message information.
+  @Deprecated(
+    'No longer supported. Use KeyEvent instead. '
+    'This feature was deprecated after v3.18.0-2.0.pre.',
+  )
   factory RawKeyEvent.fromMessage(Map<String, Object?> message) {
     String? character;
     RawKeyEventData dataFromWeb() {
@@ -329,7 +421,6 @@ abstract class RawKeyEvent with Diagnosticable {
           if (message.containsKey('character')) {
             character = message['character'] as String?;
           }
-          break;
         case 'fuchsia':
           final int codePoint = message['codePoint'] as int? ?? 0;
           data = RawKeyEventDataFuchsia(
@@ -340,7 +431,6 @@ abstract class RawKeyEvent with Diagnosticable {
           if (codePoint != 0) {
             character = String.fromCharCode(codePoint);
           }
-          break;
         case 'macos':
           data = RawKeyEventDataMacOs(
             characters: message['characters'] as String? ?? '',
@@ -350,7 +440,6 @@ abstract class RawKeyEvent with Diagnosticable {
             specifiedLogicalKey: message['specifiedLogicalKey'] as int?,
           );
           character = message['characters'] as String?;
-          break;
         case 'ios':
           data = RawKeyEventDataIos(
             characters: message['characters'] as String? ?? '',
@@ -358,7 +447,10 @@ abstract class RawKeyEvent with Diagnosticable {
             keyCode: message['keyCode'] as int? ?? 0,
             modifiers: message['modifiers'] as int? ?? 0,
           );
-          break;
+          final Object? characters = message['characters'];
+          if (characters is String && characters.isNotEmpty) {
+            character = characters;
+          }
         case 'linux':
           final int unicodeScalarValues = message['unicodeScalarValues'] as int? ?? 0;
           data = RawKeyEventDataLinux(
@@ -373,7 +465,6 @@ abstract class RawKeyEvent with Diagnosticable {
           if (unicodeScalarValues != 0) {
             character = String.fromCharCode(unicodeScalarValues);
           }
-          break;
         case 'windows':
           final int characterCodePoint = message['characterCodePoint'] as int? ?? 0;
           data = RawKeyEventDataWindows(
@@ -385,10 +476,8 @@ abstract class RawKeyEvent with Diagnosticable {
           if (characterCodePoint != 0) {
             character = String.fromCharCode(characterCodePoint);
           }
-          break;
         case 'web':
           data = dataFromWeb();
-          break;
         default:
           /// This exception would only be hit on platforms that haven't yet
           /// implemented raw key events, but will only be triggered if the
@@ -409,13 +498,27 @@ abstract class RawKeyEvent with Diagnosticable {
     }
   }
 
-  /// Returns true if the given [KeyboardKey] is pressed.
+  /// Returns true if the given [LogicalKeyboardKey] is pressed.
+  ///
+  /// This method is deprecated and will be removed. Use
+  /// [HardwareKeyboard.isLogicalKeyPressed] instead.
+  @Deprecated(
+    'Use HardwareKeyboard.instance.isLogicalKeyPressed instead. '
+    'This feature was deprecated after v3.18.0-2.0.pre.',
+  )
   bool isKeyPressed(LogicalKeyboardKey key) => RawKeyboard.instance.keysPressed.contains(key);
 
   /// Returns true if a CTRL modifier key is pressed, regardless of which side
   /// of the keyboard it is on.
   ///
+  /// This method is deprecated and will be removed. Use
+  /// [HardwareKeyboard.isControlPressed] instead.
+  ///
   /// Use [isKeyPressed] if you need to know which control key was pressed.
+  @Deprecated(
+    'Use HardwareKeyboard.instance.isControlPressed instead. '
+    'This feature was deprecated after v3.18.0-2.0.pre.',
+  )
   bool get isControlPressed {
     return isKeyPressed(LogicalKeyboardKey.controlLeft) || isKeyPressed(LogicalKeyboardKey.controlRight);
   }
@@ -423,7 +526,14 @@ abstract class RawKeyEvent with Diagnosticable {
   /// Returns true if a SHIFT modifier key is pressed, regardless of which side
   /// of the keyboard it is on.
   ///
+  /// This method is deprecated and will be removed. Use
+  /// [HardwareKeyboard.isShiftPressed] instead.
+  ///
   /// Use [isKeyPressed] if you need to know which shift key was pressed.
+  @Deprecated(
+    'Use HardwareKeyboard.instance.isShiftPressed instead. '
+    'This feature was deprecated after v3.18.0-2.0.pre.',
+  )
   bool get isShiftPressed {
     return isKeyPressed(LogicalKeyboardKey.shiftLeft) || isKeyPressed(LogicalKeyboardKey.shiftRight);
   }
@@ -431,13 +541,20 @@ abstract class RawKeyEvent with Diagnosticable {
   /// Returns true if a ALT modifier key is pressed, regardless of which side
   /// of the keyboard it is on.
   ///
-  /// Note that the ALTGR key that appears on some keyboards is considered to be
+  /// This method is deprecated and will be removed. Use
+  /// [HardwareKeyboard.isAltPressed] instead.
+  ///
+  /// The `AltGr` key that appears on some keyboards is considered to be
   /// the same as [LogicalKeyboardKey.altRight] on some platforms (notably
   /// Android). On platforms that can distinguish between `altRight` and
-  /// `altGr`, a press of `altGr` will not return true here, and will need to be
+  /// `altGr`, a press of `AltGr` will not return true here, and will need to be
   /// tested for separately.
   ///
   /// Use [isKeyPressed] if you need to know which alt key was pressed.
+  @Deprecated(
+    'Use HardwareKeyboard.instance.isAltPressed instead. '
+    'This feature was deprecated after v3.18.0-2.0.pre.',
+  )
   bool get isAltPressed {
     return isKeyPressed(LogicalKeyboardKey.altLeft) || isKeyPressed(LogicalKeyboardKey.altRight);
   }
@@ -445,7 +562,14 @@ abstract class RawKeyEvent with Diagnosticable {
   /// Returns true if a META modifier key is pressed, regardless of which side
   /// of the keyboard it is on.
   ///
+  /// This method is deprecated and will be removed. Use
+  /// [HardwareKeyboard.isMetaPressed] instead.
+  ///
   /// Use [isKeyPressed] if you need to know which meta key was pressed.
+  @Deprecated(
+    'Use HardwareKeyboard.instance.isMetaPressed instead. '
+    'This feature was deprecated after v3.18.0-2.0.pre.',
+  )
   bool get isMetaPressed {
     return isKeyPressed(LogicalKeyboardKey.metaLeft) || isKeyPressed(LogicalKeyboardKey.metaRight);
   }
@@ -462,7 +586,7 @@ abstract class RawKeyEvent with Diagnosticable {
   ///
   /// For instance, if you wanted to make a game where the key to the right of
   /// the CAPS LOCK key made the player move left, you would be comparing the
-  /// result of this `physicalKey` with [PhysicalKeyboardKey.keyA], since that
+  /// result of this [physicalKey] with [PhysicalKeyboardKey.keyA], since that
   /// is the key next to the CAPS LOCK key on a QWERTY keyboard. This would
   /// return the same thing even on a French keyboard where the key next to the
   /// CAPS LOCK produces a "Q" when pressed.
@@ -507,7 +631,7 @@ abstract class RawKeyEvent with Diagnosticable {
   /// accurately referred to as grapheme clusters) are made up of more than one
   /// code point.
   ///
-  /// The `character` doesn't take into account edits by an input method editor
+  /// The [character] doesn't take into account edits by an input method editor
   /// (IME), or manage the visibility of the soft keyboard on touch devices. For
   /// composing text, use the [TextField] or [CupertinoTextField] widgets, since
   /// those automatically handle many of the complexities of managing keyboard
@@ -539,11 +663,22 @@ abstract class RawKeyEvent with Diagnosticable {
 
 /// The user has pressed a key on the keyboard.
 ///
+/// This class has been deprecated and will be removed. Use [KeyDownEvent]
+/// instead.
+///
 /// See also:
 ///
-///  * [RawKeyboard], which uses this interface to expose key data.
+/// * [RawKeyboard], which uses this interface to expose key data.
+@Deprecated(
+  'Use KeyDownEvent instead. '
+  'This feature was deprecated after v3.18.0-2.0.pre.',
+)
 class RawKeyDownEvent extends RawKeyEvent {
   /// Creates a key event that represents the user pressing a key.
+  @Deprecated(
+    'Use KeyDownEvent instead. '
+    'This feature was deprecated after v3.18.0-2.0.pre.',
+  )
   const RawKeyDownEvent({
     required super.data,
     super.character,
@@ -553,11 +688,22 @@ class RawKeyDownEvent extends RawKeyEvent {
 
 /// The user has released a key on the keyboard.
 ///
+/// This class has been deprecated and will be removed. Use [KeyUpEvent]
+/// instead.
+///
 /// See also:
 ///
 ///  * [RawKeyboard], which uses this interface to expose key data.
+@Deprecated(
+  'Use KeyUpEvent instead. '
+  'This feature was deprecated after v3.18.0-2.0.pre.',
+)
 class RawKeyUpEvent extends RawKeyEvent {
   /// Creates a key event that represents the user releasing a key.
+  @Deprecated(
+    'Use KeyUpEvent instead. '
+    'This feature was deprecated after v3.18.0-2.0.pre.',
+  )
   const RawKeyUpEvent({
     required super.data,
     super.character,
@@ -567,12 +713,22 @@ class RawKeyUpEvent extends RawKeyEvent {
 /// A callback type used by [RawKeyboard.keyEventHandler] to send key events to
 /// a handler that can determine if the key has been handled or not.
 ///
+/// This typedef has been deprecated and will be removed. Use [KeyEventCallback]
+/// instead.
+///
 /// The handler should return true if the key has been handled, and false if the
-/// key was not handled.  It must not return null.
+/// key was not handled. It must not return null.
+@Deprecated(
+  'Use KeyEventCallback instead. '
+  'This feature was deprecated after v3.18.0-2.0.pre.',
+)
 typedef RawKeyEventHandler = bool Function(RawKeyEvent event);
 
 /// An interface for listening to raw key events.
 ///
+/// This class is deprecated and will be removed at a future date. The
+/// [HardwareKeyboard] class is its replacement.
+///
 /// Raw key events pass through as much information as possible from the
 /// underlying platform's key events, which makes them provide a high level of
 /// fidelity but a low level of portability.
@@ -584,17 +740,44 @@ typedef RawKeyEventHandler = bool Function(RawKeyEvent event);
 /// These key events are typically only key events generated by a hardware
 /// keyboard, and not those from software keyboards or input method editors.
 ///
+/// ## Compared to [HardwareKeyboard]
+///
+/// Behavior-wise, [RawKeyboard] provides a less unified, less regular event
+/// model than [HardwareKeyboard]. For example:
+///
+/// * Down events might not be matched with an up event, and vice versa (the set
+///   of pressed keys is silently updated).
+/// * The logical key of the down event might not be the same as that of the up
+///   event.
+/// * Down events and repeat events are not easily distinguishable (must be
+///   tracked manually).
+/// * Lock modes (such as CapsLock) only have their "enabled" state recorded.
+///   There's no way to acquire their pressing state.
+///
 /// See also:
 ///
 ///  * [RawKeyDownEvent] and [RawKeyUpEvent], the classes used to describe
 ///    specific raw key events.
 ///  * [RawKeyboardListener], a widget that listens for raw key events.
-///  * [SystemChannels.keyEvent], the low-level channel used for receiving
-///    events from the system.
+///  * [SystemChannels.keyEvent], the low-level channel used for receiving events
+///    from the system.
+///  * [HardwareKeyboard], the recommended replacement.
+@Deprecated(
+  'Use HardwareKeyboard instead. '
+  'This feature was deprecated after v3.18.0-2.0.pre.',
+)
 class RawKeyboard {
+  @Deprecated(
+    'Use HardwareKeyboard instead. '
+    'This feature was deprecated after v3.18.0-2.0.pre.',
+  )
   RawKeyboard._();
 
   /// The shared instance of [RawKeyboard].
+  @Deprecated(
+    'Use HardwareKeyboard.instance instead. '
+    'This feature was deprecated after v3.18.0-2.0.pre.',
+  )
   static final RawKeyboard instance = RawKeyboard._();
 
   final List<ValueChanged<RawKeyEvent>> _listeners = <ValueChanged<RawKeyEvent>>[];
@@ -823,9 +1006,18 @@ class RawKeyboard {
         modifierKeys[physicalModifier] = _allModifiers[physicalModifier]!;
       }
     }
-    _allModifiersExceptFn.keys
-      .where((PhysicalKeyboardKey key) => !anySideKeys.contains(key))
-      .forEach(_keysPressed.remove);
+    // On Linux, CapsLock key can be mapped to a non-modifier logical key:
+    // https://github.com/flutter/flutter/issues/114591.
+    // This is also affecting Flutter Web on Linux.
+    final bool nonModifierCapsLock = (event.data is RawKeyEventDataLinux  || event.data is RawKeyEventDataWeb)
+      && _keysPressed[PhysicalKeyboardKey.capsLock] != null
+      && _keysPressed[PhysicalKeyboardKey.capsLock] != LogicalKeyboardKey.capsLock;
+    for (final PhysicalKeyboardKey physicalKey in _allModifiersExceptFn.keys) {
+      final bool skipReleasingKey = nonModifierCapsLock && physicalKey == PhysicalKeyboardKey.capsLock;
+      if (!anySideKeys.contains(physicalKey) && !skipReleasingKey) {
+        _keysPressed.remove(physicalKey);
+      }
+    }
     if (event.data is! RawKeyEventDataFuchsia && event.data is! RawKeyEventDataMacOs) {
       // On Fuchsia and macOS, the Fn key is not considered a modifier key.
       _keysPressed.remove(PhysicalKeyboardKey.fn);
@@ -835,15 +1027,24 @@ class RawKeyboard {
     // exist in the modifier list. Enforce the pressing state.
     if (event is RawKeyDownEvent && thisKeyModifier != null
         && !_keysPressed.containsKey(event.physicalKey)) {
-      // So far this inconsistancy is only found on Linux GTK for AltRight in a
-      // rare case. (See https://github.com/flutter/flutter/issues/93278 .) In
-      // other cases, this inconsistancy will be caught by an assertion later.
-      if (event.data is RawKeyEventDataLinux && event.physicalKey == PhysicalKeyboardKey.altRight) {
+      // This inconsistency is found on Linux GTK for AltRight:
+      // https://github.com/flutter/flutter/issues/93278
+      // And also on Android and iOS:
+      // https://github.com/flutter/flutter/issues/101090
+      if ((event.data is RawKeyEventDataLinux && event.physicalKey == PhysicalKeyboardKey.altRight)
+        || event.data is RawKeyEventDataIos
+        || event.data is RawKeyEventDataAndroid) {
         final LogicalKeyboardKey? logicalKey = _allModifiersExceptFn[event.physicalKey];
         if (logicalKey != null) {
           _keysPressed[event.physicalKey] = logicalKey;
         }
       }
+      // On Web, PhysicalKeyboardKey.altRight can be map to LogicalKeyboardKey.altGraph or
+      // LogicalKeyboardKey.altRight:
+      // https://github.com/flutter/flutter/issues/113836
+      if (event.data is RawKeyEventDataWeb && event.physicalKey == PhysicalKeyboardKey.altRight) {
+        _keysPressed[event.physicalKey] = event.logicalKey;
+      }
     }
   }
 
