@@ -6,22 +6,23 @@ import 'package:flutter/animation.dart';
 import 'package:flutter/widgets.dart';
 import 'package:flutter_test/flutter_test.dart';
 
-import '../foundation/leak_tracking.dart';
-
 void main() {
-  testWidgetsWithLeakTracking('awaiting animation controllers - using direct future', (WidgetTester tester) async {
+  testWidgets('awaiting animation controllers - using direct future', (WidgetTester tester) async {
     final AnimationController controller1 = AnimationController(
       duration: const Duration(milliseconds: 100),
       vsync: const TestVSync(),
     );
+    addTearDown(controller1.dispose);
     final AnimationController controller2 = AnimationController(
       duration: const Duration(milliseconds: 600),
       vsync: const TestVSync(),
     );
+    addTearDown(controller2.dispose);
     final AnimationController controller3 = AnimationController(
       duration: const Duration(milliseconds: 300),
       vsync: const TestVSync(),
     );
+    addTearDown(controller3.dispose);
     final List<String> log = <String>[];
     Future<void> runTest() async {
       log.add('a'); // t=0
@@ -58,19 +59,22 @@ void main() {
     expect(log, <String>['start', 'a', 'b', 'c', 'd', 'end']);
   });
 
-  testWidgetsWithLeakTracking('awaiting animation controllers - using orCancel', (WidgetTester tester) async {
+  testWidgets('awaiting animation controllers - using orCancel', (WidgetTester tester) async {
     final AnimationController controller1 = AnimationController(
       duration: const Duration(milliseconds: 100),
       vsync: const TestVSync(),
     );
+    addTearDown(controller1.dispose);
     final AnimationController controller2 = AnimationController(
       duration: const Duration(milliseconds: 600),
       vsync: const TestVSync(),
     );
+    addTearDown(controller2.dispose);
     final AnimationController controller3 = AnimationController(
       duration: const Duration(milliseconds: 300),
       vsync: const TestVSync(),
     );
+    addTearDown(controller3.dispose);
     final List<String> log = <String>[];
     Future<void> runTest() async {
       log.add('a'); // t=0
@@ -107,7 +111,7 @@ void main() {
     expect(log, <String>['start', 'a', 'b', 'c', 'd', 'end']);
   });
 
-  testWidgetsWithLeakTracking('awaiting animation controllers and failing', (WidgetTester tester) async {
+  testWidgets('awaiting animation controllers and failing', (WidgetTester tester) async {
     final AnimationController controller1 = AnimationController(
       duration: const Duration(milliseconds: 100),
       vsync: const TestVSync(),
@@ -135,11 +139,12 @@ void main() {
     expect(log, <String>['start', 'caught', 'end']);
   });
 
-  testWidgetsWithLeakTracking('creating orCancel future later', (WidgetTester tester) async {
+  testWidgets('creating orCancel future later', (WidgetTester tester) async {
     final AnimationController controller1 = AnimationController(
       duration: const Duration(milliseconds: 100),
       vsync: const TestVSync(),
     );
+    addTearDown(controller1.dispose);
     final TickerFuture f = controller1.forward();
     await tester.pump(); // start ticker
     await tester.pump(const Duration(milliseconds: 200)); // end ticker
@@ -148,11 +153,12 @@ void main() {
     expect(true, isTrue); // should reach here
   });
 
-  testWidgetsWithLeakTracking('creating orCancel future later', (WidgetTester tester) async {
+  testWidgets('creating orCancel future later', (WidgetTester tester) async {
     final AnimationController controller1 = AnimationController(
       duration: const Duration(milliseconds: 100),
       vsync: const TestVSync(),
     );
+    addTearDown(controller1.dispose);
     final TickerFuture f = controller1.forward();
     await tester.pump(); // start ticker
     controller1.stop(); // cancel ticker
@@ -165,11 +171,12 @@ void main() {
     expect(ok, isTrue); // should reach here
   });
 
-  testWidgetsWithLeakTracking('TickerFuture is a Future', (WidgetTester tester) async {
+  testWidgets('TickerFuture is a Future', (WidgetTester tester) async {
     final AnimationController controller1 = AnimationController(
       duration: const Duration(milliseconds: 100),
       vsync: const TestVSync(),
     );
+    addTearDown(controller1.dispose);
     final TickerFuture f = controller1.forward();
     await tester.pump(); // start ticker
     await tester.pump(const Duration(milliseconds: 200)); // end ticker
