@@ -167,4 +167,57 @@ void main() {
     final Iterable<RenderClipRRect> renderClips = tester.allRenderObjects.whereType<RenderClipRRect>();
     expect(renderClips, isEmpty);
   });
+
+  testWidgets('Does not double up padding on header', (WidgetTester tester) async {
+    const Widget header = Text('Header');
+
+    await tester.pumpWidget(
+      CupertinoApp(
+        home: Center(
+          child: CupertinoFormSection(
+            header: header,
+            children: <Widget>[CupertinoTextFormFieldRow()],
+          ),
+        ),
+      ),
+    );
+
+    expect(tester.getTopLeft(find.byWidget(header)), const Offset(20, 22));
+  });
+
+  testWidgets('Does not double up padding on footer', (WidgetTester tester) async {
+    const Widget footer = Text('Footer');
+
+    await tester.pumpWidget(
+      CupertinoApp(
+        home: Center(
+          child: CupertinoFormSection(
+            footer: footer,
+            children: <Widget>[CupertinoTextFormFieldRow()],
+          ),
+        ),
+      ),
+    );
+
+    expect(tester.getTopLeft(find.byWidget(footer)), offsetMoreOrLessEquals(const Offset(20, 65), epsilon: 1));
+  });
+
+  testWidgets('Sets custom margin', (WidgetTester tester) async {
+    final Widget child = CupertinoTextFormFieldRow();
+
+    const double margin = 35;
+
+    await tester.pumpWidget(
+      CupertinoApp(
+        home: Center(
+          child: CupertinoFormSection(
+            margin: const EdgeInsets.all(margin),
+            children: <Widget>[child],
+          ),
+        ),
+      ),
+    );
+
+    expect(tester.getTopLeft(find.byWidget(child)), offsetMoreOrLessEquals(const Offset(margin, 22 + margin), epsilon: 1));
+  });
 }
