@@ -146,14 +146,14 @@ void main() {
     expect(
       tester.getTopLeft(flying(tester, find.text('Page 1')).first),
       const Offset(
-        bool.hasEnvironment('SKPARAGRAPH_REMOVE_ROUNDING_HACK') ? 342.547737105096302912 : 342.33420100808144,
+        342.547737105096302912,
         13.5,
       ),
     );
     expect(
       tester.getTopLeft(flying(tester, find.text('Page 1')).last),
       const Offset(
-        bool.hasEnvironment('SKPARAGRAPH_REMOVE_ROUNDING_HACK') ? 342.547737105096302912 : 342.33420100808144,
+        342.547737105096302912,
         13.5,
       ),
     );
@@ -173,14 +173,14 @@ void main() {
     expect(
       tester.getTopLeft(flying(tester, find.text('Page 1')).first),
       const Offset(
-        bool.hasEnvironment('SKPARAGRAPH_REMOVE_ROUNDING_HACK') ? 357.912261979376353338 : 357.66579899191856,
+        357.912261979376353338,
         13.5,
       ),
     );
     expect(
       tester.getTopLeft(flying(tester, find.text('Page 1')).last),
       const Offset(
-        bool.hasEnvironment('SKPARAGRAPH_REMOVE_ROUNDING_HACK') ? 357.912261979376353338 : 357.66579899191856,
+        357.912261979376353338,
         13.5,
       ),
     );
@@ -218,7 +218,7 @@ void main() {
         tester.renderObject(flying(tester, find.text('Page 1')).first);
     expect(bottomMiddle.text.style!.color, const Color(0xff000306));
     expect(bottomMiddle.text.style!.fontWeight, FontWeight.w600);
-    expect(bottomMiddle.text.style!.fontFamily, '.SF Pro Text');
+    expect(bottomMiddle.text.style!.fontFamily, 'CupertinoSystemText');
     expect(bottomMiddle.text.style!.letterSpacing, -0.41);
 
     checkOpacity(tester, flying(tester, find.text('Page 1')).first, 0.9404401779174805);
@@ -229,7 +229,7 @@ void main() {
         tester.renderObject(flying(tester, find.text('Page 1')).last);
     expect(topBackLabel.text.style!.color, const Color(0xff000306));
     expect(topBackLabel.text.style!.fontWeight, FontWeight.w600);
-    expect(topBackLabel.text.style!.fontFamily, '.SF Pro Text');
+    expect(topBackLabel.text.style!.fontFamily, 'CupertinoSystemText');
     expect(topBackLabel.text.style!.letterSpacing, -0.41);
 
     checkOpacity(tester, flying(tester, find.text('Page 1')).last, 0.0);
@@ -238,14 +238,14 @@ void main() {
     await tester.pump(const Duration(milliseconds: 200));
     expect(bottomMiddle.text.style!.color, const Color(0xff005ec5));
     expect(bottomMiddle.text.style!.fontWeight, FontWeight.w400);
-    expect(bottomMiddle.text.style!.fontFamily, '.SF Pro Text');
+    expect(bottomMiddle.text.style!.fontFamily, 'CupertinoSystemText');
     expect(bottomMiddle.text.style!.letterSpacing, -0.41);
 
     checkOpacity(tester, flying(tester, find.text('Page 1')).first, 0.0);
 
     expect(topBackLabel.text.style!.color, const Color(0xff005ec5));
     expect(topBackLabel.text.style!.fontWeight, FontWeight.w400);
-    expect(topBackLabel.text.style!.fontFamily, '.SF Pro Text');
+    expect(topBackLabel.text.style!.fontFamily, 'CupertinoSystemText');
     expect(topBackLabel.text.style!.letterSpacing, -0.41);
 
     checkOpacity(tester, flying(tester, find.text('Page 1')).last, 0.5292819738388062);
@@ -266,7 +266,7 @@ void main() {
         tester.renderObject(flying(tester, find.text('Page 1')).first);
     expect(bottomMiddle.text.style!.color, const Color(0xfff8fbff));
     expect(bottomMiddle.text.style!.fontWeight, FontWeight.w600);
-    expect(bottomMiddle.text.style!.fontFamily, '.SF Pro Text');
+    expect(bottomMiddle.text.style!.fontFamily, 'CupertinoSystemText');
     expect(bottomMiddle.text.style!.letterSpacing, -0.41);
 
     checkOpacity(tester, flying(tester, find.text('Page 1')).first, 0.9404401779174805);
@@ -277,7 +277,7 @@ void main() {
         tester.renderObject(flying(tester, find.text('Page 1')).last);
     expect(topBackLabel.text.style!.color, const Color(0xfff8fbff));
     expect(topBackLabel.text.style!.fontWeight, FontWeight.w600);
-    expect(topBackLabel.text.style!.fontFamily, '.SF Pro Text');
+    expect(topBackLabel.text.style!.fontFamily, 'CupertinoSystemText');
     expect(topBackLabel.text.style!.letterSpacing, -0.41);
 
     checkOpacity(tester, flying(tester, find.text('Page 1')).last, 0.0);
@@ -286,14 +286,14 @@ void main() {
     await tester.pump(const Duration(milliseconds: 200));
     expect(bottomMiddle.text.style!.color, const Color(0xff409fff));
     expect(bottomMiddle.text.style!.fontWeight, FontWeight.w400);
-    expect(bottomMiddle.text.style!.fontFamily, '.SF Pro Text');
+    expect(bottomMiddle.text.style!.fontFamily, 'CupertinoSystemText');
     expect(bottomMiddle.text.style!.letterSpacing, -0.41);
 
     checkOpacity(tester, flying(tester, find.text('Page 1')).first, 0.0);
 
     expect(topBackLabel.text.style!.color, const Color(0xff409fff));
     expect(topBackLabel.text.style!.fontWeight, FontWeight.w400);
-    expect(topBackLabel.text.style!.fontFamily, '.SF Pro Text');
+    expect(topBackLabel.text.style!.fontFamily, 'CupertinoSystemText');
     expect(topBackLabel.text.style!.letterSpacing, -0.41);
 
     checkOpacity(tester, flying(tester, find.text('Page 1')).last, 0.5292819738388062);
@@ -372,7 +372,7 @@ void main() {
       expect(
         tester.getTopLeft(flying(tester, find.text('Page 1')).first),
         const Offset(
-          bool.hasEnvironment('SKPARAGRAPH_REMOVE_ROUNDING_HACK') ? 342.547737105096302912 : 342.33420100808144,
+          342.547737105096302912,
           13.5,
         ),
       );
@@ -385,7 +385,7 @@ void main() {
       expect(
         tester.getTopLeft(flying(tester, find.text('Page 1')).last),
         const Offset(
-          bool.hasEnvironment('SKPARAGRAPH_REMOVE_ROUNDING_HACK') ? 342.547737105096302912 : 342.33420100808144,
+          342.547737105096302912,
           13.5,
         ),
       );
@@ -423,7 +423,7 @@ void main() {
       expect(
         tester.getTopLeft(flying(tester, find.text('Page 1')).first),
         const Offset(
-          bool.hasEnvironment('SKPARAGRAPH_REMOVE_ROUNDING_HACK') ? 357.912261979376353338 : 357.66579899191856,
+          357.912261979376353338,
           13.5,
         ),
       );
@@ -436,7 +436,7 @@ void main() {
       expect(
         tester.getTopLeft(flying(tester, find.text('Page 1')).last),
         const Offset(
-          bool.hasEnvironment('SKPARAGRAPH_REMOVE_ROUNDING_HACK') ? 357.912261979376353338 : 357.66579899191856,
+          357.912261979376353338,
           13.5,
         ),
       );
@@ -665,6 +665,7 @@ void main() {
   testWidgets('Middle is shown if alwaysShowMiddle is false but the nav bar is collapsed', (WidgetTester tester) async {
     const Widget userMiddle = Placeholder();
     final ScrollController scrollController = ScrollController();
+    addTearDown(scrollController.dispose);
 
     await tester.pumpWidget(
       CupertinoApp(
@@ -737,14 +738,14 @@ void main() {
     // Come in from the right and fade in.
     checkOpacity(tester, backChevron, 0.0);
     expect(tester.getTopLeft(backChevron), const Offset(
-      bool.hasEnvironment('SKPARAGRAPH_REMOVE_ROUNDING_HACK') ? 87.2460581221158690823 : 88.04496401548386,
+      87.2460581221158690823,
       7.0,
     ));
 
     await tester.pump(const Duration(milliseconds: 200));
     checkOpacity(tester, backChevron, 0.09497911669313908);
     expect(tester.getTopLeft(backChevron), const Offset(
-      bool.hasEnvironment('SKPARAGRAPH_REMOVE_ROUNDING_HACK') ? 30.8718595298545324113 : 31.055883467197418,
+      30.8718595298545324113,
       7.0,
     ));
   });
@@ -785,7 +786,7 @@ void main() {
     expect(
       tester.getTopRight(backChevron),
       const Offset(
-        bool.hasEnvironment('SKPARAGRAPH_REMOVE_ROUNDING_HACK') ? 687.163941725296126606 : 685.9550359845161,
+        687.163941725296126606,
         7.0,
       ),
     );
@@ -795,7 +796,7 @@ void main() {
     expect(
       tester.getTopRight(backChevron),
       const Offset(
-        bool.hasEnvironment('SKPARAGRAPH_REMOVE_ROUNDING_HACK') ? 743.538140317557690651 : 742.9441165328026,
+        743.538140317557690651,
         7.0,
       ),
     );
@@ -900,7 +901,7 @@ void main() {
     expect(
       tester.getTopLeft(flying(tester, find.text('custom'))),
       const Offset(
-        bool.hasEnvironment('SKPARAGRAPH_REMOVE_ROUNDING_HACK') ? 684.459999084472656250 : 684.0,
+        684.459999084472656250,
         13.5,
       ),
     );
@@ -910,7 +911,7 @@ void main() {
     expect(
       tester.getTopLeft(flying(tester, find.text('custom'))),
       const Offset(
-        bool.hasEnvironment('SKPARAGRAPH_REMOVE_ROUNDING_HACK') ? 684.459999084472656250 : 684.0,
+        684.459999084472656250,
         13.5,
       ),
     );
@@ -942,7 +943,7 @@ void main() {
     expect(
       tester.getTopLeft(flying(tester, find.text('Page 1'))),
       const Offset(
-        bool.hasEnvironment('SKPARAGRAPH_REMOVE_ROUNDING_HACK') ? 41.3003370761871337891 : 41.71033692359924,
+        41.3003370761871337891,
         13.5,
       ),
     );
@@ -952,7 +953,7 @@ void main() {
     expect(
       tester.getTopLeft(flying(tester, find.text('Page 1'))),
       const Offset(
-        bool.hasEnvironment('SKPARAGRAPH_REMOVE_ROUNDING_HACK') ? -258.642192125320434570 : -258.2321922779083,
+        -258.642192125320434570,
         13.5,
       ),
     );
@@ -985,7 +986,7 @@ void main() {
     expect(
       tester.getTopRight(flying(tester, find.text('Page 1'))),
       const Offset(
-        bool.hasEnvironment('SKPARAGRAPH_REMOVE_ROUNDING_HACK') ? 758.699662923812866211 : 758.2896630764008,
+        758.699662923812866211,
         13.5,
       ),
     );
@@ -996,7 +997,7 @@ void main() {
       tester.getTopRight(flying(tester, find.text('Page 1'))),
       // >1000. It's now off the screen.
       const Offset(
-        bool.hasEnvironment('SKPARAGRAPH_REMOVE_ROUNDING_HACK') ? 1058.64219212532043457 : 1058.2321922779083,
+        1058.64219212532043457,
         13.5,
       ),
     );
@@ -1022,14 +1023,14 @@ void main() {
     expect(
       tester.getTopLeft(flying(tester, find.text('Page 1')).first),
       const Offset(
-        bool.hasEnvironment('SKPARAGRAPH_REMOVE_ROUNDING_HACK') ? 16.9155227761479522997 : 16.926069676876068,
+        16.9155227761479522997,
         52.73951627314091,
       ),
     );
     expect(
       tester.getTopLeft(flying(tester, find.text('Page 1')).last),
       const Offset(
-        bool.hasEnvironment('SKPARAGRAPH_REMOVE_ROUNDING_HACK') ? 16.9155227761479522997 : 16.926069676876068,
+        16.9155227761479522997,
         52.73951627314091,
       ),
     );
@@ -1041,14 +1042,14 @@ void main() {
     expect(
       tester.getTopLeft(flying(tester, find.text('Page 1')).first),
       const Offset(
-        bool.hasEnvironment('SKPARAGRAPH_REMOVE_ROUNDING_HACK') ? 43.6029094262710827934 : 43.92089730501175,
+        43.6029094262710827934,
         22.49655644595623,
       ),
     );
     expect(
       tester.getTopLeft(flying(tester, find.text('Page 1')).last),
       const Offset(
-        bool.hasEnvironment('SKPARAGRAPH_REMOVE_ROUNDING_HACK') ? 43.6029094262710827934 : 43.92089730501175,
+        43.6029094262710827934,
         22.49655644595623,
       ),
     );
@@ -1073,14 +1074,14 @@ void main() {
     expect(
       tester.getTopLeft(flying(tester, find.text('A title too long to fit'))),
       const Offset(
-        bool.hasEnvironment('SKPARAGRAPH_REMOVE_ROUNDING_HACK') ? 16.9155227761479522997 : 16.926069676876068,
+        16.9155227761479522997,
         52.73951627314091,
       ),
     );
     expect(
       tester.getTopLeft(flying(tester, find.text('Back'))),
       const Offset(
-        bool.hasEnvironment('SKPARAGRAPH_REMOVE_ROUNDING_HACK') ? 16.9155227761479522997 : 16.926069676876068,
+        16.9155227761479522997,
         52.73951627314091,
       ),
     );
@@ -1091,14 +1092,14 @@ void main() {
     expect(
       tester.getTopLeft(flying(tester, find.text('A title too long to fit'))),
       const Offset(
-        bool.hasEnvironment('SKPARAGRAPH_REMOVE_ROUNDING_HACK') ? 43.6029094262710827934 : 43.92089730501175,
+        43.6029094262710827934,
         22.49655644595623,
       ),
     );
     expect(
       tester.getTopLeft(flying(tester, find.text('Back'))),
       const Offset(
-        bool.hasEnvironment('SKPARAGRAPH_REMOVE_ROUNDING_HACK') ? 43.6029094262710827934 : 43.92089730501175,
+        43.6029094262710827934,
         22.49655644595623,
       ),
     );
@@ -1119,28 +1120,28 @@ void main() {
         tester.renderObject(flying(tester, find.text('Page 1')).first);
     expect(bottomLargeTitle.text.style!.color, const Color(0xff000306));
     expect(bottomLargeTitle.text.style!.fontWeight, FontWeight.w700);
-    expect(bottomLargeTitle.text.style!.fontFamily, '.SF Pro Display');
-    expect(bottomLargeTitle.text.style!.letterSpacing, moreOrLessEquals(0.38890619069337845));
+    expect(bottomLargeTitle.text.style!.fontFamily, 'CupertinoSystemDisplay');
+    expect(bottomLargeTitle.text.style!.letterSpacing, moreOrLessEquals(0.35967791542410854));
 
     // The top back label is styled exactly the same way.
     final RenderParagraph topBackLabel =
         tester.renderObject(flying(tester, find.text('Page 1')).last);
     expect(topBackLabel.text.style!.color, const Color(0xff000306));
     expect(topBackLabel.text.style!.fontWeight, FontWeight.w700);
-    expect(topBackLabel.text.style!.fontFamily, '.SF Pro Display');
-    expect(topBackLabel.text.style!.letterSpacing, moreOrLessEquals(0.38890619069337845));
+    expect(topBackLabel.text.style!.fontFamily, 'CupertinoSystemDisplay');
+    expect(topBackLabel.text.style!.letterSpacing, moreOrLessEquals(0.35967791542410854));
 
     // Move animation further a bit.
     await tester.pump(const Duration(milliseconds: 200));
     expect(bottomLargeTitle.text.style!.color, const Color(0xff005ec5));
     expect(bottomLargeTitle.text.style!.fontWeight, FontWeight.w500);
-    expect(bottomLargeTitle.text.style!.fontFamily, '.SF Pro Text');
-    expect(bottomLargeTitle.text.style!.letterSpacing, moreOrLessEquals(-0.2259759941697121));
+    expect(bottomLargeTitle.text.style!.fontFamily, 'CupertinoSystemText');
+    expect(bottomLargeTitle.text.style!.letterSpacing, moreOrLessEquals(-0.23270857974886894));
 
     expect(topBackLabel.text.style!.color, const Color(0xff005ec5));
     expect(topBackLabel.text.style!.fontWeight, FontWeight.w500);
-    expect(topBackLabel.text.style!.fontFamily, '.SF Pro Text');
-    expect(topBackLabel.text.style!.letterSpacing, moreOrLessEquals(-0.2259759941697121));
+    expect(topBackLabel.text.style!.fontFamily, 'CupertinoSystemText');
+    expect(topBackLabel.text.style!.letterSpacing, moreOrLessEquals(-0.23270857974886894));
   });
 
   testWidgets('Top middle fades in and slides in from the right', (WidgetTester tester) async {
@@ -1157,7 +1158,7 @@ void main() {
     expect(
       tester.getTopLeft(flying(tester, find.text('Page 2'))),
       const Offset(
-        bool.hasEnvironment('SKPARAGRAPH_REMOVE_ROUNDING_HACK') ? 739.940336465835571289 : 739.7103369235992,
+        739.940336465835571289,
         13.5,
       ),
     );
@@ -1168,7 +1169,7 @@ void main() {
     expect(
       tester.getTopLeft(flying(tester, find.text('Page 2'))),
       const Offset(
-        bool.hasEnvironment('SKPARAGRAPH_REMOVE_ROUNDING_HACK') ? 504.880443334579467773 : 504.65044379234314,
+        504.880443334579467773,
         13.5,
       ),
     );
@@ -1213,7 +1214,7 @@ void main() {
     expect(
       tester.getTopRight(flying(tester, find.text('Page 2'))),
       const Offset(
-        bool.hasEnvironment('SKPARAGRAPH_REMOVE_ROUNDING_HACK') ? 60.0596635341644287109 : 60.28966307640076,
+        60.0596635341644287109,
         13.5,
       ),
     );
@@ -1224,7 +1225,7 @@ void main() {
     expect(
       tester.getTopRight(flying(tester, find.text('Page 2'))),
       const Offset(
-        bool.hasEnvironment('SKPARAGRAPH_REMOVE_ROUNDING_HACK') ? 295.119556665420532227 : 295.34955620765686,
+        295.119556665420532227,
         13.5,
       ),
     );
@@ -1351,7 +1352,7 @@ void main() {
     expect(
       tester.getTopLeft(flying(tester, find.text('Page 2'))),
       const Offset(
-        bool.hasEnvironment('SKPARAGRAPH_REMOVE_ROUNDING_HACK') ? 353.810205429792404175 : 353.5802058875561,
+        353.810205429792404175,
         13.5,
       ),
     );
@@ -1369,7 +1370,7 @@ void main() {
     expect(
       tester.getTopLeft(flying(tester, find.text('Page 2'))),
       const Offset(
-        bool.hasEnvironment('SKPARAGRAPH_REMOVE_ROUNDING_HACK') ? 655.435583114624023438 : 655.2055835723877,
+        655.435583114624023438,
         13.5,
       ),
     );
@@ -1377,7 +1378,7 @@ void main() {
     expect(
       tester.getTopLeft(flying(tester, find.text('Page 2'))),
       const Offset(
-        bool.hasEnvironment('SKPARAGRAPH_REMOVE_ROUNDING_HACK') ? 749.863556146621704102 : 749.6335566043854,
+        749.863556146621704102,
         13.5,
       ),
     );
@@ -1410,7 +1411,7 @@ void main() {
     await tester.pump(const Duration(milliseconds: 600));
 
     // Start the gesture at the edge of the screen.
-    final TestGesture gesture =  await tester.startGesture(const Offset(5.0, 200.0));
+    final TestGesture gesture = await tester.startGesture(const Offset(5.0, 200.0));
     // Trigger the swipe.
     await gesture.moveBy(const Offset(100.0, 0.0));
 
@@ -1422,7 +1423,7 @@ void main() {
     expect(
       tester.getTopLeft(flying(tester, find.text('Page 2'))),
       const Offset(
-        bool.hasEnvironment('SKPARAGRAPH_REMOVE_ROUNDING_HACK') ? 353.810205429792404175 : 353.5802058875561,
+        353.810205429792404175,
         13.5,
       ),
     );
@@ -1434,7 +1435,7 @@ void main() {
     expect(
       tester.getTopLeft(flying(tester, find.text('Page 2'))),
       const Offset(
-        bool.hasEnvironment('SKPARAGRAPH_REMOVE_ROUNDING_HACK') ? 353.810205429792404175 : 353.5802058875561,
+        353.810205429792404175,
         13.5,
       ),
     );
@@ -1442,7 +1443,7 @@ void main() {
     expect(
       tester.getTopLeft(flying(tester, find.text('Page 2'))),
       const Offset(
-        bool.hasEnvironment('SKPARAGRAPH_REMOVE_ROUNDING_HACK') ? 350.231143206357955933 : 350.0011436641216,
+        350.231143206357955933,
         13.5,
       ),
     );
