@@ -35,14 +35,20 @@ class TestGestureFlutterBinding extends BindingBase with GestureBinding, Schedul
     return _instance!;
   }
 
-  HandleEventCallback? callback;
+  HandleEventCallback? onHandlePointerEvent;
+
+  @override
+  void handlePointerEvent(PointerEvent event) {
+    onHandlePointerEvent?.call(event);
+    super.handlePointerEvent(event);
+  }
+
+  HandleEventCallback? onHandleEvent;
 
   @override
   void handleEvent(PointerEvent event, HitTestEntry entry) {
     super.handleEvent(event, entry);
-    if (callback != null) {
-      callback?.call(event);
-    }
+    onHandleEvent?.call(event);
   }
 }
 
@@ -58,7 +64,7 @@ void main() {
     );
 
     final List<PointerEvent> events = <PointerEvent>[];
-    binding.callback = events.add;
+    TestGestureFlutterBinding.instance.onHandleEvent = events.add;
 
     GestureBinding.instance.platformDispatcher.onPointerDataPacket?.call(packet);
     expect(events.length, 2);
@@ -76,7 +82,7 @@ void main() {
     );
 
     final List<PointerEvent> events = <PointerEvent>[];
-    binding.callback = events.add;
+    binding.onHandleEvent = events.add;
 
     GestureBinding.instance.platformDispatcher.onPointerDataPacket?.call(packet);
     expect(events.length, 3);
@@ -101,7 +107,7 @@ void main() {
     GestureBinding.instance.pointerRouter.addGlobalRoute(pointerRouterEvents.add);
 
     final List<PointerEvent> events = <PointerEvent>[];
-    binding.callback = events.add;
+    binding.onHandleEvent = events.add;
 
     GestureBinding.instance.platformDispatcher.onPointerDataPacket?.call(packet);
     expect(events.length, 3);
@@ -126,7 +132,7 @@ void main() {
     );
 
     final List<PointerEvent> events = <PointerEvent>[];
-    binding.callback = events.add;
+    binding.onHandleEvent = events.add;
 
     GestureBinding.instance.platformDispatcher.onPointerDataPacket?.call(packet);
     expect(events.length, 2);
@@ -143,7 +149,7 @@ void main() {
     );
 
     final List<PointerEvent> events = <PointerEvent>[];
-    binding.callback = (PointerEvent event) {
+    binding.onHandleEvent = (PointerEvent event) {
       events.add(event);
       if (event is PointerDownEvent) {
         binding.cancelPointer(event.pointer);
@@ -156,6 +162,8 @@ void main() {
     expect(events[1], isA<PointerCancelEvent>());
   });
 
+  const double devicePixelRatio = 2.5;
+
   test('Can expand add and hover pointers', () {
     const ui.PointerDataPacket packet = ui.PointerDataPacket(
       data: <ui.PointerData>[
@@ -167,7 +175,7 @@ void main() {
       ],
     );
 
-    final List<PointerEvent> events = PointerEventConverter.expand(packet.data, GestureBinding.instance.window.devicePixelRatio).toList();
+    final List<PointerEvent> events = PointerEventConverter.expand(packet.data, (int viewId) => devicePixelRatio).toList();
 
     expect(events.length, 5);
     expect(events[0], isA<PointerAddedEvent>());
@@ -177,6 +185,47 @@ void main() {
     expect(events[4], isA<PointerHoverEvent>());
   });
 
+  test('Can handle malformed scrolling event.', () {
+    ui.PointerDataPacket packet = const ui.PointerDataPacket(
+      data: <ui.PointerData>[
+        ui.PointerData(change: ui.PointerChange.add, device: 24),
+      ],
+    );
+    List<PointerEvent> events = PointerEventConverter.expand(packet.data, (int viewId) => devicePixelRatio).toList();
+
+    expect(events.length, 1);
+    expect(events[0], isA<PointerAddedEvent>());
+
+    // Send packet contains malformed scroll events.
+    packet = const ui.PointerDataPacket(
+      data: <ui.PointerData>[
+        ui.PointerData(signalKind: ui.PointerSignalKind.scroll, device: 24, scrollDeltaX: double.infinity, scrollDeltaY: 10),
+        ui.PointerData(signalKind: ui.PointerSignalKind.scroll, device: 24, scrollDeltaX: double.nan, scrollDeltaY: 10),
+        ui.PointerData(signalKind: ui.PointerSignalKind.scroll, device: 24, scrollDeltaX: double.negativeInfinity, scrollDeltaY: 10),
+        ui.PointerData(signalKind: ui.PointerSignalKind.scroll, device: 24, scrollDeltaY: double.infinity, scrollDeltaX: 10),
+        ui.PointerData(signalKind: ui.PointerSignalKind.scroll, device: 24, scrollDeltaY: double.nan, scrollDeltaX: 10),
+        ui.PointerData(signalKind: ui.PointerSignalKind.scroll, device: 24, scrollDeltaY: double.negativeInfinity, scrollDeltaX: 10),
+      ],
+    );
+    events = PointerEventConverter.expand(packet.data, (int viewId) => devicePixelRatio).toList();
+    expect(events.length, 0);
+
+    // Send packet with a valid scroll event.
+    packet = const ui.PointerDataPacket(
+      data: <ui.PointerData>[
+        ui.PointerData(signalKind: ui.PointerSignalKind.scroll, device: 24, scrollDeltaX: 10, scrollDeltaY: 10),
+      ],
+    );
+    // Make sure PointerEventConverter can expand when device pixel ratio is valid.
+    events = PointerEventConverter.expand(packet.data, (int viewId) => devicePixelRatio).toList();
+    expect(events.length, 1);
+    expect(events[0], isA<PointerScrollEvent>());
+
+    // Make sure PointerEventConverter returns none when device pixel ratio is invalid.
+    events = PointerEventConverter.expand(packet.data, (int viewId) => 0).toList();
+    expect(events.length, 0);
+  });
+
   test('Can expand pointer scroll events', () {
     const ui.PointerDataPacket packet = ui.PointerDataPacket(
         data: <ui.PointerData>[
@@ -185,7 +234,7 @@ void main() {
         ],
     );
 
-    final List<PointerEvent> events = PointerEventConverter.expand(packet.data, GestureBinding.instance.window.devicePixelRatio).toList();
+    final List<PointerEvent> events = PointerEventConverter.expand(packet.data, (int viewId) => devicePixelRatio).toList();
 
     expect(events.length, 2);
     expect(events[0], isA<PointerAddedEvent>());
@@ -193,7 +242,7 @@ void main() {
   });
 
   test('Should synthesize kPrimaryButton for touch when no button is set', () {
-    final Offset location = const Offset(10.0, 10.0) * GestureBinding.instance.window.devicePixelRatio;
+    final Offset location = const Offset(10.0, 10.0) * devicePixelRatio;
     final ui.PointerDataPacket packet = ui.PointerDataPacket(
       data: <ui.PointerData>[
         ui.PointerData(change: ui.PointerChange.add, physicalX: location.dx, physicalY: location.dy),
@@ -204,7 +253,7 @@ void main() {
       ],
     );
 
-    final List<PointerEvent> events = PointerEventConverter.expand(packet.data, GestureBinding.instance.window.devicePixelRatio).toList();
+    final List<PointerEvent> events = PointerEventConverter.expand(packet.data, (int viewId) => devicePixelRatio).toList();
 
     expect(events.length, 5);
     expect(events[0], isA<PointerAddedEvent>());
@@ -220,7 +269,7 @@ void main() {
   });
 
   test('Should not synthesize kPrimaryButton for touch when a button is set', () {
-    final Offset location = const Offset(10.0, 10.0) * GestureBinding.instance.window.devicePixelRatio;
+    final Offset location = const Offset(10.0, 10.0) * devicePixelRatio;
     final ui.PointerDataPacket packet = ui.PointerDataPacket(
       data: <ui.PointerData>[
         ui.PointerData(change: ui.PointerChange.add, physicalX: location.dx, physicalY: location.dy),
@@ -231,7 +280,7 @@ void main() {
       ],
     );
 
-    final List<PointerEvent> events = PointerEventConverter.expand(packet.data, GestureBinding.instance.window.devicePixelRatio).toList();
+    final List<PointerEvent> events = PointerEventConverter.expand(packet.data, (int viewId) => devicePixelRatio).toList();
 
     expect(events.length, 5);
     expect(events[0], isA<PointerAddedEvent>());
@@ -247,7 +296,7 @@ void main() {
   });
 
   test('Should synthesize kPrimaryButton for stylus when no button is set', () {
-    final Offset location = const Offset(10.0, 10.0) * GestureBinding.instance.window.devicePixelRatio;
+    final Offset location = const Offset(10.0, 10.0) * devicePixelRatio;
     for (final PointerDeviceKind kind in <PointerDeviceKind>[
       PointerDeviceKind.stylus,
       PointerDeviceKind.invertedStylus,
@@ -263,7 +312,7 @@ void main() {
         ],
       );
 
-      final List<PointerEvent> events = PointerEventConverter.expand(packet.data, GestureBinding.instance.window.devicePixelRatio).toList();
+      final List<PointerEvent> events = PointerEventConverter.expand(packet.data, (int viewId) => devicePixelRatio).toList();
 
       expect(events.length, 5);
       expect(events[0], isA<PointerAddedEvent>());
@@ -280,7 +329,7 @@ void main() {
   });
 
   test('Should synthesize kPrimaryButton for unknown devices when no button is set', () {
-    final Offset location = const Offset(10.0, 10.0) * GestureBinding.instance.window.devicePixelRatio;
+    final Offset location = const Offset(10.0, 10.0) * devicePixelRatio;
     const PointerDeviceKind kind = PointerDeviceKind.unknown;
     final ui.PointerDataPacket packet = ui.PointerDataPacket(
       data: <ui.PointerData>[
@@ -292,7 +341,7 @@ void main() {
       ],
     );
 
-    final List<PointerEvent> events = PointerEventConverter.expand(packet.data, GestureBinding.instance.window.devicePixelRatio).toList();
+    final List<PointerEvent> events = PointerEventConverter.expand(packet.data, (int viewId) => devicePixelRatio).toList();
 
     expect(events.length, 5);
     expect(events[0], isA<PointerAddedEvent>());
@@ -308,7 +357,7 @@ void main() {
   });
 
   test('Should not synthesize kPrimaryButton for mouse', () {
-    final Offset location = const Offset(10.0, 10.0) * GestureBinding.instance.window.devicePixelRatio;
+    final Offset location = const Offset(10.0, 10.0) * devicePixelRatio;
     for (final PointerDeviceKind kind in <PointerDeviceKind>[
       PointerDeviceKind.mouse,
     ]) {
@@ -322,7 +371,7 @@ void main() {
         ],
       );
 
-      final List<PointerEvent> events = PointerEventConverter.expand(packet.data, GestureBinding.instance.window.devicePixelRatio).toList();
+      final List<PointerEvent> events = PointerEventConverter.expand(packet.data, (int viewId) => devicePixelRatio).toList();
 
       expect(events.length, 5);
       expect(events[0], isA<PointerAddedEvent>());
@@ -348,12 +397,65 @@ void main() {
     );
 
     final List<PointerEvent> events = <PointerEvent>[];
-    binding.callback = events.add;
+    binding.onHandleEvent = events.add;
 
-    ui.window.onPointerDataPacket?.call(packet);
+    binding.platformDispatcher.onPointerDataPacket?.call(packet);
     expect(events.length, 3);
     expect(events[0], isA<PointerPanZoomStartEvent>());
     expect(events[1], isA<PointerPanZoomUpdateEvent>());
     expect(events[2], isA<PointerPanZoomEndEvent>());
   });
+
+  test('Error handling', () {
+    const ui.PointerDataPacket packet = ui.PointerDataPacket(
+      data: <ui.PointerData>[
+        ui.PointerData(change: ui.PointerChange.down),
+        ui.PointerData(change: ui.PointerChange.up),
+      ],
+    );
+
+    final List<String> events = <String>[];
+    binding.onHandlePointerEvent = (PointerEvent event) { throw Exception('zipzapzooey $event'); };
+    FlutterError.onError = (FlutterErrorDetails details) { events.add(details.toString()); };
+    try {
+      GestureBinding.instance.platformDispatcher.onPointerDataPacket?.call(packet);
+      expect(events.length, 1);
+      expect(events[0], contains('while handling a pointer data\npacket')); // The default stringifying behavior uses 65 character wrapWidth.
+      expect(events[0], contains('zipzapzooey'));
+      expect(events[0], contains('PointerDownEvent'));
+      expect(events[0], isNot(contains('PointerUpEvent'))); // Failure happens on the first message, remaining messages aren't processed.
+    } finally {
+      binding.onHandlePointerEvent = null;
+      FlutterError.onError = FlutterError.presentError;
+    }
+  });
+
+  test('PointerEventConverter processes view IDs', () {
+    const int startID = 987654;
+    const List<ui.PointerData> data = <ui.PointerData>[
+      ui.PointerData(viewId: startID + 0, change: ui.PointerChange.cancel), // ignore: avoid_redundant_argument_values
+      ui.PointerData(viewId: startID + 1, change: ui.PointerChange.add),
+      ui.PointerData(viewId: startID + 2, change: ui.PointerChange.remove),
+      ui.PointerData(viewId: startID + 3, change: ui.PointerChange.hover),
+      ui.PointerData(viewId: startID + 4, change: ui.PointerChange.down),
+      ui.PointerData(viewId: startID + 5, change: ui.PointerChange.move),
+      ui.PointerData(viewId: startID + 6, change: ui.PointerChange.up),
+      ui.PointerData(viewId: startID + 7, change: ui.PointerChange.panZoomStart),
+      ui.PointerData(viewId: startID + 8, change: ui.PointerChange.panZoomUpdate),
+      ui.PointerData(viewId: startID + 9, change: ui.PointerChange.panZoomEnd),
+    ];
+
+    final List<int> viewIds = <int>[];
+    double devicePixelRatioGetter(int viewId) {
+      viewIds.add(viewId);
+      return viewId / 10.0;
+    }
+
+    final List<PointerEvent> events = PointerEventConverter.expand(data, devicePixelRatioGetter).toList();
+
+    final List<int> expectedViewIds = List<int>.generate(10, (int index) => startID + index);
+    expect(viewIds, expectedViewIds);
+    expect(events, hasLength(10));
+    expect(events.map((PointerEvent event) => event.viewId), expectedViewIds);
+  });
 }
