@@ -79,6 +79,7 @@ void main() {
     updatedDelta = null;
     expect(didEndScale, isFalse);
     expect(didTap, isFalse);
+    expect(scale.pointerCount, 1);
 
     // Two-finger scaling
     final TestPointer pointer2 = TestPointer(2);
@@ -87,6 +88,7 @@ void main() {
     tap.addPointer(down2);
     tester.closeArena(2);
     tester.route(down2);
+    expect(scale.pointerCount, 2);
 
     expect(didEndScale, isTrue);
     didEndScale = false;
@@ -1354,4 +1356,296 @@ void main() {
 
     scale.dispose();
   });
+
+  testGesture('ScaleStartDetails and ScaleUpdateDetails callbacks should contain their event.timestamp', (GestureTester tester) {
+    final ScaleGestureRecognizer scale = ScaleGestureRecognizer();
+    final TapGestureRecognizer tap = TapGestureRecognizer();
+
+    bool didStartScale = false;
+    Offset? updatedFocalPoint;
+    Duration? initialSourceTimestamp;
+    scale.onStart = (ScaleStartDetails details) {
+      didStartScale = true;
+      updatedFocalPoint = details.focalPoint;
+      initialSourceTimestamp = details.sourceTimeStamp;
+    };
+
+    double? updatedScale;
+    double? updatedHorizontalScale;
+    double? updatedVerticalScale;
+    Offset? updatedDelta;
+    Duration? updatedSourceTimestamp;
+    scale.onUpdate = (ScaleUpdateDetails details) {
+      updatedScale = details.scale;
+      updatedHorizontalScale = details.horizontalScale;
+      updatedVerticalScale = details.verticalScale;
+      updatedFocalPoint = details.focalPoint;
+      updatedDelta = details.focalPointDelta;
+      updatedSourceTimestamp = details.sourceTimeStamp;
+    };
+
+    bool didEndScale = false;
+    scale.onEnd = (ScaleEndDetails details) {
+      didEndScale = true;
+    };
+
+    bool didTap = false;
+    tap.onTap = () {
+      didTap = true;
+    };
+
+    final TestPointer pointer1 = TestPointer();
+
+    final PointerDownEvent down = pointer1.down(Offset.zero, timeStamp: const Duration(milliseconds: 10));
+    scale.addPointer(down);
+    tap.addPointer(down);
+
+    tester.closeArena(1);
+    expect(didStartScale, isFalse);
+    expect(updatedScale, isNull);
+    expect(updatedFocalPoint, isNull);
+    expect(updatedDelta, isNull);
+    expect(updatedSourceTimestamp, isNull);
+    expect(didEndScale, isFalse);
+    expect(didTap, isFalse);
+    expect(initialSourceTimestamp, isNull);
+
+    // One-finger panning.
+    tester.route(down);
+    expect(didStartScale, isFalse);
+    expect(updatedScale, isNull);
+    expect(updatedFocalPoint, isNull);
+    expect(updatedDelta, isNull);
+    expect(didEndScale, isFalse);
+    expect(didTap, isFalse);
+    expect(initialSourceTimestamp, isNull);
+
+    tester.route(pointer1.move(const Offset(20.0, 30.0), timeStamp: const Duration(milliseconds: 20)));
+    expect(didStartScale, isTrue);
+    didStartScale = false;
+    expect(updatedFocalPoint, const Offset(20.0, 30.0));
+    updatedFocalPoint = null;
+    expect(updatedScale, 1.0);
+    updatedScale = null;
+    expect(updatedDelta, const Offset(20.0, 30.0));
+    updatedDelta = null;
+    expect(updatedSourceTimestamp, const Duration(milliseconds: 20));
+    updatedSourceTimestamp = null;
+    expect(initialSourceTimestamp, const Duration(milliseconds: 10));
+    initialSourceTimestamp = null;
+    expect(didEndScale, isFalse);
+    expect(didTap, isFalse);
+    expect(scale.pointerCount, 1);
+
+    // Two-finger scaling.
+    final TestPointer pointer2 = TestPointer(2);
+    final PointerDownEvent down2 = pointer2.down(const Offset(10.0, 20.0, ), timeStamp: const Duration(milliseconds: 30));
+    scale.addPointer(down2);
+    tap.addPointer(down2);
+    tester.closeArena(2);
+    tester.route(down2);
+    expect(scale.pointerCount, 2);
+
+    expect(didEndScale, isTrue);
+    didEndScale = false;
+    expect(updatedScale, isNull);
+    expect(updatedFocalPoint, isNull);
+    expect(updatedDelta, isNull);
+    expect(updatedSourceTimestamp, isNull);
+    expect(didStartScale, isFalse);
+    expect(initialSourceTimestamp, isNull);
+
+    // Zoom in.
+    tester.route(pointer2.move(const Offset(0.0, 10.0), timeStamp: const Duration(milliseconds: 40)));
+    expect(didStartScale, isTrue);
+    didStartScale = false;
+    expect(updatedFocalPoint, const Offset(10.0, 20.0));
+    updatedFocalPoint = null;
+    expect(updatedScale, 2.0);
+    expect(updatedHorizontalScale, 2.0);
+    expect(updatedVerticalScale, 2.0);
+    expect(updatedDelta, const Offset(-5.0, -5.0));
+    expect(updatedSourceTimestamp, const Duration(milliseconds: 40));
+    expect(initialSourceTimestamp, const Duration(milliseconds: 40));
+    updatedScale = null;
+    updatedHorizontalScale = null;
+    updatedVerticalScale = null;
+    updatedDelta = null;
+    updatedSourceTimestamp = null;
+    initialSourceTimestamp = null;
+    expect(didEndScale, isFalse);
+    expect(didTap, isFalse);
+
+    // Zoom out.
+    tester.route(pointer2.move(const Offset(15.0, 25.0), timeStamp: const Duration(milliseconds: 50)));
+    expect(updatedFocalPoint, const Offset(17.5, 27.5));
+    expect(updatedScale, 0.5);
+    expect(updatedHorizontalScale, 0.5);
+    expect(updatedVerticalScale, 0.5);
+    expect(updatedDelta, const Offset(7.5, 7.5));
+    expect(updatedSourceTimestamp, const Duration(milliseconds: 50));
+    expect(didTap, isFalse);
+    expect(initialSourceTimestamp, isNull);
+
+
+    // Horizontal scaling.
+    tester.route(pointer2.move(const Offset(0.0, 20.0), timeStamp: const Duration(milliseconds: 60)));
+    expect(updatedHorizontalScale, 2.0);
+    expect(updatedVerticalScale, 1.0);
+    expect(updatedSourceTimestamp, const Duration(milliseconds: 60));
+    expect(initialSourceTimestamp, isNull);
+
+    // Vertical scaling.
+    tester.route(pointer2.move(const Offset(10.0, 10.0), timeStamp: const Duration(milliseconds: 70)));
+    expect(updatedHorizontalScale, 1.0);
+    expect(updatedVerticalScale, 2.0);
+    expect(updatedDelta, const Offset(5.0, -5.0));
+    expect(updatedSourceTimestamp, const Duration(milliseconds: 70));
+    expect(initialSourceTimestamp, isNull);
+    tester.route(pointer2.move(const Offset(15.0, 25.0)));
+    updatedFocalPoint = null;
+    updatedScale = null;
+    updatedDelta = null;
+    updatedSourceTimestamp = null;
+
+    // Three-finger scaling.
+    final TestPointer pointer3 = TestPointer(3);
+    final PointerDownEvent down3 = pointer3.down(const Offset(25.0, 35.0), timeStamp: const Duration(milliseconds: 80));
+    scale.addPointer(down3);
+    tap.addPointer(down3);
+    tester.closeArena(3);
+    tester.route(down3);
+
+    expect(didEndScale, isTrue);
+    didEndScale = false;
+    expect(updatedScale, isNull);
+    expect(updatedFocalPoint, isNull);
+    expect(updatedDelta, isNull);
+    expect(didStartScale, isFalse);
+    expect(initialSourceTimestamp, isNull);
+
+
+    // Zoom in.
+    tester.route(pointer3.move(const Offset(55.0, 65.0), timeStamp: const Duration(milliseconds: 90)));
+    expect(didStartScale, isTrue);
+    didStartScale = false;
+    expect(updatedFocalPoint, const Offset(30.0, 40.0));
+    updatedFocalPoint = null;
+    expect(updatedScale, 5.0);
+    updatedScale = null;
+    expect(updatedDelta, const Offset(10.0, 10.0));
+    updatedDelta = null;
+    expect(updatedSourceTimestamp, const Duration(milliseconds: 90));
+    updatedSourceTimestamp = null;
+    expect(initialSourceTimestamp, const Duration(milliseconds: 90));
+    initialSourceTimestamp = null;
+    expect(didEndScale, isFalse);
+    expect(didTap, isFalse);
+
+    // Return to original positions but with different fingers.
+    tester.route(pointer1.move(const Offset(25.0, 35.0), timeStamp: const Duration(milliseconds: 100)));
+    tester.route(pointer2.move(const Offset(20.0, 30.0), timeStamp: const Duration(milliseconds: 110)));
+    tester.route(pointer3.move(const Offset(15.0, 25.0), timeStamp: const Duration(milliseconds: 120)));
+    expect(didStartScale, isFalse);
+    expect(updatedFocalPoint, const Offset(20.0, 30.0));
+    updatedFocalPoint = null;
+    expect(updatedScale, 1.0);
+    updatedScale = null;
+    expect(updatedDelta!.dx, closeTo(-13.3, 0.1));
+    expect(updatedDelta!.dy, closeTo(-13.3, 0.1));
+    updatedDelta = null;
+    expect(didEndScale, isFalse);
+    expect(didTap, isFalse);
+    expect(updatedSourceTimestamp, const Duration(milliseconds: 120));
+    updatedSourceTimestamp = null;
+    expect(initialSourceTimestamp, isNull);
+    tester.route(pointer1.up());
+    expect(didStartScale, isFalse);
+    expect(updatedFocalPoint, isNull);
+    expect(updatedScale, isNull);
+    expect(updatedDelta, isNull);
+    expect(didEndScale, isTrue);
+    expect(updatedSourceTimestamp, isNull);
+    expect(initialSourceTimestamp, isNull);
+    didEndScale = false;
+    expect(didTap, isFalse);
+
+    // Continue scaling with two fingers.
+    tester.route(pointer3.move(const Offset(10.0, 20.0), timeStamp: const Duration(milliseconds: 130)));
+    expect(didStartScale, isTrue);
+    didStartScale = false;
+    expect(updatedFocalPoint, const Offset(15.0, 25.0));
+    updatedFocalPoint = null;
+    expect(updatedScale, 2.0);
+    updatedScale = null;
+    expect(updatedDelta, const Offset(-2.5, -2.5));
+    updatedDelta = null;
+    expect(updatedSourceTimestamp, const Duration(milliseconds: 130));
+    updatedSourceTimestamp = null;
+    expect(initialSourceTimestamp, const Duration(milliseconds: 130));
+    initialSourceTimestamp = null;
+
+    // Continue rotating with two fingers.
+    tester.route(pointer3.move(const Offset(30.0, 40.0), timeStamp: const Duration(milliseconds: 140)));
+    expect(updatedFocalPoint, const Offset(25.0, 35.0));
+    updatedFocalPoint = null;
+    expect(updatedScale, 2.0);
+    updatedScale = null;
+    expect(updatedDelta, const Offset(10.0, 10.0));
+    updatedDelta = null;
+    expect(updatedSourceTimestamp, const Duration(milliseconds: 140));
+    updatedSourceTimestamp = null;
+    expect(initialSourceTimestamp, isNull);
+    tester.route(pointer3.move(const Offset(10.0, 20.0), timeStamp: const Duration(milliseconds: 140)));
+    expect(updatedFocalPoint, const Offset(15.0, 25.0));
+    updatedFocalPoint = null;
+    expect(updatedScale, 2.0);
+    updatedScale = null;
+    expect(updatedDelta, const Offset(-10.0, -10.0));
+    updatedDelta = null;
+    expect(updatedSourceTimestamp, const Duration(milliseconds: 140));
+    updatedSourceTimestamp = null;
+    expect(initialSourceTimestamp, isNull);
+
+    tester.route(pointer2.up());
+    expect(didStartScale, isFalse);
+    expect(updatedFocalPoint, isNull);
+    expect(updatedScale, isNull);
+    expect(updatedDelta, isNull);
+    expect(updatedSourceTimestamp, isNull);
+    expect(initialSourceTimestamp, isNull);
+    expect(didEndScale, isTrue);
+    didEndScale = false;
+    expect(didTap, isFalse);
+
+    // Continue panning with one finger.
+    tester.route(pointer3.move(Offset.zero, timeStamp: const Duration(milliseconds: 150)));
+    expect(didStartScale, isTrue);
+    didStartScale = false;
+    expect(updatedFocalPoint, Offset.zero);
+    updatedFocalPoint = null;
+    expect(updatedScale, 1.0);
+    updatedScale = null;
+    expect(updatedDelta, const Offset(-10.0, -20.0));
+    updatedDelta = null;
+    expect(updatedSourceTimestamp, const Duration(milliseconds: 150));
+    updatedSourceTimestamp = null;
+    expect(initialSourceTimestamp, const Duration(milliseconds: 150));
+    initialSourceTimestamp = null;
+
+    // We are done.
+    tester.route(pointer3.up());
+    expect(didStartScale, isFalse);
+    expect(updatedFocalPoint, isNull);
+    expect(updatedScale, isNull);
+    expect(updatedDelta, isNull);
+    expect(didEndScale, isTrue);
+    expect(updatedSourceTimestamp, isNull);
+    expect(initialSourceTimestamp, isNull);
+    didEndScale = false;
+    expect(didTap, isFalse);
+
+    scale.dispose();
+    tap.dispose();
+  });
 }
