@@ -7,8 +7,6 @@ import 'package:flutter/material.dart';
 import 'package:flutter/rendering.dart';
 import 'package:flutter_test/flutter_test.dart';
 
-import '../rendering/mock_canvas.dart';
-
 RenderBox getMaterialBox(WidgetTester tester) {
   return tester.firstRenderObject<RenderBox>(
     find.descendant(
@@ -27,6 +25,16 @@ Material getMaterial(WidgetTester tester) {
   );
 }
 
+IconThemeData getIconData(WidgetTester tester) {
+  final IconTheme iconTheme = tester.firstWidget(
+    find.descendant(
+      of: find.byType(RawChip),
+      matching: find.byType(IconTheme),
+    ),
+  );
+  return iconTheme.data;
+}
+
 DefaultTextStyle getLabelStyle(WidgetTester tester) {
   return tester.widget(
     find.descendant(
@@ -42,8 +50,15 @@ void main() {
     expect(const ChipThemeData().hashCode, const ChipThemeData().copyWith().hashCode);
   });
 
+  test('ChipThemeData lerp special cases', () {
+    expect(ChipThemeData.lerp(null, null, 0), null);
+    const ChipThemeData data = ChipThemeData();
+    expect(identical(ChipThemeData.lerp(data, data, 0.5), data), true);
+  });
+
   test('ChipThemeData defaults', () {
     const ChipThemeData themeData = ChipThemeData();
+    expect(themeData.color, null);
     expect(themeData.backgroundColor, null);
     expect(themeData.deleteIconColor, null);
     expect(themeData.disabledColor, null);
@@ -63,6 +78,7 @@ void main() {
     expect(themeData.brightness, null);
     expect(themeData.elevation, null);
     expect(themeData.pressElevation, null);
+    expect(themeData.iconTheme, null);
   });
 
   testWidgets('Default ChipThemeData debugFillProperties', (WidgetTester tester) async {
@@ -80,16 +96,17 @@ void main() {
   testWidgets('ChipThemeData implements debugFillProperties', (WidgetTester tester) async {
     final DiagnosticPropertiesBuilder builder = DiagnosticPropertiesBuilder();
     const ChipThemeData(
-      backgroundColor: Color(0xfffffff0),
-      deleteIconColor: Color(0xfffffff1),
-      disabledColor: Color(0xfffffff2),
-      selectedColor: Color(0xfffffff3),
-      secondarySelectedColor: Color(0xfffffff4),
-      shadowColor: Color(0xfffffff5),
-      surfaceTintColor: Color(0xfffffff8),
-      selectedShadowColor: Color(0xfffffff6),
+      color: MaterialStatePropertyAll<Color>(Color(0xfffffff0)),
+      backgroundColor: Color(0xfffffff1),
+      deleteIconColor: Color(0xfffffff2),
+      disabledColor: Color(0xfffffff3),
+      selectedColor: Color(0xfffffff4),
+      secondarySelectedColor: Color(0xfffffff5),
+      shadowColor: Color(0xfffffff6),
+      surfaceTintColor: Color(0xfffffff7),
+      selectedShadowColor: Color(0xfffffff8),
       showCheckmark: true,
-      checkmarkColor: Color(0xfffffff7),
+      checkmarkColor: Color(0xfffffff9),
       labelPadding: EdgeInsets.all(1),
       padding: EdgeInsets.all(2),
       side: BorderSide(width: 10),
@@ -99,6 +116,7 @@ void main() {
       brightness: Brightness.dark,
       elevation: 5,
       pressElevation: 6,
+      iconTheme: IconThemeData(color: Color(0xffffff10)),
     ).debugFillProperties(builder);
 
     final List<String> description = builder.properties
@@ -106,30 +124,32 @@ void main() {
         .map((DiagnosticsNode node) => node.toString())
         .toList();
 
-    expect(description, <String>[
-      'backgroundColor: Color(0xfffffff0)',
-      'deleteIconColor: Color(0xfffffff1)',
-      'disabledColor: Color(0xfffffff2)',
-      'selectedColor: Color(0xfffffff3)',
-      'secondarySelectedColor: Color(0xfffffff4)',
-      'shadowColor: Color(0xfffffff5)',
-      'surfaceTintColor: Color(0xfffffff8)',
-      'selectedShadowColor: Color(0xfffffff6)',
+    expect(description, equalsIgnoringHashCodes(<String>[
+      'color: MaterialStatePropertyAll(Color(0xfffffff0))',
+      'backgroundColor: Color(0xfffffff1)',
+      'deleteIconColor: Color(0xfffffff2)',
+      'disabledColor: Color(0xfffffff3)',
+      'selectedColor: Color(0xfffffff4)',
+      'secondarySelectedColor: Color(0xfffffff5)',
+      'shadowColor: Color(0xfffffff6)',
+      'surfaceTintColor: Color(0xfffffff7)',
+      'selectedShadowColor: Color(0xfffffff8)',
       'showCheckmark: true',
-      'checkMarkColor: Color(0xfffffff7)',
+      'checkMarkColor: Color(0xfffffff9)',
       'labelPadding: EdgeInsets.all(1.0)',
       'padding: EdgeInsets.all(2.0)',
-      'side: BorderSide(Color(0xff000000), 10.0, BorderStyle.solid)',
-      'shape: RoundedRectangleBorder(BorderSide(Color(0xff000000), 0.0, BorderStyle.none), BorderRadius.zero)',
+      'side: BorderSide(width: 10.0)',
+      'shape: RoundedRectangleBorder(BorderSide(width: 0.0, style: none), BorderRadius.zero)',
       'labelStyle: TextStyle(inherit: true, size: 10.0)',
       'secondaryLabelStyle: TextStyle(inherit: true, size: 20.0)',
       'brightness: dark',
       'elevation: 5.0',
       'pressElevation: 6.0',
-    ]);
+      'iconTheme: IconThemeData#00000(color: Color(0xffffff10))'
+    ]));
   });
 
-  testWidgets('Chip uses ThemeData chip theme', (WidgetTester tester) async {
+  testWidgets('Material3 - Chip uses ThemeData chip theme', (WidgetTester tester) async {
     const ChipThemeData chipTheme = ChipThemeData(
       backgroundColor: Color(0xff112233),
       elevation: 4,
@@ -137,18 +157,19 @@ void main() {
       labelPadding: EdgeInsets.all(25),
       shape: RoundedRectangleBorder(),
       labelStyle: TextStyle(fontSize: 32),
+      iconTheme: IconThemeData(color: Color(0xff332211)),
     );
+    final ThemeData theme = ThemeData(chipTheme: chipTheme, useMaterial3: true);
 
     await tester.pumpWidget(
       MaterialApp(
-        theme: ThemeData.light().copyWith(
-          chipTheme: chipTheme,
-        ),
+        theme: ThemeData(chipTheme: chipTheme, useMaterial3: true),
         home: Directionality(
           textDirection: TextDirection.ltr,
           child: Material(
             child: Center(
               child: RawChip(
+                avatar: const Icon(Icons.add),
                 label: const SizedBox(width: 100, height: 100),
                 onSelected: (bool newValue) { },
               ),
@@ -159,14 +180,56 @@ void main() {
     );
 
     final RenderBox materialBox = getMaterialBox(tester);
-    expect(materialBox, paints..path(color: chipTheme.backgroundColor));
+    expect(materialBox, paints..rect(color: chipTheme.backgroundColor));
     expect(getMaterial(tester).elevation, chipTheme.elevation);
-    expect(tester.getSize(find.byType(RawChip)), const Size(250, 250)); // label + padding + labelPadding
-    expect(getMaterial(tester).shape, chipTheme.shape);
+    expect(tester.getSize(find.byType(RawChip)), const Size(402, 252)); // label + padding + labelPadding
+    expect(
+      getMaterial(tester).shape,
+      chipTheme.shape?.copyWith(side: BorderSide(color: theme.colorScheme.outline)),
+    );
     expect(getLabelStyle(tester).style.fontSize, 32);
+    expect(getIconData(tester).color, chipTheme.iconTheme!.color);
   });
 
-  testWidgets('Chip uses ChipTheme', (WidgetTester tester) async {
+  testWidgets('Material2 - Chip uses ThemeData chip theme', (WidgetTester tester) async {
+    const ChipThemeData chipTheme = ChipThemeData(
+      backgroundColor: Color(0xff112233),
+      elevation: 4,
+      padding: EdgeInsets.all(50),
+      labelPadding: EdgeInsets.all(25),
+      shape: RoundedRectangleBorder(),
+      labelStyle: TextStyle(fontSize: 32),
+      iconTheme: IconThemeData(color: Color(0xff332211)),
+    );
+
+    await tester.pumpWidget(
+      MaterialApp(
+        theme: ThemeData(chipTheme: chipTheme, useMaterial3: false),
+        home: Directionality(
+          textDirection: TextDirection.ltr,
+          child: Material(
+            child: Center(
+              child: RawChip(
+                avatar: const Icon(Icons.add),
+                label: const SizedBox(width: 100, height: 100),
+                onSelected: (bool newValue) { },
+              ),
+            ),
+          ),
+        ),
+      ),
+    );
+
+    final RenderBox materialBox = getMaterialBox(tester);
+    expect(materialBox, paints..rect(color: chipTheme.backgroundColor));
+    expect(getMaterial(tester).elevation, chipTheme.elevation);
+    expect(tester.getSize(find.byType(RawChip)), const Size(400, 250)); // label + padding + labelPadding
+    expect(getMaterial(tester).shape, chipTheme.shape);
+    expect(getLabelStyle(tester).style.fontSize, 32);
+    expect(getIconData(tester).color, chipTheme.iconTheme!.color);
+  });
+
+  testWidgets('Material3 - Chip uses local ChipTheme', (WidgetTester tester) async {
     const ChipThemeData chipTheme = ChipThemeData(
       backgroundColor: Color(0xff112233),
       elevation: 4,
@@ -174,22 +237,13 @@ void main() {
       labelPadding: EdgeInsets.all(25),
       labelStyle: TextStyle(fontSize: 32),
       shape: RoundedRectangleBorder(),
+      iconTheme: IconThemeData(color: Color(0xff332211)),
     );
-
-    const ChipThemeData shadowedChipTheme = ChipThemeData(
-      backgroundColor: Color(0xff332211),
-      elevation: 3,
-      padding: EdgeInsets.all(5),
-      labelPadding: EdgeInsets.all(10),
-      labelStyle: TextStyle(fontSize: 64),
-      shape: CircleBorder(),
-    );
+    final ThemeData theme = ThemeData(chipTheme: const ChipThemeData(), useMaterial3: true);
 
     await tester.pumpWidget(
       MaterialApp(
-        theme: ThemeData.light().copyWith(
-          chipTheme: shadowedChipTheme,
-        ),
+        theme: theme,
         home: ChipTheme(
           data: chipTheme,
           child: Builder(
@@ -199,6 +253,7 @@ void main() {
                 child: Material(
                   child: Center(
                     child: RawChip(
+                      avatar: const Icon(Icons.add),
                       label: const SizedBox(width: 100, height: 100),
                       onSelected: (bool newValue) { },
                     ),
@@ -212,32 +267,33 @@ void main() {
     );
 
     final RenderBox materialBox = getMaterialBox(tester);
-    expect(materialBox, paints..path(color: chipTheme.backgroundColor));
-    expect(tester.getSize(find.byType(RawChip)), const Size(250, 250)); // label + padding + labelPadding
+    expect(materialBox, paints..rect(color: chipTheme.backgroundColor));
+    expect(tester.getSize(find.byType(RawChip)), const Size(402, 252)); // label + padding + labelPadding
     expect(getMaterial(tester).elevation, chipTheme.elevation);
-    expect(getMaterial(tester).shape, chipTheme.shape);
+    expect(
+      getMaterial(tester).shape,
+      chipTheme.shape?.copyWith(side: BorderSide(color: theme.colorScheme.outline)),
+    );
     expect(getLabelStyle(tester).style.fontSize, 32);
+    expect(getIconData(tester).color, chipTheme.iconTheme!.color);
   });
 
-  testWidgets('Chip uses constructor parameters', (WidgetTester tester) async {
-    const ChipThemeData shadowedChipTheme = ChipThemeData(
+  testWidgets('Material2 - Chip uses local ChipTheme', (WidgetTester tester) async {
+    const ChipThemeData chipTheme = ChipThemeData(
       backgroundColor: Color(0xff112233),
       elevation: 4,
-      padding: EdgeInsets.all(5),
-      labelPadding: EdgeInsets.all(2),
-      labelStyle: TextStyle(),
+      padding: EdgeInsets.all(50),
+      labelPadding: EdgeInsets.all(25),
+      labelStyle: TextStyle(fontSize: 32),
       shape: RoundedRectangleBorder(),
+      iconTheme: IconThemeData(color: Color(0xff332211)),
     );
 
-    const Color backgroundColor = Color(0xff332211);
-    const double elevation = 3;
-    const double fontSize = 32;
-    const OutlinedBorder shape = CircleBorder();
-
     await tester.pumpWidget(
       MaterialApp(
+        theme: ThemeData(chipTheme: const ChipThemeData(), useMaterial3: false),
         home: ChipTheme(
-          data: shadowedChipTheme,
+          data: chipTheme,
           child: Builder(
             builder: (BuildContext context) {
               return Directionality(
@@ -245,12 +301,7 @@ void main() {
                 child: Material(
                   child: Center(
                     child: RawChip(
-                      backgroundColor: backgroundColor,
-                      elevation: elevation,
-                      padding: const EdgeInsets.all(50),
-                      labelPadding:const EdgeInsets.all(25),
-                      labelStyle: const TextStyle(fontSize: fontSize),
-                      shape: shape,
+                      avatar: const Icon(Icons.add),
                       label: const SizedBox(width: 100, height: 100),
                       onSelected: (bool newValue) { },
                     ),
@@ -264,11 +315,159 @@ void main() {
     );
 
     final RenderBox materialBox = getMaterialBox(tester);
-    expect(materialBox, paints..path(color: backgroundColor));
-    expect(tester.getSize(find.byType(RawChip)), const Size(250, 250)); // label + padding + labelPadding
+    expect(materialBox, paints..rect(color: chipTheme.backgroundColor));
+    expect(tester.getSize(find.byType(RawChip)), const Size(400, 250)); // label + padding + labelPadding
+    expect(getMaterial(tester).elevation, chipTheme.elevation);
+    expect(getMaterial(tester).shape, chipTheme.shape);
+    expect(getLabelStyle(tester).style.fontSize, 32);
+    expect(getIconData(tester).color, chipTheme.iconTheme!.color);
+  });
+
+  testWidgets('Chip properties overrides ChipTheme', (WidgetTester tester) async {
+    const ChipThemeData chipTheme = ChipThemeData(
+      backgroundColor: Color(0xff112233),
+      elevation: 4,
+      padding: EdgeInsets.all(50),
+      labelPadding: EdgeInsets.all(25),
+      labelStyle: TextStyle(fontSize: 32),
+      shape: RoundedRectangleBorder(),
+      iconTheme: IconThemeData(color: Color(0xff332211)),
+    );
+
+    const Color backgroundColor = Color(0xff000000);
+    const double elevation = 6.0;
+    const EdgeInsets padding = EdgeInsets.all(10);
+    const EdgeInsets labelPadding = EdgeInsets.all(5);
+    const TextStyle labelStyle = TextStyle(fontSize: 20);
+    const RoundedRectangleBorder shape = RoundedRectangleBorder(side: BorderSide(color: Color(0xff0000ff)));
+    const IconThemeData iconTheme = IconThemeData(color: Color(0xff00ff00));
+
+    await tester.pumpWidget(
+      MaterialApp(
+        theme: ThemeData(chipTheme: chipTheme),
+        home: Builder(
+          builder: (BuildContext context) {
+            return Directionality(
+              textDirection: TextDirection.ltr,
+              child: Material(
+                child: Center(
+                  child: RawChip(
+                    backgroundColor: backgroundColor,
+                    elevation: elevation,
+                    padding: padding,
+                    labelPadding: labelPadding,
+                    labelStyle: labelStyle,
+                    shape: shape,
+                    iconTheme: iconTheme,
+                    avatar: const Icon(Icons.add),
+                    label: const SizedBox(width: 100, height: 100),
+                    onSelected: (bool newValue) { },
+                  ),
+                ),
+              ),
+            );
+          },
+        ),
+      ),
+    );
+
+    final RenderBox materialBox = getMaterialBox(tester);
+    expect(materialBox, paints..rect(color: backgroundColor));
+    expect(tester.getSize(find.byType(RawChip)), const Size(242, 132)); // label + padding + labelPadding
+    expect(getMaterial(tester).elevation, elevation);
+    expect(getMaterial(tester).shape, shape);
+    expect(getLabelStyle(tester).style.fontSize, labelStyle.fontSize);
+    expect(getIconData(tester).color, iconTheme.color);
+  });
+
+  testWidgets('Material3 - Chip uses constructor parameters', (WidgetTester tester) async {
+    const Color backgroundColor = Color(0xff332211);
+    const double elevation = 3;
+    const double fontSize = 32;
+    const OutlinedBorder shape = CircleBorder(side: BorderSide(color: Color(0xff0000ff)));
+    const IconThemeData iconTheme = IconThemeData(color: Color(0xff443322));
+
+    await tester.pumpWidget(
+      MaterialApp(
+        theme: ThemeData(useMaterial3: true),
+        home: Builder(
+          builder: (BuildContext context) {
+            return Directionality(
+              textDirection: TextDirection.ltr,
+              child: Material(
+                child: Center(
+                  child: RawChip(
+                    backgroundColor: backgroundColor,
+                    elevation: elevation,
+                    padding: const EdgeInsets.all(50),
+                    labelPadding:const EdgeInsets.all(25),
+                    labelStyle: const TextStyle(fontSize: fontSize),
+                    shape: shape,
+                    iconTheme: iconTheme,
+                    avatar: const Icon(Icons.add),
+                    label: const SizedBox(width: 100, height: 100),
+                    onSelected: (bool newValue) { },
+                  ),
+                ),
+              ),
+            );
+          },
+        ),
+      ),
+    );
+
+    final RenderBox materialBox = getMaterialBox(tester);
+    expect(materialBox, paints..circle(color: backgroundColor));
+    expect(tester.getSize(find.byType(RawChip)), const Size(402, 252)); // label + padding + labelPadding
     expect(getMaterial(tester).elevation, elevation);
     expect(getMaterial(tester).shape, shape);
     expect(getLabelStyle(tester).style.fontSize, 32);
+    expect(getIconData(tester).color, iconTheme.color);
+  });
+
+  testWidgets('Material2 - Chip uses constructor parameters', (WidgetTester tester) async {
+    const Color backgroundColor = Color(0xff332211);
+    const double elevation = 3;
+    const double fontSize = 32;
+    const OutlinedBorder shape = CircleBorder();
+    const IconThemeData iconTheme = IconThemeData(color: Color(0xff443322));
+
+    await tester.pumpWidget(
+      MaterialApp(
+        theme: ThemeData(useMaterial3: false),
+        home: Builder(
+          builder: (BuildContext context) {
+            return Directionality(
+              textDirection: TextDirection.ltr,
+              child: Material(
+                child: Center(
+                  child: RawChip(
+                    backgroundColor: backgroundColor,
+                    elevation: elevation,
+                    padding: const EdgeInsets.all(50),
+                    labelPadding:const EdgeInsets.all(25),
+                    labelStyle: const TextStyle(fontSize: fontSize),
+                    shape: shape,
+                    iconTheme: iconTheme,
+                    avatar: const Icon(Icons.add),
+                    label: const SizedBox(width: 100, height: 100),
+                    onSelected: (bool newValue) { },
+                  ),
+                ),
+              ),
+            );
+          },
+        ),
+      ),
+    );
+
+    final RenderBox materialBox = getMaterialBox(tester);
+    expect(materialBox, paints..circle(color: backgroundColor));
+    expect(tester.getSize(find.byType(RawChip)), const Size(400, 250)); // label + padding + labelPadding
+    expect(getMaterial(tester).elevation, elevation);
+    expect(getMaterial(tester).shape, shape);
+    expect(getLabelStyle(tester).style.fontSize, 32);
+    expect(getIconData(tester).color, iconTheme.color);
   });
 
   testWidgets('ChipTheme.fromDefaults', (WidgetTester tester) async {
@@ -323,11 +522,10 @@ void main() {
     expect(chipTheme.pressElevation, 8.0);
   });
 
-
   testWidgets('ChipThemeData generates correct opacities for defaults', (WidgetTester tester) async {
     const Color customColor1 = Color(0xcafefeed);
     const Color customColor2 = Color(0xdeadbeef);
-    final TextStyle customStyle = ThemeData.fallback().textTheme.bodyText1!.copyWith(color: customColor2);
+    final TextStyle customStyle = ThemeData.fallback().textTheme.bodyLarge!.copyWith(color: customColor2);
 
     final ChipThemeData lightTheme = ChipThemeData.fromDefaults(
       secondaryColor: customColor1,
@@ -391,7 +589,7 @@ void main() {
     final ChipThemeData chipThemeBlack = ChipThemeData.fromDefaults(
       secondaryColor: Colors.black,
       brightness: Brightness.dark,
-      labelStyle: ThemeData.fallback().textTheme.bodyText1!.copyWith(color: Colors.black),
+      labelStyle: ThemeData.fallback().textTheme.bodyLarge!.copyWith(color: Colors.black),
     ).copyWith(
       elevation: 1.0,
       labelPadding: const EdgeInsets.symmetric(horizontal: 8.0),
@@ -407,7 +605,7 @@ void main() {
     final ChipThemeData chipThemeWhite = ChipThemeData.fromDefaults(
       secondaryColor: Colors.white,
       brightness: Brightness.light,
-      labelStyle: ThemeData.fallback().textTheme.bodyText1!.copyWith(color: Colors.white),
+      labelStyle: ThemeData.fallback().textTheme.bodyLarge!.copyWith(color: Colors.white),
     ).copyWith(
       padding: const EdgeInsets.all(2.0),
       labelPadding: const EdgeInsets.only(top: 8.0, bottom: 8.0),
@@ -443,6 +641,7 @@ void main() {
     expect(lerp.elevation, 3.0);
     expect(lerp.pressElevation, 7.0);
     expect(lerp.checkmarkColor, equals(middleGrey));
+    expect(lerp.iconTheme, isNull);
 
     expect(ChipThemeData.lerp(null, null, 0.25), isNull);
 
@@ -466,6 +665,7 @@ void main() {
     expect(lerpANull25.elevation, 1.25);
     expect(lerpANull25.pressElevation, 2.5);
     expect(lerpANull25.checkmarkColor, equals(Colors.white.withAlpha(0x40)));
+    expect(lerp.iconTheme, isNull);
 
     final ChipThemeData lerpANull75 = ChipThemeData.lerp(null, chipThemeWhite, 0.75)!;
     expect(lerpANull75.backgroundColor, equals(Colors.black.withAlpha(0x17)));
@@ -508,6 +708,7 @@ void main() {
     expect(lerpBNull25.elevation, 0.75);
     expect(lerpBNull25.pressElevation, 3.0);
     expect(lerpBNull25.checkmarkColor, equals(Colors.black.withAlpha(0xbf)));
+    expect(lerp.iconTheme, isNull);
 
     final ChipThemeData lerpBNull75 = ChipThemeData.lerp(chipThemeBlack, null, 0.75)!;
     expect(lerpBNull75.backgroundColor, equals(Colors.white.withAlpha(0x08)));
@@ -529,6 +730,7 @@ void main() {
     expect(lerpBNull75.elevation, 0.25);
     expect(lerpBNull75.pressElevation, 1.0);
     expect(lerpBNull75.checkmarkColor, equals(Colors.black.withAlpha(0x40)));
+    expect(lerp.iconTheme, isNull);
   });
 
   testWidgets('Chip uses stateful color from chip theme', (WidgetTester tester) async {
@@ -625,6 +827,8 @@ void main() {
     await tester.pumpWidget(chipWidget(enabled: false));
     await tester.pumpAndSettle();
     expect(textColor(), disabledColor);
+
+    focusNode.dispose();
   });
 
   testWidgets('Chip uses stateful border side from resolveWith pattern', (WidgetTester tester) async {
@@ -644,6 +848,7 @@ void main() {
     Widget chipWidget({ bool selected = false }) {
       return MaterialApp(
         theme: ThemeData(
+          useMaterial3: false,
           chipTheme: ThemeData.light().chipTheme.copyWith(
             side: MaterialStateBorderSide.resolveWith(getBorderSide),
           ),
@@ -660,11 +865,11 @@ void main() {
 
     // Default.
     await tester.pumpWidget(chipWidget());
-    expect(find.byType(RawChip), paints..rrect(color: defaultColor));
+    expect(find.byType(RawChip), paints..rrect()..rrect(color: defaultColor));
 
     // Selected.
     await tester.pumpWidget(chipWidget(selected: true));
-    expect(find.byType(RawChip), paints..rrect(color: selectedColor));
+    expect(find.byType(RawChip), paints..rrect()..rrect(color: selectedColor));
   });
 
   testWidgets('Chip uses stateful border side from chip theme', (WidgetTester tester) async {
@@ -689,7 +894,7 @@ void main() {
 
     Widget chipWidget({ bool selected = false }) {
       return MaterialApp(
-        theme: ThemeData(chipTheme: chipTheme),
+        theme: ThemeData(useMaterial3: false, chipTheme: chipTheme),
         home: Scaffold(
           body: ChoiceChip(
             label: const Text('Chip'),
@@ -702,11 +907,11 @@ void main() {
 
     // Default.
     await tester.pumpWidget(chipWidget());
-    expect(find.byType(RawChip), paints..rrect(color: defaultColor));
+    expect(find.byType(RawChip), paints..rrect()..rrect(color: defaultColor));
 
     // Selected.
     await tester.pumpWidget(chipWidget(selected: true));
-    expect(find.byType(RawChip), paints..rrect(color: selectedColor));
+    expect(find.byType(RawChip), paints..rrect()..rrect(color: selectedColor));
   });
 
   testWidgets('Chip uses stateful shape from chip theme', (WidgetTester tester) async {
@@ -729,7 +934,7 @@ void main() {
 
     Widget chipWidget({ bool selected = false }) {
       return MaterialApp(
-        theme: ThemeData(chipTheme: chipTheme),
+        theme: ThemeData(useMaterial3: false, chipTheme: chipTheme),
         home: Scaffold(
           body: ChoiceChip(
             label: const Text('Chip'),
@@ -748,6 +953,259 @@ void main() {
     await tester.pumpWidget(chipWidget(selected: true));
     expect(getMaterial(tester).shape, isA<RoundedRectangleBorder>());
   });
+
+  testWidgets('RawChip uses material state color from ChipTheme', (WidgetTester tester) async {
+    const Color disabledSelectedColor = Color(0xffffff00);
+    const Color disabledColor = Color(0xff00ff00);
+    const Color backgroundColor = Color(0xff0000ff);
+    const Color selectedColor = Color(0xffff0000);
+    Widget buildApp({ required bool enabled, required bool selected }) {
+      return MaterialApp(
+        theme: ThemeData(
+          chipTheme: ChipThemeData(
+            color: MaterialStateProperty.resolveWith((Set<MaterialState> states) {
+              if (states.contains(MaterialState.disabled)
+                && states.contains(MaterialState.selected)) {
+                  return disabledSelectedColor;
+              }
+              if (states.contains(MaterialState.disabled)) {
+                return disabledColor;
+              }
+              if (states.contains(MaterialState.selected)) {
+                return selectedColor;
+              }
+              return backgroundColor;
+            }),
+          ),
+          useMaterial3: true,
+        ),
+        home: Material(
+          child: RawChip(
+            isEnabled: enabled,
+            selected: selected,
+            label: const Text('RawChip'),
+          ),
+        ),
+      );
+    }
+
+    // Check theme color for enabled chip.
+    await tester.pumpWidget(buildApp(enabled: true, selected: false));
+    await tester.pumpAndSettle();
+
+    // Enabled chip should have the provided backgroundColor.
+    expect(getMaterialBox(tester), paints..rrect(color: backgroundColor));
+
+    // Check theme color for disabled chip.
+    await tester.pumpWidget(buildApp(enabled: false, selected: false));
+    await tester.pumpAndSettle();
+
+    // Disabled chip should have the provided disabledColor.
+    expect(getMaterialBox(tester),paints..rrect(color: disabledColor));
+
+    // Check theme color for enabled and selected chip.
+    await tester.pumpWidget(buildApp(enabled: true, selected: true));
+    await tester.pumpAndSettle();
+
+    // Enabled & selected chip should have the provided selectedColor.
+    expect(getMaterialBox(tester), paints..rrect(color: selectedColor));
+
+    // Check theme color for disabled & selected chip.
+    await tester.pumpWidget(buildApp(enabled: false, selected: true));
+    await tester.pumpAndSettle();
+
+    // Disabled & selected chip should have the provided disabledSelectedColor.
+    expect(getMaterialBox(tester), paints..rrect(color: disabledSelectedColor));
+  });
+
+  testWidgets('RawChip uses state colors from ChipTheme', (WidgetTester tester) async {
+    const ChipThemeData chipTheme = ChipThemeData(
+      disabledColor: Color(0xadfefafe),
+      backgroundColor: Color(0xcafefeed),
+      selectedColor: Color(0xbeefcafe),
+    );
+    Widget buildApp({ required bool enabled, required bool selected }) {
+      return MaterialApp(
+        theme: ThemeData(chipTheme: chipTheme, useMaterial3: true),
+        home: Material(
+          child: RawChip(
+            isEnabled: enabled,
+            selected: selected,
+            label: const Text('RawChip'),
+          ),
+        ),
+      );
+    }
+
+    // Check theme color for enabled chip.
+    await tester.pumpWidget(buildApp(enabled: true, selected: false));
+    await tester.pumpAndSettle();
+
+    // Enabled chip should have the provided backgroundColor.
+    expect(getMaterialBox(tester), paints..rrect(color: chipTheme.backgroundColor));
+
+    // Check theme color for disabled chip.
+    await tester.pumpWidget(buildApp(enabled: false, selected: false));
+    await tester.pumpAndSettle();
+
+    // Disabled chip should have the provided disabledColor.
+    expect(getMaterialBox(tester),paints..rrect(color: chipTheme.disabledColor));
+
+    // Check theme color for enabled and selected chip.
+    await tester.pumpWidget(buildApp(enabled: true, selected: true));
+    await tester.pumpAndSettle();
+
+    // Enabled & selected chip should have the provided selectedColor.
+    expect(getMaterialBox(tester), paints..rrect(color: chipTheme.selectedColor));
+  });
+
+  // This is a regression test for https://github.com/flutter/flutter/issues/119163.
+  testWidgets('RawChip respects checkmark properties from ChipTheme', (WidgetTester tester) async {
+    Widget buildRawChip({ChipThemeData? chipTheme}) {
+      return MaterialApp(
+        theme: ThemeData.light(useMaterial3: false).copyWith(
+          chipTheme: chipTheme,
+        ),
+        home: Directionality(
+          textDirection: TextDirection.ltr,
+          child: Material(
+          child: Center(
+              child: RawChip(
+                selected: true,
+                label: const SizedBox(width: 100, height: 100),
+                onSelected: (bool newValue) { },
+              ),
+            ),
+          ),
+        ),
+      );
+    }
+
+    // Test that the checkmark is painted.
+    await tester.pumpWidget(buildRawChip(
+      chipTheme: const ChipThemeData(
+        checkmarkColor: Color(0xffff0000),
+      ),
+    ));
+
+    RenderBox materialBox = getMaterialBox(tester);
+    expect(
+      materialBox,
+      paints..path(
+        color: const Color(0xffff0000),
+        style: PaintingStyle.stroke,
+      ),
+    );
+
+    // Test that the checkmark is not painted when ChipThemeData.showCheckmark is false.
+    await tester.pumpWidget(buildRawChip(
+      chipTheme: const ChipThemeData(
+        showCheckmark: false,
+        checkmarkColor: Color(0xffff0000),
+      ),
+    ));
+    await tester.pumpAndSettle();
+
+    materialBox = getMaterialBox(tester);
+    expect(
+      materialBox,
+      isNot(paints..path(
+        color: const Color(0xffff0000),
+        style: PaintingStyle.stroke,
+      )),
+    );
+  });
+
+  testWidgets("Material3 - RawChip.shape's side is used when provided", (WidgetTester tester) async {
+    Widget buildChip({ OutlinedBorder? shape, BorderSide? side }) {
+      return MaterialApp(
+        theme: ThemeData(
+          useMaterial3: true,
+          chipTheme: ChipThemeData(
+            shape: shape,
+            side: side,
+          ),
+        ),
+        home: const Material(
+          child: Center(
+            child: RawChip(
+              label: Text('RawChip'),
+            ),
+          ),
+        ),
+      );
+    }
+
+    // Test [RawChip.shape] with a side.
+    await tester.pumpWidget(buildChip(
+      shape: const RoundedRectangleBorder(
+        side: BorderSide(color: Color(0xffff00ff)),
+        borderRadius: BorderRadius.all(Radius.circular(7.0)),
+      )),
+    );
+
+    // Chip should have the provided shape and the side from [RawChip.shape].
+    expect(
+      getMaterial(tester).shape,
+      const RoundedRectangleBorder(
+        side: BorderSide(color: Color(0xffff00ff)),
+        borderRadius: BorderRadius.all(Radius.circular(7.0)),
+      ),
+    );
+
+    // Test [RawChip.shape] with a side and [RawChip.side].
+    await tester.pumpWidget(buildChip(
+      shape: const RoundedRectangleBorder(
+        side: BorderSide(color: Color(0xffff00ff)),
+        borderRadius: BorderRadius.all(Radius.circular(7.0)),
+      ),
+      side: const BorderSide(color: Color(0xfffff000))),
+    );
+    await tester.pumpAndSettle();
+
+    // Chip use shape from [RawChip.shape] and the side from [RawChip.side].
+    // [RawChip.shape]'s side should be ignored.
+    expect(
+      getMaterial(tester).shape,
+      const RoundedRectangleBorder(
+        side: BorderSide(color: Color(0xfffff000)),
+        borderRadius: BorderRadius.all(Radius.circular(7.0)),
+      ),
+    );
+  });
+
+  testWidgets('Material3 - ChipThemeData.iconTheme respects default iconTheme.size', (WidgetTester tester) async {
+    Widget buildChip({ IconThemeData? iconTheme }) {
+      return MaterialApp(
+        theme: ThemeData(useMaterial3: true, chipTheme: ChipThemeData(iconTheme: iconTheme)),
+        home: Directionality(
+          textDirection: TextDirection.ltr,
+          child: Material(
+            child: Center(
+              child: RawChip(
+                avatar: const Icon(Icons.add),
+                label: const SizedBox(width: 100, height: 100),
+                onSelected: (bool newValue) { },
+              ),
+            ),
+          ),
+        ),
+      );
+    }
+
+    await tester.pumpWidget(buildChip(iconTheme: const IconThemeData(color: Color(0xff332211))));
+
+    // Icon should have the default chip iconSize.
+    expect(getIconData(tester).size, 18.0);
+    expect(getIconData(tester).color, const Color(0xff332211));
+
+    // Icon should have the provided iconSize.
+    await tester.pumpWidget(buildChip(iconTheme: const IconThemeData(color: Color(0xff112233), size: 23.0)));
+    await tester.pumpAndSettle();
+
+    expect(getIconData(tester).size, 23.0);
+    expect(getIconData(tester).color, const Color(0xff112233));
+  });
 }
 
 class _MaterialStateOutlinedBorder extends StadiumBorder implements MaterialStateOutlinedBorder {
