@@ -429,7 +429,7 @@ void main() {
     }, skip: isBrowser, // [intended] uses dart:isolate and io.
   );
 
-  test('fromImageProvider() propogates TimeoutException when image cannot be rendered', () async {
+  test('fromImageProvider() propagates TimeoutException when image cannot be rendered', () async {
     final Uint8List blueSquareBytes = Uint8List.fromList(kBlueSquarePng);
 
     // Corrupt the image's bytelist so it cannot be read.
