@@ -4,11 +4,10 @@
 
 import 'dart:math' as math;
 
+import 'package:flutter/gestures.dart';
 import 'package:flutter/material.dart';
 import 'package:flutter_test/flutter_test.dart';
 
-import '../rendering/mock_canvas.dart';
-
 const List<String> menuItems = <String>['one', 'two', 'three', 'four'];
 void onChanged<T>(T _) { }
 final Type dropdownButtonType = DropdownButton<String>(
@@ -90,7 +89,7 @@ class _TestAppState extends State<TestApp> {
         DefaultMaterialLocalizations.delegate,
       ],
       child: MediaQuery(
-        data: MediaQueryData.fromWindow(WidgetsBinding.instance.window).copyWith(size: widget.mediaSize),
+        data: const MediaQueryData().copyWith(size: widget.mediaSize),
         child: Directionality(
           textDirection: widget.textDirection,
           child: Navigator(
@@ -567,6 +566,45 @@ void main() {
     }
   });
 
+  testWidgets('DropdownButtonFormField with isDense:true does not clip large scale text',
+      (WidgetTester tester) async {
+    final Key buttonKey = UniqueKey();
+    const String value = 'two';
+
+    await tester.pumpWidget(
+      TestApp(
+        textDirection: TextDirection.ltr,
+        child: Builder(
+          builder: (BuildContext context) => MediaQuery(
+            data: MediaQuery.of(context).copyWith(textScaleFactor: 3.0),
+            child: Material(
+              child: Center(
+                child: DropdownButtonFormField<String>(
+                  key: buttonKey,
+                  value: value,
+                  onChanged: onChanged,
+                  items: menuItems.map<DropdownMenuItem<String>>((String item) {
+                    return DropdownMenuItem<String>(
+                      key: ValueKey<String>(item),
+                      value: item,
+                      child: Text(item,
+                          key: ValueKey<String>('${item}Text'),
+                          style: const TextStyle(fontSize: 20.0)),
+                    );
+                  }).toList(),
+                ),
+              ),
+            ),
+          ),
+        ),
+      ),
+    );
+
+    final RenderBox box =
+    tester.renderObject<RenderBox>(find.byType(dropdownButtonType));
+    expect(box.size.height, 72.0);
+  });
+
   testWidgets('DropdownButtonFormField.isDense is true by default', (WidgetTester tester) async {
     // Regression test for https://github.com/flutter/flutter/issues/46844
     final Key buttonKey = UniqueKey();
@@ -1095,4 +1133,149 @@ void main() {
       selectedItemBox.localToGlobal(Offset(selectedItemBox.size.width / 2.0, selectedItemBox.size.height / 2.0)),
     );
   });
+
+  testWidgets('InputDecoration borders are used for clipping', (WidgetTester tester) async {
+    const BorderRadius errorBorderRadius = BorderRadius.all(Radius.circular(5.0));
+    const BorderRadius focusedErrorBorderRadius = BorderRadius.all(Radius.circular(6.0));
+    const BorderRadius focusedBorder = BorderRadius.all(Radius.circular(7.0));
+    const BorderRadius enabledBorder = BorderRadius.all(Radius.circular(9.0));
+
+    final FocusNode focusNode = FocusNode();
+    addTearDown(focusNode.dispose);
+
+    const String errorText = 'This is an error';
+    bool showError = false;
+
+    await tester.pumpWidget(
+      MaterialApp(
+        theme: ThemeData(
+          inputDecorationTheme: const InputDecorationTheme(
+            errorBorder: OutlineInputBorder(
+              borderRadius: errorBorderRadius,
+            ),
+            focusedErrorBorder: OutlineInputBorder(
+              borderRadius: focusedErrorBorderRadius,
+            ),
+            focusedBorder: OutlineInputBorder(
+              borderRadius: focusedBorder,
+            ),
+            enabledBorder: OutlineInputBorder(
+              borderRadius: enabledBorder,
+            ),
+          ),
+        ),
+        home: Material(
+          child: Center(
+            child: StatefulBuilder(
+              builder: (BuildContext context, StateSetter setState) {
+                return DropdownButtonFormField<String>(
+                  value: 'two',
+                  onChanged:(String? value) {
+                    setState(() {
+                      if (value == 'three') {
+                        showError = true;
+                      } else {
+                        showError = false;
+                      }
+                    });
+                  },
+                  decoration: InputDecoration(
+                    errorText: showError ? errorText : null,
+                  ),
+                  focusNode: focusNode,
+                  items: menuItems.map<DropdownMenuItem<String>>((String item) {
+                    return DropdownMenuItem<String>(
+                      key: ValueKey<String>(item),
+                      value: item,
+                      child: Text(item, key: ValueKey<String>('${item}Text')),
+                    );
+                  }).toList(),
+                );
+              }
+            ),
+          ),
+        ),
+      ),
+    );
+
+    // Test enabled border.
+    InkWell inkWell = tester.widget<InkWell>(find.byType(InkWell));
+    expect(inkWell.borderRadius, enabledBorder);
+
+    // Test focused border.
+    focusNode.requestFocus();
+    await tester.pump();
+
+    inkWell = tester.widget<InkWell>(find.byType(InkWell));
+    expect(inkWell.borderRadius, focusedBorder);
+
+    // Test focused error border.
+    await tester.tap(find.text('two'), warnIfMissed: false);
+    await tester.pumpAndSettle();
+    await tester.tap(find.text('three').last);
+    await tester.pumpAndSettle();
+
+    inkWell = tester.widget<InkWell>(find.byType(InkWell));
+    expect(inkWell.borderRadius, focusedErrorBorderRadius);
+
+    // Test error border with no focus.
+    focusNode.unfocus();
+    await tester.pump();
+
+    // Hovering over the widget should show the error border.
+    final TestGesture gesture = await tester.createGesture(kind: PointerDeviceKind.mouse);
+    await gesture.moveTo(tester.getCenter(find.text('three').last));
+    await tester.pumpAndSettle();
+
+    inkWell = tester.widget<InkWell>(find.byType(InkWell));
+    expect(inkWell.borderRadius, errorBorderRadius);
+  });
+
+  testWidgets('DropdownButtonFormField onChanged is called when the form is reset', (WidgetTester tester) async {
+    // Regression test for https://github.com/flutter/flutter/issues/123009.
+    final GlobalKey<FormFieldState<String>> stateKey = GlobalKey<FormFieldState<String>>();
+    final GlobalKey<FormState> formKey = GlobalKey<FormState>();
+    String? value;
+
+    await tester.pumpWidget(
+      MaterialApp(
+        home: Material(
+          child: Form(
+            key: formKey,
+            child: DropdownButtonFormField<String>(
+              key: stateKey,
+              value: 'One',
+              items: <String>['One', 'Two', 'Free', 'Four']
+                .map<DropdownMenuItem<String>>((String value) {
+                  return DropdownMenuItem<String>(
+                    value: value,
+                    child: Text(value),
+                  );
+              }).toList(),
+              onChanged: (String? newValue) {
+                value = newValue;
+              },
+            ),
+          ),
+        ),
+      ),
+    );
+
+    // Initial value is 'One'.
+    expect(value, isNull);
+    expect(stateKey.currentState!.value, equals('One'));
+
+    // Select 'Two'.
+    await tester.tap(find.text('One'));
+    await tester.pumpAndSettle();
+    await tester.tap(find.text('Two').last);
+    await tester.pumpAndSettle();
+    expect(value, equals('Two'));
+    expect(stateKey.currentState!.value, equals('Two'));
+
+    // Should be back to 'One' when the form is reset.
+    formKey.currentState!.reset();
+    expect(value, equals('One'));
+    expect(stateKey.currentState!.value, equals('One'));
+  });
 }
