@@ -16,6 +16,11 @@ void main() {
     }
   });
 
+  test('Typography lerp special cases', () {
+    final Typography typography = Typography();
+    expect(identical(Typography.lerp(typography, typography, 0.5), typography), true);
+  });
+
   test('Typography on non-Apple platforms defaults to the correct font', () {
     expect(Typography.material2018().black.titleLarge!.fontFamily, 'Roboto');
     expect(Typography.material2018(platform: TargetPlatform.fuchsia).black.titleLarge!.fontFamily, 'Roboto');
@@ -31,11 +36,11 @@ void main() {
 
   // Ref: https://developer.apple.com/ios/human-interface-guidelines/visual-design/typography/
   final Matcher isSanFranciscoDisplayFont = predicate((TextStyle s) {
-    return s.fontFamily == '.SF UI Display';
+    return s.fontFamily == 'CupertinoSystemDisplay';
   }, 'Uses SF Display font');
 
   final Matcher isSanFranciscoTextFont = predicate((TextStyle s) {
-    return s.fontFamily == '.SF UI Text';
+    return s.fontFamily == 'CupertinoSystemText';
   }, 'Uses SF Text font');
 
   final Matcher isMacOSSanFranciscoMetaFont = predicate((TextStyle s) {
@@ -101,6 +106,26 @@ void main() {
     expect(nonDefaultPropertyNames, <String>['black', 'white', 'englishLike', 'dense', 'tall']);
   });
 
+  test('Can lerp between different typographies', () {
+    final List<Typography> all = <Typography>[
+      for (final TargetPlatform platform in TargetPlatform.values) Typography.material2014(platform: platform),
+      for (final TargetPlatform platform in TargetPlatform.values) Typography.material2018(platform: platform),
+      for (final TargetPlatform platform in TargetPlatform.values) Typography.material2021(platform: platform),
+    ];
+
+    for (final Typography fromTypography in all) {
+      for (final Typography toTypography in all) {
+        Object? error;
+        try {
+          Typography.lerp(fromTypography, toTypography, 0.5);
+        } catch (e) {
+          error = e;
+        }
+        expect(error, isNull);
+      }
+    }
+  });
+
   test('englishLike2018 TextTheme matches Material Design spec', () {
     // Check the default material text theme against the style values
     // shown https://material.io/design/typography/#type-scale.
@@ -343,4 +368,46 @@ void main() {
     expect(theme.bodySmall!.textBaseline, TextBaseline.alphabetic);
     expect(theme.bodySmall!.leadingDistribution, TextLeadingDistribution.even);
   });
+
+  test('Default M3 light textTheme styles all use onSurface', () {
+    final ThemeData theme = ThemeData(useMaterial3: true);
+    final TextTheme textTheme = theme.textTheme;
+    final Color dark = theme.colorScheme.onSurface;
+    expect(textTheme.displayLarge!.color, dark);
+    expect(textTheme.displayMedium!.color, dark);
+    expect(textTheme.displaySmall!.color, dark);
+    expect(textTheme.headlineLarge!.color, dark);
+    expect(textTheme.headlineMedium!.color, dark);
+    expect(textTheme.headlineSmall!.color, dark);
+    expect(textTheme.titleLarge!.color, dark);
+    expect(textTheme.titleMedium!.color, dark);
+    expect(textTheme.titleSmall!.color, dark);
+    expect(textTheme.bodyLarge!.color, dark);
+    expect(textTheme.bodyMedium!.color, dark);
+    expect(textTheme.bodySmall!.color, dark);
+    expect(textTheme.labelLarge!.color, dark);
+    expect(textTheme.labelMedium!.color, dark);
+    expect(textTheme.labelSmall!.color, dark);
+  });
+
+  test('Default M3 dark textTheme styles all use onSurface', () {
+    final ThemeData theme = ThemeData(useMaterial3: true, brightness: Brightness.dark);
+    final TextTheme textTheme = theme.textTheme;
+    final Color light = theme.colorScheme.onSurface;
+    expect(textTheme.displayLarge!.color, light);
+    expect(textTheme.displayMedium!.color, light);
+    expect(textTheme.displaySmall!.color, light);
+    expect(textTheme.headlineLarge!.color, light);
+    expect(textTheme.headlineMedium!.color, light);
+    expect(textTheme.headlineSmall!.color, light);
+    expect(textTheme.titleLarge!.color, light);
+    expect(textTheme.titleMedium!.color, light);
+    expect(textTheme.titleSmall!.color, light);
+    expect(textTheme.bodyLarge!.color, light);
+    expect(textTheme.bodyMedium!.color, light);
+    expect(textTheme.bodySmall!.color, light);
+    expect(textTheme.labelLarge!.color, light);
+    expect(textTheme.labelMedium!.color, light);
+    expect(textTheme.labelSmall!.color, light);
+  });
 }
