@@ -16,6 +16,7 @@ import '../rendering/rendering_tester.dart';
 
 void main() {
   TestRenderingFlutterBinding.ensureInitialized();
+  HttpOverrides.global = _FakeHttpOverrides();
 
   Future<Codec>  basicDecoder(ImmutableBuffer buffer, {int? cacheWidth, int? cacheHeight, bool? allowUpscaling}) {
     return PaintingBinding.instance.instantiateImageCodecFromBuffer(buffer, cacheWidth: cacheWidth, cacheHeight: cacheHeight, allowUpscaling: allowUpscaling ?? false);
@@ -24,12 +25,14 @@ void main() {
   late _FakeHttpClient httpClient;
 
   setUp(() {
+    _FakeHttpOverrides.createHttpClientCalls = 0;
     httpClient = _FakeHttpClient();
     debugNetworkImageHttpClientProvider = () => httpClient;
   });
 
   tearDown(() {
     debugNetworkImageHttpClientProvider = null;
+    expect(_FakeHttpOverrides.createHttpClientCalls, 0);
     PaintingBinding.instance.imageCache.clear();
     PaintingBinding.instance.imageCache.clearLiveImages();
   });
@@ -70,6 +73,42 @@ void main() {
     expect(httpClient.request.response.drained, true);
   }, skip: isBrowser); // [intended] Browser implementation does not use HTTP client but an <img> tag.
 
+  test('Expect thrown exception with statusCode - evicts from cache and drains, when using ResizeImage', () async {
+    const int errorStatusCode = HttpStatus.notFound;
+    const String requestUrl = 'foo-url';
+
+    httpClient.request.response.statusCode = errorStatusCode;
+
+    final Completer<dynamic> caughtError = Completer<dynamic>();
+
+    final ImageProvider imageProvider = ResizeImage(NetworkImage(nonconst(requestUrl)), width: 5, height: 5);
+    expect(imageCache.pendingImageCount, 0);
+    expect(imageCache.statusForKey(imageProvider).untracked, true);
+
+    final ImageStream result = imageProvider.resolve(ImageConfiguration.empty);
+
+    expect(imageCache.pendingImageCount, 1);
+
+    result.addListener(ImageStreamListener((ImageInfo info, bool syncCall) {},
+        onError: (dynamic error, StackTrace? stackTrace) {
+      caughtError.complete(error);
+    }));
+
+    final Object? err = await caughtError.future;
+    await Future<void>.delayed(Duration.zero);
+
+    expect(imageCache.pendingImageCount, 0);
+    expect(imageCache.statusForKey(imageProvider).untracked, true);
+
+    expect(
+      err,
+      isA<NetworkImageLoadException>()
+          .having((NetworkImageLoadException e) => e.statusCode, 'statusCode', errorStatusCode)
+          .having((NetworkImageLoadException e) => e.uri, 'uri', Uri.base.resolve(requestUrl)),
+    );
+    expect(httpClient.request.response.drained, true);
+  }, skip: isBrowser); // [intended] Browser implementation does not use HTTP client but an <img> tag.
+
   test('Uses the HttpClient provided by debugNetworkImageHttpClientProvider if set', () async {
     httpClient.thrownError = 'client1';
     final List<dynamic> capturedErrors = <dynamic>[];
@@ -177,7 +216,7 @@ void main() {
       },
     ));
 
-    final dynamic err = await caughtError.future;
+    final Object? err = await caughtError.future;
 
     expect(err, isA<SocketException>());
 
@@ -212,6 +251,22 @@ void main() {
   });
 }
 
+/// Override `HttpClient()` to throw an error.
+///
+/// This ensures that these tests never cause a call to the [HttpClient]
+/// constructor.
+///
+/// Regression test for <https://github.com/flutter/flutter/issues/129532>.
+class _FakeHttpOverrides extends HttpOverrides {
+  static int createHttpClientCalls = 0;
+
+  @override
+  HttpClient createHttpClient(SecurityContext? context) {
+    createHttpClientCalls++;
+    throw Exception('This test tried to create an HttpClient.');
+  }
+}
+
 class _FakeHttpClient extends Fake implements HttpClient {
   final _FakeHttpClientRequest request = _FakeHttpClientRequest();
   Object? thrownError;
@@ -224,6 +279,7 @@ class _FakeHttpClient extends Fake implements HttpClient {
     return request;
   }
 }
+
 class _FakeHttpClientRequest extends Fake implements HttpClientRequest {
   final _FakeHttpClientResponse response = _FakeHttpClientResponse();
 
