@@ -9,6 +9,7 @@ import 'package:flutter/foundation.dart';
 import 'package:flutter/painting.dart';
 import 'package:flutter/scheduler.dart' show SchedulerBinding, timeDilation;
 import 'package:flutter_test/flutter_test.dart';
+import 'package:leak_tracker_flutter_testing/leak_tracker_flutter_testing.dart';
 
 import '../image_data.dart';
 import 'fake_codec.dart';
@@ -870,4 +871,21 @@ void main() {
 
     expect(synchronouslyCalled, false);
   });
+
+  test('ImageStreamCompleterHandle dispatches memory events', () async {
+    await expectLater(
+      await memoryEvents(
+            () {
+              final StreamController<ImageChunkEvent> streamController = StreamController<ImageChunkEvent>();
+              addTearDown(streamController.close);
+              final ImageStreamCompleterHandle imageStreamCompleterHandle = FakeEventReportingImageStreamCompleter(
+                chunkEvents: streamController.stream,
+              ).keepAlive();
+              imageStreamCompleterHandle.dispose();
+            },
+        ImageStreamCompleterHandle,
+      ),
+      areCreateAndDispose,
+    );
+  });
 }
