@@ -7,6 +7,7 @@ import 'dart:ui' as ui;
 import 'package:flutter/foundation.dart';
 import 'package:flutter/widgets.dart';
 import 'package:flutter_test/flutter_test.dart';
+import 'package:leak_tracker_flutter_testing/leak_tracker_flutter_testing.dart';
 
 void _checkCaretOffsetsLtrAt(String text, List<int> boundaries) {
   expect(boundaries.first, 0);
@@ -400,27 +401,27 @@ void main() {
     painter.dispose();
   });
 
-  test('TextPainter textScaleFactor test', () {
+  test('TextPainter textScaler test', () {
     final TextPainter painter = TextPainter(
       text: const TextSpan(
         text: 'X',
         style: TextStyle(inherit: false, fontSize: 10.0),
       ),
       textDirection: TextDirection.ltr,
-      textScaleFactor: 2.0,
+      textScaler: const TextScaler.linear(2.0),
     );
     painter.layout();
     expect(painter.size, const Size(20.0, 20.0));
     painter.dispose();
   });
 
-  test('TextPainter textScaleFactor null style test', () {
+  test('TextPainter textScaler null style test', () {
     final TextPainter painter = TextPainter(
       text: const TextSpan(
         text: 'X',
       ),
       textDirection: TextDirection.ltr,
-      textScaleFactor: 2.0,
+      textScaler: const TextScaler.linear(2.0),
     );
     painter.layout();
     expect(painter.size, const Size(28.0, 28.0));
@@ -1192,9 +1193,9 @@ void main() {
   }, skip: isBrowser && !isCanvasKit); // https://github.com/flutter/flutter/issues/87543
 
   test('TextPainter handles invalid UTF-16', () {
-    Object? exception;
+    FlutterErrorDetails? error;
     FlutterError.onError = (FlutterErrorDetails details) {
-      exception = details.exception;
+      error = details;
     };
 
     final TextPainter painter = TextPainter()
@@ -1206,7 +1207,8 @@ void main() {
     painter.layout();
     // The layout should include one replacement character.
     expect(painter.width, equals(fontSize));
-    expect(exception, isNotNull);
+    expect(error!.exception, isNotNull);
+    expect(error!.silent, isTrue);
     painter.dispose();
   }, skip: kIsWeb); // https://github.com/flutter/flutter/issues/87544
 
@@ -1509,9 +1511,6 @@ void main() {
   });
 
   test('TextPainter line breaking does not round to integers', () {
-    if (! const bool.hasEnvironment('SKPARAGRAPH_REMOVE_ROUNDING_HACK')) {
-      return;
-    }
     const double fontSize = 1.25;
     const String text = '12345';
     assert((fontSize * text.length).truncate() != fontSize * text.length);
@@ -1528,6 +1527,13 @@ void main() {
         expect(metrics, hasLength(1));
     }
   }, skip: kIsWeb && !isCanvasKit); // [intended] Browsers seem to always round font/glyph metrics.
+
+  test('TextPainter dispatches memory events', () async {
+    await expectLater(
+      await memoryEvents(() => TextPainter().dispose(), TextPainter),
+      areCreateAndDispose,
+    );
+  });
 }
 
 class MockCanvas extends Fake implements Canvas {
