@@ -8,9 +8,10 @@ import 'package:flutter/rendering.dart';
 import 'package:flutter_test/flutter_test.dart';
 
 void main() {
-  setUp(() => _GestureBindingSpy());
+  final TestWidgetsFlutterBinding binding = _GestureBindingSpy();
 
-  test('attach and detach correctly handle gesture', () {
+  testWidgets('attach and detach correctly handle gesture', (_) async {
+    expect(WidgetsBinding.instance, binding);
     final TextSelectionDelegate delegate = FakeEditableTextState();
     final RenderEditable editable = RenderEditable(
       backgroundCursorColor: Colors.grey,
