@@ -45,7 +45,11 @@ class FakePlatformViewController extends PlatformViewController {
 }
 
 class FakeAndroidViewController implements AndroidViewController {
-  FakeAndroidViewController(this.viewId, {this.requiresSize = false});
+  FakeAndroidViewController(
+    this.viewId, {
+    this.requiresSize = false,
+    this.requiresViewComposition = false,
+  });
 
   bool disposed = false;
   bool focusCleared = false;
@@ -56,6 +60,10 @@ class FakeAndroidViewController implements AndroidViewController {
 
   bool _createCalledSuccessfully = false;
 
+  Offset? createPosition;
+
+  final List<PlatformViewCreatedCallback> _createdCallbacks = <PlatformViewCreatedCallback>[];
+
   /// Events that are dispatched.
   List<PointerEvent> dispatchedPointerEvents = <PointerEvent>[];
 
@@ -106,10 +114,13 @@ class FakeAndroidViewController implements AndroidViewController {
   @override
   void addOnPlatformViewCreatedListener(PlatformViewCreatedCallback listener) {
     created = true;
+    createdCallbacks.add(listener);
   }
 
   @override
-  void removeOnPlatformViewCreatedListener(PlatformViewCreatedCallback listener) {}
+  void removeOnPlatformViewCreatedListener(PlatformViewCreatedCallback listener) {
+    createdCallbacks.remove(listener);
+  }
 
   @override
   Future<void> sendMotionEvent(AndroidMotionEvent event) {
@@ -122,21 +133,25 @@ class FakeAndroidViewController implements AndroidViewController {
   }
 
   @override
-  Future<void> create({Size? size}) async {
+  Future<void> create({Size? size, Offset? position}) async {
     assert(!_createCalledSuccessfully);
     if (requiresSize && size != null) {
       assert(!size.isEmpty);
     }
-    _createCalledSuccessfully = size != null || !requiresSize;
+    _createCalledSuccessfully = size != null && position != null || !requiresSize;
+    createPosition = position;
   }
 
   @override
-  List<PlatformViewCreatedCallback> get createdCallbacks => <PlatformViewCreatedCallback>[];
+  List<PlatformViewCreatedCallback> get createdCallbacks => _createdCallbacks;
+
+  @override
+  bool requiresViewComposition;
 }
 
 class FakeAndroidPlatformViewsController {
   FakeAndroidPlatformViewsController() {
-    TestDefaultBinaryMessengerBinding.instance!.defaultBinaryMessenger.setMockMethodCallHandler(SystemChannels.platform_views, _onMethodCall);
+    TestDefaultBinaryMessengerBinding.instance.defaultBinaryMessenger.setMockMethodCallHandler(SystemChannels.platform_views, _onMethodCall);
   }
 
   Iterable<FakeAndroidPlatformView> get views => _views.values;
@@ -156,6 +171,11 @@ class FakeAndroidPlatformViewsController {
 
   Map<int, Offset> offsets = <int, Offset>{};
 
+  /// True if Texture Layer Hybrid Composition mode should be enabled.
+  ///
+  /// When false, `create` will simulate the engine's fallback mode.
+  bool allowTextureLayerMode = true;
+
   void registerViewType(String viewType) {
     _registeredViewTypes.add(viewType);
   }
@@ -163,12 +183,12 @@ class FakeAndroidPlatformViewsController {
   void invokeViewFocused(int viewId) {
     final MethodCodec codec = SystemChannels.platform_views.codec;
     final ByteData data = codec.encodeMethodCall(MethodCall('viewFocused', viewId));
-    ServicesBinding.instance.defaultBinaryMessenger
+    TestDefaultBinaryMessengerBinding.instance.defaultBinaryMessenger
         .handlePlatformMessage(SystemChannels.platform_views.name, data, (ByteData? data) {});
   }
 
   Future<dynamic> _onMethodCall(MethodCall call) {
-    switch(call.method) {
+    switch (call.method) {
       case 'create':
         return _create(call);
       case 'dispose':
@@ -195,7 +215,10 @@ class FakeAndroidPlatformViewsController {
     final double? height = args['height'] as double?;
     final int layoutDirection = args['direction'] as int;
     final bool? hybrid = args['hybrid'] as bool?;
+    final bool? hybridFallback = args['hybridFallback'] as bool?;
     final Uint8List? creationParams = args['params'] as Uint8List?;
+    final double? top = args['top'] as double?;
+    final double? left = args['left'] as double?;
 
     if (_views.containsKey(id)) {
       throw PlatformException(
@@ -218,11 +241,22 @@ class FakeAndroidPlatformViewsController {
     _views[id] = FakeAndroidPlatformView(id, viewType,
         width != null && height != null ? Size(width, height) : null,
         layoutDirection,
-        hybrid,
-        creationParams,
+        hybrid: hybrid,
+        hybridFallback: hybridFallback,
+        creationParams: creationParams,
+        position: left != null && top != null ? Offset(left, top) : null,
     );
+    // Return a hybrid result (null rather than a texture ID) if:
+    final bool hybridResult =
+      // hybrid was explicitly requested, or
+      (hybrid ?? false) ||
+      // hybrid fallback was requested and simulated.
+      (!allowTextureLayerMode && (hybridFallback ?? false));
+    if (hybridResult) {
+      return Future<void>.value();
+    }
     final int textureId = _textureCounter++;
-    return Future<int>.sync(() => textureId);
+    return Future<int>.value(textureId);
   }
 
   Future<dynamic> _dispose(MethodCall call) {
@@ -336,7 +370,7 @@ class FakeAndroidPlatformViewsController {
 
 class FakeIosPlatformViewsController {
   FakeIosPlatformViewsController() {
-    TestDefaultBinaryMessengerBinding.instance!.defaultBinaryMessenger.setMockMethodCallHandler(SystemChannels.platform_views, _onMethodCall);
+    TestDefaultBinaryMessengerBinding.instance.defaultBinaryMessenger.setMockMethodCallHandler(SystemChannels.platform_views, _onMethodCall);
   }
 
   Iterable<FakeUiKitView> get views => _views.values;
@@ -361,12 +395,12 @@ class FakeIosPlatformViewsController {
   void invokeViewFocused(int viewId) {
     final MethodCodec codec = SystemChannels.platform_views.codec;
     final ByteData data = codec.encodeMethodCall(MethodCall('viewFocused', viewId));
-    ServicesBinding.instance.defaultBinaryMessenger
+    TestDefaultBinaryMessengerBinding.instance.defaultBinaryMessenger
         .handlePlatformMessage(SystemChannels.platform_views.name, data, (ByteData? data) {});
   }
 
   Future<dynamic> _onMethodCall(MethodCall call) {
-    switch(call.method) {
+    switch (call.method) {
       case 'create':
         return _create(call);
       case 'dispose':
@@ -437,38 +471,59 @@ class FakeIosPlatformViewsController {
   }
 }
 
-class FakeHtmlPlatformViewsController {
-  FakeHtmlPlatformViewsController() {
-    TestDefaultBinaryMessengerBinding.instance!.defaultBinaryMessenger.setMockMethodCallHandler(SystemChannels.platform_views, _onMethodCall);
+class FakeMacosPlatformViewsController {
+  FakeMacosPlatformViewsController() {
+    TestDefaultBinaryMessengerBinding.instance.defaultBinaryMessenger.setMockMethodCallHandler(SystemChannels.platform_views, _onMethodCall);
   }
 
-  Iterable<FakeHtmlPlatformView> get views => _views.values;
-  final Map<int, FakeHtmlPlatformView> _views = <int, FakeHtmlPlatformView>{};
+  Iterable<FakeAppKitView> get views => _views.values;
+  final Map<int, FakeAppKitView> _views = <int, FakeAppKitView>{};
 
   final Set<String> _registeredViewTypes = <String>{};
 
-  late Completer<void> resizeCompleter;
+  // When this completer is non null, the 'create' method channel call will be
+  // delayed until it completes.
+  Completer<void>? creationDelay;
 
-  Completer<void>? createCompleter;
+  // Maps a view id to the number of gestures it accepted so far.
+  final Map<int, int> gesturesAccepted = <int, int>{};
+
+  // Maps a view id to the number of gestures it rejected so far.
+  final Map<int, int> gesturesRejected = <int, int>{};
 
   void registerViewType(String viewType) {
     _registeredViewTypes.add(viewType);
   }
 
+  void invokeViewFocused(int viewId) {
+    final MethodCodec codec = SystemChannels.platform_views.codec;
+    final ByteData data = codec.encodeMethodCall(MethodCall('viewFocused', viewId));
+    TestDefaultBinaryMessengerBinding.instance.defaultBinaryMessenger
+        .handlePlatformMessage(SystemChannels.platform_views.name, data, (ByteData? data) {});
+  }
+
   Future<dynamic> _onMethodCall(MethodCall call) {
-    switch(call.method) {
+    switch (call.method) {
       case 'create':
         return _create(call);
       case 'dispose':
         return _dispose(call);
+      case 'acceptGesture':
+        return _acceptGesture(call);
+      case 'rejectGesture':
+        return _rejectGesture(call);
     }
     return Future<dynamic>.sync(() => null);
   }
 
   Future<dynamic> _create(MethodCall call) async {
+    if (creationDelay != null) {
+      await creationDelay!.future;
+    }
     final Map<dynamic, dynamic> args = call.arguments as Map<dynamic, dynamic>;
     final int id = args['id'] as int;
     final String viewType = args['viewType'] as String;
+    final Uint8List? creationParams = args['params'] as Uint8List?;
 
     if (_views.containsKey(id)) {
       throw PlatformException(
@@ -484,11 +539,23 @@ class FakeHtmlPlatformViewsController {
       );
     }
 
-    if (createCompleter != null) {
-      await createCompleter!.future;
-    }
+    _views[id] = FakeAppKitView(id, viewType, creationParams);
+    gesturesAccepted[id] = 0;
+    gesturesRejected[id] = 0;
+    return Future<int?>.sync(() => null);
+  }
 
-    _views[id] = FakeHtmlPlatformView(id, viewType);
+  Future<dynamic> _acceptGesture(MethodCall call) async {
+    final Map<dynamic, dynamic> args = call.arguments as Map<dynamic, dynamic>;
+    final int id = args['id'] as int;
+    gesturesAccepted[id] = gesturesAccepted[id]! + 1;
+    return Future<int?>.sync(() => null);
+  }
+
+  Future<dynamic> _rejectGesture(MethodCall call) async {
+    final Map<dynamic, dynamic> args = call.arguments as Map<dynamic, dynamic>;
+    final int id = args['id'] as int;
+    gesturesRejected[id] = gesturesRejected[id]! + 1;
     return Future<int?>.sync(() => null);
   }
 
@@ -509,7 +576,8 @@ class FakeHtmlPlatformViewsController {
 
 @immutable
 class FakeAndroidPlatformView {
-  const FakeAndroidPlatformView(this.id, this.type, this.size, this.layoutDirection, this.hybrid, [this.creationParams]);
+  const FakeAndroidPlatformView(this.id, this.type, this.size, this.layoutDirection,
+    {this.hybrid, this.hybridFallback, this.creationParams, this.position});
 
   final int id;
   final String type;
@@ -517,14 +585,18 @@ class FakeAndroidPlatformView {
   final Size? size;
   final int layoutDirection;
   final bool? hybrid;
+  final bool? hybridFallback;
+  final Offset? position;
 
   FakeAndroidPlatformView copyWith({Size? size, int? layoutDirection}) => FakeAndroidPlatformView(
     id,
     type,
     size ?? this.size,
     layoutDirection ?? this.layoutDirection,
-    hybrid,
-    creationParams,
+    hybrid: hybrid,
+    hybridFallback: hybridFallback,
+    creationParams: creationParams,
+    position: position,
   );
 
   @override
@@ -538,7 +610,9 @@ class FakeAndroidPlatformView {
         && listEquals<int>(other.creationParams, creationParams)
         && other.size == size
         && other.hybrid == hybrid
-        && other.layoutDirection == layoutDirection;
+        && other.hybridFallback == hybridFallback
+        && other.layoutDirection == layoutDirection
+        && other.position == position;
   }
 
   @override
@@ -549,11 +623,15 @@ class FakeAndroidPlatformView {
     size,
     layoutDirection,
     hybrid,
+    hybridFallback,
+    position,
   );
 
   @override
   String toString() {
-    return 'FakeAndroidPlatformView(id: $id, type: $type, size: $size, layoutDirection: $layoutDirection, hybrid: $hybrid, creationParams: $creationParams)';
+    return 'FakeAndroidPlatformView(id: $id, type: $type, size: $size, '
+      'layoutDirection: $layoutDirection, hybrid: $hybrid, '
+      'hybridFallback: $hybridFallback, creationParams: $creationParams, position: $position)';
   }
 }
 
@@ -612,20 +690,22 @@ class FakeUiKitView {
 }
 
 @immutable
-class FakeHtmlPlatformView {
-  const FakeHtmlPlatformView(this.id, this.type);
+class FakeAppKitView {
+  const FakeAppKitView(this.id, this.type, [this.creationParams]);
 
   final int id;
   final String type;
+  final Uint8List? creationParams;
 
   @override
   bool operator ==(Object other) {
     if (other.runtimeType != runtimeType) {
       return false;
     }
-    return other is FakeHtmlPlatformView
+    return other is FakeAppKitView
         && other.id == id
-        && other.type == type;
+        && other.type == type
+        && other.creationParams == creationParams;
   }
 
   @override
@@ -633,6 +713,6 @@ class FakeHtmlPlatformView {
 
   @override
   String toString() {
-    return 'FakeHtmlPlatformView(id: $id, type: $type)';
+    return 'FakeAppKitView(id: $id, type: $type, creationParams: $creationParams)';
   }
 }
