@@ -8,10 +8,18 @@ import 'package:flutter/scheduler.dart';
 import 'package:flutter/services.dart';
 import 'package:flutter/widgets.dart';
 import 'package:flutter_test/flutter_test.dart';
+import 'package:leak_tracker_flutter_testing/leak_tracker_flutter_testing.dart';
 
 import 'restoration.dart';
 
 void main() {
+  testWidgets('$RestorationManager dispatches memory events', (WidgetTester tester) async {
+    await expectLater(
+      await memoryEvents(() => RestorationManager().dispose(), RestorationManager),
+      areCreateAndDispose,
+    );
+  });
+
   group('RestorationManager', () {
     testWidgets('root bucket retrieval', (WidgetTester tester) async {
       final List<MethodCall> callsToEngine = <MethodCall>[];
@@ -22,6 +30,7 @@ void main() {
       });
 
       final RestorationManager manager = RestorationManager();
+      addTearDown(manager.dispose);
       final Future<RestorationBucket?> rootBucketFuture = manager.rootBucket;
       RestorationBucket? rootBucket;
       rootBucketFuture.then((RestorationBucket? bucket) {
@@ -48,6 +57,7 @@ void main() {
       expect(rootBucket!.read<int>('value1'), 10);
       expect(rootBucket!.read<String>('value2'), 'Hello');
       final RestorationBucket child = rootBucket!.claimChild('child1', debugOwner: null);
+      addTearDown(child.dispose);
       expect(child.read<int>('another value'), 22);
 
       // Accessing the root bucket again completes synchronously with same bucket.
@@ -67,6 +77,7 @@ void main() {
         return null;
       });
       final RestorationManager manager = RestorationManager();
+      addTearDown(manager.dispose);
 
       await _pushDataFromEngine(_createEncodedRestorationData1());
 
@@ -87,6 +98,7 @@ void main() {
         return result.future;
       });
       final RestorationManager manager = RestorationManager();
+      addTearDown(manager.dispose);
 
       RestorationBucket? rootBucket;
       manager.rootBucket.then((RestorationBucket? bucket) => rootBucket = bucket);
@@ -113,6 +125,7 @@ void main() {
         return _createEncodedRestorationData1();
       });
       final RestorationManager manager = RestorationManager();
+      addTearDown(manager.dispose);
       RestorationBucket? rootBucket;
       manager.rootBucket.then((RestorationBucket? bucket) {
         rootBucket = bucket;
@@ -145,6 +158,7 @@ void main() {
       expect(newRoot!.read<int>('foo'), 33);
       expect(newRoot!.read<int>('value1'), null);
       final RestorationBucket newChild = newRoot!.claimChild('childFoo', debugOwner: null);
+      addTearDown(newChild.dispose);
       expect(newChild.read<String>('bar'), 'Hello');
     });
 
@@ -159,6 +173,7 @@ void main() {
       final RestorationManager manager = RestorationManager()..addListener(() {
         listenerCount++;
       });
+      addTearDown(manager.dispose);
       RestorationBucket? rootBucket;
       bool rootBucketResolved = false;
       manager.rootBucket.then((RestorationBucket? bucket) {
@@ -200,6 +215,7 @@ void main() {
       });
 
       final RestorationManager manager = RestorationManager();
+      addTearDown(manager.dispose);
       final Future<RestorationBucket?> rootBucketFuture = manager.rootBucket;
       RestorationBucket? rootBucket;
       rootBucketFuture.then((RestorationBucket? bucket) {
@@ -234,6 +250,7 @@ void main() {
       });
 
       final TestRestorationManager manager = TestRestorationManager();
+      addTearDown(manager.dispose);
       expect(manager.isReplacing, isFalse);
 
       RestorationBucket? rootBucket;
@@ -304,7 +321,7 @@ void main() {
 }
 
 Future<void> _pushDataFromEngine(Map<dynamic, dynamic> data) async {
-  await ServicesBinding.instance.defaultBinaryMessenger.handlePlatformMessage(
+  await TestDefaultBinaryMessengerBinding.instance.defaultBinaryMessenger.handlePlatformMessage(
     'flutter/restoration',
     const StandardMethodCodec().encodeMethodCall(MethodCall('push', data)),
     (_) { },
