@@ -339,6 +339,8 @@ void main() {
       bool hovering = false;
       bool focusing = false;
 
+      addTearDown(focusNode.dispose);
+
       Future<void> buildTest(bool enabled) async {
         await tester.pumpWidget(
           Center(
@@ -756,6 +758,10 @@ void main() {
       );
     });
 
+    tearDown(() async {
+      focusNode.dispose();
+    });
+
     testWidgets('FocusableActionDetector keeps track of focus and hover even when disabled.', (WidgetTester tester) async {
       FocusManager.instance.highlightStrategy = FocusHighlightStrategy.alwaysTraditional;
       final GlobalKey containerKey = GlobalKey();
@@ -860,6 +866,8 @@ void main() {
       (WidgetTester tester) async {
         final FocusNode buttonNode = FocusNode(debugLabel: 'Test');
 
+        addTearDown(buttonNode.dispose);
+
         await tester.pumpWidget(
           MaterialApp(
             home: FocusableActionDetector(
@@ -904,10 +912,18 @@ void main() {
           (WidgetTester tester) async {
         final FocusNode buttonNode1 = FocusNode(debugLabel: 'Button Node 1');
         final FocusNode buttonNode2 = FocusNode(debugLabel: 'Button Node 2');
+        final FocusNode skipTraversalNode = FocusNode(skipTraversal: true);
+
+        addTearDown(() {
+          buttonNode1.dispose();
+          buttonNode2.dispose();
+          skipTraversalNode.dispose();
+        });
 
         await tester.pumpWidget(
           MaterialApp(
             home: FocusableActionDetector(
+              focusNode: skipTraversalNode,
               child: Column(
                 children: <Widget>[
                   ElevatedButton(
@@ -938,6 +954,7 @@ void main() {
         await tester.pumpWidget(
           MaterialApp(
             home: FocusableActionDetector(
+              focusNode: skipTraversalNode,
               descendantsAreTraversable: false,
               child: Column(
                 children: <Widget>[
@@ -963,7 +980,7 @@ void main() {
         expect(buttonNode2.hasFocus, isFalse);
         primaryFocus!.nextFocus();
         await tester.pump();
-        expect(buttonNode1.hasFocus, isTrue);
+        expect(buttonNode1.hasFocus, isFalse);
         expect(buttonNode2.hasFocus, isFalse);
       },
     );
