@@ -6,12 +6,12 @@
 //   This file is run as part of a reduced test set in CI on Mac and Windows
 //   machines.
 @Tags(<String>['reduced-test-set'])
+library;
 
 import 'package:flutter/material.dart';
 import 'package:flutter/rendering.dart';
 import 'package:flutter_test/flutter_test.dart';
 
-import '../rendering/mock_canvas.dart';
 import 'test_border.dart' show TestBorder;
 
 final List<String> log = <String>[];
@@ -360,7 +360,7 @@ void main() {
     await tester.pumpWidget(
       Center(
         child: RepaintBoundary(
-          child: Container(
+          child: ColoredBox(
             color: Colors.white,
             child: Padding(
               padding: const EdgeInsets.all(100.0),
@@ -370,9 +370,9 @@ void main() {
                 child: Transform.rotate(
                   angle: 1.0, // radians
                   child: ClipRect(
-                    child: Container(
+                    child: ColoredBox(
                       color: Colors.red,
-                      child: Container(
+                      child: ColoredBox(
                         color: Colors.white,
                         child: RepaintBoundary(
                           child: Center(
@@ -442,7 +442,7 @@ void main() {
     await tester.pumpWidget(
       Center(
         child: RepaintBoundary(
-          child: Container(
+          child: ColoredBox(
             color: Colors.white,
             child: Padding(
               padding: const EdgeInsets.all(100.0),
@@ -458,9 +458,9 @@ void main() {
                       bottomLeft: Radius.elliptical(2.5, 12.0),
                       bottomRight: Radius.elliptical(15.0, 6.0),
                     ),
-                    child: Container(
+                    child: ColoredBox(
                       color: Colors.red,
-                      child: Container(
+                      child: ColoredBox(
                         color: Colors.white,
                         child: RepaintBoundary(
                           child: Center(
@@ -491,7 +491,7 @@ void main() {
     await tester.pumpWidget(
       Center(
         child: RepaintBoundary(
-          child: Container(
+          child: ColoredBox(
             color: Colors.white,
             child: Padding(
               padding: const EdgeInsets.all(100.0),
@@ -501,9 +501,9 @@ void main() {
                 child: Transform.rotate(
                   angle: 1.0, // radians
                   child: ClipOval(
-                    child: Container(
+                    child: ColoredBox(
                       color: Colors.red,
-                      child: Container(
+                      child: ColoredBox(
                         color: Colors.white,
                         child: RepaintBoundary(
                           child: Center(
@@ -534,7 +534,7 @@ void main() {
     await tester.pumpWidget(
       Center(
         child: RepaintBoundary(
-          child: Container(
+          child: ColoredBox(
             color: Colors.white,
             child: Padding(
               padding: const EdgeInsets.all(100.0),
@@ -549,9 +549,9 @@ void main() {
                         borderRadius: BorderRadius.all(Radius.circular(20.0)),
                       ),
                     ),
-                    child: Container(
+                    child: ColoredBox(
                       color: Colors.red,
-                      child: Container(
+                      child: ColoredBox(
                         color: Colors.white,
                         child: RepaintBoundary(
                           child: Center(
@@ -581,7 +581,7 @@ void main() {
   Center genPhysicalModel(Clip clipBehavior) {
     return Center(
       child: RepaintBoundary(
-        child: Container(
+        child: ColoredBox(
           color: Colors.white,
           child: Padding(
             padding: const EdgeInsets.all(100.0),
@@ -594,7 +594,7 @@ void main() {
                   borderRadius: const BorderRadius.all(Radius.circular(20.0)),
                   color: Colors.red,
                   clipBehavior: clipBehavior,
-                  child: Container(
+                  child: ColoredBox(
                     color: Colors.white,
                     child: RepaintBoundary(
                       child: Center(
@@ -645,7 +645,7 @@ void main() {
     await tester.pumpWidget(
       Center(
         child: RepaintBoundary(
-          child: Container(
+          child: ColoredBox(
             color: Colors.white,
             child: Padding(
               padding: const EdgeInsets.all(100.0),
@@ -657,7 +657,7 @@ void main() {
                   child: PhysicalModel(
                     borderRadius: const BorderRadius.all(Radius.circular(20.0)),
                     color: Colors.red,
-                    child: Container(
+                    child: ColoredBox(
                       color: Colors.white,
                       child: RepaintBoundary(
                         child: Center(
@@ -686,7 +686,7 @@ void main() {
   Center genPhysicalShape(Clip clipBehavior) {
     return Center(
       child: RepaintBoundary(
-        child: Container(
+        child: ColoredBox(
           color: Colors.white,
           child: Padding(
             padding: const EdgeInsets.all(100.0),
@@ -703,7 +703,7 @@ void main() {
                   ),
                   clipBehavior: clipBehavior,
                   color: Colors.red,
-                  child: Container(
+                  child: ColoredBox(
                     color: Colors.white,
                     child: RepaintBoundary(
                       child: Center(
@@ -752,7 +752,7 @@ void main() {
     await tester.pumpWidget(
       Center(
         child: RepaintBoundary(
-          child: Container(
+          child: ColoredBox(
             color: Colors.white,
             child: Padding(
               padding: const EdgeInsets.all(100.0),
@@ -768,7 +768,7 @@ void main() {
                       ),
                     ),
                     color: Colors.red,
-                    child: Container(
+                    child: ColoredBox(
                       color: Colors.white,
                       child: RepaintBoundary(
                         child: Center(
@@ -853,6 +853,7 @@ void main() {
 
   testWidgets('CustomClipper reclips when notified', (WidgetTester tester) async {
     final ValueNotifier<Rect> clip = ValueNotifier<Rect>(const Rect.fromLTWH(50.0, 50.0, 100.0, 100.0));
+    addTearDown(clip.dispose);
 
     await tester.pumpWidget(
       ClipRect(
@@ -883,4 +884,90 @@ void main() {
       ..restore(),
     );
   });
+
+  testWidgets('ClipRRect supports BorderRadiusDirectional', (WidgetTester tester) async {
+    const Radius startRadius = Radius.circular(15.0);
+    const Radius endRadius = Radius.circular(30.0);
+
+    Widget buildClipRRect(TextDirection textDirection) {
+      return Directionality(
+        textDirection: textDirection,
+        child: const ClipRRect(
+          borderRadius: BorderRadiusDirectional.horizontal(
+            start: startRadius,
+            end: endRadius,
+          ),
+        ),
+      );
+    }
+
+    for (final TextDirection textDirection in TextDirection.values) {
+      await tester.pumpWidget(buildClipRRect(textDirection));
+      final RenderClipRRect renderClip = tester.allRenderObjects.whereType<RenderClipRRect>().first;
+      final bool isRtl = textDirection == TextDirection.rtl;
+      expect(renderClip.borderRadius.resolve(textDirection).topLeft, isRtl ? endRadius : startRadius);
+      expect(renderClip.borderRadius.resolve(textDirection).topRight, isRtl ? startRadius : endRadius);
+      expect(renderClip.borderRadius.resolve(textDirection).bottomLeft, isRtl ? endRadius : startRadius);
+      expect(renderClip.borderRadius.resolve(textDirection).bottomRight, isRtl ? startRadius : endRadius);
+    }
+  });
+
+  testWidgets('ClipRRect is direction-aware', (WidgetTester tester) async {
+    const Radius startRadius = Radius.circular(15.0);
+    const Radius endRadius = Radius.circular(30.0);
+    TextDirection textDirection = TextDirection.ltr;
+
+    Widget buildClipRRect(TextDirection textDirection) {
+      return Directionality(
+        textDirection: textDirection,
+        child: const ClipRRect(
+          borderRadius: BorderRadiusDirectional.horizontal(
+            start: startRadius,
+            end: endRadius,
+          ),
+        ),
+      );
+    }
+
+    Widget buildStatefulClipRRect() {
+      return StatefulBuilder(
+        builder: (BuildContext context, StateSetter setState) {
+          return Directionality(
+            textDirection: textDirection,
+            child: SizedBox(
+              width: 100.0,
+              height: 100.0,
+              child: ClipRRect(
+                borderRadius: const BorderRadiusDirectional.horizontal(
+                  start: startRadius,
+                  end: endRadius,
+                ),
+                child: GestureDetector(
+                  onTap: () {
+                    setState(() {
+                      textDirection = TextDirection.rtl;
+                    });
+                  },
+                ),
+              ),
+            ),
+          );
+        },
+      );
+    }
+
+    for (final TextDirection textDirection in TextDirection.values) {
+      await tester.pumpWidget(buildClipRRect(textDirection));
+      final RenderClipRRect renderClip = tester.allRenderObjects.whereType<RenderClipRRect>().first;
+      final bool isRtl = textDirection == TextDirection.rtl;
+      expect(renderClip.textDirection, isRtl ? TextDirection.rtl : TextDirection.ltr);
+    }
+
+    await tester.pumpWidget(buildStatefulClipRRect());
+    final RenderClipRRect renderClip = tester.allRenderObjects.whereType<RenderClipRRect>().first;
+    expect(renderClip.textDirection, TextDirection.ltr);
+    await tester.tapAt(Offset.zero);
+    await tester.pump();
+    expect(renderClip.textDirection, TextDirection.rtl);
+  });
 }
