@@ -40,11 +40,23 @@ class _TestSliverPersistentHeaderDelegate extends SliverPersistentHeaderDelegate
 void main() {
   const TextStyle textStyle = TextStyle();
   const Color cursorColor = Color.fromARGB(0xFF, 0xFF, 0x00, 0x00);
-    final FocusNode focusNode = FocusNode();
+
+  late TextEditingController controller;
+  late FocusNode focusNode;
+
+  setUp(() {
+    controller = TextEditingController();
+    focusNode = FocusNode();
+  });
+
+  tearDown(() {
+    controller.dispose();
+    focusNode.dispose();
+  });
 
   testWidgets('tapping on a partly visible editable brings it fully on screen', (WidgetTester tester) async {
     final ScrollController scrollController = ScrollController();
-    final TextEditingController controller = TextEditingController();
+    addTearDown(scrollController.dispose);
 
     await tester.pumpWidget(MaterialApp(
       home: Center(
@@ -82,8 +94,7 @@ void main() {
 
   testWidgets('tapping on a partly visible editable brings it fully on screen with scrollInsets', (WidgetTester tester) async {
     final ScrollController scrollController = ScrollController();
-    final TextEditingController controller = TextEditingController();
-    final FocusNode focusNode = FocusNode();
+    addTearDown(scrollController.dispose);
 
     await tester.pumpWidget(MaterialApp(
       home: Center(
@@ -128,8 +139,7 @@ void main() {
 
   testWidgets('editable comes back on screen when entering text while it is off-screen', (WidgetTester tester) async {
     final ScrollController scrollController = ScrollController(initialScrollOffset: 100.0);
-    final TextEditingController controller = TextEditingController();
-    final FocusNode focusNode = FocusNode();
+    addTearDown(scrollController.dispose);
 
     await tester.pumpWidget(MaterialApp(
       home: Center(
@@ -175,10 +185,8 @@ void main() {
 
   testWidgets('entering text does not scroll when scrollPhysics.allowImplicitScrolling = false', (WidgetTester tester) async {
     // regression test for https://github.com/flutter/flutter/issues/19523
-
     final ScrollController scrollController = ScrollController(initialScrollOffset: 100.0);
-    final TextEditingController controller = TextEditingController();
-    final FocusNode focusNode = FocusNode();
+    addTearDown(scrollController.dispose);
 
     await tester.pumpWidget(MaterialApp(
       home: Center(
@@ -225,9 +233,8 @@ void main() {
 
   testWidgets('entering text does not scroll a surrounding PageView', (WidgetTester tester) async {
     // regression test for https://github.com/flutter/flutter/issues/19523
-
-    final TextEditingController textController = TextEditingController();
     final PageController pageController = PageController(initialPage: 1);
+    addTearDown(pageController.dispose);
 
     await tester.pumpWidget(
       MaterialApp(
@@ -245,7 +252,7 @@ void main() {
                   ColoredBox(
                     color: Colors.green,
                     child: TextField(
-                      controller: textController,
+                      controller: controller,
                     ),
                   ),
                   Container(
@@ -261,7 +268,7 @@ void main() {
 
     await tester.showKeyboard(find.byType(EditableText));
     await tester.pumpAndSettle();
-    expect(textController.text, '');
+    expect(controller.text, '');
     tester.testTextInput.enterText('H');
     final int frames = await tester.pumpAndSettle();
 
@@ -269,13 +276,12 @@ void main() {
     // that the surrounding PageView is incorrectly scrolling back-and-forth.
     expect(frames, 1);
 
-    expect(textController.text, 'H');
+    expect(controller.text, 'H');
   });
 
   testWidgets('focused multi-line editable scrolls caret back into view when typing', (WidgetTester tester) async {
     final ScrollController scrollController = ScrollController();
-    final TextEditingController controller = TextEditingController();
-    final FocusNode focusNode = FocusNode();
+    addTearDown(scrollController.dispose);
     controller.text = "Start${'\n' * 39}End";
 
     await tester.pumpWidget(MaterialApp(
@@ -324,8 +330,7 @@ void main() {
 
   testWidgets('focused multi-line editable does not scroll to old position when non-collapsed selection set', (WidgetTester tester) async {
     final ScrollController scrollController = ScrollController();
-    final TextEditingController controller = TextEditingController();
-    final FocusNode focusNode = FocusNode();
+    addTearDown(scrollController.dispose);
     final String text = "Start${'\n' * 39}End";
     controller.value = TextEditingValue(text: text, selection: TextSelection.collapsed(offset: text.length - 3));
 
@@ -360,7 +365,7 @@ void main() {
     expect(render.size.height, greaterThan(500.0));
     expect(scrollController.offset, 0.0);
 
-    // Change selection to non-collapased so that cursor isn't shown
+    // Change selection to non-collapsed so that cursor isn't shown
     // and the location requires a bit of scroll.
     tester.testTextInput.updateEditingValue(TextEditingValue(
       text: text,
@@ -376,9 +381,7 @@ void main() {
 
   testWidgets('scrolls into view with scrollInserts after the keyboard pops up', (WidgetTester tester) async {
     final ScrollController scrollController = ScrollController();
-    final TextEditingController controller = TextEditingController();
-    final FocusNode focusNode = FocusNode();
-
+    addTearDown(scrollController.dispose);
     const Key container = Key('container');
 
     await tester.pumpWidget(MaterialApp(
@@ -422,11 +425,10 @@ void main() {
     'A pinned persistent header should not scroll when its descendant EditableText gains focus',
     (WidgetTester tester) async {
       // Regression test for https://github.com/flutter/flutter/issues/25507.
-      ScrollController controller;
-      final TextEditingController textEditingController = TextEditingController();
-      final FocusNode focusNode = FocusNode();
-
+      final ScrollController scrollController = ScrollController();
+      addTearDown(scrollController.dispose);
       const Key headerKey = Key('header');
+
       await tester.pumpWidget(
         MaterialApp(
           home: Center(
@@ -434,7 +436,7 @@ void main() {
               height: 600.0,
               width: 600.0,
               child: CustomScrollView(
-                controller: controller = ScrollController(),
+                controller: scrollController,
                 slivers: List<Widget>.generate(50, (int i) {
                   return i == 10
                   ? SliverPersistentHeader(
@@ -447,7 +449,7 @@ void main() {
                         child: EditableText(
                           key: headerKey,
                           backgroundCursorColor: Colors.grey,
-                          controller: textEditingController,
+                          controller: controller,
                           focusNode: focusNode,
                           style: textStyle,
                           cursorColor: cursorColor,
@@ -469,14 +471,14 @@ void main() {
       );
 
       // The persistent header should now be pinned at the top.
-      controller.jumpTo(100.0 * 15);
+      scrollController.jumpTo(100.0 * 15);
       await tester.pumpAndSettle();
-      expect(controller.offset, 100.0 * 15);
+      expect(scrollController.offset, 100.0 * 15);
 
       focusNode.requestFocus();
       await tester.pumpAndSettle();
       // The scroll offset should remain the same.
-      expect(controller.offset, 100.0 * 15);
+      expect(scrollController.offset, 100.0 * 15);
     },
   );
 
@@ -484,9 +486,8 @@ void main() {
     'A pinned persistent header should not scroll when its descendant EditableText gains focus (no animation)',
     (WidgetTester tester) async {
       // Regression test for https://github.com/flutter/flutter/issues/25507.
-      ScrollController controller;
-      final TextEditingController textEditingController = TextEditingController();
-      final FocusNode focusNode = FocusNode();
+      final ScrollController scrollController = ScrollController();
+      addTearDown(scrollController.dispose);
 
       const Key headerKey = Key('header');
       await tester.pumpWidget(
@@ -496,7 +497,7 @@ void main() {
               height: 600.0,
               width: 600.0,
               child: CustomScrollView(
-                controller: controller = ScrollController(),
+                controller: scrollController,
                 slivers: List<Widget>.generate(50, (int i) {
                   return i == 10
                     ? SliverPersistentHeader(
@@ -510,7 +511,7 @@ void main() {
                           child: EditableText(
                             key: headerKey,
                             backgroundCursorColor: Colors.grey,
-                            controller: textEditingController,
+                            controller: controller,
                             focusNode: focusNode,
                             style: textStyle,
                             cursorColor: cursorColor,
@@ -532,20 +533,19 @@ void main() {
       );
 
       // The persistent header should now be pinned at the top.
-      controller.jumpTo(100.0 * 15);
+      scrollController.jumpTo(100.0 * 15);
       await tester.pumpAndSettle();
-      expect(controller.offset, 100.0 * 15);
+      expect(scrollController.offset, 100.0 * 15);
 
       focusNode.requestFocus();
       await tester.pumpAndSettle();
       // The scroll offset should remain the same.
-      expect(controller.offset, 100.0 * 15);
+      expect(scrollController.offset, 100.0 * 15);
     },
   );
 
   void testShowCaretOnScreen({ required bool readOnly }) {
     group('EditableText._showCaretOnScreen, readOnly=$readOnly', () {
-      final TextEditingController textEditingController = TextEditingController();
       final TextInputFormatter rejectEverythingFormatter = TextInputFormatter.withFunction((TextEditingValue old, TextEditingValue value) => old);
 
       bool isCaretOnScreen(WidgetTester tester) {
@@ -574,7 +574,7 @@ void main() {
                 const SizedBox(height: 599),
                 EditableText(
                   backgroundCursorColor: Colors.grey,
-                  controller: textEditingController,
+                  controller: controller,
                   scrollController: editableScrollController,
                   inputFormatters: <TextInputFormatter>[if (rejectUserInputs) rejectEverythingFormatter],
                   focusNode: focusNode,
@@ -589,10 +589,12 @@ void main() {
       }
 
       testWidgets('focus-triggered showCaretOnScreen', (WidgetTester tester) async {
-        textEditingController.text = 'a' * 100;
-        textEditingController.selection = const TextSelection.collapsed(offset: 100);
+        controller.text = 'a' * 100;
+        controller.selection = const TextSelection.collapsed(offset: 100);
         final ScrollController scrollController = ScrollController();
+        addTearDown(scrollController.dispose);
         final ScrollController editableScrollController = ScrollController();
+        addTearDown(editableScrollController.dispose);
 
         await tester.pumpWidget(
           buildEditableText(
@@ -618,10 +620,12 @@ void main() {
       });
 
       testWidgets('selection-triggered showCaretOnScreen: virtual keyboard', (WidgetTester tester) async {
-        textEditingController.text = 'a' * 100;
-        textEditingController.selection = const TextSelection.collapsed(offset: 80);
+        controller.text = 'a' * 100;
+        controller.selection = const TextSelection.collapsed(offset: 80);
         final ScrollController scrollController = ScrollController();
+        addTearDown(scrollController.dispose);
         final ScrollController editableScrollController = ScrollController();
+        addTearDown(editableScrollController.dispose);
 
         await tester.pumpWidget(
           buildEditableText(
@@ -676,10 +680,12 @@ void main() {
       });
 
       testWidgets('selection-triggered showCaretOnScreen: text selection delegate', (WidgetTester tester) async {
-        textEditingController.text = 'a' * 100;
-        textEditingController.selection = const TextSelection.collapsed(offset: 80);
+        controller.text = 'a' * 100;
+        controller.selection = const TextSelection.collapsed(offset: 80);
         final ScrollController scrollController = ScrollController();
+        addTearDown(scrollController.dispose);
         final ScrollController editableScrollController = ScrollController();
+        addTearDown(editableScrollController.dispose);
 
         await tester.pumpWidget(
           buildEditableText(
@@ -740,9 +746,10 @@ void main() {
 
       // Regression text for https://github.com/flutter/flutter/pull/74722.
       testWidgets('does NOT randomly trigger when cursor blinks', (WidgetTester tester) async {
-        textEditingController.text = 'a' * 100;
-        textEditingController.selection = const TextSelection.collapsed(offset: 0);
+        controller.text = 'a' * 100;
+        controller.selection = const TextSelection.collapsed(offset: 0);
         final ScrollController editableScrollController = ScrollController();
+        addTearDown(editableScrollController.dispose);
         final bool deterministicCursor = EditableText.debugDeterministicCursor;
         EditableText.debugDeterministicCursor = false;
 
@@ -751,7 +758,7 @@ void main() {
             home: Scaffold(
               body: EditableText(
                 backgroundCursorColor: Colors.grey,
-                controller: textEditingController,
+                controller: controller,
                 scrollController: editableScrollController,
                 focusNode: focusNode,
                 style: textStyle,
@@ -770,7 +777,7 @@ void main() {
         expect(editableScrollController.offset, 0.0);
 
         // Change the text but keep the cursor location.
-        state.updateEditingValue(textEditingController.value.copyWith(
+        state.updateEditingValue(controller.value.copyWith(
           text: 'a' * 101,
         ));
 
