@@ -30,16 +30,6 @@ class FadeInImageParts {
     expect(animatedFadeOutFadeInElement, isNotNull);
     return animatedFadeOutFadeInElement!.state;
   }
-
-  Element? get semanticsElement {
-    Element? result;
-    fadeInImageElement.visitChildren((Element child) {
-      if (child.widget is Semantics) {
-        result = child;
-      }
-    });
-    return result;
-  }
 }
 
 class FadeInImageElements {
@@ -51,6 +41,8 @@ class FadeInImageElements {
   double get opacity => rawImage.opacity?.value ?? 1.0;
   BoxFit? get fit => rawImage.fit;
   FilterQuality? get filterQuality => rawImage.filterQuality;
+  Color? get color => rawImage.color;
+  BlendMode? get colorBlendMode => rawImage.colorBlendMode;
 }
 
 class LoadTestImageProvider extends ImageProvider<Object> {
@@ -68,7 +60,7 @@ class LoadTestImageProvider extends ImageProvider<Object> {
   }
 
   @override
-  ImageStreamCompleter load(Object key, DecoderCallback decode) {
+  ImageStreamCompleter loadImage(Object key, ImageDecoderCallback decode) {
     throw UnimplementedError();
   }
 }
@@ -100,11 +92,23 @@ FadeInImageParts findFadeInImage(WidgetTester tester) {
   }
 }
 
-Future<void> main() async {
+void main() {
   // These must run outside test zone to complete
-  final ui.Image targetImage = await createTestImage();
-  final ui.Image placeholderImage = await createTestImage();
-  final ui.Image replacementImage = await createTestImage();
+  late final ui.Image targetImage;
+  late final ui.Image placeholderImage;
+  late final ui.Image replacementImage;
+
+  setUpAll(() async {
+    targetImage = await createTestImage();
+    placeholderImage = await createTestImage();
+    replacementImage = await createTestImage();
+  });
+
+  tearDownAll(() {
+    targetImage.dispose();
+    placeholderImage.dispose();
+    replacementImage.dispose();
+  });
 
   group('FadeInImage', () {
     testWidgets('animates an uncached image', (WidgetTester tester) async {
@@ -375,6 +379,29 @@ Future<void> main() async {
       expect(findFadeInImage(tester).target.opacity, moreOrLessEquals(1));
     });
 
+    testWidgets('Image color and colorBlend parameters', (WidgetTester tester) async {
+      final TestImageProvider placeholderProvider = TestImageProvider(placeholderImage);
+      final TestImageProvider imageProvider = TestImageProvider(targetImage);
+
+      await tester.pumpWidget(FadeInImage(
+        placeholder: placeholderProvider,
+        image: imageProvider,
+        color: const Color(0xFF00FF00),
+        colorBlendMode: BlendMode.clear,
+        placeholderColor: const Color(0xFF0000FF),
+        placeholderColorBlendMode: BlendMode.modulate,
+        fadeOutDuration: animationDuration,
+        fadeInDuration: animationDuration,
+        excludeFromSemantics: true,
+      ));
+
+      expect(findFadeInImage(tester).placeholder?.color, const Color(0xFF0000FF));
+      expect(findFadeInImage(tester).placeholder?.colorBlendMode, BlendMode.modulate);
+      await tester.pump(animationDuration);
+      expect(findFadeInImage(tester).target.color, const Color(0xFF00FF00));
+      expect(findFadeInImage(tester).target.colorBlendMode, BlendMode.clear);
+    });
+
     group('ImageProvider', () {
 
       test('memory placeholder cacheWidth and cacheHeight is passed through', () async {
