@@ -155,6 +155,7 @@ void main() {
   const String image = 'assets/image.png';
 
   final Map<double, ui.Image> images = <double, ui.Image>{};
+
   setUpAll(() async {
     for (final double scale in const <double>[0.5, 1.0, 1.5, 2.0, 4.0, 10.0]) {
       final int dimension = (48 * scale).floor();
@@ -162,6 +163,12 @@ void main() {
     }
   });
 
+  tearDownAll(() {
+    for (final ui.Image image in images.values) {
+      image.dispose();
+    }
+  });
+
   testWidgets('Image for device pixel ratio 1.0', (WidgetTester tester) async {
     const double ratio = 1.0;
     Key key = GlobalKey();
