@@ -30,7 +30,7 @@ class TestWidgetState extends State<TestWidget> {
 void main() {
   testWidgets('initState() is called when we are in the tree', (WidgetTester tester) async {
     await tester.pumpWidget(const Parent(child: TestWidget()));
-    expect(ancestors, containsAllInOrder(<String>['Parent', 'View', 'RenderObjectToWidgetAdapter<RenderBox>']));
+    expect(ancestors, containsAllInOrder(<String>['Parent', 'View', 'RootWidget']));
   });
 }
 
