@@ -10,7 +10,7 @@ import 'package:flutter_test/flutter_test.dart';
 
 class _MediaQueryAspectCase {
   const _MediaQueryAspectCase(this.method, this.data);
-  final Function(BuildContext) method;
+  final void Function(BuildContext) method;
   final MediaQueryData data;
 }
 
@@ -44,26 +44,25 @@ class _MediaQueryAspectVariant extends TestVariant<_MediaQueryAspectCase> {
 
 void main() {
   testWidgets('MediaQuery does not have a default', (WidgetTester tester) async {
-    bool tested = false;
+    late final FlutterError error;
     // Cannot use tester.pumpWidget here because it wraps the widget in a View,
     // which introduces a MediaQuery ancestor.
     await pumpWidgetWithoutViewWrapper(
       tester: tester,
       widget: Builder(
         builder: (BuildContext context) {
-          tested = true;
-          MediaQuery.of(context); // should throw
-          return Container();
+          try {
+            MediaQuery.of(context);
+          } on FlutterError catch (e) {
+            error = e;
+          }
+          return View(
+            view: tester.view,
+            child: const SizedBox(),
+          );
         },
       ),
     );
-    expect(tested, isTrue);
-    final dynamic exception = tester.takeException();
-    expect(exception, isNotNull);
-    expect(exception ,isFlutterError);
-    final FlutterError error = exception as FlutterError;
-    expect(error.diagnostics.length, 5);
-    expect(error.diagnostics.last, isA<ErrorHint>());
     expect(
       error.toStringDeep(),
       startsWith(
@@ -119,7 +118,10 @@ void main() {
           final MediaQueryData? data = MediaQuery.maybeOf(context);
           expect(data, isNull);
           tested = true;
-          return Container();
+          return View(
+            view: tester.view,
+            child: const SizedBox(),
+          );
         },
       ),
     );
@@ -154,6 +156,7 @@ void main() {
     expect(data.disableAnimations, false);
     expect(data.boldText, false);
     expect(data.highContrast, false);
+    expect(data.onOffSwitchLabels, false);
     expect(data.platformBrightness, Brightness.light);
     expect(data.gestureSettings.touchSlop, null);
     expect(data.displayFeatures, isEmpty);
@@ -168,6 +171,7 @@ void main() {
       disableAnimations: true,
       boldText: true,
       highContrast: true,
+      onOffSwitchLabels: true,
       alwaysUse24HourFormat: true,
       navigationMode: NavigationMode.directional,
     );
@@ -177,7 +181,7 @@ void main() {
     expect(data.hashCode, data.copyWith().hashCode);
     expect(data.size, tester.view.physicalSize / tester.view.devicePixelRatio);
     expect(data.devicePixelRatio, tester.view.devicePixelRatio);
-    expect(data.textScaleFactor, platformData.textScaleFactor);
+    expect(data.textScaler, TextScaler.linear(platformData.textScaleFactor));
     expect(data.platformBrightness, platformData.platformBrightness);
     expect(data.padding, EdgeInsets.fromViewPadding(tester.view.padding, tester.view.devicePixelRatio));
     expect(data.viewPadding, EdgeInsets.fromViewPadding(tester.view.viewPadding, tester.view.devicePixelRatio));
@@ -188,6 +192,7 @@ void main() {
     expect(data.disableAnimations, platformData.disableAnimations);
     expect(data.boldText, platformData.boldText);
     expect(data.highContrast, platformData.highContrast);
+    expect(data.onOffSwitchLabels, platformData.onOffSwitchLabels);
     expect(data.alwaysUse24HourFormat, platformData.alwaysUse24HourFormat);
     expect(data.navigationMode, platformData.navigationMode);
     expect(data.gestureSettings, DeviceGestureSettings.fromView(tester.view));
@@ -206,7 +211,7 @@ void main() {
     expect(data.hashCode, data.copyWith().hashCode);
     expect(data.size, tester.view.physicalSize / tester.view.devicePixelRatio);
     expect(data.devicePixelRatio, tester.view.devicePixelRatio);
-    expect(data.textScaleFactor, tester.platformDispatcher.textScaleFactor);
+    expect(data.textScaler, TextScaler.linear(tester.platformDispatcher.textScaleFactor));
     expect(data.platformBrightness, tester.platformDispatcher.platformBrightness);
     expect(data.padding, EdgeInsets.fromViewPadding(tester.view.padding, tester.view.devicePixelRatio));
     expect(data.viewPadding, EdgeInsets.fromViewPadding(tester.view.viewPadding, tester.view.devicePixelRatio));
@@ -217,6 +222,7 @@ void main() {
     expect(data.disableAnimations, tester.platformDispatcher.accessibilityFeatures.disableAnimations);
     expect(data.boldText, tester.platformDispatcher.accessibilityFeatures.boldText);
     expect(data.highContrast, tester.platformDispatcher.accessibilityFeatures.highContrast);
+    expect(data.onOffSwitchLabels, tester.platformDispatcher.accessibilityFeatures.onOffSwitchLabels);
     expect(data.alwaysUse24HourFormat, tester.platformDispatcher.alwaysUse24HourFormat);
     expect(data.navigationMode, NavigationMode.traditional);
     expect(data.gestureSettings, DeviceGestureSettings.fromView(tester.view));
@@ -232,6 +238,7 @@ void main() {
       disableAnimations: true,
       boldText: true,
       highContrast: true,
+      onOffSwitchLabels: true,
       alwaysUse24HourFormat: true,
       navigationMode: NavigationMode.directional,
     );
@@ -253,7 +260,7 @@ void main() {
     expect(data, isNot(platformData));
     expect(data.size, tester.view.physicalSize / tester.view.devicePixelRatio);
     expect(data.devicePixelRatio, tester.view.devicePixelRatio);
-    expect(data.textScaleFactor, platformData.textScaleFactor);
+    expect(data.textScaler, TextScaler.linear(platformData.textScaleFactor));
     expect(data.platformBrightness, platformData.platformBrightness);
     expect(data.padding, EdgeInsets.fromViewPadding(tester.view.padding, tester.view.devicePixelRatio));
     expect(data.viewPadding, EdgeInsets.fromViewPadding(tester.view.viewPadding, tester.view.devicePixelRatio));
@@ -264,6 +271,7 @@ void main() {
     expect(data.disableAnimations, platformData.disableAnimations);
     expect(data.boldText, platformData.boldText);
     expect(data.highContrast, platformData.highContrast);
+    expect(data.onOffSwitchLabels, platformData.onOffSwitchLabels);
     expect(data.alwaysUse24HourFormat, platformData.alwaysUse24HourFormat);
     expect(data.navigationMode, platformData.navigationMode);
     expect(data.gestureSettings, DeviceGestureSettings.fromView(tester.view));
@@ -289,7 +297,10 @@ void main() {
               child: Builder(
                 builder: (BuildContext context) {
                   data = MediaQuery.of(context);
-                  return const Placeholder();
+                  return View(
+                    view: tester.view,
+                    child: const SizedBox(),
+                  );
                 },
               )
           );
@@ -300,7 +311,7 @@ void main() {
     expect(outerData, isNull);
     expect(data.size, tester.view.physicalSize / tester.view.devicePixelRatio);
     expect(data.devicePixelRatio, tester.view.devicePixelRatio);
-    expect(data.textScaleFactor, tester.platformDispatcher.textScaleFactor);
+    expect(data.textScaler, TextScaler.linear(tester.platformDispatcher.textScaleFactor));
     expect(data.platformBrightness, tester.platformDispatcher.platformBrightness);
     expect(data.padding, EdgeInsets.fromViewPadding(tester.view.padding, tester.view.devicePixelRatio));
     expect(data.viewPadding, EdgeInsets.fromViewPadding(tester.view.viewPadding, tester.view.devicePixelRatio));
@@ -311,6 +322,7 @@ void main() {
     expect(data.disableAnimations, tester.platformDispatcher.accessibilityFeatures.disableAnimations);
     expect(data.boldText, tester.platformDispatcher.accessibilityFeatures.boldText);
     expect(data.highContrast, tester.platformDispatcher.accessibilityFeatures.highContrast);
+    expect(data.onOffSwitchLabels, tester.platformDispatcher.accessibilityFeatures.onOffSwitchLabels);
     expect(data.alwaysUse24HourFormat, tester.platformDispatcher.alwaysUse24HourFormat);
     expect(data.navigationMode, NavigationMode.traditional);
     expect(data.gestureSettings, DeviceGestureSettings.fromView(tester.view));
@@ -341,7 +353,10 @@ void main() {
                 builder: (BuildContext context) {
                   rebuildCount++;
                   data = MediaQuery.of(context);
-                  return const Placeholder();
+                  return View(
+                    view: tester.view,
+                    child: const SizedBox(),
+                  );
                 },
               ),
           );
@@ -352,10 +367,10 @@ void main() {
     expect(outerData, isNull);
     expect(rebuildCount, 1);
 
-    expect(data.textScaleFactor, 123);
+    expect(data.textScaler, const TextScaler.linear(123));
     tester.platformDispatcher.textScaleFactorTestValue = 456;
     await tester.pump();
-    expect(data.textScaleFactor, 456);
+    expect(data.textScaler, const TextScaler.linear(456));
     expect(rebuildCount, 2);
 
     expect(data.platformBrightness, Brightness.dark);
@@ -392,7 +407,7 @@ void main() {
     await tester.pumpWidget(
       MediaQuery(
         data: const MediaQueryData(
-          textScaleFactor: 44,
+          textScaler: TextScaler.linear(44),
           platformBrightness: Brightness.dark,
           accessibleNavigation: true,
         ),
@@ -411,10 +426,10 @@ void main() {
 
     expect(rebuildCount, 1);
 
-    expect(data.textScaleFactor, 44);
+    expect(data.textScaler, const TextScaler.linear(44));
     tester.platformDispatcher.textScaleFactorTestValue = 456;
     await tester.pump();
-    expect(data.textScaleFactor, 44);
+    expect(data.textScaler, const TextScaler.linear(44));
     expect(rebuildCount, 1);
 
     expect(data.platformBrightness, Brightness.dark);
@@ -439,14 +454,14 @@ void main() {
   testWidgets('MediaQuery.fromView updates when parent data changes', (WidgetTester tester) async {
     late MediaQueryData data;
     int rebuildCount = 0;
-    double textScaleFactor = 55;
+    TextScaler textScaler = const TextScaler.linear(55);
     late StateSetter stateSetter;
     await tester.pumpWidget(
       StatefulBuilder(
         builder: (BuildContext context, StateSetter setState) {
           stateSetter = setState;
           return MediaQuery(
-            data: MediaQueryData(textScaleFactor: textScaleFactor),
+            data: MediaQueryData(textScaler: textScaler),
             child: MediaQuery.fromView(
               view: tester.view,
               child: Builder(
@@ -463,13 +478,13 @@ void main() {
     );
 
     expect(rebuildCount, 1);
-    expect(data.textScaleFactor, 55);
+    expect(data.textScaler, const TextScaler.linear(55));
 
     stateSetter(() {
-      textScaleFactor = 66;
+      textScaler = const TextScaler.linear(66);
     });
     await tester.pump();
-    expect(data.textScaleFactor, 66);
+    expect(data.textScaler, const TextScaler.linear(66));
     expect(rebuildCount, 2);
   });
 
@@ -478,7 +493,7 @@ void main() {
     final MediaQueryData copied = data.copyWith();
     expect(copied.size, data.size);
     expect(copied.devicePixelRatio, data.devicePixelRatio);
-    expect(copied.textScaleFactor, data.textScaleFactor);
+    expect(copied.textScaler, data.textScaler);
     expect(copied.padding, data.padding);
     expect(copied.viewPadding, data.viewPadding);
     expect(copied.viewInsets, data.viewInsets);
@@ -489,6 +504,7 @@ void main() {
     expect(copied.disableAnimations, data.disableAnimations);
     expect(copied.boldText, data.boldText);
     expect(copied.highContrast, data.highContrast);
+    expect(copied.onOffSwitchLabels, data.onOffSwitchLabels);
     expect(copied.platformBrightness, data.platformBrightness);
     expect(copied.gestureSettings, data.gestureSettings);
     expect(copied.displayFeatures, data.displayFeatures);
@@ -499,7 +515,7 @@ void main() {
     // values are copied over exactly
     const Size customSize = Size(3.14, 2.72);
     const double customDevicePixelRatio = 1.41;
-    const double customTextScaleFactor = 1.62;
+    const TextScaler customTextScaler = TextScaler.linear(1.23);
     const EdgeInsets customPadding = EdgeInsets.all(9.10938);
     const EdgeInsets customViewPadding = EdgeInsets.all(11.24031);
     const EdgeInsets customViewInsets = EdgeInsets.all(1.67262);
@@ -517,7 +533,7 @@ void main() {
     final MediaQueryData copied = data.copyWith(
       size: customSize,
       devicePixelRatio: customDevicePixelRatio,
-      textScaleFactor: customTextScaleFactor,
+      textScaler: customTextScaler,
       padding: customPadding,
       viewPadding: customViewPadding,
       viewInsets: customViewInsets,
@@ -528,6 +544,7 @@ void main() {
       disableAnimations: true,
       boldText: true,
       highContrast: true,
+      onOffSwitchLabels: true,
       platformBrightness: Brightness.dark,
       navigationMode: NavigationMode.directional,
       gestureSettings: gestureSettings,
@@ -535,7 +552,7 @@ void main() {
     );
     expect(copied.size, customSize);
     expect(copied.devicePixelRatio, customDevicePixelRatio);
-    expect(copied.textScaleFactor, customTextScaleFactor);
+    expect(copied.textScaler, customTextScaler);
     expect(copied.padding, customPadding);
     expect(copied.viewPadding, customViewPadding);
     expect(copied.viewInsets, customViewInsets);
@@ -546,6 +563,7 @@ void main() {
     expect(copied.disableAnimations, true);
     expect(copied.boldText, true);
     expect(copied.highContrast, true);
+    expect(copied.onOffSwitchLabels, true);
     expect(copied.platformBrightness, Brightness.dark);
     expect(copied.navigationMode, NavigationMode.directional);
     expect(copied.gestureSettings, gestureSettings);
@@ -555,7 +573,7 @@ void main() {
   testWidgets('MediaQuery.removePadding removes specified padding', (WidgetTester tester) async {
     const Size size = Size(2.0, 4.0);
     const double devicePixelRatio = 2.0;
-    const double textScaleFactor = 1.2;
+    const TextScaler textScaler = TextScaler.linear(1.2);
     const EdgeInsets padding = EdgeInsets.only(top: 1.0, right: 2.0, left: 3.0, bottom: 4.0);
     const EdgeInsets viewPadding = EdgeInsets.only(top: 6.0, right: 8.0, left: 10.0, bottom: 12.0);
     const EdgeInsets viewInsets = EdgeInsets.only(top: 5.0, right: 6.0, left: 7.0, bottom: 8.0);
@@ -573,7 +591,7 @@ void main() {
         data: const MediaQueryData(
           size: size,
           devicePixelRatio: devicePixelRatio,
-          textScaleFactor: textScaleFactor,
+          textScaler: textScaler,
           padding: padding,
           viewPadding: viewPadding,
           viewInsets: viewInsets,
@@ -583,6 +601,7 @@ void main() {
           disableAnimations: true,
           boldText: true,
           highContrast: true,
+          onOffSwitchLabels: true,
           navigationMode: NavigationMode.directional,
           displayFeatures: displayFeatures,
         ),
@@ -608,7 +627,7 @@ void main() {
 
     expect(unpadded.size, size);
     expect(unpadded.devicePixelRatio, devicePixelRatio);
-    expect(unpadded.textScaleFactor, textScaleFactor);
+    expect(unpadded.textScaler, textScaler);
     expect(unpadded.padding, EdgeInsets.zero);
     expect(unpadded.viewPadding, viewInsets);
     expect(unpadded.viewInsets, viewInsets);
@@ -618,6 +637,7 @@ void main() {
     expect(unpadded.disableAnimations, true);
     expect(unpadded.boldText, true);
     expect(unpadded.highContrast, true);
+    expect(unpadded.onOffSwitchLabels, true);
     expect(unpadded.navigationMode, NavigationMode.directional);
     expect(unpadded.displayFeatures, displayFeatures);
   });
@@ -625,7 +645,7 @@ void main() {
   testWidgets('MediaQuery.removePadding only removes specified padding', (WidgetTester tester) async {
     const Size size = Size(2.0, 4.0);
     const double devicePixelRatio = 2.0;
-    const double textScaleFactor = 1.2;
+    const TextScaler textScaler = TextScaler.linear(1.2);
     const EdgeInsets padding = EdgeInsets.only(top: 1.0, right: 2.0, left: 3.0, bottom: 4.0);
     const EdgeInsets viewPadding = EdgeInsets.only(top: 6.0, right: 8.0, left: 10.0, bottom: 12.0);
     const EdgeInsets viewInsets = EdgeInsets.only(top: 5.0, right: 6.0, left: 7.0, bottom: 8.0);
@@ -643,7 +663,7 @@ void main() {
         data: const MediaQueryData(
           size: size,
           devicePixelRatio: devicePixelRatio,
-          textScaleFactor: textScaleFactor,
+          textScaler: textScaler,
           padding: padding,
           viewPadding: viewPadding,
           viewInsets: viewInsets,
@@ -653,6 +673,7 @@ void main() {
           disableAnimations: true,
           boldText: true,
           highContrast: true,
+          onOffSwitchLabels: true,
           navigationMode: NavigationMode.directional,
           displayFeatures: displayFeatures,
         ),
@@ -675,7 +696,7 @@ void main() {
 
     expect(unpadded.size, size);
     expect(unpadded.devicePixelRatio, devicePixelRatio);
-    expect(unpadded.textScaleFactor, textScaleFactor);
+    expect(unpadded.textScaler, textScaler);
     expect(unpadded.padding, padding.copyWith(top: 0));
     expect(unpadded.viewPadding, viewPadding.copyWith(top: viewInsets.top));
     expect(unpadded.viewInsets, viewInsets);
@@ -685,6 +706,7 @@ void main() {
     expect(unpadded.disableAnimations, true);
     expect(unpadded.boldText, true);
     expect(unpadded.highContrast, true);
+    expect(unpadded.onOffSwitchLabels, true);
     expect(unpadded.navigationMode, NavigationMode.directional);
     expect(unpadded.displayFeatures, displayFeatures);
   });
@@ -692,7 +714,7 @@ void main() {
   testWidgets('MediaQuery.removeViewInsets removes specified viewInsets', (WidgetTester tester) async {
     const Size size = Size(2.0, 4.0);
     const double devicePixelRatio = 2.0;
-    const double textScaleFactor = 1.2;
+    const TextScaler textScaler = TextScaler.linear(1.2);
     const EdgeInsets padding = EdgeInsets.only(top: 5.0, right: 6.0, left: 7.0, bottom: 8.0);
     const EdgeInsets viewPadding = EdgeInsets.only(top: 6.0, right: 8.0, left: 10.0, bottom: 12.0);
     const EdgeInsets viewInsets = EdgeInsets.only(top: 1.0, right: 2.0, left: 3.0, bottom: 4.0);
@@ -710,7 +732,7 @@ void main() {
         data: const MediaQueryData(
           size: size,
           devicePixelRatio: devicePixelRatio,
-          textScaleFactor: textScaleFactor,
+          textScaler: textScaler,
           padding: padding,
           viewPadding: viewPadding,
           viewInsets: viewInsets,
@@ -720,6 +742,7 @@ void main() {
           disableAnimations: true,
           boldText: true,
           highContrast: true,
+          onOffSwitchLabels: true,
           navigationMode: NavigationMode.directional,
           displayFeatures: displayFeatures,
         ),
@@ -745,7 +768,7 @@ void main() {
 
     expect(unpadded.size, size);
     expect(unpadded.devicePixelRatio, devicePixelRatio);
-    expect(unpadded.textScaleFactor, textScaleFactor);
+    expect(unpadded.textScaler, textScaler);
     expect(unpadded.padding, padding);
     expect(unpadded.viewPadding, padding);
     expect(unpadded.viewInsets, EdgeInsets.zero);
@@ -755,6 +778,7 @@ void main() {
     expect(unpadded.disableAnimations, true);
     expect(unpadded.boldText, true);
     expect(unpadded.highContrast, true);
+    expect(unpadded.onOffSwitchLabels, true);
     expect(unpadded.navigationMode, NavigationMode.directional);
     expect(unpadded.displayFeatures, displayFeatures);
   });
@@ -762,7 +786,7 @@ void main() {
   testWidgets('MediaQuery.removeViewInsets removes only specified viewInsets', (WidgetTester tester) async {
     const Size size = Size(2.0, 4.0);
     const double devicePixelRatio = 2.0;
-    const double textScaleFactor = 1.2;
+    const TextScaler textScaler = TextScaler.linear(1.2);
     const EdgeInsets padding = EdgeInsets.only(top: 5.0, right: 6.0, left: 7.0, bottom: 8.0);
     const EdgeInsets viewPadding = EdgeInsets.only(top: 6.0, right: 8.0, left: 10.0, bottom: 12.0);
     const EdgeInsets viewInsets = EdgeInsets.only(top: 1.0, right: 2.0, left: 3.0, bottom: 4.0);
@@ -780,7 +804,7 @@ void main() {
         data: const MediaQueryData(
           size: size,
           devicePixelRatio: devicePixelRatio,
-          textScaleFactor: textScaleFactor,
+          textScaler: textScaler,
           padding: padding,
           viewPadding: viewPadding,
           viewInsets: viewInsets,
@@ -790,6 +814,7 @@ void main() {
           disableAnimations: true,
           boldText: true,
           highContrast: true,
+          onOffSwitchLabels: true,
           navigationMode: NavigationMode.directional,
           displayFeatures: displayFeatures,
         ),
@@ -812,7 +837,7 @@ void main() {
 
     expect(unpadded.size, size);
     expect(unpadded.devicePixelRatio, devicePixelRatio);
-    expect(unpadded.textScaleFactor, textScaleFactor);
+    expect(unpadded.textScaler, textScaler);
     expect(unpadded.padding, padding);
     expect(unpadded.viewPadding, viewPadding.copyWith(bottom: 8));
     expect(unpadded.viewInsets, viewInsets.copyWith(bottom: 0));
@@ -822,6 +847,7 @@ void main() {
     expect(unpadded.disableAnimations, true);
     expect(unpadded.boldText, true);
     expect(unpadded.highContrast, true);
+    expect(unpadded.onOffSwitchLabels, true);
     expect(unpadded.navigationMode, NavigationMode.directional);
     expect(unpadded.displayFeatures, displayFeatures);
   });
@@ -829,7 +855,7 @@ void main() {
   testWidgets('MediaQuery.removeViewPadding removes specified viewPadding', (WidgetTester tester) async {
     const Size size = Size(2.0, 4.0);
     const double devicePixelRatio = 2.0;
-    const double textScaleFactor = 1.2;
+    const TextScaler textScaler = TextScaler.linear(1.2);
     const EdgeInsets padding = EdgeInsets.only(top: 5.0, right: 6.0, left: 7.0, bottom: 8.0);
     const EdgeInsets viewPadding = EdgeInsets.only(top: 6.0, right: 8.0, left: 10.0, bottom: 12.0);
     const EdgeInsets viewInsets = EdgeInsets.only(top: 1.0, right: 2.0, left: 3.0, bottom: 4.0);
@@ -847,7 +873,7 @@ void main() {
         data: const MediaQueryData(
           size: size,
           devicePixelRatio: devicePixelRatio,
-          textScaleFactor: textScaleFactor,
+          textScaler: textScaler,
           padding: padding,
           viewPadding: viewPadding,
           viewInsets: viewInsets,
@@ -857,6 +883,7 @@ void main() {
           disableAnimations: true,
           boldText: true,
           highContrast: true,
+          onOffSwitchLabels: true,
           navigationMode: NavigationMode.directional,
           displayFeatures: displayFeatures,
         ),
@@ -882,7 +909,7 @@ void main() {
 
     expect(unpadded.size, size);
     expect(unpadded.devicePixelRatio, devicePixelRatio);
-    expect(unpadded.textScaleFactor, textScaleFactor);
+    expect(unpadded.textScaler, textScaler);
     expect(unpadded.padding, EdgeInsets.zero);
     expect(unpadded.viewPadding, EdgeInsets.zero);
     expect(unpadded.viewInsets, viewInsets);
@@ -892,6 +919,7 @@ void main() {
     expect(unpadded.disableAnimations, true);
     expect(unpadded.boldText, true);
     expect(unpadded.highContrast, true);
+    expect(unpadded.onOffSwitchLabels, true);
     expect(unpadded.navigationMode, NavigationMode.directional);
     expect(unpadded.displayFeatures, displayFeatures);
   });
@@ -899,7 +927,7 @@ void main() {
   testWidgets('MediaQuery.removeViewPadding removes only specified viewPadding', (WidgetTester tester) async {
     const Size size = Size(2.0, 4.0);
     const double devicePixelRatio = 2.0;
-    const double textScaleFactor = 1.2;
+    const TextScaler textScaler = TextScaler.linear(1.2);
     const EdgeInsets padding = EdgeInsets.only(top: 5.0, right: 6.0, left: 7.0, bottom: 8.0);
     const EdgeInsets viewPadding = EdgeInsets.only(top: 6.0, right: 8.0, left: 10.0, bottom: 12.0);
     const EdgeInsets viewInsets = EdgeInsets.only(top: 1.0, right: 2.0, left: 3.0, bottom: 4.0);
@@ -917,7 +945,7 @@ void main() {
         data: const MediaQueryData(
           size: size,
           devicePixelRatio: devicePixelRatio,
-          textScaleFactor: textScaleFactor,
+          textScaler: textScaler,
           padding: padding,
           viewPadding: viewPadding,
           viewInsets: viewInsets,
@@ -927,6 +955,7 @@ void main() {
           disableAnimations: true,
           boldText: true,
           highContrast: true,
+          onOffSwitchLabels: true,
           navigationMode: NavigationMode.directional,
           displayFeatures: displayFeatures,
         ),
@@ -949,7 +978,7 @@ void main() {
 
     expect(unpadded.size, size);
     expect(unpadded.devicePixelRatio, devicePixelRatio);
-    expect(unpadded.textScaleFactor, textScaleFactor);
+    expect(unpadded.textScaler, textScaler);
     expect(unpadded.padding, padding.copyWith(left: 0));
     expect(unpadded.viewPadding, viewPadding.copyWith(left: 0));
     expect(unpadded.viewInsets, viewInsets);
@@ -959,25 +988,26 @@ void main() {
     expect(unpadded.disableAnimations, true);
     expect(unpadded.boldText, true);
     expect(unpadded.highContrast, true);
+    expect(unpadded.onOffSwitchLabels, true);
     expect(unpadded.navigationMode, NavigationMode.directional);
     expect(unpadded.displayFeatures, displayFeatures);
   });
 
-  testWidgets('MediaQuery.textScaleFactorOf', (WidgetTester tester) async {
-    late double outsideTextScaleFactor;
-    late double insideTextScaleFactor;
+  testWidgets('MediaQuery.textScalerOf', (WidgetTester tester) async {
+    late TextScaler outsideTextScaler;
+    late TextScaler insideTextScaler;
 
     await tester.pumpWidget(
       Builder(
         builder: (BuildContext context) {
-          outsideTextScaleFactor = MediaQuery.textScaleFactorOf(context);
+          outsideTextScaler = MediaQuery.textScalerOf(context);
           return MediaQuery(
             data: const MediaQueryData(
-              textScaleFactor: 4.0,
+              textScaler: TextScaler.linear(4.0),
             ),
             child: Builder(
               builder: (BuildContext context) {
-                insideTextScaleFactor = MediaQuery.textScaleFactorOf(context);
+                insideTextScaler = MediaQuery.textScalerOf(context);
                 return Container();
               },
             ),
@@ -986,8 +1016,8 @@ void main() {
       ),
     );
 
-    expect(outsideTextScaleFactor, 1.0);
-    expect(insideTextScaleFactor, 4.0);
+    expect(outsideTextScaler, TextScaler.noScaling);
+    expect(insideTextScaler, const TextScaler.linear(4.0));
   });
 
   testWidgets('MediaQuery.platformBrightnessOf', (WidgetTester tester) async {
@@ -1044,6 +1074,33 @@ void main() {
     expect(insideHighContrast, true);
   });
 
+  testWidgets('MediaQuery.onOffSwitchLabelsOf', (WidgetTester tester) async {
+    late bool outsideOnOffSwitchLabels;
+    late bool insideOnOffSwitchLabels;
+
+    await tester.pumpWidget(
+      Builder(
+        builder: (BuildContext context) {
+          outsideOnOffSwitchLabels = MediaQuery.onOffSwitchLabelsOf(context);
+          return MediaQuery(
+            data: const MediaQueryData(
+              onOffSwitchLabels: true,
+            ),
+            child: Builder(
+              builder: (BuildContext context) {
+                insideOnOffSwitchLabels = MediaQuery.onOffSwitchLabelsOf(context);
+                return Container();
+              },
+            ),
+          );
+        },
+      ),
+    );
+
+    expect(outsideOnOffSwitchLabels, false);
+    expect(insideOnOffSwitchLabels, true);
+  });
+
   testWidgets('MediaQuery.boldTextOf', (WidgetTester tester) async {
     late bool outsideBoldTextOverride;
     late bool insideBoldTextOverride;
@@ -1135,7 +1192,7 @@ void main() {
   testWidgets('MediaQuery.removeDisplayFeatures removes specified display features and padding', (WidgetTester tester) async {
     const Size size = Size(82.0, 40.0);
     const double devicePixelRatio = 2.0;
-    const double textScaleFactor = 1.2;
+    const TextScaler textScaler = TextScaler.linear(1.2);
     const EdgeInsets padding = EdgeInsets.only(top: 1.0, right: 2.0, left: 3.0, bottom: 4.0);
     const EdgeInsets viewPadding = EdgeInsets.only(top: 6.0, right: 8.0, left: 10.0, bottom: 12.0);
     const EdgeInsets viewInsets = EdgeInsets.only(top: 5.0, right: 6.0, left: 7.0, bottom: 8.0);
@@ -1161,7 +1218,7 @@ void main() {
         data: const MediaQueryData(
           size: size,
           devicePixelRatio: devicePixelRatio,
-          textScaleFactor: textScaleFactor,
+          textScaler: textScaler,
           padding: padding,
           viewPadding: viewPadding,
           viewInsets: viewInsets,
@@ -1171,6 +1228,7 @@ void main() {
           disableAnimations: true,
           boldText: true,
           highContrast: true,
+          onOffSwitchLabels: true,
           displayFeatures: displayFeatures,
         ),
         child: Builder(
@@ -1191,7 +1249,7 @@ void main() {
 
     expect(subScreenMediaQuery.size, size);
     expect(subScreenMediaQuery.devicePixelRatio, devicePixelRatio);
-    expect(subScreenMediaQuery.textScaleFactor, textScaleFactor);
+    expect(subScreenMediaQuery.textScaler, textScaler);
     expect(subScreenMediaQuery.padding, EdgeInsets.zero);
     expect(subScreenMediaQuery.viewPadding, EdgeInsets.zero);
     expect(subScreenMediaQuery.viewInsets, EdgeInsets.zero);
@@ -1201,13 +1259,14 @@ void main() {
     expect(subScreenMediaQuery.disableAnimations, true);
     expect(subScreenMediaQuery.boldText, true);
     expect(subScreenMediaQuery.highContrast, true);
+    expect(subScreenMediaQuery.onOffSwitchLabels, true);
     expect(subScreenMediaQuery.displayFeatures, isEmpty);
   });
 
   testWidgets('MediaQuery.removePadding only removes specified display features and padding', (WidgetTester tester) async {
     const Size size = Size(82.0, 40.0);
     const double devicePixelRatio = 2.0;
-    const double textScaleFactor = 1.2;
+    const TextScaler textScaler = TextScaler.linear(1.2);
     const EdgeInsets padding = EdgeInsets.only(top: 1.0, right: 2.0, left: 3.0, bottom: 4.0);
     const EdgeInsets viewPadding = EdgeInsets.only(top: 6.0, right: 8.0, left: 46.0, bottom: 12.0);
     const EdgeInsets viewInsets = EdgeInsets.only(top: 5.0, right: 6.0, left: 7.0, bottom: 8.0);
@@ -1234,7 +1293,7 @@ void main() {
         data: const MediaQueryData(
           size: size,
           devicePixelRatio: devicePixelRatio,
-          textScaleFactor: textScaleFactor,
+          textScaler: textScaler,
           padding: padding,
           viewPadding: viewPadding,
           viewInsets: viewInsets,
@@ -1244,6 +1303,7 @@ void main() {
           disableAnimations: true,
           boldText: true,
           highContrast: true,
+          onOffSwitchLabels: true,
           displayFeatures: displayFeatures,
         ),
         child: Builder(
@@ -1264,7 +1324,7 @@ void main() {
 
     expect(subScreenMediaQuery.size, size);
     expect(subScreenMediaQuery.devicePixelRatio, devicePixelRatio);
-    expect(subScreenMediaQuery.textScaleFactor, textScaleFactor);
+    expect(subScreenMediaQuery.textScaler, textScaler);
     expect(
       subScreenMediaQuery.padding,
       const EdgeInsets.only(top: 1.0, right: 2.0, bottom: 4.0),
@@ -1283,6 +1343,7 @@ void main() {
     expect(subScreenMediaQuery.disableAnimations, true);
     expect(subScreenMediaQuery.boldText, true);
     expect(subScreenMediaQuery.highContrast, true);
+    expect(subScreenMediaQuery.onOffSwitchLabels, true);
     expect(subScreenMediaQuery.displayFeatures, <DisplayFeature>[cutoutDisplayFeature]);
   });
 
@@ -1296,11 +1357,11 @@ void main() {
   testWidgets('MediaQuery can be partially depended-on', (WidgetTester tester) async {
     MediaQueryData data = const MediaQueryData(
       size: Size(800, 600),
-      textScaleFactor: 1.1
+      textScaler: TextScaler.linear(1.1),
     );
 
     int sizeBuildCount = 0;
-    int textScaleFactorBuildCount = 0;
+    int textScalerBuildCount = 0;
 
     final Widget showSize = Builder(
       builder: (BuildContext context) {
@@ -1309,10 +1370,10 @@ void main() {
       }
     );
 
-    final Widget showTextScaleFactor = Builder(
+    final Widget showTextScaler = Builder(
       builder: (BuildContext context) {
-        textScaleFactorBuildCount++;
-        return Text('textScaleFactor: ${MediaQuery.textScaleFactorOf(context).toStringAsFixed(1)}');
+        textScalerBuildCount++;
+        return Text('textScaler: ${MediaQuery.textScalerOf(context)}');
       }
     );
 
@@ -1324,7 +1385,7 @@ void main() {
             child: Column(
               children: <Widget>[
                 showSize,
-                showTextScaleFactor,
+                showTextScaler,
                 ElevatedButton(
                   onPressed: () {
                     setState(() {
@@ -1336,10 +1397,10 @@ void main() {
                 ElevatedButton(
                   onPressed: () {
                     setState(() {
-                      data = data.copyWith(textScaleFactor: data.textScaleFactor + 0.1);
+                      data = data.copyWith(textScaler: TextScaler.noScaling);
                     });
                   },
-                  child: const Text('Increase textScaleFactor by 0.1')
+                  child: const Text('Disable text scaling')
                 )
               ]
             )
@@ -1350,23 +1411,23 @@ void main() {
 
     await tester.pumpWidget(MaterialApp(home: page));
     expect(find.text('size: Size(800.0, 600.0)'), findsOneWidget);
-    expect(find.text('textScaleFactor: 1.1'), findsOneWidget);
+    expect(find.text('textScaler: linear (1.1x)'), findsOneWidget);
     expect(sizeBuildCount, 1);
-    expect(textScaleFactorBuildCount, 1);
+    expect(textScalerBuildCount, 1);
 
     await tester.tap(find.text('Increase width by 100'));
     await tester.pumpAndSettle();
     expect(find.text('size: Size(900.0, 600.0)'), findsOneWidget);
-    expect(find.text('textScaleFactor: 1.1'), findsOneWidget);
+    expect(find.text('textScaler: linear (1.1x)'), findsOneWidget);
     expect(sizeBuildCount, 2);
-    expect(textScaleFactorBuildCount, 1);
+    expect(textScalerBuildCount, 1);
 
-    await tester.tap(find.text('Increase textScaleFactor by 0.1'));
+    await tester.tap(find.text('Disable text scaling'));
     await tester.pumpAndSettle();
     expect(find.text('size: Size(900.0, 600.0)'), findsOneWidget);
-    expect(find.text('textScaleFactor: 1.2'), findsOneWidget);
+    expect(find.text('textScaler: no scaling'), findsOneWidget);
     expect(sizeBuildCount, 2);
-    expect(textScaleFactorBuildCount, 2);
+    expect(textScalerBuildCount, 2);
   });
 
   testWidgets('MediaQuery partial dependencies', (WidgetTester tester) async {
@@ -1435,6 +1496,8 @@ void main() {
       const _MediaQueryAspectCase(MediaQuery.maybeDevicePixelRatioOf, MediaQueryData(devicePixelRatio: 1.1)),
       const _MediaQueryAspectCase(MediaQuery.textScaleFactorOf, MediaQueryData(textScaleFactor: 1.1)),
       const _MediaQueryAspectCase(MediaQuery.maybeTextScaleFactorOf, MediaQueryData(textScaleFactor: 1.1)),
+      const _MediaQueryAspectCase(MediaQuery.textScalerOf, MediaQueryData(textScaler: TextScaler.linear(1.1))),
+      const _MediaQueryAspectCase(MediaQuery.maybeTextScalerOf, MediaQueryData(textScaler: TextScaler.linear(1.1))),
       const _MediaQueryAspectCase(MediaQuery.platformBrightnessOf, MediaQueryData(platformBrightness: Brightness.dark)),
       const _MediaQueryAspectCase(MediaQuery.maybePlatformBrightnessOf, MediaQueryData(platformBrightness: Brightness.dark)),
       const _MediaQueryAspectCase(MediaQuery.paddingOf, MediaQueryData(padding: EdgeInsets.all(1))),
@@ -1453,6 +1516,8 @@ void main() {
       const _MediaQueryAspectCase(MediaQuery.maybeInvertColorsOf, MediaQueryData(invertColors: true)),
       const _MediaQueryAspectCase(MediaQuery.highContrastOf, MediaQueryData(highContrast: true)),
       const _MediaQueryAspectCase(MediaQuery.maybeHighContrastOf, MediaQueryData(highContrast: true)),
+      const _MediaQueryAspectCase(MediaQuery.onOffSwitchLabelsOf, MediaQueryData(onOffSwitchLabels: true)),
+      const _MediaQueryAspectCase(MediaQuery.maybeOnOffSwitchLabelsOf, MediaQueryData(onOffSwitchLabels: true)),
       const _MediaQueryAspectCase(MediaQuery.disableAnimationsOf, MediaQueryData(disableAnimations: true)),
       const _MediaQueryAspectCase(MediaQuery.maybeDisableAnimationsOf, MediaQueryData(disableAnimations: true)),
       const _MediaQueryAspectCase(MediaQuery.boldTextOf, MediaQueryData(boldText: true)),
