@@ -24,6 +24,7 @@ void main() {
     }) {
     return MaterialApp(
       theme:  ThemeData(
+        useMaterial3: false,
         materialTapTargetSize: MaterialTapTargetSize.padded,
       ),
       home: Scaffold(
@@ -40,6 +41,7 @@ void main() {
     group('hasScrollBody: true, default', () {
       testWidgets('no siblings', (WidgetTester tester) async {
         final ScrollController controller = ScrollController();
+        addTearDown(controller.dispose);
         await tester.pumpWidget(
           Directionality(
             textDirection: TextDirection.ltr,
@@ -80,6 +82,7 @@ void main() {
 
       testWidgets('one sibling', (WidgetTester tester) async {
         final ScrollController controller = ScrollController();
+        addTearDown(controller.dispose);
         await tester.pumpWidget(
           Directionality(
             textDirection: TextDirection.ltr,
@@ -121,6 +124,7 @@ void main() {
 
       testWidgets('scrolls beyond viewportMainAxisExtent', (WidgetTester tester) async {
         final ScrollController controller = ScrollController();
+        addTearDown(controller.dispose);
         final List<Widget> slivers = <Widget>[
           sliverBox,
           SliverFillRemaining(
@@ -140,6 +144,7 @@ void main() {
     group('hasScrollBody: false', () {
       testWidgets('does not extend past viewportMainAxisExtent', (WidgetTester tester) async {
         final ScrollController controller = ScrollController();
+        addTearDown(controller.dispose);
         final List<Widget> slivers = <Widget>[
           sliverBox,
           SliverFillRemaining(
@@ -184,7 +189,7 @@ void main() {
           sliverBox,
           SliverFillRemaining(
             hasScrollBody: false,
-            child: Container(
+            child: ColoredBox(
               key: key,
               color: Colors.blue,
               child: Align(
@@ -387,7 +392,7 @@ void main() {
             SliverFillRemaining(
               hasScrollBody: false,
               fillOverscroll: true,
-              child: Container(
+              child: ColoredBox(
                 key: key,
                 color: Colors.blue,
                 child: Align(
@@ -430,6 +435,7 @@ void main() {
         testWidgets('extent is overridden by child size and overscroll if precedingScrollExtent > viewportMainAxisExtent', (WidgetTester tester) async {
           final GlobalKey key = GlobalKey();
           final ScrollController controller = ScrollController();
+          addTearDown(controller.dispose);
           final List<Widget> slivers = <Widget>[
             SliverFixedExtentList(
               itemExtent: 150,
@@ -494,6 +500,7 @@ void main() {
         testWidgets('fillOverscroll works when child has no size and precedingScrollExtent > viewportMainAxisExtent', (WidgetTester tester) async {
           final GlobalKey key = GlobalKey();
           final ScrollController controller = ScrollController();
+          addTearDown(controller.dispose);
           final List<Widget> slivers = <Widget>[
             SliverFixedExtentList(
               itemExtent: 150,
@@ -673,7 +680,7 @@ void main() {
             SliverFillRemaining(
               hasScrollBody: false,
               fillOverscroll: true,
-              child: Container(
+              child: ColoredBox(
                 key: key,
                 color: Colors.blue,
                 child: Align(
@@ -708,6 +715,7 @@ void main() {
         testWidgets('extent is overridden by child size if precedingScrollExtent > viewportMainAxisExtent', (WidgetTester tester) async {
           final GlobalKey key = GlobalKey();
           final ScrollController controller = ScrollController();
+          addTearDown(controller.dispose);
           final List<Widget> slivers = <Widget>[
             SliverFixedExtentList(
               itemExtent: 150,
@@ -765,6 +773,7 @@ void main() {
         testWidgets('child has no size and precedingScrollExtent > viewportMainAxisExtent', (WidgetTester tester) async {
           final GlobalKey key = GlobalKey();
           final ScrollController controller = ScrollController();
+          addTearDown(controller.dispose);
           final List<Widget> slivers = <Widget>[
             SliverFixedExtentList(
               itemExtent: 150,
