@@ -6,6 +6,9 @@ import 'package:flutter/material.dart';
 import 'package:flutter/rendering.dart';
 import 'package:flutter_test/flutter_test.dart';
 
+import '../rendering/sliver_utils.dart';
+
+
 const double VIEWPORT_HEIGHT = 600;
 const double VIEWPORT_WIDTH = 300;
 
@@ -13,6 +16,7 @@ void main() {
   testWidgets('SliverMainAxisGroup is laid out properly', (WidgetTester tester) async {
     final List<int> items = List<int>.generate(20, (int i) => i);
     final ScrollController controller = ScrollController();
+    addTearDown(controller.dispose);
 
     await tester.pumpWidget(
       _buildSliverMainAxisGroup(
@@ -64,6 +68,7 @@ void main() {
   testWidgets('SliverMainAxisGroup is laid out properly when reversed', (WidgetTester tester) async {
     final List<int> items = List<int>.generate(20, (int i) => i);
     final ScrollController controller = ScrollController();
+    addTearDown(controller.dispose);
 
     await tester.pumpWidget(
       _buildSliverMainAxisGroup(
@@ -116,6 +121,7 @@ void main() {
   testWidgets('SliverMainAxisGroup is laid out properly when horizontal', (WidgetTester tester) async {
     final List<int> items = List<int>.generate(20, (int i) => i);
     final ScrollController controller = ScrollController();
+    addTearDown(controller.dispose);
 
     await tester.pumpWidget(
       _buildSliverMainAxisGroup(
@@ -173,6 +179,7 @@ void main() {
   testWidgets('SliverMainAxisGroup is laid out properly when horizontal, reversed', (WidgetTester tester) async {
     final List<int> items = List<int>.generate(20, (int i) => i);
     final ScrollController controller = ScrollController();
+    addTearDown(controller.dispose);
 
     await tester.pumpWidget(
       _buildSliverMainAxisGroup(
@@ -231,6 +238,7 @@ void main() {
   testWidgets('Hit test works properly on various parts of SliverMainAxisGroup', (WidgetTester tester) async {
     final List<int> items = List<int>.generate(20, (int i) => i);
     final ScrollController controller = ScrollController();
+    addTearDown(controller.dispose);
 
     String? clickedTile;
 
@@ -318,6 +326,8 @@ void main() {
 
   testWidgets('visitChildrenForSemantics visits children in the correct order', (WidgetTester tester) async {
     final ScrollController controller = ScrollController();
+    addTearDown(controller.dispose);
+
     await tester.pumpWidget(_buildSliverMainAxisGroup(
       controller: controller,
       slivers: const <Widget>[
@@ -343,6 +353,8 @@ void main() {
 
   testWidgets('SliverPinnedPersistentHeader is painted within bounds of SliverMainAxisGroup', (WidgetTester tester) async {
     final ScrollController controller = ScrollController();
+    addTearDown(controller.dispose);
+
     await tester.pumpWidget(_buildSliverMainAxisGroup(
       controller: controller,
       slivers: <Widget>[
@@ -370,6 +382,8 @@ void main() {
 
   testWidgets('SliverFloatingPersistentHeader is painted within bounds of SliverMainAxisGroup', (WidgetTester tester) async {
     final ScrollController controller = ScrollController();
+    addTearDown(controller.dispose);
+
     await tester.pumpWidget(_buildSliverMainAxisGroup(
       controller: controller,
       slivers: <Widget>[
@@ -399,6 +413,8 @@ void main() {
 
   testWidgets('SliverPinnedPersistentHeader is painted within bounds of SliverMainAxisGroup with different minExtent/maxExtent', (WidgetTester tester) async {
     final ScrollController controller = ScrollController();
+    addTearDown(controller.dispose);
+
     await tester.pumpWidget(_buildSliverMainAxisGroup(
       controller: controller,
       slivers: <Widget>[
@@ -428,6 +444,8 @@ void main() {
 
   testWidgets('SliverFloatingPersistentHeader is painted within bounds of SliverMainAxisGroup with different minExtent/maxExtent', (WidgetTester tester) async {
     final ScrollController controller = ScrollController();
+    addTearDown(controller.dispose);
+
     await tester.pumpWidget(_buildSliverMainAxisGroup(
       controller: controller,
       slivers: <Widget>[
@@ -464,6 +482,8 @@ void main() {
 
   testWidgets('SliverPinnedFloatingPersistentHeader is painted within bounds of SliverMainAxisGroup with different minExtent/maxExtent', (WidgetTester tester) async {
     final ScrollController controller = ScrollController();
+    addTearDown(controller.dispose);
+
     await tester.pumpWidget(_buildSliverMainAxisGroup(
       controller: controller,
       slivers: <Widget>[
@@ -501,6 +521,8 @@ void main() {
 
   testWidgets('SliverAppBar with floating: false, pinned: false, snap: false is painted within bounds of SliverMainAxisGroup', (WidgetTester tester) async {
     final ScrollController controller = ScrollController();
+    addTearDown(controller.dispose);
+
     await tester.pumpWidget(_buildSliverMainAxisGroup(
       controller: controller,
       slivers: <Widget>[
@@ -531,6 +553,8 @@ void main() {
 
     testWidgets('SliverAppBar with floating: true, pinned: false, snap: true is painted within bounds of SliverMainAxisGroup', (WidgetTester tester) async {
     final ScrollController controller = ScrollController();
+    addTearDown(controller.dispose);
+
     await tester.pumpWidget(_buildSliverMainAxisGroup(
       controller: controller,
       slivers: <Widget>[
@@ -571,6 +595,8 @@ void main() {
 
   testWidgets('SliverAppBar with floating: true, pinned: true, snap: true is painted within bounds of SliverMainAxisGroup', (WidgetTester tester) async {
     final ScrollController controller = ScrollController();
+    addTearDown(controller.dispose);
+
     await tester.pumpWidget(_buildSliverMainAxisGroup(
       controller: controller,
       slivers: <Widget>[
@@ -608,6 +634,64 @@ void main() {
     expect(renderHeader.geometry!.paintExtent, equals(60.0));
     expect((renderHeader.parentData! as SliverPhysicalParentData).paintOffset.dy, equals(-50.0));
   });
+
+  testWidgets('SliverMainAxisGroup skips painting invisible children', (WidgetTester tester) async {
+    final ScrollController controller = ScrollController();
+    addTearDown(controller.dispose);
+
+    int counter = 0;
+    void incrementCounter() {
+      counter += 1;
+    }
+
+    await tester.pumpWidget(
+      _buildSliverMainAxisGroup(
+        controller: controller,
+        slivers: <Widget>[
+          MockSliverToBoxAdapter(
+            incrementCounter: incrementCounter,
+            child: Container(
+              height: 1000,
+              decoration: const BoxDecoration(color: Colors.amber),
+            ),
+          ),
+          MockSliverToBoxAdapter(
+            incrementCounter: incrementCounter,
+            child: Container(
+              height: 400,
+              decoration: const BoxDecoration(color: Colors.amber)
+            ),
+          ),
+          MockSliverToBoxAdapter(
+            incrementCounter: incrementCounter,
+            child: Container(
+              height: 500,
+              decoration: const BoxDecoration(color: Colors.amber)
+            ),
+          ),
+          MockSliverToBoxAdapter(
+            incrementCounter: incrementCounter,
+            child: Container(
+              height: 300,
+              decoration: const BoxDecoration(color: Colors.amber)
+            ),
+          ),
+        ],
+      ),
+    );
+
+    // Can only see top sliver.
+    expect(counter, equals(1));
+
+    // Reset paint counter.
+    counter = 0;
+    controller.jumpTo(1000);
+    await tester.pumpAndSettle();
+
+    // Can only see second and third slivers.
+    expect(controller.offset, 1000);
+    expect(counter, equals(2));
+  });
 }
 
 Widget _buildSliverList({
