@@ -7,7 +7,7 @@ import 'package:flutter/rendering.dart';
 import 'package:flutter_test/flutter_test.dart';
 
 class TestStatefulWidget extends StatefulWidget {
-  const TestStatefulWidget({ Key? key }) : super(key: key);
+  const TestStatefulWidget({ super.key });
 
   @override
   TestStatefulWidgetState createState() => TestStatefulWidgetState();
@@ -19,7 +19,7 @@ class TestStatefulWidgetState extends State<TestStatefulWidget> {
 }
 
 class TestChildWidget extends StatefulWidget {
-  const TestChildWidget({ Key? key }) : super(key: key);
+  const TestChildWidget({ super.key });
 
   @override
   TestChildState createState() => TestChildState();
@@ -45,6 +45,7 @@ void main() {
       ),
     );
   });
+
   testWidgets('Table widget - control test', (WidgetTester tester) async {
     Future<void> run(TextDirection textDirection) async {
       await tester.pumpWidget(
@@ -879,24 +880,24 @@ void main() {
     expect(
       element.toStringDeep(minLevel: DiagnosticLevel.info),
       equalsIgnoringHashCodes(
-        'Table-[GlobalKey#00000](dependencies: [Directionality], renderObject: RenderTable#00000)\n'
-        '├Text("A")\n'
+        'Table-[GlobalKey#00000](dependencies: [Directionality, MediaQuery], renderObject: RenderTable#00000)\n'
+        '├Text("A", dependencies: [MediaQuery])\n'
         '│└RichText(softWrap: wrapping at box width, maxLines: unlimited, text: "A", dependencies: [Directionality], renderObject: RenderParagraph#00000 relayoutBoundary=up1)\n'
-        '├Text("B")\n'
+        '├Text("B", dependencies: [MediaQuery])\n'
         '│└RichText(softWrap: wrapping at box width, maxLines: unlimited, text: "B", dependencies: [Directionality], renderObject: RenderParagraph#00000 relayoutBoundary=up1)\n'
-        '├Text("C")\n'
+        '├Text("C", dependencies: [MediaQuery])\n'
         '│└RichText(softWrap: wrapping at box width, maxLines: unlimited, text: "C", dependencies: [Directionality], renderObject: RenderParagraph#00000 relayoutBoundary=up1)\n'
-        '├Text("D")\n'
+        '├Text("D", dependencies: [MediaQuery])\n'
         '│└RichText(softWrap: wrapping at box width, maxLines: unlimited, text: "D", dependencies: [Directionality], renderObject: RenderParagraph#00000 relayoutBoundary=up1)\n'
-        '├Text("EEE")\n'
+        '├Text("EEE", dependencies: [MediaQuery])\n'
         '│└RichText(softWrap: wrapping at box width, maxLines: unlimited, text: "EEE", dependencies: [Directionality], renderObject: RenderParagraph#00000 relayoutBoundary=up1)\n'
-        '├Text("F")\n'
+        '├Text("F", dependencies: [MediaQuery])\n'
         '│└RichText(softWrap: wrapping at box width, maxLines: unlimited, text: "F", dependencies: [Directionality], renderObject: RenderParagraph#00000 relayoutBoundary=up1)\n'
-        '├Text("G")\n'
+        '├Text("G", dependencies: [MediaQuery])\n'
         '│└RichText(softWrap: wrapping at box width, maxLines: unlimited, text: "G", dependencies: [Directionality], renderObject: RenderParagraph#00000 relayoutBoundary=up1)\n'
-        '├Text("H")\n'
+        '├Text("H", dependencies: [MediaQuery])\n'
         '│└RichText(softWrap: wrapping at box width, maxLines: unlimited, text: "H", dependencies: [Directionality], renderObject: RenderParagraph#00000 relayoutBoundary=up1)\n'
-        '└Text("III")\n'
+        '└Text("III", dependencies: [MediaQuery])\n'
         ' └RichText(softWrap: wrapping at box width, maxLines: unlimited, text: "III", dependencies: [Directionality], renderObject: RenderParagraph#00000 relayoutBoundary=up1)\n',
       ),
     );
@@ -940,7 +941,7 @@ void main() {
   });
 
   testWidgets(
-    'Table widget requires all TableRows to have non-null children',
+    'Table widget requires all TableRows to have same number of children',
     (WidgetTester tester) async {
       FlutterError? error;
       try {
@@ -959,7 +960,7 @@ void main() {
         error = e;
       } finally {
         expect(error, isNotNull);
-        expect(error!.toStringDeep(), contains('The children property of TableRow must not be null.'));
+        expect(error!.toStringDeep(), contains('Table contains irregular row lengths.'));
       }
     },
   );
@@ -1033,5 +1034,92 @@ void main() {
     expect(find.text('Hello'), findsOneWidget);
   });
 
+  testWidgets('TableRow with no children throws an error message', (WidgetTester tester) async {
+    // Regression test for https://github.com/flutter/flutter/issues/119541.
+    String result = 'no exception';
+
+    // Test TableRow with children.
+    try {
+      await tester.pumpWidget(Directionality(
+        textDirection: TextDirection.ltr,
+        child: Table(
+          children: const <TableRow>[
+            TableRow(
+              children: <Widget>[
+                Text('A'),
+              ],
+            ),
+          ],
+        ),
+      ));
+    } on FlutterError catch (e) {
+      result = e.toString();
+    }
+
+    expect(result, 'no exception');
+
+    // Test TableRow with no children.
+    try {
+      await tester.pumpWidget(Directionality(
+        textDirection: TextDirection.ltr,
+        child: Table(
+          children: const <TableRow>[
+            TableRow(),
+          ],
+        ),
+      ));
+    } on FlutterError catch (e) {
+      result = e.toString();
+    }
+
+    expect(
+      result,
+      'One or more TableRow have no children.\n'
+      'Every TableRow in a Table must have at least one child, so there is no empty row.',
+    );
+  });
+
+    testWidgets('Set defaultVerticalAlignment to intrisic height and check their heights', (WidgetTester tester) async {
+    final Widget table = Directionality(
+      textDirection: TextDirection.ltr,
+      child: Table(
+        defaultVerticalAlignment: TableCellVerticalAlignment.intrinsicHeight,
+        children: const <TableRow>[
+          TableRow(
+            children: <Widget>[
+              SizedBox(height: 100, child: Text('A')),
+              SizedBox(height: 200, child: Text('B')),
+            ],
+          ),
+          TableRow(
+            children: <Widget>[
+              SizedBox(height: 200, child: Text('C')),
+              SizedBox(height: 300, child: Text('D')),
+            ],
+          ),
+        ],
+      ),
+    );
+
+    // load and check if render object was created.
+    await tester.pumpWidget(table);
+    expect(find.byWidget(table), findsOneWidget);
+
+    final RenderBox boxA = tester.renderObject(find.text('A'));
+    final RenderBox boxB = tester.renderObject(find.text('B'));
+
+    // boxA and boxB must be the same height, even though boxB is higher than boxA initially.
+    expect(boxA.size.height, equals(boxB.size.height));
+
+    final RenderBox boxC = tester.renderObject(find.text('C'));
+    final RenderBox boxD = tester.renderObject(find.text('D'));
+
+    // boxC and boxD must be the same height, even though boxD is higher than boxC initially.
+    expect(boxC.size.height, equals(boxD.size.height));
+
+    // boxD (300.0h) should be higher than boxA (200.0h) which has the same height of boxB.
+    expect(boxD.size.height, greaterThan(boxA.size.height));
+  });
+
   // TODO(ianh): Test handling of TableCell object
 }
