@@ -146,6 +146,7 @@ void main() {
 
   testWidgets('AlignTransition animates', (WidgetTester tester) async {
     final AnimationController controller = AnimationController(vsync: const TestVSync());
+    addTearDown(controller.dispose);
     final Animation<Alignment> alignmentTween = AlignmentTween(
       begin: Alignment.centerLeft,
       end: Alignment.bottomRight,
@@ -170,6 +171,7 @@ void main() {
 
   testWidgets('RelativePositionedTransition animates', (WidgetTester tester) async {
     final AnimationController controller = AnimationController(vsync: const TestVSync());
+    addTearDown(controller.dispose);
     final Animation<Rect?> rectTween = RectTween(
       begin: const Rect.fromLTWH(0, 0, 30, 40),
       end: const Rect.fromLTWH(100, 200, 100, 200),
@@ -216,6 +218,7 @@ void main() {
 
   testWidgets('AlignTransition keeps width and height factors', (WidgetTester tester) async {
     final AnimationController controller = AnimationController(vsync: const TestVSync());
+    addTearDown(controller.dispose);
     final Animation<Alignment> alignmentTween = AlignmentTween(
       begin: Alignment.centerLeft,
       end: Alignment.bottomRight,
@@ -237,12 +240,14 @@ void main() {
 
   testWidgets('SizeTransition clamps negative size factors - vertical axis', (WidgetTester tester) async {
     final AnimationController controller = AnimationController(vsync: const TestVSync());
+    addTearDown(controller.dispose);
     final Animation<double> animation = Tween<double>(begin: -1.0, end: 1.0).animate(controller);
 
     final Widget widget =  Directionality(
       textDirection: TextDirection.ltr,
       child: SizeTransition(
         sizeFactor: animation,
+        fixedCrossAxisSizeFactor: 2.0,
         child: const Text('Ready'),
       ),
     );
@@ -251,22 +256,27 @@ void main() {
 
     final RenderPositionedBox actualPositionedBox = tester.renderObject(find.byType(Align));
     expect(actualPositionedBox.heightFactor, 0.0);
+    expect(actualPositionedBox.widthFactor, 2.0);
 
     controller.value = 0.0;
     await tester.pump();
     expect(actualPositionedBox.heightFactor, 0.0);
+    expect(actualPositionedBox.widthFactor, 2.0);
 
     controller.value = 0.75;
     await tester.pump();
     expect(actualPositionedBox.heightFactor, 0.5);
+    expect(actualPositionedBox.widthFactor, 2.0);
 
     controller.value = 1.0;
     await tester.pump();
     expect(actualPositionedBox.heightFactor, 1.0);
+    expect(actualPositionedBox.widthFactor, 2.0);
   });
 
   testWidgets('SizeTransition clamps negative size factors - horizontal axis', (WidgetTester tester) async {
     final AnimationController controller = AnimationController(vsync: const TestVSync());
+    addTearDown(controller.dispose);
     final Animation<double> animation = Tween<double>(begin: -1.0, end: 1.0).animate(controller);
 
     final Widget widget =  Directionality(
@@ -274,6 +284,7 @@ void main() {
       child: SizeTransition(
         axis: Axis.horizontal,
         sizeFactor: animation,
+        fixedCrossAxisSizeFactor: 1.0,
         child: const Text('Ready'),
       ),
     );
@@ -282,22 +293,209 @@ void main() {
 
     final RenderPositionedBox actualPositionedBox = tester.renderObject(find.byType(Align));
     expect(actualPositionedBox.widthFactor, 0.0);
+    expect(actualPositionedBox.heightFactor, 1.0);
 
     controller.value = 0.0;
     await tester.pump();
     expect(actualPositionedBox.widthFactor, 0.0);
+    expect(actualPositionedBox.heightFactor, 1.0);
 
     controller.value = 0.75;
     await tester.pump();
     expect(actualPositionedBox.widthFactor, 0.5);
+    expect(actualPositionedBox.heightFactor, 1.0);
 
     controller.value = 1.0;
     await tester.pump();
     expect(actualPositionedBox.widthFactor, 1.0);
+    expect(actualPositionedBox.heightFactor, 1.0);
+  });
+
+  testWidgets('SizeTransition with fixedCrossAxisSizeFactor should size its cross axis from its children - vertical axis', (WidgetTester tester) async {
+    final AnimationController controller = AnimationController(vsync: const TestVSync());
+    addTearDown(controller.dispose);
+    final Animation<double> animation = Tween<double>(begin: 0, end: 1.0).animate(controller);
+
+    const Key key = Key('key');
+
+    final Widget widget =  Directionality(
+      textDirection: TextDirection.ltr,
+      child: Center(
+        child: SizedBox(
+          key: key,
+          child: SizeTransition(
+            sizeFactor: animation,
+            fixedCrossAxisSizeFactor: 1.0,
+            child: const SizedBox.square(dimension: 100),
+          ),
+        ),
+      ),
+    );
+
+    await tester.pumpWidget(widget);
+
+    final RenderPositionedBox actualPositionedBox = tester.renderObject(find.byType(Align));
+    expect(actualPositionedBox.heightFactor, 0.0);
+    expect(actualPositionedBox.widthFactor, 1.0);
+    expect(tester.getSize(find.byKey(key)), const Size(100, 0));
+
+    controller.value = 0.0;
+    await tester.pump();
+    expect(actualPositionedBox.heightFactor, 0.0);
+    expect(actualPositionedBox.widthFactor, 1.0);
+    expect(tester.getSize(find.byKey(key)), const Size(100, 0));
+
+    controller.value = 0.5;
+    await tester.pump();
+    expect(actualPositionedBox.heightFactor, 0.5);
+    expect(actualPositionedBox.widthFactor, 1.0);
+    expect(tester.getSize(find.byKey(key)), const Size(100, 50));
+
+    controller.value = 1.0;
+    await tester.pump();
+    expect(actualPositionedBox.heightFactor, 1.0);
+    expect(actualPositionedBox.widthFactor, 1.0);
+    expect(tester.getSize(find.byKey(key)), const Size.square(100));
+
+    controller.value = 0.5;
+    await tester.pump();
+    expect(actualPositionedBox.heightFactor, 0.5);
+    expect(actualPositionedBox.widthFactor, 1.0);
+    expect(tester.getSize(find.byKey(key)), const Size(100, 50));
+
+    controller.value = 0.0;
+    await tester.pump();
+    expect(actualPositionedBox.heightFactor, 0.0);
+    expect(actualPositionedBox.widthFactor, 1.0);
+    expect(tester.getSize(find.byKey(key)), const Size(100, 0));
+  });
+
+  testWidgets('SizeTransition with fixedCrossAxisSizeFactor should size its cross axis from its children - horizontal axis', (WidgetTester tester) async {
+    final AnimationController controller = AnimationController(vsync: const TestVSync());
+    addTearDown(controller.dispose);
+    final Animation<double> animation = Tween<double>(begin: 0.0, end: 1.0).animate(controller);
+
+    const Key key = Key('key');
+
+    final Widget widget =  Directionality(
+      textDirection: TextDirection.ltr,
+      child: Center(
+        child: SizedBox(
+          key: key,
+          child: SizeTransition(
+            axis: Axis.horizontal,
+            sizeFactor: animation,
+            fixedCrossAxisSizeFactor: 1.0,
+            child: const SizedBox.square(dimension: 100),
+          ),
+        ),
+      ),
+    );
+
+    await tester.pumpWidget(widget);
+
+    final RenderPositionedBox actualPositionedBox = tester.renderObject(find.byType(Align));
+    expect(actualPositionedBox.heightFactor, 1.0);
+    expect(actualPositionedBox.widthFactor, 0.0);
+    expect(tester.getSize(find.byKey(key)), const Size(0, 100));
+
+    controller.value = 0.0;
+    await tester.pump();
+    expect(actualPositionedBox.heightFactor, 1.0);
+    expect(actualPositionedBox.widthFactor, 0.0);
+    expect(tester.getSize(find.byKey(key)), const Size(0, 100));
+
+    controller.value = 0.5;
+    await tester.pump();
+    expect(actualPositionedBox.heightFactor, 1.0);
+    expect(actualPositionedBox.widthFactor, 0.5);
+    expect(tester.getSize(find.byKey(key)), const Size(50, 100));
+
+    controller.value = 1.0;
+    await tester.pump();
+    expect(actualPositionedBox.heightFactor, 1.0);
+    expect(actualPositionedBox.widthFactor, 1.0);
+    expect(tester.getSize(find.byKey(key)), const Size.square(100));
+
+    controller.value = 0.5;
+    await tester.pump();
+    expect(actualPositionedBox.heightFactor, 1.0);
+    expect(actualPositionedBox.widthFactor, 0.5);
+    expect(tester.getSize(find.byKey(key)), const Size(50, 100));
+
+    controller.value = 0.0;
+    await tester.pump();
+    expect(actualPositionedBox.heightFactor, 1.0);
+    expect(actualPositionedBox.widthFactor, 0.0);
+    expect(tester.getSize(find.byKey(key)), const Size(0, 100));
+  });
+
+  testWidgets('MatrixTransition animates', (WidgetTester tester) async {
+    final AnimationController controller = AnimationController(vsync: const TestVSync());
+    addTearDown(controller.dispose);
+    final Widget widget = MatrixTransition(
+      alignment: Alignment.topRight,
+      onTransform: (double value) => Matrix4.translationValues(value, value, value),
+      animation: controller,
+      child: const Text(
+        'Matrix',
+        textDirection: TextDirection.ltr,
+      ),
+    );
+
+    await tester.pumpWidget(widget);
+    Transform actualTransformedBox = tester.widget(find.byType(Transform));
+    Matrix4 actualTransform = actualTransformedBox.transform;
+    expect(actualTransform, equals(Matrix4.rotationZ(0.0)));
+
+    controller.value = 0.5;
+    await tester.pump();
+    actualTransformedBox = tester.widget(find.byType(Transform));
+    actualTransform = actualTransformedBox.transform;
+    expect(actualTransform, Matrix4.fromList(<double>[
+      1.0,  0.0, 0.0, 0.5,
+      0.0,  1.0, 0.0, 0.5,
+      0.0,  0.0, 1.0, 0.5,
+      0.0,  0.0, 0.0, 1.0,
+    ])..transpose());
+
+    controller.value = 0.75;
+    await tester.pump();
+    actualTransformedBox = tester.widget(find.byType(Transform));
+    actualTransform = actualTransformedBox.transform;
+    expect(actualTransform, Matrix4.fromList(<double>[
+      1.0, 0.0, 0.0, 0.75,
+      0.0, 1.0, 0.0, 0.75,
+      0.0, 0.0, 1.0, 0.75,
+      0.0, 0.0, 0.0, 1.0,
+    ])..transpose());
+  });
+
+  testWidgets('MatrixTransition maintains chosen alignment during animation', (WidgetTester tester) async {
+    final AnimationController controller = AnimationController(vsync: const TestVSync());
+    addTearDown(controller.dispose);
+    final Widget widget = MatrixTransition(
+      alignment: Alignment.topRight,
+      onTransform: (double value) => Matrix4.identity(),
+      animation: controller,
+      child: const Text('Matrix', textDirection: TextDirection.ltr),
+    );
+
+    await tester.pumpWidget(widget);
+    MatrixTransition actualTransformedBox = tester.widget(find.byType(MatrixTransition));
+    Alignment actualAlignment = actualTransformedBox.alignment;
+    expect(actualAlignment, Alignment.topRight);
+
+    controller.value = 0.5;
+    await tester.pump();
+    actualTransformedBox = tester.widget(find.byType(MatrixTransition));
+    actualAlignment = actualTransformedBox.alignment;
+    expect(actualAlignment, Alignment.topRight);
   });
 
   testWidgets('RotationTransition animates', (WidgetTester tester) async {
     final AnimationController controller = AnimationController(vsync: const TestVSync());
+    addTearDown(controller.dispose);
     final Widget widget = RotationTransition(
       alignment: Alignment.topRight,
       turns: controller,
@@ -316,27 +514,28 @@ void main() {
     await tester.pump();
     actualRotatedBox = tester.widget(find.byType(Transform));
     actualTurns = actualRotatedBox.transform;
-    expect(actualTurns, Matrix4.fromList(<double>[
+    expect(actualTurns, matrixMoreOrLessEquals(Matrix4.fromList(<double>[
      -1.0,  0.0, 0.0, 0.0,
       0.0, -1.0, 0.0, 0.0,
       0.0,  0.0, 1.0, 0.0,
       0.0,  0.0, 0.0, 1.0,
-    ])..transpose());
+    ])..transpose()));
 
     controller.value = 0.75;
     await tester.pump();
     actualRotatedBox = tester.widget(find.byType(Transform));
     actualTurns = actualRotatedBox.transform;
-    expect(actualTurns, Matrix4.fromList(<double>[
+    expect(actualTurns, matrixMoreOrLessEquals(Matrix4.fromList(<double>[
       0.0, 1.0, 0.0, 0.0,
      -1.0, 0.0, 0.0, 0.0,
       0.0, 0.0, 1.0, 0.0,
       0.0, 0.0, 0.0, 1.0,
-    ])..transpose());
+    ])..transpose()));
   });
 
   testWidgets('RotationTransition maintains chosen alignment during animation', (WidgetTester tester) async {
     final AnimationController controller = AnimationController(vsync: const TestVSync());
+    addTearDown(controller.dispose);
     final Widget widget = RotationTransition(
       alignment: Alignment.topRight,
       turns: controller,
@@ -367,6 +566,7 @@ void main() {
     }
     testWidgets('animates', (WidgetTester tester) async {
       final AnimationController controller = AnimationController(vsync: const TestVSync());
+      addTearDown(controller.dispose);
       final Animation<double> animation = Tween<double>(begin: 0.0, end: 1.0).animate(controller);
       final Widget widget =  Directionality(
         textDirection: TextDirection.ltr,
@@ -410,6 +610,7 @@ void main() {
     }
     testWidgets('animates', (WidgetTester tester) async {
       final AnimationController controller = AnimationController(vsync: const TestVSync());
+      addTearDown(controller.dispose);
       final Animation<double> animation = Tween<double>(begin: 0.0, end: 1.0).animate(controller);
       final Widget widget = Localizations(
         locale: const Locale('en', 'us'),
@@ -457,9 +658,74 @@ void main() {
     });
   });
 
+  group('MatrixTransition', () {
+    testWidgets('uses ImageFilter when provided with FilterQuality argument', (WidgetTester tester) async {
+      final AnimationController controller = AnimationController(vsync: const TestVSync());
+      addTearDown(controller.dispose);
+      final Animation<double> animation = Tween<double>(begin: 0.0, end: 1.0).animate(controller);
+      final Widget widget = Directionality(
+        textDirection: TextDirection.ltr,
+        child: MatrixTransition(
+          animation: animation,
+          onTransform: (double value) => Matrix4.identity(),
+          filterQuality: FilterQuality.none,
+          child: const Text('Matrix Transition'),
+        ),
+      );
+
+      await tester.pumpWidget(widget);
+
+      // Validate that expensive layer is not left in tree before animation has started.
+      expect(tester.layers, isNot(contains(isA<ImageFilterLayer>())));
+
+      controller.value = 0.25;
+      await tester.pump();
+
+      expect(
+          tester.layers,
+          contains(isA<ImageFilterLayer>().having(
+            (ImageFilterLayer layer) => layer.imageFilter.toString(),
+            'image filter',
+            startsWith('ImageFilter.matrix('),
+          )),
+      );
+
+      controller.value = 0.5;
+      await tester.pump();
+
+      expect(
+          tester.layers,
+          contains(isA<ImageFilterLayer>().having(
+            (ImageFilterLayer layer) => layer.imageFilter.toString(),
+            'image filter',
+            startsWith('ImageFilter.matrix('),
+          )),
+      );
+
+      controller.value = 0.75;
+      await tester.pump();
+
+      expect(
+          tester.layers,
+          contains(isA<ImageFilterLayer>().having(
+            (ImageFilterLayer layer) => layer.imageFilter.toString(),
+            'image filter',
+            startsWith('ImageFilter.matrix('),
+          )),
+      );
+
+      controller.value = 1;
+      await tester.pump();
+
+      // Validate that expensive layer is not left in tree after animation has finished.
+      expect(tester.layers, isNot(contains(isA<ImageFilterLayer>())));
+    });
+  });
+
   group('ScaleTransition', () {
     testWidgets('uses ImageFilter when provided with FilterQuality argument', (WidgetTester tester) async {
       final AnimationController controller = AnimationController(vsync: const TestVSync());
+      addTearDown(controller.dispose);
       final Animation<double> animation = Tween<double>(begin: 0.0, end: 1.0).animate(controller);
       final Widget widget =  Directionality(
         textDirection: TextDirection.ltr,
@@ -513,6 +779,7 @@ void main() {
   group('RotationTransition', () {
     testWidgets('uses ImageFilter when provided with FilterQuality argument', (WidgetTester tester) async {
       final AnimationController controller = AnimationController(vsync: const TestVSync());
+      addTearDown(controller.dispose);
       final Animation<double> animation = Tween<double>(begin: 0.0, end: 1.0).animate(controller);
       final Widget widget =  Directionality(
         textDirection: TextDirection.ltr,
@@ -567,6 +834,8 @@ void main() {
     testWidgets('AnimatedBuilder rebuilds when changed', (WidgetTester tester) async {
       final GlobalKey<RedrawCounterState> redrawKey = GlobalKey<RedrawCounterState>();
       final ChangeNotifier notifier = ChangeNotifier();
+      addTearDown(notifier.dispose);
+
       await tester.pumpWidget(
         Directionality(
           textDirection: TextDirection.ltr,
@@ -596,6 +865,8 @@ void main() {
       final GlobalKey<RedrawCounterState> redrawKey = GlobalKey<RedrawCounterState>();
       final GlobalKey<RedrawCounterState> redrawKeyChild = GlobalKey<RedrawCounterState>();
       final ChangeNotifier notifier = ChangeNotifier();
+      addTearDown(notifier.dispose);
+
       await tester.pumpWidget(
         Directionality(
           textDirection: TextDirection.ltr,
@@ -629,6 +900,8 @@ void main() {
     testWidgets('ListenableBuilder rebuilds when changed', (WidgetTester tester) async {
       final GlobalKey<RedrawCounterState> redrawKey = GlobalKey<RedrawCounterState>();
       final ChangeNotifier notifier = ChangeNotifier();
+      addTearDown(notifier.dispose);
+
       await tester.pumpWidget(
         Directionality(
           textDirection: TextDirection.ltr,
@@ -658,6 +931,8 @@ void main() {
       final GlobalKey<RedrawCounterState> redrawKey = GlobalKey<RedrawCounterState>();
       final GlobalKey<RedrawCounterState> redrawKeyChild = GlobalKey<RedrawCounterState>();
       final ChangeNotifier notifier = ChangeNotifier();
+      addTearDown(notifier.dispose);
+
       await tester.pumpWidget(
         Directionality(
           textDirection: TextDirection.ltr,
