@@ -13,7 +13,6 @@ import 'package:flutter/foundation.dart';
 /// features are enabled, consider the [FakeAccessibilityFeatures.allOn]
 /// constant.
 @immutable
-// ignore: avoid_implementing_value_types
 class FakeAccessibilityFeatures implements AccessibilityFeatures {
   /// Creates a test instance of [AccessibilityFeatures].
   ///
@@ -259,7 +258,7 @@ class TestPlatformDispatcher implements PlatformDispatcher {
   set onPlatformBrightnessChanged(VoidCallback? callback) {
     _platformDispatcher.onPlatformBrightnessChanged = callback;
   }
-  /// Hides the real text scale factor and reports the given
+  /// Hides the real platform brightness and reports the given
   /// [platformBrightnessTestValue] instead.
   set platformBrightnessTestValue(Brightness platformBrightnessTestValue) { // ignore: avoid_setters_without_getters
     _platformBrightnessTestValue = platformBrightnessTestValue;
@@ -442,21 +441,6 @@ class TestPlatformDispatcher implements PlatformDispatcher {
     _platformDispatcher.sendPlatformMessage(name, data, callback);
   }
 
-  @Deprecated(
-    'Instead of calling this callback, use ServicesBinding.instance.channelBuffers.push. '
-    'This feature was deprecated after v2.1.0-10.0.pre.'
-  )
-  @override
-  PlatformMessageCallback? get onPlatformMessage => _platformDispatcher.onPlatformMessage;
-  @Deprecated(
-    'Instead of setting this callback, use ServicesBinding.instance.defaultBinaryMessenger.setMessageHandler. '
-    'This feature was deprecated after v2.1.0-10.0.pre.'
-  )
-  @override
-  set onPlatformMessage(PlatformMessageCallback? callback) {
-    _platformDispatcher.onPlatformMessage = callback;
-  }
-
   /// Delete any test value properties that have been set on this [TestPlatformDispatcher]
   /// and return to reporting the real [PlatformDispatcher] values for all
   /// [PlatformDispatcher] properties.
@@ -760,79 +744,19 @@ class TestFlutterView implements FlutterView {
     platformDispatcher.onMetricsChanged?.call();
   }
 
-  /// The physical geometry to use for this test.
-  ///
-  /// Defaults to the value provided by [FlutterView.physicalGeometry]. This
-  /// can only be set in a test environment to emulate different view
-  /// configurations. A standard [FlutterView] is not mutable from the framework.
-  ///
-  /// This property and [physicalSize] are dependent on one another. If both
-  /// properties are set through their test setters, the final result will be
-  /// that [physicalGeometry] determines the location and [physicalSize]
-  /// determines the size of the [physicalGeometry] [Rect]. If only
-  /// [physicalSize] is set, the final result is that the default value of
-  /// [physicalGeometry] determines the location and [physicalSize] determines
-  /// the size of the [physicalGeometry] [Rect]. If only [physicalGeometry]
-  /// is set, it will determine both the location and size of the
-  /// [physicalGeometry] [Rect].
-  ///
-  /// See also:
-  ///
-  ///   * [FlutterView.physicalGeometry] for the standard implementation
-  ///   * [resetPhysicalGeometry] to reset this value specifically
-  ///   * [reset] to reset all test values for this view
-  @override
-  Rect get physicalGeometry {
-    Rect value = _physicalGeometry ?? _view.physicalGeometry;
-    if (_physicalSize != null) {
-      value = value.topLeft & _physicalSize!;
-    }
-    return value;
-  }
-  Rect? _physicalGeometry;
-  set physicalGeometry(Rect value) {
-    _physicalGeometry = value;
-    platformDispatcher.onMetricsChanged?.call();
-  }
-
-  /// Resets [physicalGeometry] to the default value for this view.
-  ///
-  /// This will also reset [physicalSize] as the values are dependent
-  /// on one another.
-  void resetPhysicalGeometry() {
-    _physicalGeometry = null;
-    _physicalSize = null;
-    platformDispatcher.onMetricsChanged?.call();
-  }
-
   /// The physical size to use for this test.
   ///
   /// Defaults to the value provided by [FlutterView.physicalSize]. This
   /// can only be set in a test environment to emulate different view
   /// configurations. A standard [FlutterView] is not mutable from the framework.
   ///
-  /// This property and [physicalGeometry] are dependent on one another. If both
-  /// properties are set through their test setters, the final result will be
-  /// that [physicalGeometry] determines the location and [physicalSize]
-  /// determines the size of the [physicalGeometry] [Rect]. If only
-  /// [physicalSize] is set, the final result is that the default value of
-  /// [physicalGeometry] determines the location and [physicalSize] determines
-  /// the size of the [physicalGeometry] [Rect]. If only [physicalGeometry]
-  /// is set, it will determine both the location and size of the
-  /// [physicalGeometry] [Rect].
-  ///
   /// See also:
   ///
   ///   * [FlutterView.physicalSize] for the standard implementation
   ///   * [resetPhysicalSize] to reset this value specifically
   ///   * [reset] to reset all test values for this view
   @override
-  Size get physicalSize {
-    // This has to be able to default to `_view.physicalSize` as web_ui handles
-    // `physicalSize` and `physicalGeometry` differently than dart:ui, where
-    // the values are both based off of `physicalGeometry`.
-    return _physicalSize ?? _physicalGeometry?.size ?? _view.physicalSize;
-  }
+  Size get physicalSize => _physicalSize ?? _view.physicalSize;
   Size? _physicalSize;
   set physicalSize(Size value) {
     _physicalSize = value;
@@ -840,11 +764,9 @@ class TestFlutterView implements FlutterView {
   }
 
   /// Resets [physicalSize] to the default value for this view.
-  ///
-  /// This will also reset [physicalGeometry] as the values are dependent
-  /// on one another.
   void resetPhysicalSize() {
-    resetPhysicalGeometry();
+    _physicalSize = null;
+    platformDispatcher.onMetricsChanged?.call();
   }
 
   /// The system gesture insets to use for this test.
@@ -951,7 +873,8 @@ class TestFlutterView implements FlutterView {
   }
 
   @override
-  void render(Scene scene) {
+  void render(Scene scene, {Size? size}) {
+    // TODO(goderbauer): Wire through size after https://github.com/flutter/engine/pull/48090 rolled in.
     _view.render(scene);
   }
 
@@ -967,7 +890,6 @@ class TestFlutterView implements FlutterView {
   ///   * [resetDevicePixelRatio] to reset [devicePixelRatio] specifically
   ///   * [resetDisplayFeatures]  to reset [displayFeatures] specifically
   ///   * [resetPadding] to reset [padding] specifically
-  ///   * [resetPhysicalGeometry] to reset [physicalGeometry] specifically
   ///   * [resetPhysicalSize] to reset [physicalSize] specifically
   ///   * [resetSystemGestureInsets] to reset [systemGestureInsets] specifically
   ///   * [resetViewInsets] to reset [viewInsets] specifically
@@ -977,8 +899,7 @@ class TestFlutterView implements FlutterView {
     resetDevicePixelRatio();
     resetDisplayFeatures();
     resetPadding();
-    resetPhysicalGeometry();
-    // Skipping resetPhysicalSize because resetPhysicalGeometry resets both values.
+    resetPhysicalSize();
     resetSystemGestureInsets();
     resetViewInsets();
     resetViewPadding();
@@ -1173,7 +1094,7 @@ class _UnsupportedDisplay implements TestDisplay {
 ///   // Fake the desired properties of the TestWindow. All code running
 ///   // within this test will perceive the following fake text scale
 ///   // factor as the real text scale factor of the window.
-///   testBinding.window.textScaleFactorFakeValue = 2.5;
+///   testBinding.platformDispatcher.textScaleFactorTestValue = 2.5;
 ///
 ///   // Test code that depends on text scale factor here.
 /// });
@@ -1186,7 +1107,7 @@ class _UnsupportedDisplay implements TestDisplay {
 /// If a test needs to override a real [SingletonFlutterWindow] property and
 /// then later return to using the real [SingletonFlutterWindow] property,
 /// [TestWindow] provides methods to clear each individual test value, e.g.,
-/// [clearLocaleTestValue].
+/// [clearDevicePixelRatioTestValue].
 ///
 /// To clear all fake test values in a [TestWindow], consider using
 /// [clearAllTestValues].
@@ -1507,22 +1428,6 @@ class TestWindow implements SingletonFlutterWindow {
   )
   @override
   Locale get locale => platformDispatcher.locale;
-  /// Hides the real locale and reports the given [localeTestValue] instead.
-  @Deprecated(
-    'Use WidgetTester.platformDispatcher.localeTestValue instead. '
-    'This feature was deprecated after v2.11.0-0.0.pre.'
-  )
-  set localeTestValue(Locale localeTestValue) { // ignore: avoid_setters_without_getters
-    platformDispatcher.localeTestValue = localeTestValue;
-  }
-  @Deprecated(
-    'Use WidgetTester.platformDispatcher.clearLocaleTestValue() instead. '
-    'This feature was deprecated after v2.11.0-0.0.pre.'
-  )
-  /// Deletes any existing test locale and returns to using the real locale.
-  void clearLocaleTestValue() {
-    platformDispatcher.clearLocaleTestValue();
-  }
 
   @Deprecated(
     'Use WidgetTester.platformDispatcher.locales instead. '
@@ -1531,22 +1436,6 @@ class TestWindow implements SingletonFlutterWindow {
   )
   @override
   List<Locale> get locales => platformDispatcher.locales;
-  /// Hides the real locales and reports the given [localesTestValue] instead.
-  @Deprecated(
-    'Use WidgetTester.platformDispatcher.localesTestValue instead. '
-    'This feature was deprecated after v2.11.0-0.0.pre.'
-  )
-  set localesTestValue(List<Locale> localesTestValue) { // ignore: avoid_setters_without_getters
-    platformDispatcher.localesTestValue = localesTestValue;
-  }
-  /// Deletes any existing test locales and returns to using the real locales.
-  @Deprecated(
-    'Use WidgetTester.platformDispatcher.clearLocalesTestValue() instead. '
-    'This feature was deprecated after v2.11.0-0.0.pre.'
-  )
-  void clearLocalesTestValue() {
-    platformDispatcher.clearLocalesTestValue();
-  }
 
   @Deprecated(
     'Use WidgetTester.platformDispatcher.onLocaleChanged instead. '
@@ -1572,14 +1461,6 @@ class TestWindow implements SingletonFlutterWindow {
   )
   @override
   String get initialLifecycleState => platformDispatcher.initialLifecycleState;
-  /// Sets a faked initialLifecycleState for testing.
-  @Deprecated(
-    'Use WidgetTester.platformDispatcher.initialLifecycleStateTestValue instead. '
-    'This feature was deprecated after v2.11.0-0.0.pre.'
-  )
-  set initialLifecycleStateTestValue(String state) { // ignore: avoid_setters_without_getters
-    platformDispatcher.initialLifecycleStateTestValue = state;
-  }
 
   @Deprecated(
     'Use WidgetTester.platformDispatcher.textScaleFactor instead. '
@@ -1588,24 +1469,6 @@ class TestWindow implements SingletonFlutterWindow {
   )
   @override
   double get textScaleFactor => platformDispatcher.textScaleFactor;
-  /// Hides the real text scale factor and reports the given
-  /// [textScaleFactorTestValue] instead.
-  @Deprecated(
-    'Use WidgetTester.platformDispatcher.textScaleFactorTestValue instead. '
-    'This feature was deprecated after v2.11.0-0.0.pre.'
-  )
-  set textScaleFactorTestValue(double textScaleFactorTestValue) { // ignore: avoid_setters_without_getters
-    platformDispatcher.textScaleFactorTestValue = textScaleFactorTestValue;
-  }
-  /// Deletes any existing test text scale factor and returns to using the real
-  /// text scale factor.
-  @Deprecated(
-    'Use WidgetTester.platformDispatcher.clearTextScaleFactorTestValue() instead. '
-    'This feature was deprecated after v2.11.0-0.0.pre.'
-  )
-  void clearTextScaleFactorTestValue() {
-    platformDispatcher.clearTextScaleFactorTestValue();
-  }
 
   @Deprecated(
     'Use WidgetTester.platformDispatcher.platformBrightness instead. '
@@ -1630,24 +1493,6 @@ class TestWindow implements SingletonFlutterWindow {
   set onPlatformBrightnessChanged(VoidCallback? callback) {
     platformDispatcher.onPlatformBrightnessChanged = callback;
   }
-  /// Hides the real text scale factor and reports the given
-  /// [platformBrightnessTestValue] instead.
-  @Deprecated(
-    'Use WidgetTester.platformDispatcher.platformBrightnessTestValue instead. '
-    'This feature was deprecated after v2.11.0-0.0.pre.'
-  )
-  set platformBrightnessTestValue(Brightness platformBrightnessTestValue) { // ignore: avoid_setters_without_getters
-    platformDispatcher.platformBrightnessTestValue = platformBrightnessTestValue;
-  }
-  /// Deletes any existing test platform brightness and returns to using the
-  /// real platform brightness.
-  @Deprecated(
-    'Use WidgetTester.platformDispatcher.clearPlatformBrightnessTestValue() instead. '
-    'This feature was deprecated after v2.11.0-0.0.pre.'
-  )
-  void clearPlatformBrightnessTestValue() {
-    platformDispatcher.clearPlatformBrightnessTestValue();
-  }
 
   @Deprecated(
     'Use WidgetTester.platformDispatcher.alwaysUse24HourFormat instead. '
@@ -1656,24 +1501,6 @@ class TestWindow implements SingletonFlutterWindow {
   )
   @override
   bool get alwaysUse24HourFormat => platformDispatcher.alwaysUse24HourFormat;
-  /// Hides the real clock format and reports the given
-  /// [alwaysUse24HourFormatTestValue] instead.
-  @Deprecated(
-    'Use WidgetTester.platformDispatcher.alwaysUse24HourFormatTestValue instead. '
-    'This feature was deprecated after v2.11.0-0.0.pre.'
-  )
-  set alwaysUse24HourFormatTestValue(bool alwaysUse24HourFormatTestValue) { // ignore: avoid_setters_without_getters
-    platformDispatcher.alwaysUse24HourFormatTestValue = alwaysUse24HourFormatTestValue;
-  }
-  /// Deletes any existing test clock format and returns to using the real clock
-  /// format.
-  @Deprecated(
-    'Use WidgetTester.platformDispatcher.clearAlwaysUse24HourTestValue() instead. '
-    'This feature was deprecated after v2.11.0-0.0.pre.'
-  )
-  void clearAlwaysUse24HourTestValue() {
-    platformDispatcher.clearAlwaysUse24HourTestValue();
-  }
 
   @Deprecated(
     'Use WidgetTester.platformDispatcher.onTextScaleFactorChanged instead. '
@@ -1715,15 +1542,6 @@ class TestWindow implements SingletonFlutterWindow {
   )
   @override
   bool get brieflyShowPassword => platformDispatcher.brieflyShowPassword;
-  /// Hides the real [brieflyShowPassword] and reports the given
-  /// `brieflyShowPasswordTestValue` instead.
-  @Deprecated(
-    'Use WidgetTester.platformDispatcher.brieflyShowPasswordTestValue instead. '
-    'This feature was deprecated after v2.11.0-0.0.pre.'
-  )
-  set brieflyShowPasswordTestValue(bool brieflyShowPasswordTestValue) { // ignore: avoid_setters_without_getters
-    platformDispatcher.brieflyShowPasswordTestValue = brieflyShowPasswordTestValue;
-  }
 
   @Deprecated(
     'Use WidgetTester.platformDispatcher.onBeginFrame instead. '
@@ -1800,24 +1618,6 @@ class TestWindow implements SingletonFlutterWindow {
   )
   @override
   String get defaultRouteName => platformDispatcher.defaultRouteName;
-  /// Hides the real default route name and reports the given
-  /// [defaultRouteNameTestValue] instead.
-  @Deprecated(
-    'Use WidgetTester.platformDispatcher.defaultRouteNameTestValue instead. '
-    'This feature was deprecated after v2.11.0-0.0.pre.'
-  )
-  set defaultRouteNameTestValue(String defaultRouteNameTestValue) { // ignore: avoid_setters_without_getters
-    platformDispatcher.defaultRouteNameTestValue = defaultRouteNameTestValue;
-  }
-  /// Deletes any existing test default route name and returns to using the real
-  /// default route name.
-  @Deprecated(
-    'Use WidgetTester.platformDispatcher.clearDefaultRouteNameTestValue() instead. '
-    'This feature was deprecated after v2.11.0-0.0.pre.'
-  )
-  void clearDefaultRouteNameTestValue() {
-    platformDispatcher.clearDefaultRouteNameTestValue();
-  }
 
   @Deprecated(
     'Use WidgetTester.platformDispatcher.scheduleFrame() instead. '
@@ -1835,7 +1635,8 @@ class TestWindow implements SingletonFlutterWindow {
     'This feature was deprecated after v3.9.0-0.1.pre.'
   )
   @override
-  void render(Scene scene) {
+  void render(Scene scene, {Size? size}) {
+    // TODO(goderbauer): Wire through size after https://github.com/flutter/engine/pull/48090 rolled in.
     _view.render(scene);
   }
 
@@ -1846,24 +1647,6 @@ class TestWindow implements SingletonFlutterWindow {
   )
   @override
   bool get semanticsEnabled => platformDispatcher.semanticsEnabled;
-  /// Hides the real semantics enabled and reports the given
-  /// [semanticsEnabledTestValue] instead.
-  @Deprecated(
-    'Use WidgetTester.platformDispatcher.semanticsEnabledTestValue instead. '
-    'This feature was deprecated after v2.11.0-0.0.pre.'
-  )
-  set semanticsEnabledTestValue(bool semanticsEnabledTestValue) { // ignore: avoid_setters_without_getters
-    platformDispatcher.semanticsEnabledTestValue = semanticsEnabledTestValue;
-  }
-  /// Deletes any existing test semantics enabled and returns to using the real
-  /// semantics enabled.
-  @Deprecated(
-    'Use WidgetTester.platformDispatcher.clearSemanticsEnabledTestValue() instead. '
-    'This feature was deprecated after v2.11.0-0.0.pre.'
-  )
-  void clearSemanticsEnabledTestValue() {
-    platformDispatcher.clearSemanticsEnabledTestValue();
-  }
 
   @Deprecated(
     'Use WidgetTester.platformDispatcher.onSemanticsEnabledChanged instead. '
@@ -1889,27 +1672,6 @@ class TestWindow implements SingletonFlutterWindow {
   )
   @override
   AccessibilityFeatures get accessibilityFeatures => platformDispatcher.accessibilityFeatures;
-  /// Hides the real accessibility features and reports the given
-  /// [accessibilityFeaturesTestValue] instead.
-  ///
-  /// Consider using [FakeAccessibilityFeatures] to provide specific
-  /// values for the various accessibility features under test.
-  @Deprecated(
-    'Use WidgetTester.platformDispatcher.accessibilityFeaturesTestValue instead. '
-    'This feature was deprecated after v2.11.0-0.0.pre.'
-  )
-  set accessibilityFeaturesTestValue(AccessibilityFeatures accessibilityFeaturesTestValue) { // ignore: avoid_setters_without_getters
-    platformDispatcher.accessibilityFeaturesTestValue = accessibilityFeaturesTestValue;
-  }
-  /// Deletes any existing test accessibility features and returns to using the
-  /// real accessibility features.
-  @Deprecated(
-    'Use WidgetTester.platformDispatcher.clearAccessibilityFeaturesTestValue() instead. '
-    'This feature was deprecated after v2.11.0-0.0.pre.'
-  )
-  void clearAccessibilityFeaturesTestValue() {
-    platformDispatcher.clearAccessibilityFeaturesTestValue();
-  }
 
   @Deprecated(
     'Use WidgetTester.platformDispatcher.onAccessibilityFeaturesChanged instead. '
@@ -1962,21 +1724,6 @@ class TestWindow implements SingletonFlutterWindow {
     platformDispatcher.sendPlatformMessage(name, data, callback);
   }
 
-  @Deprecated(
-    'Instead of calling this callback, use ServicesBinding.instance.channelBuffers.push. '
-    'This feature was deprecated after v2.1.0-10.0.pre.'
-  )
-  @override
-  PlatformMessageCallback? get onPlatformMessage => platformDispatcher.onPlatformMessage;
-  @Deprecated(
-    'Instead of setting this callback, use ServicesBinding.instance.defaultBinaryMessenger.setMessageHandler. '
-    'This feature was deprecated after v2.1.0-10.0.pre.'
-  )
-  @override
-  set onPlatformMessage(PlatformMessageCallback? callback) {
-    platformDispatcher.onPlatformMessage = callback;
-  }
-
   /// Delete any test value properties that have been set on this [TestWindow]
   /// as well as its [platformDispatcher].
   ///
@@ -1984,7 +1731,7 @@ class TestWindow implements SingletonFlutterWindow {
   /// [PlatformDispatcher] values are reported again.
   ///
   /// If desired, clearing of properties can be done on an individual basis,
-  /// e.g., [clearLocaleTestValue].
+  /// e.g., [clearDevicePixelRatioTestValue].
   @Deprecated(
     'Use WidgetTester.platformDispatcher.clearAllTestValues() and WidgetTester.view.reset() instead. '
     'Deprecated to prepare for the upcoming multi-window support. '
@@ -2069,14 +1816,6 @@ class TestWindow implements SingletonFlutterWindow {
   @override
   FrameData get frameData => platformDispatcher.frameData;
 
-  @Deprecated(
-    'Use WidgetTester.platformDispatcher.physicalGeometry instead. '
-    'Deprecated to prepare for the upcoming multi-window support. '
-    'This feature was deprecated after v3.9.0-0.1.pre.'
-  )
-  @override
-  Rect get physicalGeometry => _view.physicalGeometry;
-
   @Deprecated(
     'Use WidgetTester.platformDispatcher.systemFontFamily instead. '
     'Deprecated to prepare for the upcoming multi-window support. '
