@@ -227,6 +227,7 @@ class KernelCompiler {
     TargetModel targetModel = TargetModel.flutter,
     bool linkPlatformKernelIn = false,
     bool aot = false,
+    String? frontendServerStarterPath,
     List<String>? extraFrontEndOptions,
     List<String>? fileSystemRoots,
     String? fileSystemScheme,
@@ -240,20 +241,13 @@ class KernelCompiler {
     required bool trackWidgetCreation,
     required List<String> dartDefines,
     required PackageConfig packageConfig,
+    String? nativeAssets,
   }) async {
     final TargetPlatform? platform = targetModel == TargetModel.dartdevc ? TargetPlatform.web_javascript : null;
-    final String frontendServer = _artifacts.getArtifactPath(
-      Artifact.frontendServerSnapshotForEngineDartSdk,
-      platform: platform,
-    );
     // This is a URI, not a file path, so the forward slash is correct even on Windows.
     if (!sdkRoot.endsWith('/')) {
       sdkRoot = '$sdkRoot/';
     }
-    final String engineDartPath = _artifacts.getArtifactPath(Artifact.engineDartBinary, platform: platform);
-    if (!_processManager.canRun(engineDartPath)) {
-      throwToolExit('Unable to find Dart binary at $engineDartPath');
-    }
     String? mainUri;
     final File mainFile = _fileSystem.file(mainPath);
     final Uri mainFileUri = mainFile.uri;
@@ -278,10 +272,33 @@ class KernelCompiler {
         toMultiRootPath(dartPluginRegistrantFileUri, _fileSystemScheme, _fileSystemRoots, _fileSystem.path.separator == r'\');
     }
 
-    final List<String> command = <String>[
-      engineDartPath,
-      '--disable-dart-dev',
-      frontendServer,
+    final List<String> commandToStartFrontendServer;
+    if (frontendServerStarterPath != null && frontendServerStarterPath.isNotEmpty) {
+      final String engineDartPath = _artifacts.getArtifactPath(Artifact.engineDartBinary, platform: platform);
+      if (!_processManager.canRun(engineDartPath)) {
+        throwToolExit('Unable to find Dart binary at $engineDartPath');
+      }
+      commandToStartFrontendServer = <String>[
+        engineDartPath,
+        '--disable-dart-dev',
+        frontendServerStarterPath,
+      ];
+    } else {
+      final String engineDartAotRuntimePath = _artifacts.getArtifactPath(Artifact.engineDartAotRuntime, platform: platform);
+      if (!_processManager.canRun(engineDartAotRuntimePath)) {
+        throwToolExit('Unable to find dartaotruntime binary at $engineDartAotRuntimePath');
+      }
+      commandToStartFrontendServer = <String>[
+        engineDartAotRuntimePath,
+        '--disable-dart-dev',
+        _artifacts.getArtifactPath(
+          Artifact.frontendServerSnapshotForEngineDartSdk,
+          platform: platform,
+        ),
+      ];
+    }
+
+    final List<String> command = commandToStartFrontendServer + <String>[
       '--sdk-root',
       sdkRoot,
       '--target=$targetModel',
@@ -337,6 +354,10 @@ class KernelCompiler {
         'package:flutter/src/dart_plugin_registrant.dart',
         '-Dflutter.dart_plugin_registrant=$dartPluginRegistrantUri',
       ],
+      if (nativeAssets != null) ...<String>[
+        '--native-assets',
+        nativeAssets,
+      ],
       // See: https://github.com/flutter/flutter/issues/103994
       '--verbosity=error',
       ...?extraFrontEndOptions,
@@ -381,9 +402,10 @@ class _RecompileRequest extends _CompilationRequest {
     this.invalidatedFiles,
     this.outputPath,
     this.packageConfig,
-    this.suppressErrors,
-    {this.additionalSourceUri}
-  );
+    this.suppressErrors, {
+    this.additionalSourceUri,
+    this.nativeAssetsYamlUri,
+  });
 
   Uri mainUri;
   List<Uri>? invalidatedFiles;
@@ -391,6 +413,7 @@ class _RecompileRequest extends _CompilationRequest {
   PackageConfig packageConfig;
   bool suppressErrors;
   final Uri? additionalSourceUri;
+  final Uri? nativeAssetsYamlUri;
 
   @override
   Future<CompilerOutput?> _run(DefaultResidentCompiler compiler) async =>
@@ -483,6 +506,7 @@ abstract class ResidentCompiler {
     bool assumeInitializeFromDillUpToDate,
     TargetModel targetModel,
     bool unsafePackageSerialization,
+    String? frontendServerStarterPath,
     List<String> extraFrontEndOptions,
     String platformDill,
     List<String>? dartDefines,
@@ -515,6 +539,7 @@ abstract class ResidentCompiler {
     bool suppressErrors = false,
     bool checkDartPluginRegistry = false,
     File? dartPluginRegistrant,
+    Uri? nativeAssetsYaml,
   });
 
   Future<CompilerOutput?> compileExpression(
@@ -585,7 +610,7 @@ class DefaultResidentCompiler implements ResidentCompiler {
     required this.buildMode,
     required Logger logger,
     required ProcessManager processManager,
-    required Artifacts artifacts,
+    required this.artifacts,
     required Platform platform,
     required FileSystem fileSystem,
     this.testCompilation = false,
@@ -597,6 +622,7 @@ class DefaultResidentCompiler implements ResidentCompiler {
     this.assumeInitializeFromDillUpToDate = false,
     this.targetModel = TargetModel.flutter,
     this.unsafePackageSerialization = false,
+    this.frontendServerStarterPath,
     this.extraFrontEndOptions,
     this.platformDill,
     List<String>? dartDefines,
@@ -604,7 +630,6 @@ class DefaultResidentCompiler implements ResidentCompiler {
     @visibleForTesting StdoutHandler? stdoutHandler,
   }) : _logger = logger,
        _processManager = processManager,
-       _artifacts = artifacts,
        _stdoutHandler = stdoutHandler ?? StdoutHandler(logger: logger, fileSystem: fileSystem),
        _platform = platform,
        dartDefines = dartDefines ?? const <String>[],
@@ -615,7 +640,7 @@ class DefaultResidentCompiler implements ResidentCompiler {
 
   final Logger _logger;
   final ProcessManager _processManager;
-  final Artifacts _artifacts;
+  final Artifacts artifacts;
   final Platform _platform;
 
   final bool testCompilation;
@@ -628,6 +653,7 @@ class DefaultResidentCompiler implements ResidentCompiler {
   final String? initializeFromDill;
   final bool assumeInitializeFromDillUpToDate;
   final bool unsafePackageSerialization;
+  final String? frontendServerStarterPath;
   final List<String>? extraFrontEndOptions;
   final List<String> dartDefines;
   final String? librariesSpec;
@@ -664,6 +690,7 @@ class DefaultResidentCompiler implements ResidentCompiler {
     File? dartPluginRegistrant,
     String? projectRootPath,
     FileSystem? fs,
+    Uri? nativeAssetsYaml,
   }) async {
     if (!_controller.hasListener) {
       _controller.stream.listen(_handleCompilationRequest);
@@ -682,6 +709,7 @@ class DefaultResidentCompiler implements ResidentCompiler {
       packageConfig,
       suppressErrors,
       additionalSourceUri: additionalSourceUri,
+      nativeAssetsYamlUri: nativeAssetsYaml,
     ));
     return completer.future;
   }
@@ -700,12 +728,22 @@ class DefaultResidentCompiler implements ResidentCompiler {
         toMultiRootPath(request.additionalSourceUri!, fileSystemScheme, fileSystemRoots, _platform.isWindows);
     }
 
+    final String? nativeAssets = request.nativeAssetsYamlUri?.toString();
     final Process? server = _server;
     if (server == null) {
-      return _compile(mainUri, request.outputPath, additionalSourceUri: additionalSourceUri);
+      return _compile(
+        mainUri,
+        request.outputPath,
+        additionalSourceUri: additionalSourceUri,
+        nativeAssetsUri: nativeAssets,
+      );
     }
     final String inputKey = Uuid().generateV4();
 
+    if (nativeAssets != null && nativeAssets.isNotEmpty) {
+      server.stdin.writeln('native-assets $nativeAssets');
+      _logger.printTrace('<- native-assets $nativeAssets');
+    }
     server.stdin.writeln('recompile $mainUri $inputKey');
     _logger.printTrace('<- recompile $mainUri $inputKey');
     final List<Uri>? invalidatedFiles = request.invalidatedFiles;
@@ -747,18 +785,30 @@ class DefaultResidentCompiler implements ResidentCompiler {
 
   Future<CompilerOutput?> _compile(
     String scriptUri,
-    String? outputPath,
-    {String? additionalSourceUri}
-  ) async {
+    String? outputPath, {
+    String? additionalSourceUri,
+    String? nativeAssetsUri,
+  }) async {
     final TargetPlatform? platform = (targetModel == TargetModel.dartdevc) ? TargetPlatform.web_javascript : null;
-    final String frontendServer = _artifacts.getArtifactPath(
-      Artifact.frontendServerSnapshotForEngineDartSdk,
-      platform: platform,
-    );
-    final List<String> command = <String>[
-      _artifacts.getArtifactPath(Artifact.engineDartBinary, platform: platform),
-      '--disable-dart-dev',
-      frontendServer,
+    late final List<String> commandToStartFrontendServer;
+    if (frontendServerStarterPath != null && frontendServerStarterPath!.isNotEmpty) {
+      commandToStartFrontendServer = <String>[
+        artifacts.getArtifactPath(Artifact.engineDartBinary, platform: platform),
+        '--disable-dart-dev',
+        frontendServerStarterPath!,
+      ];
+    } else {
+      commandToStartFrontendServer = <String>[
+        artifacts.getArtifactPath(Artifact.engineDartAotRuntime, platform: platform),
+        '--disable-dart-dev',
+        artifacts.getArtifactPath(
+          Artifact.frontendServerSnapshotForEngineDartSdk,
+          platform: platform,
+        ),
+      ];
+    }
+
+    final List<String> command = commandToStartFrontendServer + <String>[
       '--sdk-root',
       sdkRoot,
       '--incremental',
@@ -807,6 +857,10 @@ class DefaultResidentCompiler implements ResidentCompiler {
         'package:flutter/src/dart_plugin_registrant.dart',
         '-Dflutter.dart_plugin_registrant=$additionalSourceUri',
       ],
+      if (nativeAssetsUri != null) ...<String>[
+        '--native-assets',
+        nativeAssetsUri,
+      ],
       if (platformDill != null) ...<String>[
         '--platform',
         platformDill!,
@@ -843,6 +897,11 @@ class DefaultResidentCompiler implements ResidentCompiler {
       }
     }));
 
+    if (nativeAssetsUri != null && nativeAssetsUri.isNotEmpty) {
+      _server?.stdin.writeln('native-assets $nativeAssetsUri');
+      _logger.printTrace('<- native-assets $nativeAssetsUri');
+    }
+
     _server?.stdin.writeln('compile $scriptUri');
     _logger.printTrace('<- compile $scriptUri');
 
