@@ -33,6 +33,7 @@ import 'package:flutter_tools/src/runner/flutter_command.dart';
 import 'package:flutter_tools/src/vmservice.dart';
 import 'package:flutter_tools/src/web/compile.dart';
 import 'package:test/fake.dart';
+import 'package:unified_analytics/unified_analytics.dart' as analytics;
 import 'package:vm_service/vm_service.dart';
 
 import '../../src/common.dart';
@@ -47,11 +48,13 @@ void main() {
   });
 
   group('run', () {
+    late BufferLogger logger;
     late TestDeviceManager testDeviceManager;
     late FileSystem fileSystem;
 
     setUp(() {
-      testDeviceManager = TestDeviceManager(logger: BufferLogger.test());
+      logger = BufferLogger.test();
+      testDeviceManager = TestDeviceManager(logger: logger);
       fileSystem = MemoryFileSystem.test();
     });
 
@@ -64,7 +67,7 @@ void main() {
     }, overrides: <Type, Generator>{
       FileSystem: () => fileSystem,
       ProcessManager: () => FakeProcessManager.any(),
-      Logger: () => BufferLogger.test(),
+      Logger: () => logger,
     });
 
     testUsingContext('does not support --no-sound-null-safety by default', () async {
@@ -88,7 +91,7 @@ void main() {
     }, overrides: <Type, Generator>{
       FileSystem: () => fileSystem,
       ProcessManager: () => FakeProcessManager.any(),
-      Logger: () => BufferLogger.test(),
+      Logger: () => logger,
     });
 
     testUsingContext('supports --no-sound-null-safety with an overridden NonNullSafeBuilds', () async {
@@ -108,7 +111,7 @@ void main() {
     }, overrides: <Type, Generator>{
       DeviceManager: () => testDeviceManager,
       FileSystem: () => fileSystem,
-      Logger: () => BufferLogger.test(),
+      Logger: () => logger,
       NonNullSafeBuilds: () => NonNullSafeBuilds.allowed,
       ProcessManager: () => FakeProcessManager.any(),
     });
@@ -136,7 +139,7 @@ void main() {
     }, overrides: <Type, Generator>{
       FileSystem: () => fileSystem,
       ProcessManager: () => FakeProcessManager.any(),
-      Logger: () => BufferLogger.test(),
+      Logger: () => logger,
     });
 
     testUsingContext('Walks upward looking for a pubspec.yaml and succeeds if found', () async {
@@ -164,7 +167,7 @@ void main() {
     }, overrides: <Type, Generator>{
       FileSystem: () => fileSystem,
       ProcessManager: () => FakeProcessManager.any(),
-      Logger: () => BufferLogger.test(),
+      Logger: () => logger,
     });
 
     testUsingContext('Walks upward looking for a pubspec.yaml and exits if missing', () async {
@@ -184,7 +187,7 @@ void main() {
     }, overrides: <Type, Generator>{
       FileSystem: () => fileSystem,
       ProcessManager: () => FakeProcessManager.any(),
-      Logger: () => BufferLogger.test(),
+      Logger: () => logger,
     });
 
     group('run app', () {
@@ -192,6 +195,7 @@ void main() {
       late Artifacts artifacts;
       late TestUsage usage;
       late FakeAnsiTerminal fakeTerminal;
+      late analytics.FakeAnalytics fakeAnalytics;
 
       setUpAll(() {
         Cache.disableLocking();
@@ -211,6 +215,10 @@ void main() {
         libDir.createSync();
         final File mainFile = libDir.childFile('main.dart');
         mainFile.writeAsStringSync('void main() {}');
+        fakeAnalytics = getInitializedFakeAnalyticsInstance(
+          fs: fs,
+          fakeFlutterVersion: FakeFlutterVersion(),
+        );
       });
 
       testUsingContext('exits with a user message when no supported devices attached', () async {
@@ -425,6 +433,36 @@ void main() {
         Cache: () => Cache.test(processManager: FakeProcessManager.any()),
       });
 
+      testUsingContext('prints warning when --flavor is used with an unsupported target platform', () async {
+        const List<String> runCommand = <String>[
+          'run',
+          '--no-pub',
+          '--no-hot',
+          '--flavor=vanilla',
+          '-d',
+          'all',
+        ];
+        // Useful for test readability.
+        // ignore: avoid_redundant_argument_values
+        final FakeDevice deviceWithoutFlavorSupport = FakeDevice(supportsFlavors: false);
+        final FakeDevice deviceWithFlavorSupport = FakeDevice(supportsFlavors: true);
+        testDeviceManager.devices = <Device>[deviceWithoutFlavorSupport, deviceWithFlavorSupport];
+
+        await createTestCommandRunner(TestRunCommandThatOnlyValidates()).run(runCommand);
+
+        expect(logger.warningText, contains(
+          '--flavor is only supported for Android, macOS, and iOS devices. '
+          'Flavor-related features may not function properly and could '
+          'behave differently in a future release.'
+        ));
+      }, overrides: <Type, Generator>{
+        DeviceManager: () => testDeviceManager,
+        FileSystem: () => fs,
+        ProcessManager: () => FakeProcessManager.any(),
+        Cache: () => Cache.test(processManager: FakeProcessManager.any()),
+        Logger: () => logger,
+      });
+
       testUsingContext('forwards --uninstall-only to DebuggingOptions', () async {
         final RunCommand command = RunCommand();
         final FakeDevice mockDevice = FakeDevice(
@@ -478,6 +516,23 @@ void main() {
             'cd58': 'false',
           })
         )));
+        expect(
+          fakeAnalytics.sentEvents,
+          contains(
+            analytics.Event.commandUsageValues(
+              workflow: 'run',
+              commandHasTerminal: globals.stdio.hasTerminal,
+              runIsEmulator: false,
+              runTargetName: 'ios',
+              runTargetOsVersion: 'iOS 13',
+              runModeName: 'debug',
+              runProjectModule: false,
+              runProjectHostLanguage: 'swift',
+              runIOSInterfaceType: 'usb',
+              runIsTest: false,
+            ),
+          ),
+        );
       }, overrides: <Type, Generator>{
         AnsiTerminal: () => fakeTerminal,
         Artifacts: () => artifacts,
@@ -487,6 +542,7 @@ void main() {
         ProcessManager: () => FakeProcessManager.any(),
         Stdio: () => FakeStdio(),
         Usage: () => usage,
+        analytics.Analytics: () => fakeAnalytics,
       });
 
       testUsingContext('correctly reports tests to usage', () async {
@@ -513,6 +569,23 @@ void main() {
             'cd58': 'true',
           })),
         ));
+        expect(
+          fakeAnalytics.sentEvents,
+          contains(
+            analytics.Event.commandUsageValues(
+              workflow: 'run',
+              commandHasTerminal: globals.stdio.hasTerminal,
+              runIsEmulator: false,
+              runTargetName: 'ios',
+              runTargetOsVersion: 'iOS 13',
+              runModeName: 'debug',
+              runProjectModule: false,
+              runProjectHostLanguage: 'swift',
+              runIOSInterfaceType: 'usb',
+              runIsTest: true,
+            ),
+          ),
+        );
       }, overrides: <Type, Generator>{
         AnsiTerminal: () => fakeTerminal,
         Artifacts: () => artifacts,
@@ -522,6 +595,7 @@ void main() {
         ProcessManager: () => FakeProcessManager.any(),
         Stdio: () => FakeStdio(),
         Usage: () => usage,
+        analytics.Analytics: () => fakeAnalytics,
       });
 
       group('--machine', () {
@@ -606,7 +680,7 @@ void main() {
           ProcessManager: () => FakeProcessManager.any(),
           Usage: () => usage,
           Stdio: () => FakeStdio(),
-          Logger: () => AppRunLogger(parent: BufferLogger.test()),
+          Logger: () => AppRunLogger(parent: logger),
         });
 
         testUsingContext('can disable devtools with --no-devtools', () async {
@@ -634,7 +708,7 @@ void main() {
           ProcessManager: () => FakeProcessManager.any(),
           Usage: () => usage,
           Stdio: () => FakeStdio(),
-          Logger: () => AppRunLogger(parent: BufferLogger.test()),
+          Logger: () => AppRunLogger(parent: logger),
         });
       });
     });
@@ -919,6 +993,99 @@ void main() {
         ProcessManager: () => FakeProcessManager.any(),
       });
     });
+
+    group('--web-header', () {
+      setUp(() {
+        fileSystem.file('lib/main.dart').createSync(recursive: true);
+        fileSystem.file('pubspec.yaml').createSync();
+        fileSystem.file('.packages').createSync();
+        final FakeDevice device = FakeDevice(isLocalEmulator: true, platformType: PlatformType.android);
+        testDeviceManager.devices = <Device>[device];
+      });
+
+      testUsingContext('can accept simple, valid values', () async {
+        final RunCommand command = RunCommand();
+        await expectLater(
+          () => createTestCommandRunner(command).run(<String>[
+            'run',
+            '--no-pub', '--no-hot',
+            '--web-header', 'foo = bar',
+          ]), throwsToolExit());
+
+        final DebuggingOptions options = await command.createDebuggingOptions(true);
+        expect(options.webHeaders, <String, String>{'foo': 'bar'});
+      }, overrides: <Type, Generator>{
+        FileSystem: () => fileSystem,
+        ProcessManager: () => FakeProcessManager.any(),
+        Logger: () => logger,
+        DeviceManager: () => testDeviceManager,
+      });
+
+      testUsingContext('throws a ToolExit when no value is provided', () async {
+        final RunCommand command = RunCommand();
+        await expectLater(
+          () => createTestCommandRunner(command).run(<String>[
+            'run',
+            '--no-pub', '--no-hot',
+            '--web-header',
+            'foo',
+          ]), throwsToolExit(message: 'Invalid web headers: foo'));
+
+        await expectLater(
+          () => command.createDebuggingOptions(true),
+          throwsToolExit(),
+        );
+      }, overrides: <Type, Generator>{
+        FileSystem: () => fileSystem,
+        ProcessManager: () => FakeProcessManager.any(),
+        Logger: () => logger,
+        DeviceManager: () => testDeviceManager,
+      });
+
+      testUsingContext('throws a ToolExit when value includes delimiter characters', () async {
+        fileSystem.file('lib/main.dart').createSync(recursive: true);
+        fileSystem.file('pubspec.yaml').createSync();
+        fileSystem.file('.packages').createSync();
+
+        final RunCommand command = RunCommand();
+        await expectLater(
+          () => createTestCommandRunner(command).run(<String>[
+            'run',
+            '--no-pub', '--no-hot',
+            '--web-header', 'hurray/headers=flutter',
+          ]), throwsToolExit());
+
+        await expectLater(
+          () => command.createDebuggingOptions(true),
+          throwsToolExit(message: 'Invalid web headers: hurray/headers=flutter'),
+        );
+      }, overrides: <Type, Generator>{
+        FileSystem: () => fileSystem,
+        ProcessManager: () => FakeProcessManager.any(),
+        Logger: () => logger,
+        DeviceManager: () => testDeviceManager,
+      });
+
+      testUsingContext('accepts headers with commas in them', () async {
+        final RunCommand command = RunCommand();
+        await expectLater(
+          () => createTestCommandRunner(command).run(<String>[
+            'run',
+            '--no-pub', '--no-hot',
+            '--web-header', 'hurray=flutter,flutter=hurray',
+          ]), throwsToolExit());
+
+        final DebuggingOptions options = await command.createDebuggingOptions(true);
+        expect(options.webHeaders, <String, String>{
+          'hurray': 'flutter,flutter=hurray'
+        });
+      }, overrides: <Type, Generator>{
+        FileSystem: () => fileSystem,
+        ProcessManager: () => FakeProcessManager.any(),
+        Logger: () => logger,
+        DeviceManager: () => testDeviceManager,
+      });
+    });
   });
 
   group('dart-defines and web-renderer options', () {
@@ -1086,12 +1253,12 @@ void main() {
       '--use-test-fonts',
       '--trace-skia',
       '--trace-systrace',
+      '--trace-to-file=path/to/trace.binpb',
       '--verbose-system-logs',
       '--null-assertions',
       '--native-null-assertions',
       '--enable-impeller',
       '--enable-vulkan-validation',
-      '--impeller-force-gl',
       '--trace-systrace',
       '--enable-software-rendering',
       '--skia-deterministic-rendering',
@@ -1106,13 +1273,13 @@ void main() {
     expect(options.useTestFonts, true);
     expect(options.traceSkia, true);
     expect(options.traceSystrace, true);
+    expect(options.traceToFile, 'path/to/trace.binpb');
     expect(options.verboseSystemLogs, true);
     expect(options.nullAssertions, true);
     expect(options.nativeNullAssertions, true);
     expect(options.traceSystrace, true);
     expect(options.enableImpeller, ImpellerStatus.enabled);
     expect(options.enableVulkanValidation, true);
-    expect(options.impellerForceGL, true);
     expect(options.enableSoftwareRendering, true);
     expect(options.skiaDeterministicRendering, true);
     expect(options.usingCISystem, true);
@@ -1164,9 +1331,6 @@ class FakeAndroidSdk extends Fake implements AndroidSdk {
   String get adbPath => 'adb';
 }
 
-// Unfortunately Device, despite not being immutable, has an `operator ==`.
-// Until we fix that, we have to also ignore related lints here.
-// ignore: avoid_implementing_value_types
 class FakeDevice extends Fake implements Device {
   FakeDevice({
     bool isLocalEmulator = false,
@@ -1174,11 +1338,13 @@ class FakeDevice extends Fake implements Device {
     String sdkNameAndVersion = '',
     PlatformType platformType = PlatformType.ios,
     bool isSupported = true,
+    bool supportsFlavors = false,
   }): _isLocalEmulator = isLocalEmulator,
       _targetPlatform = targetPlatform,
       _sdkNameAndVersion = sdkNameAndVersion,
       _platformType = platformType,
-      _isSupported = isSupported;
+      _isSupported = isSupported,
+      _supportsFlavors = supportsFlavors;
 
   static const int kSuccess = 1;
   static const int kFailure = -1;
@@ -1187,6 +1353,7 @@ class FakeDevice extends Fake implements Device {
   final String _sdkNameAndVersion;
   final PlatformType _platformType;
   final bool _isSupported;
+  final bool _supportsFlavors;
 
   @override
   Category get category => Category.mobile;
@@ -1214,6 +1381,9 @@ class FakeDevice extends Fake implements Device {
   @override
   bool get supportsFastStart => false;
 
+  @override
+  bool get supportsFlavors => _supportsFlavors;
+
   @override
   bool get ephemeral => true;
 
@@ -1302,9 +1472,6 @@ class FakeDevice extends Fake implements Device {
   }
 }
 
-// Unfortunately Device, despite not being immutable, has an `operator ==`.
-// Until we fix that, we have to also ignore related lints here.
-// ignore: avoid_implementing_value_types
 class FakeIOSDevice extends Fake implements IOSDevice {
   FakeIOSDevice({
     this.connectionInterface = DeviceConnectionInterface.attached,
@@ -1335,13 +1502,10 @@ class FakeIOSDevice extends Fake implements IOSDevice {
 
 class TestRunCommandForUsageValues extends RunCommand {
   TestRunCommandForUsageValues({
-   this.devices,
-  });
-
-  @override
-  // devices is not set within usageValues, so we override the field
-  // ignore: overridden_fields
-  List<Device>? devices;
+    List<Device>? devices,
+  }) {
+    this.devices = devices;
+  }
 
   @override
   Future<BuildInfo> getBuildInfo({ BuildMode? forcedBuildMode, File? forcedTargetFile }) async {
@@ -1433,6 +1597,7 @@ class CapturingAppDomain extends AppDomain {
     bool machine = true,
     String? userIdentifier,
     bool enableDevTools = true,
+    String? flavor,
   }) async {
     this.multidexEnabled = multidexEnabled;
     this.userIdentifier = userIdentifier;
