@@ -28,7 +28,7 @@ const FakeCommand kARMCheckCommand = FakeCommand(
 );
 
 const List<String> kDefaultClang = <String>[
-  '-miphoneos-version-min=11.0',
+  '-miphoneos-version-min=12.0',
   '-isysroot',
   'path/to/sdk',
   '-dynamiclib',
@@ -40,6 +40,7 @@ const List<String> kDefaultClang = <String>[
   '-rpath',
   '-Xlinker',
   '@loader_path/Frameworks',
+  '-fapplication-extension',
   '-install_name',
   '@rpath/App.framework/App',
   '-o',
@@ -165,7 +166,6 @@ void main() {
         buildMode: BuildMode.debug,
         mainPath: 'main.dill',
         outputPath: outputPath,
-        bitcode: false,
         dartObfuscation: false,
       ), isNot(equals(0)));
     });
@@ -178,7 +178,6 @@ void main() {
         buildMode: BuildMode.debug,
         mainPath: 'main.dill',
         outputPath: outputPath,
-        bitcode: false,
         dartObfuscation: false,
       ), isNot(0));
     });
@@ -191,85 +190,10 @@ void main() {
         buildMode: BuildMode.debug,
         mainPath: 'main.dill',
         outputPath: outputPath,
-        bitcode: false,
         dartObfuscation: false,
       ), isNot(0));
     });
 
-    testWithoutContext('builds iOS with bitcode', () async {
-      final String outputPath = fileSystem.path.join('build', 'foo');
-      final String assembly = fileSystem.path.join(outputPath, 'snapshot_assembly.S');
-      final String genSnapshotPath = artifacts.getArtifactPath(
-        Artifact.genSnapshot,
-        platform: TargetPlatform.ios,
-        mode: BuildMode.profile,
-      );
-      processManager.addCommands(<FakeCommand>[
-        FakeCommand(command: <String>[
-          '${genSnapshotPath}_arm64',
-          '--deterministic',
-          '--snapshot_kind=app-aot-assembly',
-          '--assembly=$assembly',
-          '--strip',
-          'main.dill',
-        ]),
-        kWhichSysctlCommand,
-        kARMCheckCommand,
-        const FakeCommand(command: <String>[
-          'xcrun',
-          'cc',
-          '-arch',
-          'arm64',
-          '-miphoneos-version-min=11.0',
-          '-isysroot',
-          'path/to/sdk',
-          '-fembed-bitcode',
-          '-c',
-          'build/foo/snapshot_assembly.S',
-          '-o',
-          'build/foo/snapshot_assembly.o',
-        ]),
-        const FakeCommand(command: <String>[
-          'xcrun',
-          'clang',
-          '-arch',
-          'arm64',
-          '-miphoneos-version-min=11.0',
-          '-isysroot',
-          'path/to/sdk',
-          '-dynamiclib',
-          '-Xlinker',
-          '-rpath',
-          '-Xlinker',
-          '@executable_path/Frameworks',
-          '-Xlinker',
-          '-rpath',
-          '-Xlinker',
-          '@loader_path/Frameworks',
-          '-install_name',
-          '@rpath/App.framework/App',
-          '-fembed-bitcode',
-          '-o',
-          'build/foo/App.framework/App',
-          'build/foo/snapshot_assembly.o',
-        ]),
-      ]);
-
-      final int genSnapshotExitCode = await snapshotter.build(
-        platform: TargetPlatform.ios,
-        buildMode: BuildMode.profile,
-        mainPath: 'main.dill',
-        outputPath: outputPath,
-        darwinArch: DarwinArch.arm64,
-        sdkRoot: 'path/to/sdk',
-        bitcode: true,
-        dartObfuscation: false,
-      );
-
-      expect(genSnapshotExitCode, 0);
-      expect(processManager, hasNoRemainingExpectations);
-    });
-
     testWithoutContext('builds iOS snapshot with dwarfStackTraces', () async {
       final String outputPath = fileSystem.path.join('build', 'foo');
       final String assembly = fileSystem.path.join(outputPath, 'snapshot_assembly.S');
@@ -285,8 +209,8 @@ void main() {
           '--deterministic',
           '--snapshot_kind=app-aot-assembly',
           '--assembly=$assembly',
-          '--strip',
           '--dwarf-stack-traces',
+          '--resolve-dwarf-paths',
           '--save-debugging-info=$debugPath',
           'main.dill',
         ]),
@@ -297,7 +221,7 @@ void main() {
           'cc',
           '-arch',
           'arm64',
-          '-miphoneos-version-min=11.0',
+          '-miphoneos-version-min=12.0',
           '-isysroot',
           'path/to/sdk',
           '-c',
@@ -312,6 +236,21 @@ void main() {
           'arm64',
           ...kDefaultClang,
         ]),
+        const FakeCommand(command: <String>[
+          'xcrun',
+          'dsymutil',
+          '-o',
+          'build/foo/App.framework.dSYM',
+          'build/foo/App.framework/App',
+        ]),
+        const FakeCommand(command: <String>[
+          'xcrun',
+          'strip',
+          '-x',
+          'build/foo/App.framework/App',
+          '-o',
+          'build/foo/App.framework/App',
+        ]),
       ]);
 
       final int genSnapshotExitCode = await snapshotter.build(
@@ -321,7 +260,6 @@ void main() {
         outputPath: outputPath,
         darwinArch: DarwinArch.arm64,
         sdkRoot: 'path/to/sdk',
-        bitcode: false,
         splitDebugInfo: 'foo',
         dartObfuscation: false,
       );
@@ -344,7 +282,6 @@ void main() {
           '--deterministic',
           '--snapshot_kind=app-aot-assembly',
           '--assembly=$assembly',
-          '--strip',
           '--obfuscate',
           'main.dill',
         ]),
@@ -355,7 +292,7 @@ void main() {
           'cc',
           '-arch',
           'arm64',
-          '-miphoneos-version-min=11.0',
+          '-miphoneos-version-min=12.0',
           '-isysroot',
           'path/to/sdk',
           '-c',
@@ -370,6 +307,21 @@ void main() {
           'arm64',
           ...kDefaultClang,
         ]),
+        const FakeCommand(command: <String>[
+          'xcrun',
+          'dsymutil',
+          '-o',
+          'build/foo/App.framework.dSYM',
+          'build/foo/App.framework/App',
+        ]),
+        const FakeCommand(command: <String>[
+          'xcrun',
+          'strip',
+          '-x',
+          'build/foo/App.framework/App',
+          '-o',
+          'build/foo/App.framework/App',
+        ]),
       ]);
 
       final int genSnapshotExitCode = await snapshotter.build(
@@ -379,7 +331,6 @@ void main() {
         outputPath: outputPath,
         darwinArch: DarwinArch.arm64,
         sdkRoot: 'path/to/sdk',
-        bitcode: false,
         dartObfuscation: true,
       );
 
@@ -400,7 +351,6 @@ void main() {
           '--deterministic',
           '--snapshot_kind=app-aot-assembly',
           '--assembly=${fileSystem.path.join(outputPath, 'snapshot_assembly.S')}',
-          '--strip',
           'main.dill',
         ]),
         kWhichSysctlCommand,
@@ -410,7 +360,7 @@ void main() {
           'cc',
           '-arch',
           'arm64',
-          '-miphoneos-version-min=11.0',
+          '-miphoneos-version-min=12.0',
           '-isysroot',
           'path/to/sdk',
           '-c',
@@ -425,6 +375,21 @@ void main() {
           'arm64',
           ...kDefaultClang,
         ]),
+        const FakeCommand(command: <String>[
+          'xcrun',
+          'dsymutil',
+          '-o',
+          'build/foo/App.framework.dSYM',
+          'build/foo/App.framework/App',
+        ]),
+        const FakeCommand(command: <String>[
+          'xcrun',
+          'strip',
+          '-x',
+          'build/foo/App.framework/App',
+          '-o',
+          'build/foo/App.framework/App',
+        ]),
       ]);
 
       final int genSnapshotExitCode = await snapshotter.build(
@@ -434,7 +399,6 @@ void main() {
         outputPath: outputPath,
         darwinArch: DarwinArch.arm64,
         sdkRoot: 'path/to/sdk',
-        bitcode: false,
         dartObfuscation: false,
       );
 
@@ -462,7 +426,6 @@ void main() {
         buildMode: BuildMode.release,
         mainPath: 'main.dill',
         outputPath: outputPath,
-        bitcode: false,
         dartObfuscation: false,
       );
 
@@ -483,6 +446,7 @@ void main() {
           '--no-sim-use-hardfp',
           '--no-use-integer-division',
           '--dwarf-stack-traces',
+          '--resolve-dwarf-paths',
           '--save-debugging-info=$debugPath',
           'main.dill',
         ]
@@ -493,7 +457,6 @@ void main() {
         buildMode: BuildMode.release,
         mainPath: 'main.dill',
         outputPath: outputPath,
-        bitcode: false,
         splitDebugInfo: 'foo',
         dartObfuscation: false,
       );
@@ -523,7 +486,6 @@ void main() {
         buildMode: BuildMode.release,
         mainPath: 'main.dill',
         outputPath: outputPath,
-        bitcode: false,
         dartObfuscation: true,
       );
 
@@ -551,7 +513,6 @@ void main() {
         buildMode: BuildMode.release,
         mainPath: 'main.dill',
         outputPath: outputPath,
-        bitcode: false,
         splitDebugInfo: '',
         dartObfuscation: false,
       );
@@ -578,7 +539,6 @@ void main() {
         buildMode: BuildMode.release,
         mainPath: 'main.dill',
         outputPath: outputPath,
-        bitcode: false,
         dartObfuscation: false,
       );
 
@@ -603,7 +563,6 @@ void main() {
         buildMode: BuildMode.release,
         mainPath: 'main.dill',
         outputPath: outputPath,
-        bitcode: false,
         dartObfuscation: false,
         extraGenSnapshotOptions: const <String>['--no-strip'],
       );
