@@ -39,7 +39,7 @@ void main() {
     androidEnvironment = Environment.test(
       fileSystem.currentDirectory,
       defines: <String, String>{
-        kBuildMode: getNameForBuildMode(BuildMode.profile),
+        kBuildMode: BuildMode.profile.cliName,
         kTargetPlatform: getNameForTargetPlatform(TargetPlatform.android_arm),
       },
       inputs: <String, String>{},
@@ -52,7 +52,7 @@ void main() {
     iosEnvironment = Environment.test(
       fileSystem.currentDirectory,
       defines: <String, String>{
-        kBuildMode: getNameForBuildMode(BuildMode.profile),
+        kBuildMode: BuildMode.profile.cliName,
         kTargetPlatform: getNameForTargetPlatform(TargetPlatform.ios),
       },
       inputs: <String, String>{},
@@ -71,10 +71,21 @@ void main() {
       throwsA(isA<MissingDefineException>()));
   });
 
+  const String emptyNativeAssets = '''
+format-version:
+  - 1
+  - 0
+  - 0
+native-assets: {}
+''';
+
   testWithoutContext('KernelSnapshot handles null result from kernel compilation', () async {
     fileSystem.file('.dart_tool/package_config.json')
       ..createSync(recursive: true)
       ..writeAsStringSync('{"configVersion": 2, "packages":[]}');
+    androidEnvironment.buildDir.childFile('native_assets.yaml')
+      ..createSync(recursive: true)
+      ..writeAsStringSync(emptyNativeAssets);
     final String build = androidEnvironment.buildDir.path;
     final String flutterPatchedSdkPath = artifacts.getArtifactPath(
       Artifact.flutterPatchedSdkPath,
@@ -83,7 +94,7 @@ void main() {
     );
     processManager.addCommands(<FakeCommand>[
       FakeCommand(command: <String>[
-        artifacts.getHostArtifact(HostArtifact.engineDartBinary).path,
+        artifacts.getArtifactPath(Artifact.engineDartAotRuntime),
         '--disable-dart-dev',
         artifacts.getArtifactPath(Artifact.frontendServerSnapshotForEngineDartSdk),
         '--sdk-root',
@@ -94,12 +105,16 @@ void main() {
         '--track-widget-creation',
         '--aot',
         '--tfa',
+        '--target-os',
+        'android',
         '--packages',
         '/.dart_tool/package_config.json',
         '--output-dill',
         '$build/app.dill',
         '--depfile',
         '$build/kernel_snapshot.d',
+        '--native-assets',
+        '$build/native_assets.yaml',
         '--verbosity=error',
         'file:///lib/main.dart',
       ], exitCode: 1),
@@ -113,6 +128,9 @@ void main() {
     fileSystem.file('.dart_tool/package_config.json')
       ..createSync(recursive: true)
       ..writeAsStringSync('{"configVersion": 2, "packages":[]}');
+    androidEnvironment.buildDir.childFile('native_assets.yaml')
+      ..createSync(recursive: true)
+      ..writeAsStringSync(emptyNativeAssets);
     final String build = androidEnvironment.buildDir.path;
     final String flutterPatchedSdkPath = artifacts.getArtifactPath(
       Artifact.flutterPatchedSdkPath,
@@ -121,7 +139,7 @@ void main() {
     );
     processManager.addCommands(<FakeCommand>[
       FakeCommand(command: <String>[
-        artifacts.getHostArtifact(HostArtifact.engineDartBinary).path,
+        artifacts.getArtifactPath(Artifact.engineDartAotRuntime),
         '--disable-dart-dev',
         artifacts.getArtifactPath(Artifact.frontendServerSnapshotForEngineDartSdk),
         '--sdk-root',
@@ -132,12 +150,16 @@ void main() {
         '--track-widget-creation',
         '--aot',
         '--tfa',
+        '--target-os',
+        'android',
         '--packages',
         '/.dart_tool/package_config.json',
         '--output-dill',
         '$build/app.dill',
         '--depfile',
         '$build/kernel_snapshot.d',
+        '--native-assets',
+        '$build/native_assets.yaml',
         '--verbosity=error',
         'file:///lib/main.dart',
       ], stdout: 'result $kBoundaryKey\n$kBoundaryKey\n$kBoundaryKey $build/app.dill 0\n'),
@@ -152,6 +174,9 @@ void main() {
     fileSystem.file('.dart_tool/package_config.json')
       ..createSync(recursive: true)
       ..writeAsStringSync('{"configVersion": 2, "packages":[]}');
+    androidEnvironment.buildDir.childFile('native_assets.yaml')
+      ..createSync(recursive: true)
+      ..writeAsStringSync(emptyNativeAssets);
     final String build = androidEnvironment.buildDir.path;
     final String flutterPatchedSdkPath = artifacts.getArtifactPath(
       Artifact.flutterPatchedSdkPath,
@@ -160,7 +185,7 @@ void main() {
     );
     processManager.addCommands(<FakeCommand>[
       FakeCommand(command: <String>[
-        artifacts.getHostArtifact(HostArtifact.engineDartBinary).path,
+        artifacts.getArtifactPath(Artifact.engineDartAotRuntime),
         '--disable-dart-dev',
         artifacts.getArtifactPath(Artifact.frontendServerSnapshotForEngineDartSdk),
         '--sdk-root',
@@ -171,12 +196,16 @@ void main() {
         '--track-widget-creation',
         '--aot',
         '--tfa',
+        '--target-os',
+        'android',
         '--packages',
         '/.dart_tool/package_config.json',
         '--output-dill',
         '$build/app.dill',
         '--depfile',
         '$build/kernel_snapshot.d',
+        '--native-assets',
+        '$build/native_assets.yaml',
         '--verbosity=error',
         'file:///lib/main.dart',
       ], stdout: 'result $kBoundaryKey\n$kBoundaryKey\n$kBoundaryKey $build/app.dill 0\n'),
@@ -188,10 +217,13 @@ void main() {
     expect(processManager, hasNoRemainingExpectations);
   });
 
-  testWithoutContext('KernelSnapshot correctly forwards ExtraFrontEndOptions', () async {
+  testWithoutContext('KernelSnapshot correctly forwards FrontendServerStarterPath', () async {
     fileSystem.file('.dart_tool/package_config.json')
       ..createSync(recursive: true)
       ..writeAsStringSync('{"configVersion": 2, "packages":[]}');
+    androidEnvironment.buildDir.childFile('native_assets.yaml')
+      ..createSync(recursive: true)
+      ..writeAsStringSync(emptyNativeAssets);
     final String build = androidEnvironment.buildDir.path;
     final String flutterPatchedSdkPath = artifacts.getArtifactPath(
       Artifact.flutterPatchedSdkPath,
@@ -200,7 +232,54 @@ void main() {
     );
     processManager.addCommands(<FakeCommand>[
       FakeCommand(command: <String>[
-        artifacts.getHostArtifact(HostArtifact.engineDartBinary).path,
+        artifacts.getArtifactPath(Artifact.engineDartBinary),
+        '--disable-dart-dev',
+        'path/to/frontend_server_starter.dart',
+        '--sdk-root',
+        '$flutterPatchedSdkPath/',
+        '--target=flutter',
+        '--no-print-incremental-dependencies',
+        ...buildModeOptions(BuildMode.profile, <String>[]),
+        '--track-widget-creation',
+        '--aot',
+        '--tfa',
+        '--target-os',
+        'android',
+        '--packages',
+        '/.dart_tool/package_config.json',
+        '--output-dill',
+        '$build/app.dill',
+        '--depfile',
+        '$build/kernel_snapshot.d',
+        '--native-assets',
+        '$build/native_assets.yaml',
+        '--verbosity=error',
+        'file:///lib/main.dart',
+      ], stdout: 'result $kBoundaryKey\n$kBoundaryKey\n$kBoundaryKey $build/app.dill 0\n'),
+    ]);
+
+    await const KernelSnapshot()
+      .build(androidEnvironment..defines[kFrontendServerStarterPath] = 'path/to/frontend_server_starter.dart');
+
+    expect(processManager, hasNoRemainingExpectations);
+  });
+
+  testWithoutContext('KernelSnapshot correctly forwards ExtraFrontEndOptions', () async {
+    fileSystem.file('.dart_tool/package_config.json')
+      ..createSync(recursive: true)
+      ..writeAsStringSync('{"configVersion": 2, "packages":[]}');
+    androidEnvironment.buildDir.childFile('native_assets.yaml')
+      ..createSync(recursive: true)
+      ..writeAsStringSync(emptyNativeAssets);
+    final String build = androidEnvironment.buildDir.path;
+    final String flutterPatchedSdkPath = artifacts.getArtifactPath(
+      Artifact.flutterPatchedSdkPath,
+      platform: TargetPlatform.android_arm,
+      mode: BuildMode.profile,
+    );
+    processManager.addCommands(<FakeCommand>[
+      FakeCommand(command: <String>[
+        artifacts.getArtifactPath(Artifact.engineDartAotRuntime),
         '--disable-dart-dev',
         artifacts.getArtifactPath(Artifact.frontendServerSnapshotForEngineDartSdk),
         '--sdk-root',
@@ -211,12 +290,16 @@ void main() {
         '--track-widget-creation',
         '--aot',
         '--tfa',
+        '--target-os',
+        'android',
         '--packages',
         '/.dart_tool/package_config.json',
         '--output-dill',
         '$build/app.dill',
         '--depfile',
         '$build/kernel_snapshot.d',
+        '--native-assets',
+        '$build/native_assets.yaml',
         '--verbosity=error',
         'foo',
         'bar',
@@ -234,6 +317,9 @@ void main() {
     fileSystem.file('.dart_tool/package_config.json')
       ..createSync(recursive: true)
       ..writeAsStringSync('{"configVersion": 2, "packages":[]}');
+    androidEnvironment.buildDir.childFile('native_assets.yaml')
+      ..createSync(recursive: true)
+      ..writeAsStringSync(emptyNativeAssets);
     final String build = androidEnvironment.buildDir.path;
     final String flutterPatchedSdkPath = artifacts.getArtifactPath(
       Artifact.flutterPatchedSdkPath,
@@ -242,7 +328,7 @@ void main() {
     );
     processManager.addCommands(<FakeCommand>[
       FakeCommand(command: <String>[
-        artifacts.getHostArtifact(HostArtifact.engineDartBinary).path,
+        artifacts.getArtifactPath(Artifact.engineDartAotRuntime),
         '--disable-dart-dev',
         artifacts.getArtifactPath(Artifact.frontendServerSnapshotForEngineDartSdk),
         '--sdk-root',
@@ -260,13 +346,15 @@ void main() {
         '--incremental',
         '--initialize-from-dill',
         '$build/app.dill',
+        '--native-assets',
+        '$build/native_assets.yaml',
         '--verbosity=error',
         'file:///lib/main.dart',
       ], stdout: 'result $kBoundaryKey\n$kBoundaryKey\n$kBoundaryKey $build/app.dill 0\n'),
     ]);
 
     await const KernelSnapshot().build(androidEnvironment
-      ..defines[kBuildMode] = getNameForBuildMode(BuildMode.debug)
+      ..defines[kBuildMode] = BuildMode.debug.cliName
       ..defines[kTrackWidgetCreation] = 'false');
 
     expect(processManager, hasNoRemainingExpectations);
@@ -276,6 +364,9 @@ void main() {
     fileSystem.file('.dart_tool/package_config.json')
       ..createSync(recursive: true)
       ..writeAsStringSync('{"configVersion": 2, "packages":[]}');
+    androidEnvironment.buildDir.childFile('native_assets.yaml')
+      ..createSync(recursive: true)
+      ..writeAsStringSync(emptyNativeAssets);
     final String build = androidEnvironment.buildDir.path;
     final String flutterPatchedSdkPath = artifacts.getArtifactPath(
       Artifact.flutterPatchedSdkPath,
@@ -284,7 +375,7 @@ void main() {
     );
     processManager.addCommands(<FakeCommand>[
       FakeCommand(command: <String>[
-        artifacts.getHostArtifact(HostArtifact.engineDartBinary).path,
+        artifacts.getArtifactPath(Artifact.engineDartAotRuntime),
         '--disable-dart-dev',
         artifacts.getArtifactPath(Artifact.frontendServerSnapshotForEngineDartSdk),
         '--sdk-root',
@@ -301,6 +392,8 @@ void main() {
         '--incremental',
         '--initialize-from-dill',
         '$build/app.dill',
+        '--native-assets',
+        '$build/native_assets.yaml',
         '--verbosity=error',
         'file:///lib/main.dart',
       ], stdout: 'result $kBoundaryKey\n$kBoundaryKey\n$kBoundaryKey $build/app.dill 0\n'),
@@ -308,7 +401,7 @@ void main() {
 
     await const KernelSnapshot().build(androidEnvironment
       ..defines[kTargetPlatform]  = getNameForTargetPlatform(TargetPlatform.darwin)
-      ..defines[kBuildMode] = getNameForBuildMode(BuildMode.debug)
+      ..defines[kBuildMode] = BuildMode.debug.cliName
       ..defines[kTrackWidgetCreation] = 'false'
     );
 
@@ -322,7 +415,7 @@ void main() {
     final Environment testEnvironment = Environment.test(
       fileSystem.currentDirectory,
       defines: <String, String>{
-        kBuildMode: getNameForBuildMode(BuildMode.debug),
+        kBuildMode: BuildMode.debug.cliName,
         kTargetPlatform: getNameForTargetPlatform(TargetPlatform.android_arm),
       },
       processManager: processManager,
@@ -330,6 +423,9 @@ void main() {
       fileSystem: fileSystem,
       logger: logger,
     );
+    testEnvironment.buildDir.childFile('native_assets.yaml')
+      ..createSync(recursive: true)
+      ..writeAsStringSync(emptyNativeAssets);
     final String build = testEnvironment.buildDir.path;
     final String flutterPatchedSdkPath = artifacts.getArtifactPath(
       Artifact.flutterPatchedSdkPath,
@@ -338,7 +434,7 @@ void main() {
     );
     processManager.addCommands(<FakeCommand>[
       FakeCommand(command: <String>[
-        artifacts.getHostArtifact(HostArtifact.engineDartBinary).path,
+        artifacts.getArtifactPath(Artifact.engineDartAotRuntime),
         '--disable-dart-dev',
         artifacts.getArtifactPath(Artifact.frontendServerSnapshotForEngineDartSdk),
         '--sdk-root',
@@ -357,6 +453,8 @@ void main() {
         '--incremental',
         '--initialize-from-dill',
         '$build/app.dill',
+        '--native-assets',
+        '$build/native_assets.yaml',
         '--verbosity=error',
         'file:///lib/main.dart',
       ], stdout: 'result $kBoundaryKey\n$kBoundaryKey\n$kBoundaryKey /build/653e11a8e6908714056a57cd6b4f602a/app.dill 0\n'),
@@ -465,84 +563,10 @@ void main() {
     ProcessManager: () => processManager,
   });
 
-  testUsingContext('AotAssemblyProfile with bitcode sends correct argument to snapshotter', () async {
-    iosEnvironment.defines[kIosArchs] = 'arm64';
-    iosEnvironment.defines[kBitcodeFlag] = 'true';
-    iosEnvironment.defines[kSdkRoot] = 'path/to/iPhoneOS.sdk';
-    final String build = iosEnvironment.buildDir.path;
-    processManager.addCommands(<FakeCommand>[
-      FakeCommand(command: <String>[
-        // This path is not known by the cache due to the iOS gen_snapshot split.
-        'Artifact.genSnapshot.TargetPlatform.ios.profile_arm64',
-        '--deterministic',
-        kAssemblyAot,
-        '--assembly=$build/arm64/snapshot_assembly.S',
-        '--strip',
-        '$build/app.dill',
-      ]),
-      FakeCommand(command: <String>[
-        'xcrun',
-        'cc',
-        '-arch',
-        'arm64',
-        '-miphoneos-version-min=11.0',
-        '-isysroot',
-        'path/to/iPhoneOS.sdk',
-        // Contains bitcode flag.
-        '-fembed-bitcode',
-        '-c',
-        '$build/arm64/snapshot_assembly.S',
-        '-o',
-        '$build/arm64/snapshot_assembly.o',
-      ]),
-      FakeCommand(command: <String>[
-        'xcrun',
-        'clang',
-        '-arch',
-        'arm64',
-        '-miphoneos-version-min=11.0',
-        '-isysroot',
-        'path/to/iPhoneOS.sdk',
-        '-dynamiclib',
-        '-Xlinker',
-        '-rpath',
-        '-Xlinker',
-        '@executable_path/Frameworks',
-        '-Xlinker',
-        '-rpath',
-        '-Xlinker',
-        '@loader_path/Frameworks',
-        '-install_name',
-        '@rpath/App.framework/App',
-        // Contains bitcode flag.
-        '-fembed-bitcode',
-        '-o',
-        '$build/arm64/App.framework/App',
-        '$build/arm64/snapshot_assembly.o',
-      ]),
-      FakeCommand(command: <String>[
-        'lipo',
-        '$build/arm64/App.framework/App',
-        '-create',
-        '-output',
-        '$build/App.framework/App',
-      ]),
-    ]);
-
-    await const AotAssemblyProfile().build(iosEnvironment);
-
-    expect(processManager, hasNoRemainingExpectations);
-  }, overrides: <Type, Generator>{
-    Platform: () => macPlatform,
-    FileSystem: () => fileSystem,
-    ProcessManager: () => processManager,
-  });
-
   testUsingContext('AotAssemblyRelease configures gen_snapshot with code size directory', () async {
     iosEnvironment.defines[kCodeSizeDirectory] = 'code_size_1';
     iosEnvironment.defines[kIosArchs] = 'arm64';
     iosEnvironment.defines[kSdkRoot] = 'path/to/iPhoneOS.sdk';
-    iosEnvironment.defines[kBitcodeFlag] = 'true';
     final String build = iosEnvironment.buildDir.path;
     processManager.addCommands(<FakeCommand>[
       FakeCommand(command: <String>[
@@ -553,7 +577,6 @@ void main() {
         '--trace-precompiler-to=code_size_1/trace.arm64.json',
         kAssemblyAot,
         '--assembly=$build/arm64/snapshot_assembly.S',
-        '--strip',
         '$build/app.dill',
       ]),
       FakeCommand(command: <String>[
@@ -561,11 +584,9 @@ void main() {
         'cc',
         '-arch',
         'arm64',
-        '-miphoneos-version-min=11.0',
+        '-miphoneos-version-min=12.0',
         '-isysroot',
         'path/to/iPhoneOS.sdk',
-        // Contains bitcode flag.
-        '-fembed-bitcode',
         '-c',
         '$build/arm64/snapshot_assembly.S',
         '-o',
@@ -576,7 +597,7 @@ void main() {
         'clang',
         '-arch',
         'arm64',
-        '-miphoneos-version-min=11.0',
+        '-miphoneos-version-min=12.0',
         '-isysroot',
         'path/to/iPhoneOS.sdk',
         '-dynamiclib',
@@ -588,14 +609,28 @@ void main() {
         '-rpath',
         '-Xlinker',
         '@loader_path/Frameworks',
+        '-fapplication-extension',
         '-install_name',
         '@rpath/App.framework/App',
-        // Contains bitcode flag.
-        '-fembed-bitcode',
         '-o',
         '$build/arm64/App.framework/App',
         '$build/arm64/snapshot_assembly.o',
       ]),
+      FakeCommand(command: <String>[
+        'xcrun',
+        'dsymutil',
+        '-o',
+        '$build/arm64/App.framework.dSYM',
+        '$build/arm64/App.framework/App',
+      ]),
+      FakeCommand(command: <String>[
+        'xcrun',
+        'strip',
+        '-x',
+        '$build/arm64/App.framework/App',
+        '-o',
+        '$build/arm64/App.framework/App',
+      ]),
       FakeCommand(command: <String>[
         'lipo',
         '$build/arm64/App.framework/App',
@@ -616,7 +651,7 @@ void main() {
 
   testUsingContext('kExtraGenSnapshotOptions passes values to gen_snapshot', () async {
     androidEnvironment.defines[kExtraGenSnapshotOptions] = 'foo,bar,baz=2';
-    androidEnvironment.defines[kBuildMode] = getNameForBuildMode(BuildMode.profile);
+    androidEnvironment.defines[kBuildMode] = BuildMode.profile.cliName;
     final String build = androidEnvironment.buildDir.path;
 
     processManager.addCommands(<FakeCommand>[
