@@ -334,6 +334,28 @@ void main() {
       expect(logger.warningText, contains('Flutter assets will be downloaded from $baseUrl'));
       expect(logger.statusText, isEmpty);
     });
+
+    testWithoutContext('a non-empty realm is included in the storage url', () async {
+      final MemoryFileSystem fileSystem = MemoryFileSystem.test();
+      final Directory internalDir = fileSystem.currentDirectory
+        .childDirectory('cache')
+        .childDirectory('bin')
+        .childDirectory('internal');
+      final File engineVersionFile = internalDir.childFile('engine.version');
+      engineVersionFile.createSync(recursive: true);
+      engineVersionFile.writeAsStringSync('abcdef');
+
+      final File engineRealmFile = internalDir.childFile('engine.realm');
+      engineRealmFile.createSync(recursive: true);
+      engineRealmFile.writeAsStringSync('flutter_archives_v2');
+
+      final Cache cache = Cache.test(
+        processManager: FakeProcessManager.any(),
+        fileSystem: fileSystem,
+      );
+
+      expect(cache.storageBaseUrl, contains('flutter_archives_v2'));
+    });
   });
 
   testWithoutContext('flattenNameSubdirs', () {
@@ -1267,7 +1289,7 @@ class FakeAndroidSdk extends Fake implements AndroidSdk {
   bool reinitialized = false;
 
   @override
-  void reinitialize() {
+  void reinitialize({FileSystem? fileSystem}) {
     reinitialized = true;
   }
 }
