@@ -75,6 +75,8 @@ FakeCommand attachDebuggerCommand({
   String stdout = '(lldb)     run\nsuccess',
   Completer<void>? completer,
   bool isWirelessDevice = false,
+  bool uninstallFirst = false,
+  bool skipInstall = false,
 }) {
   return FakeCommand(
     command: <String>[
@@ -87,6 +89,10 @@ FakeCommand attachDebuggerCommand({
       '123',
       '--bundle',
       '/',
+      if (uninstallFirst)
+        '--uninstall',
+      if (skipInstall)
+        '--noinstall',
       '--debug',
       if (!isWirelessDevice) '--no-wifi',
       '--args',
@@ -207,7 +213,7 @@ void main() {
     completer.complete();
     expect(secondLaunchResult.started, true);
     expect(secondLaunchResult.hasVmService, true);
-    expect(await device.stopApp(iosApp), false);
+    expect(await device.stopApp(iosApp), true);
   });
 
   testWithoutContext('IOSDevice.startApp launches in debug mode via log reading on <iOS 13', () async {
@@ -285,7 +291,7 @@ void main() {
 
     expect(launchResult.started, true);
     expect(launchResult.hasVmService, true);
-    expect(await device.stopApp(iosApp), false);
+    expect(await device.stopApp(iosApp), true);
     expect(logger.errorText, contains('The Dart VM Service was not discovered after 30 seconds. This is taking much longer than expected...'));
     expect(utf8.decoder.convert(stdin.writes.first), contains('process interrupt'));
     completer.complete();
@@ -330,7 +336,7 @@ void main() {
 
     expect(launchResult.started, true);
     expect(launchResult.hasVmService, true);
-    expect(await device.stopApp(iosApp), false);
+    expect(await device.stopApp(iosApp), true);
     expect(logger.errorText, contains('The Dart VM Service was not discovered after 45 seconds. This is taking much longer than expected...'));
     expect(logger.errorText, contains('Click "Allow" to the prompt asking if you would like to find and connect devices on your local network.'));
     completer.complete();
@@ -339,6 +345,88 @@ void main() {
     MDnsVmServiceDiscovery: () => FakeMDnsVmServiceDiscovery(),
   });
 
+  testWithoutContext('IOSDevice.startApp retries when ios-deploy loses connection the first time in CI', () async {
+    final BufferLogger logger = BufferLogger.test();
+    final FileSystem fileSystem = MemoryFileSystem.test();
+    final Completer<void> completer = Completer<void>();
+    final FakeProcessManager processManager = FakeProcessManager.list(<FakeCommand>[
+      attachDebuggerCommand(
+        stdout: '(lldb)     run\nsuccess\nProcess 525 exited with status = -1 (0xffffffff) lost connection',
+        uninstallFirst: true,
+      ),
+      attachDebuggerCommand(
+        stdout: '(lldb)     run\nsuccess\nThe Dart VM service is listening on http://127.0.0.1:456',
+        completer: completer,
+        skipInstall: true,
+      ),
+    ]);
+    final IOSDevice device = setUpIOSDevice(
+      processManager: processManager,
+      fileSystem: fileSystem,
+      logger: logger,
+    );
+    final IOSApp iosApp = PrebuiltIOSApp(
+      projectBundleId: 'app',
+      bundleName: 'Runner',
+      uncompressedBundle: fileSystem.currentDirectory,
+      applicationPackage: fileSystem.currentDirectory,
+    );
+
+    device.portForwarder = const NoOpDevicePortForwarder();
+
+    final LaunchResult launchResult = await device.startApp(iosApp,
+      prebuiltApplication: true,
+      debuggingOptions: DebuggingOptions.enabled(
+        BuildInfo.debug,
+        usingCISystem: true,
+        uninstallFirst: true,
+      ),
+      platformArgs: <String, dynamic>{},
+    );
+    completer.complete();
+
+    expect(processManager, hasNoRemainingExpectations);
+    expect(launchResult.started, true);
+    expect(launchResult.hasVmService, true);
+    expect(await device.stopApp(iosApp), true);
+  });
+
+  testWithoutContext('IOSDevice.startApp does not retry when ios-deploy loses connection if not in CI', () async {
+    final BufferLogger logger = BufferLogger.test();
+    final FileSystem fileSystem = MemoryFileSystem.test();
+    final FakeProcessManager processManager = FakeProcessManager.list(<FakeCommand>[
+      attachDebuggerCommand(
+        stdout: '(lldb)     run\nsuccess\nProcess 525 exited with status = -1 (0xffffffff) lost connection',
+      ),
+    ]);
+    final IOSDevice device = setUpIOSDevice(
+      processManager: processManager,
+      fileSystem: fileSystem,
+      logger: logger,
+    );
+    final IOSApp iosApp = PrebuiltIOSApp(
+      projectBundleId: 'app',
+      bundleName: 'Runner',
+      uncompressedBundle: fileSystem.currentDirectory,
+      applicationPackage: fileSystem.currentDirectory,
+    );
+
+    device.portForwarder = const NoOpDevicePortForwarder();
+
+    final LaunchResult launchResult = await device.startApp(iosApp,
+      prebuiltApplication: true,
+      debuggingOptions: DebuggingOptions.enabled(
+        BuildInfo.debug,
+      ),
+      platformArgs: <String, dynamic>{},
+    );
+
+    expect(processManager, hasNoRemainingExpectations);
+    expect(launchResult.started, false);
+    expect(launchResult.hasVmService, false);
+    expect(await device.stopApp(iosApp), false);
+  });
+
   testWithoutContext('IOSDevice.startApp succeeds in release mode', () async {
     final FileSystem fileSystem = MemoryFileSystem.test();
     final FakeProcessManager processManager = FakeProcessManager.list(<FakeCommand>[
@@ -397,6 +485,7 @@ void main() {
             '--verify-entry-points',
             '--enable-software-rendering',
             '--trace-systrace',
+            '--trace-to-file="path/to/trace.binpb"',
             '--skia-deterministic-rendering',
             '--trace-skia',
             '--trace-allowlist="foo"',
@@ -453,6 +542,7 @@ void main() {
         traceAllowlist: 'foo',
         traceSkiaAllowlist: 'skia.a,skia.b',
         traceSystrace: true,
+        traceToFile: 'path/to/trace.binpb',
         endlessTraceBuffer: true,
         dumpSkpOnShaderCompilation: true,
         cacheSkSL: true,
@@ -814,6 +904,68 @@ void main() {
       }, overrides: <Type, Generator>{
         MDnsVmServiceDiscovery: () => FakeMDnsVmServiceDiscovery(),
       });
+
+      group('IOSDevice.startApp attaches in debug mode via device logging', () {
+        late FakeMDnsVmServiceDiscovery mdnsDiscovery;
+        setUp(() {
+          mdnsDiscovery = FakeMDnsVmServiceDiscovery(returnsNull: true);
+        });
+
+        testUsingContext('when mDNS fails', () async {
+          final FileSystem fileSystem = MemoryFileSystem.test();
+          final FakeProcessManager processManager = FakeProcessManager.empty();
+
+          final Directory temporaryXcodeProjectDirectory = fileSystem.systemTempDirectory.childDirectory('flutter_empty_xcode.rand0');
+          final Directory bundleLocation = fileSystem.currentDirectory;
+          final IOSDevice device = setUpIOSDevice(
+            processManager: processManager,
+            fileSystem: fileSystem,
+            isCoreDevice: true,
+            coreDeviceControl: FakeIOSCoreDeviceControl(),
+            xcodeDebug: FakeXcodeDebug(
+              expectedProject: XcodeDebugProject(
+                scheme: 'Runner',
+                xcodeWorkspace: temporaryXcodeProjectDirectory.childDirectory('Runner.xcworkspace'),
+                xcodeProject: temporaryXcodeProjectDirectory.childDirectory('Runner.xcodeproj'),
+                hostAppProjectName: 'Runner',
+              ),
+              expectedDeviceId: '123',
+              expectedLaunchArguments: <String>['--enable-dart-profiling'],
+              expectedBundlePath: bundleLocation.path,
+            )
+          );
+          final IOSApp iosApp = PrebuiltIOSApp(
+            projectBundleId: 'app',
+            bundleName: 'Runner',
+            uncompressedBundle: bundleLocation,
+            applicationPackage: bundleLocation,
+          );
+          final FakeDeviceLogReader deviceLogReader = FakeDeviceLogReader();
+
+          device.portForwarder = const NoOpDevicePortForwarder();
+          device.setLogReader(iosApp, deviceLogReader);
+
+          unawaited(mdnsDiscovery.completer.future.whenComplete(() {
+            // Start writing messages to the log reader.
+            Timer.run(() {
+              deviceLogReader.addLine('Foo');
+              deviceLogReader.addLine('The Dart VM service is listening on http://127.0.0.1:456');
+            });
+          }));
+
+          final LaunchResult launchResult = await device.startApp(iosApp,
+            prebuiltApplication: true,
+            debuggingOptions: DebuggingOptions.enabled(BuildInfo.debug),
+            platformArgs: <String, dynamic>{},
+          );
+
+          expect(launchResult.started, true);
+          expect(launchResult.hasVmService, true);
+          expect(await device.stopApp(iosApp), true);
+        }, overrides: <Type, Generator>{
+          MDnsVmServiceDiscovery: () => mdnsDiscovery,
+        });
+      });
     });
   });
 }
@@ -884,6 +1036,10 @@ class FakeDevicePortForwarder extends Fake implements DevicePortForwarder {
 }
 
 class FakeMDnsVmServiceDiscovery extends Fake implements MDnsVmServiceDiscovery {
+  FakeMDnsVmServiceDiscovery({this.returnsNull = false});
+  bool returnsNull;
+
+  Completer<void> completer = Completer<void>();
   @override
   Future<Uri?> getVMServiceUriForLaunch(
     String applicationId,
@@ -894,6 +1050,11 @@ class FakeMDnsVmServiceDiscovery extends Fake implements MDnsVmServiceDiscovery
     bool useDeviceIPAsHost = false,
     Duration timeout = Duration.zero,
   }) async {
+    completer.complete();
+    if (returnsNull) {
+      return null;
+    }
+
     return Uri.tryParse('http://0.0.0.0:1234');
   }
 }
