@@ -23,16 +23,25 @@ import 'package:flutter_tools/src/migrations/xcode_thin_binary_build_phase_input
 import 'package:flutter_tools/src/reporting/reporting.dart';
 import 'package:flutter_tools/src/xcode_project.dart';
 import 'package:test/fake.dart';
+import 'package:unified_analytics/unified_analytics.dart';
 
 import '../../src/common.dart';
 import '../../src/fake_process_manager.dart';
+import '../../src/fakes.dart';
 
 void main () {
   group('iOS migration', () {
     late TestUsage testUsage;
+    late FakeAnalytics fakeAnalytics;
 
     setUp(() {
       testUsage = TestUsage();
+
+      final MemoryFileSystem fs = MemoryFileSystem.test();
+      fakeAnalytics = getInitializedFakeAnalyticsInstance(
+        fs: fs,
+        fakeFlutterVersion: FakeFlutterVersion(),
+      );
     });
 
     testWithoutContext('migrators succeed', () {
@@ -59,10 +68,12 @@ void main () {
         final RemoveFrameworkLinkAndEmbeddingMigration iosProjectMigration = RemoveFrameworkLinkAndEmbeddingMigration(
           project,
           testLogger,
-          testUsage
+          testUsage,
+          fakeAnalytics,
         );
         iosProjectMigration.migrate();
         expect(testUsage.events, isEmpty);
+        expect(fakeAnalytics.sentEvents, isEmpty);
 
         expect(xcodeProjectInfoFile.existsSync(), isFalse);
 
@@ -79,9 +90,11 @@ void main () {
           project,
           testLogger,
           testUsage,
+          fakeAnalytics,
         );
         iosProjectMigration.migrate();
         expect(testUsage.events, isEmpty);
+        expect(fakeAnalytics.sentEvents, isEmpty);
 
         expect(xcodeProjectInfoFile.lastModifiedSync(), projectLastModified);
         expect(xcodeProjectInfoFile.readAsStringSync(), contents);
@@ -99,6 +112,7 @@ shellScript = "/bin/sh \"$FLUTTER_ROOT/packages/flutter_tools/bin/xcode_backend.
           project,
           testLogger,
           testUsage,
+          fakeAnalytics,
         );
         iosProjectMigration.migrate();
         expect(xcodeProjectInfoFile.readAsStringSync(), contents);
@@ -126,9 +140,11 @@ keep this 2
           project,
           testLogger,
           testUsage,
+          fakeAnalytics,
         );
         iosProjectMigration.migrate();
         expect(testUsage.events, isEmpty);
+        expect(fakeAnalytics.sentEvents, isEmpty);
 
         expect(xcodeProjectInfoFile.readAsStringSync(), r'''
 keep this 1
@@ -147,12 +163,20 @@ keep this 2
           project,
           testLogger,
           testUsage,
+          fakeAnalytics,
         );
 
         expect(iosProjectMigration.migrate, throwsToolExit(message: 'Your Xcode project requires migration'));
         expect(testUsage.events, contains(
           const TestUsageEvent('ios-migration', 'remove-frameworks', label: 'failure'),
         ));
+        expect(fakeAnalytics.sentEvents, contains(
+          Event.appleUsageEvent(
+              workflow: 'ios-migration',
+              parameter: 'remove-frameworks',
+              result: 'failure',
+            )
+        ));
       });
 
       testWithoutContext('migration fails with leftover Flutter.framework reference', () {
@@ -164,11 +188,19 @@ keep this 2
           project,
           testLogger,
           testUsage,
+          fakeAnalytics,
         );
         expect(iosProjectMigration.migrate, throwsToolExit(message: 'Your Xcode project requires migration'));
         expect(testUsage.events, contains(
           const TestUsageEvent('ios-migration', 'remove-frameworks', label: 'failure'),
         ));
+        expect(fakeAnalytics.sentEvents, contains(
+          Event.appleUsageEvent(
+              workflow: 'ios-migration',
+              parameter: 'remove-frameworks',
+              result: 'failure',
+            )
+        ));
       });
 
       testWithoutContext('migration fails without Xcode installed', () {
@@ -180,11 +212,19 @@ keep this 2
           project,
           testLogger,
           testUsage,
+          fakeAnalytics,
         );
         expect(iosProjectMigration.migrate, throwsToolExit(message: 'Your Xcode project requires migration'));
         expect(testUsage.events, contains(
           const TestUsageEvent('ios-migration', 'remove-frameworks', label: 'failure'),
         ));
+        expect(fakeAnalytics.sentEvents, contains(
+          Event.appleUsageEvent(
+              workflow: 'ios-migration',
+              parameter: 'remove-frameworks',
+              result: 'failure',
+            )
+        ));
       });
     });
 
@@ -546,18 +586,18 @@ keep this 3
       });
 
       testWithoutContext('skipped if nothing to upgrade', () {
-        const String xcodeProjectInfoFileContents = 'IPHONEOS_DEPLOYMENT_TARGET = 11.0;';
+        const String xcodeProjectInfoFileContents = 'IPHONEOS_DEPLOYMENT_TARGET = 12.0;';
         xcodeProjectInfoFile.writeAsStringSync(xcodeProjectInfoFileContents);
 
         const String appFrameworkInfoPlistContents = '''
   <key>MinimumOSVersion</key>
-  <string>11.0</string>
+  <string>12.0</string>
 ''';
         appFrameworkInfoPlist.writeAsStringSync(appFrameworkInfoPlistContents);
 
         final DateTime projectLastModified = xcodeProjectInfoFile.lastModifiedSync();
 
-        const String podfileFileContents = "# platform :ios, '11.0'";
+        const String podfileFileContents = "# platform :ios, '12.0'";
         podfile.writeAsStringSync(podfileFileContents);
         final DateTime podfileLastModified = podfile.lastModifiedSync();
 
@@ -576,7 +616,7 @@ keep this 3
         expect(testLogger.statusText, isEmpty);
       });
 
-      testWithoutContext('Xcode project is migrated to 11', () {
+      testWithoutContext('Xcode project is migrated to 12', () {
         xcodeProjectInfoFile.writeAsStringSync('''
 				GCC_WARN_UNUSED_VARIABLE = YES;
 				IPHONEOS_DEPLOYMENT_TARGET = 8.0;
@@ -585,6 +625,7 @@ keep this 3
 
 				IPHONEOS_DEPLOYMENT_TARGET = 8.0;
 				IPHONEOS_DEPLOYMENT_TARGET = 11.0;
+				IPHONEOS_DEPLOYMENT_TARGET = 12.0;
 ''');
 
         appFrameworkInfoPlist.writeAsStringSync('''
@@ -598,6 +639,8 @@ keep this 3
   <string>8.0</string>
   <key>MinimumOSVersion</key>
   <string>11.0</string>
+  <key>MinimumOSVersion</key>
+  <string>12.0</string>
 </dict>
 </plist>
 ''');
@@ -605,6 +648,8 @@ keep this 3
         podfile.writeAsStringSync('''
 # platform :ios, '9.0'
 platform :ios, '9.0'
+# platform :ios, '11.0'
+platform :ios, '11.0'
 ''');
 
         final IOSDeploymentTargetMigration iosProjectMigration = IOSDeploymentTargetMigration(
@@ -615,12 +660,13 @@ platform :ios, '9.0'
 
         expect(xcodeProjectInfoFile.readAsStringSync(), '''
 				GCC_WARN_UNUSED_VARIABLE = YES;
-				IPHONEOS_DEPLOYMENT_TARGET = 11.0;
+				IPHONEOS_DEPLOYMENT_TARGET = 12.0;
 				MTL_ENABLE_DEBUG_INFO = YES;
 				ONLY_ACTIVE_ARCH = YES;
 
-				IPHONEOS_DEPLOYMENT_TARGET = 11.0;
-				IPHONEOS_DEPLOYMENT_TARGET = 11.0;
+				IPHONEOS_DEPLOYMENT_TARGET = 12.0;
+				IPHONEOS_DEPLOYMENT_TARGET = 12.0;
+				IPHONEOS_DEPLOYMENT_TARGET = 12.0;
 ''');
 
         expect(appFrameworkInfoPlist.readAsStringSync(), '''
@@ -631,19 +677,23 @@ platform :ios, '9.0'
   <key>CFBundleVersion</key>
   <string>1.0</string>
   <key>MinimumOSVersion</key>
-  <string>11.0</string>
+  <string>12.0</string>
   <key>MinimumOSVersion</key>
-  <string>11.0</string>
+  <string>12.0</string>
+  <key>MinimumOSVersion</key>
+  <string>12.0</string>
 </dict>
 </plist>
 ''');
 
         expect(podfile.readAsStringSync(), '''
-# platform :ios, '11.0'
-platform :ios, '11.0'
+# platform :ios, '12.0'
+platform :ios, '12.0'
+# platform :ios, '12.0'
+platform :ios, '12.0'
 ''');
         // Only print once even though 2 lines were changed.
-        expect('Updating minimum iOS deployment target to 11.0'.allMatches(testLogger.statusText).length, 1);
+        expect('Updating minimum iOS deployment target to 12.0'.allMatches(testLogger.statusText).length, 1);
       });
     });
 
@@ -662,7 +712,7 @@ platform :ios, '11.0'
         project.xcodeProjectInfoFile = xcodeProjectInfoFile;
 
         xcodeProjectSchemeFile = memoryFileSystem.file('Runner.xcscheme');
-        project.xcodeProjectSchemeFile = xcodeProjectSchemeFile;
+        project.schemeFile = xcodeProjectSchemeFile;
       });
 
       testWithoutContext('skipped if files are missing', () {
@@ -686,13 +736,13 @@ platform :ios, '11.0'
 	objectVersion = 54;
 	objects = {
 			attributes = {
-				LastUpgradeCheck = 1430;
+				LastUpgradeCheck = 1510;
 				ORGANIZATIONNAME = "";
       ''';
         xcodeProjectInfoFile.writeAsStringSync(xcodeProjectInfoFileContents);
 
         const String xcodeProjectSchemeFileContents = '''
-   LastUpgradeVersion = "1430"
+   LastUpgradeVersion = "1510"
 ''';
         xcodeProjectSchemeFile.writeAsStringSync(xcodeProjectSchemeFileContents);
 
@@ -718,13 +768,13 @@ platform :ios, '11.0'
 	objectVersion = 46;
 	objects = {
 			attributes = {
-				LastUpgradeCheck = 1020;
+				LastUpgradeCheck = 1430;
 				ORGANIZATIONNAME = "";
 ''');
 
         xcodeProjectSchemeFile.writeAsStringSync('''
 <Scheme
-   LastUpgradeVersion = "1020"
+   LastUpgradeVersion = "1430"
    version = "1.3">
 ''');
 
@@ -740,13 +790,13 @@ platform :ios, '11.0'
 	objectVersion = 54;
 	objects = {
 			attributes = {
-				LastUpgradeCheck = 1430;
+				LastUpgradeCheck = 1510;
 				ORGANIZATIONNAME = "";
 ''');
 
         expect(xcodeProjectSchemeFile.readAsStringSync(), '''
 <Scheme
-   LastUpgradeVersion = "1430"
+   LastUpgradeVersion = "1510"
    version = "1.3">
 ''');
         // Only print once even though 3 lines were changed.
@@ -864,7 +914,7 @@ platform :ios, '11.0'
       });
 
       testWithoutContext('skipped if nothing to upgrade', () {
-        const String xcodeProjectInfoFileContents = 'IPHONEOS_DEPLOYMENT_TARGET = 11.0;';
+        const String xcodeProjectInfoFileContents = 'IPHONEOS_DEPLOYMENT_TARGET = 12.0;';
         xcodeProjectInfoFile.writeAsStringSync(xcodeProjectInfoFileContents);
         final DateTime projectLastModified = xcodeProjectInfoFile.lastModifiedSync();
 
@@ -1370,8 +1420,10 @@ class FakeIosProject extends Fake implements IosProject {
   @override
   File xcodeProjectInfoFile = MemoryFileSystem.test().file('xcodeProjectInfoFile');
 
+  File? schemeFile;
+
   @override
-  File xcodeProjectSchemeFile = MemoryFileSystem.test().file('xcodeProjectSchemeFile');
+  File xcodeProjectSchemeFile({String? scheme}) => schemeFile ?? MemoryFileSystem.test().file('xcodeProjectSchemeFile');
 
   @override
   File appFrameworkInfoPlist = MemoryFileSystem.test().file('appFrameworkInfoPlist');
