@@ -2,27 +2,30 @@
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
-// @dart = 2.8
-
-// ignore_for_file: avoid_redundant_argument_values
-
+import 'dart:async';
 import 'dart:io' hide Directory, File;
 
+import 'package:dwds/dwds.dart';
+import 'package:fake_async/fake_async.dart';
 import 'package:flutter_tools/src/artifacts.dart';
 import 'package:flutter_tools/src/base/file_system.dart';
 import 'package:flutter_tools/src/base/logger.dart';
 import 'package:flutter_tools/src/base/platform.dart';
 import 'package:flutter_tools/src/build_info.dart';
-import 'package:flutter_tools/src/build_system/targets/web.dart';
+import 'package:flutter_tools/src/build_system/targets/shader_compiler.dart';
 import 'package:flutter_tools/src/compile.dart';
 import 'package:flutter_tools/src/convert.dart';
+import 'package:flutter_tools/src/devfs.dart';
+import 'package:flutter_tools/src/device.dart';
 import 'package:flutter_tools/src/globals.dart' as globals;
+import 'package:flutter_tools/src/html_utils.dart';
 import 'package:flutter_tools/src/isolated/devfs_web.dart';
 import 'package:flutter_tools/src/web/compile.dart';
 import 'package:logging/logging.dart' as logging;
 import 'package:package_config/package_config.dart';
 import 'package:shelf/shelf.dart';
 import 'package:test/fake.dart';
+import 'package:vm_service/vm_service.dart' as vm_service;
 
 import '../../src/common.dart';
 import '../../src/testbed.dart';
@@ -36,14 +39,14 @@ const List<int> kTransparentImage = <int>[
 ];
 
 void main() {
-  Testbed testbed;
-  WebAssetServer webAssetServer;
-  ReleaseAssetServer releaseAssetServer;
-  Platform linux;
-  PackageConfig packages;
-  Platform windows;
-  FakeHttpServer httpServer;
-  BufferLogger logger;
+  late Testbed testbed;
+  late WebAssetServer webAssetServer;
+  late ReleaseAssetServer releaseAssetServer;
+  late Platform linux;
+  late PackageConfig packages;
+  late Platform windows;
+  late FakeHttpServer httpServer;
+  late BufferLogger logger;
 
   setUpAll(() async {
     packages = PackageConfig(<Package>[
@@ -61,17 +64,16 @@ void main() {
         httpServer,
         packages,
         InternetAddress.loopbackIPv4,
-        null,
-        null,
-        null,
+        <String, String>{},
+        <String, String>{},
+        NullSafetyMode.unsound,
       );
       releaseAssetServer = ReleaseAssetServer(
         globals.fs.file('main.dart').uri,
-        fileSystem: null,
+        fileSystem: globals.fs,
         flutterRoot: null,
-        platform: null,
+        platform: FakePlatform(),
         webBuildDirectory: null,
-        basePath: null,
       );
     }, overrides: <Type, Generator>{
       Logger: () => logger,
@@ -178,10 +180,10 @@ void main() {
       }));
     webAssetServer.write(source, manifest, sourcemap, metadata);
 
-    final String merged = await webAssetServer.metadataContents('main_module.ddc_merged_metadata');
+    final String? merged = await webAssetServer.metadataContents('main_module.ddc_merged_metadata');
     expect(merged, equals(metadataContents));
 
-    final String single = await webAssetServer.metadataContents('foo.js.metadata');
+    final String? single = await webAssetServer.metadataContents('foo.js.metadata');
     expect(single, equals(metadataContents));
   }, overrides: <Type, Generator>{
     Platform: () => linux,
@@ -287,9 +289,9 @@ void main() {
       httpServer,
       packages,
       InternetAddress.loopbackIPv4,
-      null,
-      null,
-      null,
+      <String, String>{},
+      <String, String>{},
+      NullSafetyMode.unsound,
     );
 
     expect(webAssetServer.basePath, 'foo/bar');
@@ -306,9 +308,9 @@ void main() {
       httpServer,
       packages,
       InternetAddress.loopbackIPv4,
-      null,
-      null,
-      null,
+      <String, String>{},
+      <String, String>{},
+      NullSafetyMode.unsound,
     );
 
     // Defaults to "/" when there's no base element.
@@ -327,9 +329,9 @@ void main() {
         httpServer,
         packages,
         InternetAddress.loopbackIPv4,
-        null,
-        null,
-        null,
+        <String, String>{},
+        <String, String>{},
+        NullSafetyMode.unsound,
       ),
       throwsToolExit(),
     );
@@ -347,9 +349,9 @@ void main() {
         httpServer,
         packages,
         InternetAddress.loopbackIPv4,
-        null,
-        null,
-        null,
+        <String, String>{},
+        <String, String>{},
+        NullSafetyMode.unsound,
       ),
       throwsToolExit(),
     );
@@ -375,7 +377,7 @@ void main() {
 
     final Response response = await webAssetServer
       .handleRequest(Request('GET', Uri.parse('http://foobar/foo.js')));
-    final String etag = response.headers[HttpHeaders.etagHeader];
+    final String etag = response.headers[HttpHeaders.etagHeader]!;
 
     final Response cachedResponse = await webAssetServer
       .handleRequest(Request('GET', Uri.parse('http://foobar/foo.js'), headers: <String, String>{
@@ -590,12 +592,12 @@ void main() {
 
     expect(response.headers, allOf(<Matcher>[
       containsPair(HttpHeaders.contentLengthHeader, '0'),
-      containsPair(HttpHeaders.contentTypeHeader, 'application/javascript'),
+      containsPair(HttpHeaders.contentTypeHeader, 'text/javascript'),
     ]));
     expect((await response.read().toList()).first, source.readAsBytesSync());
   }));
 
-  test('serves asset files files from in filesystem with unknown mime type', () => testbed.run(() async {
+  test('serves asset files from in filesystem with unknown mime type', () => testbed.run(() async {
     final File source = globals.fs.file(globals.fs.path.join('build', 'flutter_assets', 'foo'))
       ..createSync(recursive: true)
       ..writeAsBytesSync(List<int>.filled(100, 0));
@@ -617,14 +619,14 @@ void main() {
 
     final Response response = await webAssetServer
       .handleRequest(Request('GET', Uri.parse('http://foobar/assets/fooπ')));
-    final String etag = response.headers[HttpHeaders.etagHeader];
+    final String etag = response.headers[HttpHeaders.etagHeader]!;
 
     expect(etag.runes, everyElement(predicate((int char) => char < 255)));
   }));
 
   test('serves /packages/<package>/<path> files as if they were '
        'package:<package>/<path> uris', () => testbed.run(() async {
-    final Uri expectedUri = packages.resolve(
+    final Uri? expectedUri = packages.resolve(
         Uri.parse('package:flutter_tools/foo.dart'));
     final File source = globals.fs.file(globals.fs.path.fromUri(expectedUri))
       ..createSync(recursive: true)
@@ -660,6 +662,8 @@ void main() {
     final WebDevFS webDevFS = WebDevFS(
       hostname: 'localhost',
       port: 0,
+      tlsCertPath: null,
+      tlsCertKeyPath: null,
       packagesFilePath: '.packages',
       urlTunneller: null,
       useSseForDebugProxy: true,
@@ -678,21 +682,23 @@ void main() {
       entrypoint: Uri.base,
       testMode: true,
       expressionCompiler: null,
+      extraHeaders: const <String, String>{},
       chromiumLauncher: null,
       nullSafetyMode: NullSafetyMode.unsound,
     );
     webDevFS.requireJS.createSync(recursive: true);
+    webDevFS.flutterJs.createSync(recursive: true);
     webDevFS.stackTraceMapper.createSync(recursive: true);
 
     final Uri uri = await webDevFS.create();
     webDevFS.webAssetServer.entrypointCacheDirectory = globals.fs.currentDirectory;
-    final String webPrecompiledSdk = globals.artifacts
+    final String webPrecompiledSdk = globals.artifacts!
       .getHostArtifact(HostArtifact.webPrecompiledSdk).path;
-    final String webPrecompiledSdkSourcemaps = globals.artifacts
+    final String webPrecompiledSdkSourcemaps = globals.artifacts!
       .getHostArtifact(HostArtifact.webPrecompiledSdkSourcemaps).path;
-    final String webPrecompiledCanvaskitSdk = globals.artifacts
+    final String webPrecompiledCanvaskitSdk = globals.artifacts!
       .getHostArtifact(HostArtifact.webPrecompiledCanvaskitSdk).path;
-    final String webPrecompiledCanvaskitSdkSourcemaps = globals.artifacts
+    final String webPrecompiledCanvaskitSdkSourcemaps = globals.artifacts!
       .getHostArtifact(HostArtifact.webPrecompiledCanvaskitSdkSourcemaps).path;
     globals.fs.currentDirectory
       .childDirectory('lib')
@@ -721,6 +727,7 @@ void main() {
       packageConfig: PackageConfig.empty,
       pathToReload: '',
       dillOutputPath: 'out.dill',
+      shaderCompiler: const FakeShaderCompiler(),
     );
 
     expect(webDevFS.webAssetServer.getFile('require.js'), isNotNull);
@@ -772,6 +779,8 @@ void main() {
     final WebDevFS webDevFS = WebDevFS(
       hostname: 'localhost',
       port: 0,
+      tlsCertPath: null,
+      tlsCertKeyPath: null,
       packagesFilePath: '.packages',
       urlTunneller: null,
       useSseForDebugProxy: true,
@@ -789,10 +798,12 @@ void main() {
       entrypoint: Uri.base,
       testMode: true,
       expressionCompiler: null,
+      extraHeaders: const <String, String>{},
       chromiumLauncher: null,
       nullSafetyMode: NullSafetyMode.sound,
     );
     webDevFS.requireJS.createSync(recursive: true);
+    webDevFS.flutterJs.createSync(recursive: true);
     webDevFS.stackTraceMapper.createSync(recursive: true);
 
     final Uri uri = await webDevFS.create();
@@ -802,13 +813,13 @@ void main() {
       .childFile('web_entrypoint.dart')
       ..createSync(recursive: true)
       ..writeAsStringSync('GENERATED');
-    final String webPrecompiledSdk = globals.artifacts
+    final String webPrecompiledSdk = globals.artifacts!
       .getHostArtifact(HostArtifact.webPrecompiledSoundSdk).path;
-    final String webPrecompiledSdkSourcemaps = globals.artifacts
+    final String webPrecompiledSdkSourcemaps = globals.artifacts!
       .getHostArtifact(HostArtifact.webPrecompiledSoundSdkSourcemaps).path;
-    final String webPrecompiledCanvaskitSdk = globals.artifacts
+    final String webPrecompiledCanvaskitSdk = globals.artifacts!
       .getHostArtifact(HostArtifact.webPrecompiledCanvaskitSoundSdk).path;
-    final String webPrecompiledCanvaskitSdkSourcemaps = globals.artifacts
+    final String webPrecompiledCanvaskitSdkSourcemaps = globals.artifacts!
       .getHostArtifact(HostArtifact.webPrecompiledCanvaskitSoundSdkSourcemaps).path;
     globals.fs.file(webPrecompiledSdk)
       ..createSync(recursive: true)
@@ -832,6 +843,7 @@ void main() {
       packageConfig: PackageConfig.empty,
       pathToReload: '',
       dillOutputPath: '',
+      shaderCompiler: const FakeShaderCompiler(),
     );
 
     expect(webDevFS.webAssetServer.getFile('require.js'), isNotNull);
@@ -867,6 +879,74 @@ void main() {
     Artifacts: () => Artifacts.test(),
   }));
 
+  test('.connect() will never call vmServiceFactory twice', () => testbed.run(() async {
+    await FakeAsync().run<Future<void>>((FakeAsync time) {
+      final File outputFile = globals.fs.file(globals.fs.path.join('lib', 'main.dart'))
+        ..createSync(recursive: true);
+      outputFile.parent.childFile('a.sources').writeAsStringSync('');
+      outputFile.parent.childFile('a.json').writeAsStringSync('{}');
+      outputFile.parent.childFile('a.map').writeAsStringSync('{}');
+      outputFile.parent.childFile('a.metadata').writeAsStringSync('{}');
+
+      final WebDevFS webDevFS = WebDevFS(
+        // if this is any other value, we will do a real ip lookup
+        hostname: 'any',
+        port: 0,
+        tlsCertPath: null,
+        tlsCertKeyPath: null,
+        packagesFilePath: '.packages',
+        urlTunneller: null,
+        useSseForDebugProxy: true,
+        useSseForDebugBackend: true,
+        useSseForInjectedClient: true,
+        nullAssertions: true,
+        nativeNullAssertions: true,
+        buildInfo: const BuildInfo(
+          BuildMode.debug,
+          '',
+          treeShakeIcons: false,
+        ),
+        enableDwds: true,
+        enableDds: false,
+        entrypoint: Uri.base,
+        testMode: true,
+        expressionCompiler: null,
+        extraHeaders: const <String, String>{},
+        chromiumLauncher: null,
+        nullSafetyMode: NullSafetyMode.sound,
+      );
+      webDevFS.requireJS.createSync(recursive: true);
+      webDevFS.stackTraceMapper.createSync(recursive: true);
+      final FakeAppConnection firstConnection = FakeAppConnection();
+      final FakeAppConnection secondConnection = FakeAppConnection();
+
+      final Future<void> done = webDevFS.create().then<void>((Uri _) {
+        // In non-test mode, webDevFS.create() would have initialized DWDS
+        webDevFS.webAssetServer.dwds = FakeDwds(<AppConnection>[firstConnection, secondConnection]);
+
+        int vmServiceFactoryInvocationCount = 0;
+        Future<vm_service.VmService> vmServiceFactory(Uri uri, {CompressionOptions? compression, required Logger logger}) {
+          if (vmServiceFactoryInvocationCount > 0) {
+            fail('Called vmServiceFactory twice!');
+          }
+          vmServiceFactoryInvocationCount += 1;
+          return Future<vm_service.VmService>.delayed(
+            const Duration(seconds: 2),
+            () => FakeVmService(),
+          );
+        }
+        return webDevFS.connect(false, vmServiceFactory: vmServiceFactory).then<void>((ConnectionResult? firstConnectionResult) {
+          return webDevFS.destroy();
+        });
+      });
+      time.elapse(const Duration(seconds: 1));
+      time.elapse(const Duration(seconds: 2));
+      return done;
+    });
+  }, overrides: <Type, Generator>{
+    Artifacts: () => Artifacts.test(),
+  }));
+
   test('Can start web server with hostname any', () => testbed.run(() async {
     final File outputFile = globals.fs.file(globals.fs.path.join('lib', 'main.dart'))
       ..createSync(recursive: true);
@@ -877,6 +957,8 @@ void main() {
     final WebDevFS webDevFS = WebDevFS(
       hostname: 'any',
       port: 0,
+      tlsCertPath: null,
+      tlsCertKeyPath: null,
       packagesFilePath: '.packages',
       urlTunneller: null,
       useSseForDebugProxy: true,
@@ -888,6 +970,7 @@ void main() {
       entrypoint: Uri.base,
       testMode: true,
       expressionCompiler: null,
+      extraHeaders: const <String, String>{},
       chromiumLauncher: null,
       nullAssertions: true,
       nativeNullAssertions: true,
@@ -912,6 +995,8 @@ void main() {
     final WebDevFS webDevFS = WebDevFS(
       hostname: 'localhost',
       port: 0,
+      tlsCertPath: null,
+      tlsCertKeyPath: null,
       packagesFilePath: '.packages',
       urlTunneller: null,
       useSseForDebugProxy: true,
@@ -932,6 +1017,7 @@ void main() {
       entrypoint: Uri.base,
       testMode: true,
       expressionCompiler: null,
+      extraHeaders: const <String, String>{},
       chromiumLauncher: null,
       nullSafetyMode: NullSafetyMode.sound,
     );
@@ -955,6 +1041,8 @@ void main() {
     final WebDevFS webDevFS = WebDevFS(
       hostname: 'localhost',
       port: 0,
+      tlsCertPath: null,
+      tlsCertKeyPath: null,
       packagesFilePath: '.packages',
       urlTunneller: null,
       useSseForDebugProxy: true,
@@ -975,6 +1063,7 @@ void main() {
       entrypoint: Uri.base,
       testMode: true,
       expressionCompiler: null,
+      extraHeaders: const <String, String>{},
       chromiumLauncher: null,
       nullSafetyMode: NullSafetyMode.sound,
     );
@@ -983,17 +1072,69 @@ void main() {
 
     await webDevFS.create();
 
-    expect(webDevFS.webAssetServer.webRenderer, WebRendererMode.autoDetect);
+    expect(webDevFS.webAssetServer.webRenderer, WebRendererMode.auto);
 
     await webDevFS.destroy();
   }));
 
+  test('Can start web server with tls connection', () => testbed.run(() async {
+    final String dataPath = globals.fs.path.join(
+      getFlutterRoot(),
+      'packages',
+      'flutter_tools',
+      'test',
+      'data',
+      'asset_test',
+    );
+
+    final String dummyCertPath =
+        globals.fs.path.join(dataPath, 'tls_cert', 'dummy-cert.pem');
+    final String dummyCertKeyPath =
+        globals.fs.path.join(dataPath, 'tls_cert', 'dummy-key.pem');
+
+    final WebDevFS webDevFS = WebDevFS(
+      hostname: 'localhost',
+      port: 0,
+      tlsCertPath: dummyCertPath,
+      tlsCertKeyPath: dummyCertKeyPath,
+      packagesFilePath: '.packages',
+      urlTunneller: null,
+      useSseForDebugProxy: true,
+      useSseForDebugBackend: true,
+      useSseForInjectedClient: true,
+      nullAssertions: true,
+      nativeNullAssertions: true,
+      buildInfo: BuildInfo.debug,
+      enableDwds: false,
+      enableDds: false,
+      entrypoint: Uri.base,
+      testMode: true,
+      expressionCompiler: null,
+      extraHeaders: const <String, String>{},
+      chromiumLauncher: null,
+      nullSafetyMode: NullSafetyMode.unsound,
+    );
+    webDevFS.requireJS.createSync(recursive: true);
+    webDevFS.stackTraceMapper.createSync(recursive: true);
+
+    final Uri uri = await webDevFS.create();
+
+    // Ensure the connection established is secure
+    expect(uri.scheme, 'https');
+
+    await webDevFS.destroy();
+  }, overrides: <Type, Generator>{
+    Artifacts: () => Artifacts.test(),
+  }));
+
   test('allows frame embedding', () async {
     final WebAssetServer webAssetServer = await WebAssetServer.start(
       null,
       'localhost',
       0,
       null,
+      null,
+      null,
       true,
       true,
       true,
@@ -1006,13 +1147,47 @@ void main() {
       false,
       Uri.base,
       null,
-      null,
+      const <String, String>{},
+      NullSafetyMode.unsound,
       testMode: true);
 
     expect(webAssetServer.defaultResponseHeaders['x-frame-options'], null);
     await webAssetServer.dispose();
   });
 
+  test('passes on extra headers', () async {
+    const String extraHeaderKey = 'hurray';
+    const String extraHeaderValue = 'flutter';
+    final WebAssetServer webAssetServer = await WebAssetServer.start(
+      null,
+      'localhost',
+      0,
+      null,
+      null,
+      null,
+      true,
+      true,
+      true,
+      const BuildInfo(
+        BuildMode.debug,
+        '',
+        treeShakeIcons: false,
+      ),
+      false,
+      false,
+      Uri.base,
+      null,
+      const <String, String>{
+        extraHeaderKey: extraHeaderValue,
+      },
+      NullSafetyMode.unsound,
+      testMode: true);
+
+    expect(webAssetServer.defaultResponseHeaders[extraHeaderKey], <String>[extraHeaderValue]);
+
+    await webAssetServer.dispose();
+  });
+
   test('WebAssetServer responds to POST requests with 404 not found', () => testbed.run(() async {
     final Response response = await webAssetServer.handleRequest(
       Request('POST', Uri.parse('http://foobar/something')),
@@ -1065,6 +1240,8 @@ void main() {
     final WebDevFS webDevFS = WebDevFS(
       hostname: 'localhost',
       port: 0,
+      tlsCertPath: null,
+      tlsCertKeyPath: null,
       packagesFilePath: '.packages',
       urlTunneller: null,
       useSseForDebugProxy: true,
@@ -1078,6 +1255,7 @@ void main() {
       entrypoint: Uri.base,
       testMode: true,
       expressionCompiler: null,
+      extraHeaders: const <String, String>{},
       chromiumLauncher: null,
       nullSafetyMode: NullSafetyMode.unsound,
     );
@@ -1107,20 +1285,64 @@ class FakeHttpServer extends Fake implements HttpServer {
 }
 
 class FakeResidentCompiler extends Fake implements ResidentCompiler {
-  CompilerOutput output;
+  CompilerOutput? output;
 
   @override
   void addFileSystemRoot(String root) { }
 
   @override
-  Future<CompilerOutput> recompile(Uri mainUri, List<Uri> invalidatedFiles, {
-    String outputPath,
-    PackageConfig packageConfig,
-    String projectRootPath,
-    FileSystem fs,
+  Future<CompilerOutput?> recompile(Uri mainUri, List<Uri>? invalidatedFiles, {
+    String? outputPath,
+    PackageConfig? packageConfig,
+    String? projectRootPath,
+    FileSystem? fs,
     bool suppressErrors = false,
     bool checkDartPluginRegistry = false,
+    File? dartPluginRegistrant,
+    Uri? nativeAssetsYaml,
   }) async {
     return output;
   }
 }
+
+class FakeShaderCompiler implements DevelopmentShaderCompiler {
+  const FakeShaderCompiler();
+
+  @override
+  void configureCompiler(
+    TargetPlatform? platform, {
+    required ImpellerStatus impellerStatus,
+  }) { }
+
+  @override
+  Future<DevFSContent> recompileShader(DevFSContent inputShader) {
+    throw UnimplementedError();
+  }
+}
+
+class FakeDwds extends Fake implements Dwds {
+  FakeDwds(Iterable<AppConnection> connectedAppsIterable) :
+    connectedApps = Stream<AppConnection>.fromIterable(connectedAppsIterable);
+
+  @override
+  final Stream<AppConnection> connectedApps;
+
+  @override
+  Future<DebugConnection> debugConnection(AppConnection appConnection) => Future<DebugConnection>.value(FakeDebugConnection());
+}
+
+class FakeAppConnection extends Fake implements AppConnection {
+  @override
+  void runMain() {}
+}
+
+class FakeDebugConnection extends Fake implements DebugConnection {
+  FakeDebugConnection({
+    this.uri = 'http://foo',
+  });
+
+  @override
+  final String uri;
+}
+
+class FakeVmService extends Fake implements vm_service.VmService {}
