@@ -4,6 +4,7 @@
 
 import 'package:file/file.dart';
 import 'package:file_testing/file_testing.dart';
+import 'package:flutter_tools/src/base/io.dart';
 
 import '../src/common.dart';
 import 'test_utils.dart';
@@ -68,7 +69,7 @@ void main() {
     // Build example APK
     final Directory exampleDir = projectRoot.childDirectory('example');
 
-    processManager.runSync(<String>[
+    final ProcessResult result = processManager.runSync(<String>[
       flutterBin,
       ...getLocalEngineArguments(),
       'build',
@@ -78,6 +79,8 @@ void main() {
       '--verbose',
     ], workingDirectory: exampleDir.path);
 
+    expect(result, const ProcessResultMatcher());
+
     final String exampleAppApk = fileSystem.path.join(
       exampleDir.path,
       'build',
