@@ -77,7 +77,7 @@ class DeferredComponentModule {
     apply plugin: "com.android.dynamic-feature"
 
     android {
-        compileSdkVersion 31
+        compileSdk 34
 
         sourceSets {
             applicationVariants.all { variant ->
@@ -88,7 +88,7 @@ class DeferredComponentModule {
 
         defaultConfig {
             minSdkVersion 19
-            targetSdkVersion 31
+            targetSdkVersion 33
             versionCode flutterVersionCode.toInteger()
             versionName flutterVersionName
         }
