@@ -3,7 +3,9 @@
 // found in the LICENSE file.
 
 import 'dart:async';
+import 'dart:io' as io;
 
+import 'package:fake_async/fake_async.dart';
 import 'package:file/memory.dart';
 import 'package:file_testing/file_testing.dart';
 import 'package:flutter_tools/src/base/file_system.dart';
@@ -11,10 +13,12 @@ import 'package:flutter_tools/src/base/logger.dart';
 import 'package:flutter_tools/src/base/os.dart';
 import 'package:flutter_tools/src/base/platform.dart';
 import 'package:flutter_tools/src/web/chrome.dart';
+import 'package:test/fake.dart';
+import 'package:webkit_inspection_protocol/webkit_inspection_protocol.dart';
 
 import '../src/common.dart';
 import '../src/fake_process_manager.dart';
-import '../src/fakes.dart';
+import '../src/fakes.dart' hide FakeProcess;
 
 const List<String> kChromeArgs = <String>[
   '--disable-background-timer-throttling',
@@ -42,6 +46,7 @@ void main() {
   late Platform platform;
   late FakeProcessManager processManager;
   late OperatingSystemUtils operatingSystemUtils;
+  late BufferLogger testLogger;
 
   setUp(() {
     exceptionHandler = FileExceptionHandler();
@@ -57,13 +62,41 @@ void main() {
       processManager: processManager,
       operatingSystemUtils: operatingSystemUtils,
       browserFinder: findChromeExecutable,
-      logger: BufferLogger.test(),
+      logger: testLogger = BufferLogger.test(),
     );
   });
 
+  Future<Chromium> testLaunchChrome(String userDataDir, FakeProcessManager processManager, ChromiumLauncher chromeLauncher) {
+    if (testLogger.isVerbose) {
+      processManager.addCommand(const FakeCommand(
+        command: <String>[
+          'example_chrome',
+          '--version',
+        ],
+        stdout: 'Chromium 115',
+      ));
+    }
+
+    processManager.addCommand(FakeCommand(
+      command: <String>[
+        'example_chrome',
+        '--user-data-dir=$userDataDir',
+        '--remote-debugging-port=12345',
+        ...kChromeArgs,
+        'example_url',
+      ],
+      stderr: kDevtoolsStderr,
+    ));
+
+    return chromeLauncher.launch(
+      'example_url',
+      skipCheck: true,
+    );
+  }
+
   testWithoutContext('can launch chrome and connect to the devtools', () async {
     await expectReturnsNormallyLater(
-      _testLaunchChrome(
+      testLaunchChrome(
         '/.tmp_rand0/flutter_tools_chrome_device.rand0',
         processManager,
         chromeLauncher,
@@ -71,15 +104,44 @@ void main() {
     );
   });
 
+  testWithoutContext('can launch chrome in verbose mode', () async {
+    chromeLauncher = ChromiumLauncher(
+      fileSystem: fileSystem,
+      platform: platform,
+      processManager: processManager,
+      operatingSystemUtils: operatingSystemUtils,
+      browserFinder: findChromeExecutable,
+      logger: testLogger = BufferLogger.test(verbose: true),
+    );
+
+    await expectReturnsNormallyLater(
+      testLaunchChrome(
+        '/.tmp_rand0/flutter_tools_chrome_device.rand0',
+        processManager,
+        chromeLauncher,
+      )
+    );
+
+    expect(
+      testLogger.traceText.trim(),
+      'Launching Chromium (url = example_url, headless = false, skipCheck = true, debugPort = null)\n'
+      'Will use Chromium executable at example_chrome\n'
+      'Using Chromium 115\n'
+      '[CHROME]: \n'
+      '[CHROME]: \n'
+      '[CHROME]: DevTools listening',
+    );
+  });
+
   testWithoutContext('cannot have two concurrent instances of chrome', () async {
-    await _testLaunchChrome(
+    await testLaunchChrome(
       '/.tmp_rand0/flutter_tools_chrome_device.rand0',
       processManager,
       chromeLauncher,
     );
 
     await expectToolExitLater(
-      _testLaunchChrome(
+      testLaunchChrome(
         '/.tmp_rand0/flutter_tools_chrome_device.rand1',
         processManager,
         chromeLauncher,
@@ -89,7 +151,7 @@ void main() {
   });
 
   testWithoutContext('can launch new chrome after stopping a previous chrome', () async {
-    final Chromium chrome = await _testLaunchChrome(
+    final Chromium chrome = await testLaunchChrome(
       '/.tmp_rand0/flutter_tools_chrome_device.rand0',
       processManager,
       chromeLauncher,
@@ -97,7 +159,7 @@ void main() {
     await chrome.close();
 
     await expectReturnsNormallyLater(
-      _testLaunchChrome(
+      testLaunchChrome(
         '/.tmp_rand0/flutter_tools_chrome_device.rand1',
         processManager,
         chromeLauncher,
@@ -105,6 +167,116 @@ void main() {
     );
   });
 
+  testWithoutContext('exits normally using SIGTERM', () async {
+    final BufferLogger logger = BufferLogger.test();
+    final FakeAsync fakeAsync = FakeAsync();
+
+    fakeAsync.run((_) {
+      () async {
+        final FakeChromeConnection chromeConnection = FakeChromeConnection(maxRetries: 4);
+        final ChromiumLauncher chromiumLauncher = ChromiumLauncher(
+          fileSystem: fileSystem,
+          platform: platform,
+          processManager: processManager,
+          operatingSystemUtils: operatingSystemUtils,
+          browserFinder: findChromeExecutable,
+          logger: logger,
+        );
+
+        final FakeProcess process = FakeProcess(
+          duration: const Duration(seconds: 3),
+        );
+
+        final Chromium chrome = Chromium(0, chromeConnection, chromiumLauncher: chromiumLauncher, process: process, logger: logger);
+
+        final Future<void> closeFuture = chrome.close();
+        fakeAsync.elapse(const Duration(seconds: 4));
+        await closeFuture;
+
+        expect(process.signals, <io.ProcessSignal>[io.ProcessSignal.sigterm]);
+      }();
+    });
+
+    fakeAsync.flushTimers();
+    expect(logger.warningText, isEmpty);
+  });
+
+  testWithoutContext('falls back to SIGKILL if SIGTERM did not work', () async {
+    final BufferLogger logger = BufferLogger.test();
+    final FakeAsync fakeAsync = FakeAsync();
+
+    fakeAsync.run((_) {
+      () async {
+        final FakeChromeConnection chromeConnection = FakeChromeConnection(maxRetries: 4);
+        final ChromiumLauncher chromiumLauncher = ChromiumLauncher(
+          fileSystem: fileSystem,
+          platform: platform,
+          processManager: processManager,
+          operatingSystemUtils: operatingSystemUtils,
+          browserFinder: findChromeExecutable,
+          logger: logger,
+        );
+
+        final FakeProcess process = FakeProcess(
+          duration: const Duration(seconds: 6),
+        );
+
+        final Chromium chrome = Chromium(0, chromeConnection, chromiumLauncher: chromiumLauncher, process: process, logger: logger);
+
+        final Future<void> closeFuture = chrome.close();
+        fakeAsync.elapse(const Duration(seconds: 7));
+        await closeFuture;
+
+        expect(process.signals, <io.ProcessSignal>[io.ProcessSignal.sigterm, io.ProcessSignal.sigkill]);
+      }();
+    });
+
+    fakeAsync.flushTimers();
+    expect(
+      logger.warningText,
+      'Failed to exit Chromium (pid: 1234) using SIGTERM. Will try sending SIGKILL instead.\n',
+    );
+  });
+
+  testWithoutContext('falls back to a warning if SIGKILL did not work', () async {
+    final BufferLogger logger = BufferLogger.test();
+    final FakeAsync fakeAsync = FakeAsync();
+
+    fakeAsync.run((_) {
+      () async {
+        final FakeChromeConnection chromeConnection = FakeChromeConnection(maxRetries: 4);
+        final ChromiumLauncher chromiumLauncher = ChromiumLauncher(
+          fileSystem: fileSystem,
+          platform: platform,
+          processManager: processManager,
+          operatingSystemUtils: operatingSystemUtils,
+          browserFinder: findChromeExecutable,
+          logger: logger,
+        );
+
+        final FakeProcess process = FakeProcess(
+          duration: const Duration(seconds: 20),
+        );
+
+        final Chromium chrome = Chromium(0, chromeConnection, chromiumLauncher: chromiumLauncher, process: process, logger: logger);
+
+        final Future<void> closeFuture = chrome.close();
+        fakeAsync.elapse(const Duration(seconds: 30));
+        await closeFuture;
+        expect(process.signals, <io.ProcessSignal>[io.ProcessSignal.sigterm, io.ProcessSignal.sigkill]);
+      }();
+    });
+
+    fakeAsync.flushTimers();
+    expect(
+      logger.warningText,
+      'Failed to exit Chromium (pid: 1234) using SIGTERM. Will try sending SIGKILL instead.\n'
+      'Failed to exit Chromium (pid: 1234) using SIGKILL. Giving up. Will continue, assuming '
+      'Chromium has exited successfully, but it is possible that this left a dangling Chromium '
+      'process running on the system.\n',
+    );
+  });
+
   testWithoutContext('does not crash if saving profile information fails due to a file system exception.', () async {
     final BufferLogger logger = BufferLogger.test();
     chromeLauncher = ChromiumLauncher(
@@ -214,7 +386,7 @@ void main() {
           'example_url',
         ],
         stderr: kDevtoolsStderr,
-      )
+      ),
     ]);
 
     await expectReturnsNormallyLater(
@@ -226,7 +398,7 @@ void main() {
   });
 
   testWithoutContext('can launch x86_64 Chrome on ARM macOS', () async {
-    final OperatingSystemUtils macOSUtils = FakeOperatingSystemUtils(hostPlatform: HostPlatform.darwin_arm);
+    final OperatingSystemUtils macOSUtils = FakeOperatingSystemUtils(hostPlatform: HostPlatform.darwin_arm64);
     final ChromiumLauncher chromiumLauncher = ChromiumLauncher(
       fileSystem: fileSystem,
       platform: platform,
@@ -253,7 +425,7 @@ void main() {
           'example_url',
         ],
         stderr: kDevtoolsStderr,
-      )
+      ),
     ]);
 
     await expectReturnsNormallyLater(
@@ -265,7 +437,7 @@ void main() {
   });
 
   testWithoutContext('can launch ARM Chrome natively on ARM macOS when installed', () async {
-    final OperatingSystemUtils macOSUtils = FakeOperatingSystemUtils(hostPlatform: HostPlatform.darwin_arm);
+    final OperatingSystemUtils macOSUtils = FakeOperatingSystemUtils(hostPlatform: HostPlatform.darwin_arm64);
     final ChromiumLauncher chromiumLauncher = ChromiumLauncher(
       fileSystem: fileSystem,
       platform: platform,
@@ -326,6 +498,32 @@ void main() {
     );
   });
 
+  testWithoutContext('can launch chrome with arbitrary flags', () async {
+    processManager.addCommand(const FakeCommand(
+      command: <String>[
+        'example_chrome',
+        '--user-data-dir=/.tmp_rand0/flutter_tools_chrome_device.rand0',
+        '--remote-debugging-port=12345',
+        ...kChromeArgs,
+        '--autoplay-policy=no-user-gesture-required',
+        '--incognito',
+        '--auto-select-desktop-capture-source="Entire screen"',
+        'example_url',
+      ],
+      stderr: kDevtoolsStderr,
+    ));
+
+    await expectReturnsNormallyLater(chromeLauncher.launch(
+      'example_url',
+      skipCheck: true,
+      webBrowserFlags: <String>[
+        '--autoplay-policy=no-user-gesture-required',
+        '--incognito',
+        '--auto-select-desktop-capture-source="Entire screen"',
+      ],
+    ));
+  });
+
   testWithoutContext('can launch chrome headless', () async {
     processManager.addCommand(const FakeCommand(
       command: <String>[
@@ -391,13 +589,6 @@ void main() {
       cacheDir: dataDir,
     );
 
-    exitCompleter.complete();
-    await Future<void>.delayed(const Duration(milliseconds: 1));
-
-    // writes non-crash back to dart_tool
-    expect(preferencesFile.readAsStringSync(), '"exit_type":"Normal"');
-
-
     // validate any Default content is copied
     final Directory defaultContentDir = fileSystem
         .directory('.tmp_rand0/flutter_tools_chrome_device.rand0')
@@ -406,6 +597,12 @@ void main() {
 
     expect(defaultContentDir, exists);
 
+    exitCompleter.complete();
+    await Future<void>.delayed(const Duration(milliseconds: 1));
+
+    // writes non-crash back to dart_tool
+    expect(preferencesFile.readAsStringSync(), '"exit_type":"Normal"');
+
     // Validate cache dirs are not copied.
     for (final String cache in kCodeCache) {
       expect(fileSystem
@@ -413,6 +610,9 @@ void main() {
         .childDirectory('Default')
         .childDirectory(cache), isNot(exists));
     }
+
+    // validate defaultContentDir is deleted after exit, data is in cache
+    expect(defaultContentDir, isNot(exists));
   });
 
   testWithoutContext('can retry launch when glibc bug happens', () async {
@@ -556,22 +756,81 @@ void main() {
     );
     expect(logger.errorText, contains('SocketException'));
   });
+
+  test('can recover if getTabs throws a connection exception', () async {
+    final BufferLogger logger = BufferLogger.test();
+    final FakeChromeConnection chromeConnection = FakeChromeConnection(maxRetries: 4);
+    final ChromiumLauncher chromiumLauncher = ChromiumLauncher(
+      fileSystem: fileSystem,
+      platform: platform,
+      processManager: processManager,
+      operatingSystemUtils: operatingSystemUtils,
+      browserFinder: findChromeExecutable,
+      logger: logger,
+    );
+    final FakeProcess process = FakeProcess();
+    final Chromium chrome = Chromium(0, chromeConnection, chromiumLauncher: chromiumLauncher, process: process, logger: logger);
+    expect(await chromiumLauncher.connect(chrome, false), equals(chrome));
+    expect(logger.errorText, isEmpty);
+  });
+
+  test('exits if getTabs throws a connection exception consistently', () async {
+    final BufferLogger logger = BufferLogger.test();
+    final FakeChromeConnection chromeConnection = FakeChromeConnection();
+    final ChromiumLauncher chromiumLauncher = ChromiumLauncher(
+      fileSystem: fileSystem,
+      platform: platform,
+      processManager: processManager,
+      operatingSystemUtils: operatingSystemUtils,
+      browserFinder: findChromeExecutable,
+      logger: logger,
+    );
+    final FakeProcess process = FakeProcess();
+    final Chromium chrome = Chromium(0, chromeConnection, chromiumLauncher: chromiumLauncher, process: process, logger: logger);
+    await expectToolExitLater(
+      chromiumLauncher.connect(chrome, false),
+        allOf(
+          contains('Unable to connect to Chrome debug port'),
+          contains('incorrect format'),
+        ));
+    expect(logger.errorText,
+      allOf(
+          contains('incorrect format'),
+          contains('OK'),
+          contains('<html> ...'),
+        ));
+  });
 }
 
-Future<Chromium> _testLaunchChrome(String userDataDir, FakeProcessManager processManager, ChromiumLauncher chromeLauncher) {
-  processManager.addCommand(FakeCommand(
-    command: <String>[
-      'example_chrome',
-      '--user-data-dir=$userDataDir',
-      '--remote-debugging-port=12345',
-      ...kChromeArgs,
-      'example_url',
-    ],
-    stderr: kDevtoolsStderr,
-  ));
+/// Fake chrome connection that fails to get tabs a few times.
+class FakeChromeConnection extends Fake implements ChromeConnection {
 
-  return chromeLauncher.launch(
-    'example_url',
-    skipCheck: true,
-  );
+  /// Create a connection that throws a connection exception on first
+  /// [maxRetries] calls to [getTabs].
+  /// If [maxRetries] is `null`, [getTabs] calls never succeed.
+  FakeChromeConnection({this.maxRetries}): _retries = 0;
+
+  final List<ChromeTab> tabs = <ChromeTab>[];
+  final int? maxRetries;
+  int _retries;
+
+  @override
+  Future<ChromeTab?> getTab(bool Function(ChromeTab tab) accept, {Duration? retryFor}) async {
+    return tabs.firstWhere(accept);
+  }
+
+  @override
+  Future<List<ChromeTab>> getTabs({Duration? retryFor}) async {
+    _retries ++;
+    if (maxRetries == null || _retries < maxRetries!) {
+      throw ConnectionException(
+        formatException: const FormatException('incorrect format'),
+        responseStatus: 'OK,',
+        responseBody: '<html> ...');
+    }
+    return tabs;
+  }
+
+  @override
+  void close() {}
 }
