@@ -15,6 +15,7 @@ import 'package:webkit_inspection_protocol/webkit_inspection_protocol.dart';
 
 import 'fixtures/context.dart';
 import 'fixtures/project.dart';
+import 'fixtures/utilities.dart';
 
 void testAll({
   required TestSdkConfigurationProvider provider,
@@ -68,10 +69,12 @@ void testAll({
       setUpAll(() async {
         setCurrentLogWriter(debug: debug);
         await context.setUp(
-          compilationMode: compilationMode,
-          enableExpressionEvaluation: true,
-          useDebuggerModuleNames: useDebuggerModuleNames,
-          verboseCompiler: debug,
+          testSettings: TestSettings(
+            compilationMode: compilationMode,
+            enableExpressionEvaluation: true,
+            useDebuggerModuleNames: useDebuggerModuleNames,
+            verboseCompiler: debug,
+          ),
         );
       });
 
@@ -155,11 +158,15 @@ void testAll({
               scope: scope,
             );
 
-        getInstanceRef(frame, expr, {scope}) async => await evaluateInFrame(
-              frame,
-              expr,
-              scope: scope,
-            ) as InstanceRef;
+        getInstanceRef(frame, expr, {scope}) async {
+          final result = await evaluateInFrame(
+            frame,
+            expr,
+            scope: scope,
+          );
+          expect(result, isA<InstanceRef>());
+          return result as InstanceRef;
+        }
 
         getInstance(InstanceRef ref) async =>
             await context.service.getObject(isolateId, ref.id!) as Instance;
@@ -313,7 +320,7 @@ void testAll({
                 await getInstanceRef(event.topFrame!.index!, 'stream');
             final instance = await getInstance(instanceRef);
 
-            expect(instance, matchInstance('_AsBroadcastStream<int>'));
+            expect(instance, matchInstanceClassName('_AsBroadcastStream<int>'));
           });
         });
 
@@ -660,27 +667,30 @@ void testAll({
         tearDown(() async {});
 
         evaluate(
-          libraryId,
+          targetId,
           expr, {
           scope,
         }) async =>
             await context.service.evaluate(
               isolateId,
-              libraryId,
+              targetId,
               expr,
               scope: scope,
             );
 
         getInstanceRef(
-          libraryId,
+          targetId,
           expr, {
           scope,
-        }) async =>
-            await evaluate(
-              libraryId,
-              expr,
-              scope: scope,
-            ) as InstanceRef;
+        }) async {
+          final result = await evaluate(
+            targetId,
+            expr,
+            scope: scope,
+          );
+          expect(result, isA<InstanceRef>());
+          return result as InstanceRef;
+        }
 
         String getRootLibraryId() {
           expect(isolate.rootLib, isNotNull);
@@ -688,6 +698,28 @@ void testAll({
           return isolate.rootLib!.id!;
         }
 
+        test(
+          'RecordType getters',
+          () async {
+            final libraryId = getRootLibraryId();
+
+            final type = await getInstanceRef(libraryId, '(0,1).runtimeType');
+            final result = await getInstanceRef(type.id, 'hashCode');
+
+            expect(result, matchInstanceRefKind('Double'));
+          },
+          skip: 'https://github.com/dart-lang/sdk/issues/54609',
+        );
+
+        test('Object getters', () async {
+          final libraryId = getRootLibraryId();
+
+          final type = await getInstanceRef(libraryId, 'Object()');
+          final result = await getInstanceRef(type.id, 'hashCode');
+
+          expect(result, matchInstanceRefKind('Double'));
+        });
+
         test('with scope', () async {
           final libraryId = getRootLibraryId();
 
@@ -751,15 +783,13 @@ void testAll({
           final evaluation2 = evaluate(libraryId, 'MainClass(1,1).toString()');
 
           final results = await Future.wait([evaluation1, evaluation2]);
-
           expect(
             results[0],
-            matchErrorRef(contains('No batch result object ID')),
-          );
-          expect(
-            results[1],
-            matchErrorRef(contains('No batch result object ID')),
+            matchErrorRef(
+              contains('Evaluate is called on an unsupported target'),
+            ),
           );
+          expect(results[1], matchInstanceRef('1, 1'));
         });
 
         test('with scope override', () async {
@@ -809,9 +839,11 @@ void testAll({
     setUpAll(() async {
       setCurrentLogWriter(debug: debug);
       await context.setUp(
-        compilationMode: compilationMode,
-        enableExpressionEvaluation: false,
-        verboseCompiler: debug,
+        testSettings: TestSettings(
+          compilationMode: compilationMode,
+          enableExpressionEvaluation: false,
+          verboseCompiler: debug,
+        ),
       );
     });
 
@@ -888,13 +920,20 @@ Future<String> _setBreakpointInInjectedClient(WipDebugger debugger) async {
   return result.json['result']['breakpointId'];
 }
 
-Matcher matchInstanceRef(dynamic value) => isA<InstanceRef>().having(
-      (instance) => instance.valueAsString,
-      'valueAsString',
-      value,
+Matcher matchInstanceRefKind(String kind) =>
+    isA<InstanceRef>().having((instance) => instance.kind, 'kind', kind);
+
+Matcher matchInstanceRef(dynamic value) => isA<InstanceRef>()
+    .having((instance) => instance.valueAsString, 'valueAsString', value);
+
+Matcher matchInstanceClassName(dynamic className) => isA<Instance>().having(
+      (instance) => instance.classRef!.name,
+      'class name',
+      className,
     );
 
-Matcher matchInstance(dynamic className) => isA<Instance>().having(
+Matcher matchInstanceRefClassName(dynamic className) =>
+    isA<InstanceRef>().having(
       (instance) => instance.classRef!.name,
       'class name',
       className,
