@@ -15,7 +15,9 @@ main() {
     defineReflectiveTests(DuplicateDefinitionClassTest);
     defineReflectiveTests(DuplicateDefinitionEnumTest);
     defineReflectiveTests(DuplicateDefinitionExtensionTest);
+    defineReflectiveTests(DuplicateDefinitionExtensionTypeTest);
     defineReflectiveTests(DuplicateDefinitionMixinTest);
+    defineReflectiveTests(DuplicateDefinitionUnitTest);
   });
 }
 
@@ -150,6 +152,57 @@ class C {
     ]);
   }
 
+  test_instance_method_method_augment() async {
+    final a = newFile('$testPackageLibPath/a.dart', r'''
+library augment 'test.dart';
+
+augment class A {
+  augment void foo() {}
+}
+''');
+
+    newFile(testFile.path, r'''
+import augment 'a.dart';
+
+class A {
+  void foo() {}
+}
+''');
+
+    await resolveTestFile();
+    assertNoErrorsInResult();
+
+    await resolveFile2(a.path);
+    assertNoErrorsInResult();
+  }
+
+  test_instance_method_method_inAugmentation() async {
+    final a = newFile('$testPackageLibPath/a.dart', r'''
+library augment 'test.dart';
+
+augment class A {
+  void foo() {}
+}
+''');
+
+    newFile(testFile.path, r'''
+import augment 'a.dart';
+
+class A {
+  void foo() {}
+}
+''');
+
+    await resolveTestFile();
+    assertNoErrorsInResult();
+
+    await resolveFile2(a.path);
+    assertErrorsInResult([
+      error(CompileTimeErrorCode.DUPLICATE_DEFINITION, 55, 3,
+          contextMessages: [message('/home/test/lib/test.dart', 43, 3)]),
+    ]);
+  }
+
   test_instance_method_setter() async {
     await assertErrorsInCode(r'''
 class C {
@@ -1098,6 +1151,310 @@ extension E on A {}
   }
 }
 
+@reflectiveTest
+class DuplicateDefinitionExtensionTypeTest extends PubPackageResolutionTest {
+  test_instance_getter_getter() async {
+    await assertErrorsInCode(r'''
+extension type E(int it) {
+  int get foo => 0;
+  int get foo => 0;
+}
+''', [
+      error(CompileTimeErrorCode.DUPLICATE_DEFINITION, 57, 3,
+          contextMessages: [message('/home/test/lib/test.dart', 37, 3)]),
+    ]);
+  }
+
+  test_instance_getter_method() async {
+    await assertErrorsInCode(r'''
+extension type E(int it) {
+  int get foo => 0;
+  void foo() {}
+}
+''', [
+      error(CompileTimeErrorCode.DUPLICATE_DEFINITION, 54, 3,
+          contextMessages: [message('/home/test/lib/test.dart', 37, 3)]),
+    ]);
+  }
+
+  test_instance_getter_setter() async {
+    await assertNoErrorsInCode(r'''
+extension type E(int it) {
+  int get foo => 0;
+  set foo(_) {}
+}
+''');
+  }
+
+  test_instance_method_getter() async {
+    await assertErrorsInCode(r'''
+extension type E(int it) {
+  void foo() {}
+  int get foo => 0;
+}
+''', [
+      error(CompileTimeErrorCode.DUPLICATE_DEFINITION, 53, 3,
+          contextMessages: [message('/home/test/lib/test.dart', 34, 3)]),
+    ]);
+  }
+
+  test_instance_method_method() async {
+    await assertErrorsInCode(r'''
+extension type E(int it) {
+  void foo() {}
+  void foo() {}
+}
+''', [
+      error(CompileTimeErrorCode.DUPLICATE_DEFINITION, 50, 3,
+          contextMessages: [message('/home/test/lib/test.dart', 34, 3)]),
+    ]);
+  }
+
+  test_instance_method_setter() async {
+    await assertErrorsInCode(r'''
+extension type E(int it) {
+  void foo() {}
+  set foo(_) {}
+}
+''', [
+      error(CompileTimeErrorCode.DUPLICATE_DEFINITION, 49, 3,
+          contextMessages: [message('/home/test/lib/test.dart', 34, 3)]),
+    ]);
+  }
+
+  test_instance_representation_getter() async {
+    await assertErrorsInCode(r'''
+extension type E(int it) {
+  int get it => 0;
+}
+''', [
+      error(CompileTimeErrorCode.DUPLICATE_DEFINITION, 37, 2,
+          contextMessages: [message('/home/test/lib/test.dart', 21, 2)]),
+    ]);
+  }
+
+  test_instance_representation_method() async {
+    await assertErrorsInCode(r'''
+extension type E(int it) {
+  void it() {}
+}
+''', [
+      error(CompileTimeErrorCode.DUPLICATE_DEFINITION, 34, 2,
+          contextMessages: [message('/home/test/lib/test.dart', 21, 2)]),
+    ]);
+  }
+
+  test_instance_representation_setter() async {
+    await assertNoErrorsInCode(r'''
+extension type E(int it) {
+  set it(int _) {}
+}
+''');
+  }
+
+  test_instance_setter_getter() async {
+    await assertNoErrorsInCode(r'''
+extension type E(int it) {
+  set foo(_) {}
+  int get foo => 0;
+}
+''');
+  }
+
+  test_instance_setter_method() async {
+    await assertErrorsInCode(r'''
+extension type E(int it) {
+  set foo(_) {}
+  void foo() {}
+}
+''', [
+      error(CompileTimeErrorCode.DUPLICATE_DEFINITION, 50, 3,
+          contextMessages: [message('/home/test/lib/test.dart', 33, 3)]),
+    ]);
+  }
+
+  test_instance_setter_setter() async {
+    await assertErrorsInCode(r'''
+extension type E(int it) {
+  void set foo(_) {}
+  void set foo(_) {}
+}
+''', [
+      error(CompileTimeErrorCode.DUPLICATE_DEFINITION, 59, 3,
+          contextMessages: [message('/home/test/lib/test.dart', 38, 3)]),
+    ]);
+  }
+
+  test_static_field_field() async {
+    await assertErrorsInCode(r'''
+extension type E(int it) {
+  static int foo = 0;
+  static int foo = 0;
+}
+''', [
+      error(CompileTimeErrorCode.DUPLICATE_DEFINITION, 62, 3,
+          contextMessages: [message('/home/test/lib/test.dart', 40, 3)]),
+    ]);
+  }
+
+  test_static_field_getter() async {
+    await assertErrorsInCode(r'''
+extension type E(int it) {
+  static int foo = 0;
+  static int get foo => 0;
+}
+''', [
+      error(CompileTimeErrorCode.DUPLICATE_DEFINITION, 66, 3,
+          contextMessages: [message('/home/test/lib/test.dart', 40, 3)]),
+    ]);
+  }
+
+  test_static_field_method() async {
+    await assertErrorsInCode(r'''
+extension type E(int it) {
+  static int foo = 0;
+  static void foo() {}
+}
+''', [
+      error(CompileTimeErrorCode.DUPLICATE_DEFINITION, 63, 3,
+          contextMessages: [message('/home/test/lib/test.dart', 40, 3)]),
+    ]);
+  }
+
+  test_static_fieldFinal_getter() async {
+    await assertErrorsInCode(r'''
+extension type E(int it) {
+  static final int foo = 0;
+  static int get foo => 0;
+}
+''', [
+      error(CompileTimeErrorCode.DUPLICATE_DEFINITION, 72, 3,
+          contextMessages: [message('/home/test/lib/test.dart', 46, 3)]),
+    ]);
+  }
+
+  test_static_fieldFinal_setter() async {
+    await assertNoErrorsInCode(r'''
+extension type E(int it) {
+  static final int foo = 0;
+  static set foo(int x) {}
+}
+''');
+  }
+
+  test_static_getter_getter() async {
+    await assertErrorsInCode(r'''
+extension type E(int it) {
+  static int get foo => 0;
+  static int get foo => 0;
+}
+''', [
+      error(CompileTimeErrorCode.DUPLICATE_DEFINITION, 71, 3,
+          contextMessages: [message('/home/test/lib/test.dart', 44, 3)]),
+    ]);
+  }
+
+  test_static_getter_method() async {
+    await assertErrorsInCode(r'''
+extension type E(int it) {
+  static int get foo => 0;
+  static void foo() {}
+}
+''', [
+      error(CompileTimeErrorCode.DUPLICATE_DEFINITION, 68, 3,
+          contextMessages: [message('/home/test/lib/test.dart', 44, 3)]),
+    ]);
+  }
+
+  test_static_getter_setter() async {
+    await assertNoErrorsInCode(r'''
+extension type E(int it) {
+  static int get foo => 0;
+  static set foo(_) {}
+}
+''');
+  }
+
+  test_static_method_getter() async {
+    await assertErrorsInCode(r'''
+extension type E(int it) {
+  static void foo() {}
+  static int get foo => 0;
+}
+''', [
+      error(CompileTimeErrorCode.DUPLICATE_DEFINITION, 67, 3,
+          contextMessages: [message('/home/test/lib/test.dart', 41, 3)]),
+    ]);
+  }
+
+  test_static_method_method() async {
+    await assertErrorsInCode(r'''
+extension type E(int it) {
+  static void foo() {}
+  static void foo() {}
+}
+''', [
+      error(CompileTimeErrorCode.DUPLICATE_DEFINITION, 64, 3,
+          contextMessages: [message('/home/test/lib/test.dart', 41, 3)]),
+    ]);
+  }
+
+  test_static_method_setter() async {
+    await assertErrorsInCode(r'''
+extension type E(int it) {
+  static void foo() {}
+  static set foo(_) {}
+}
+''', [
+      error(CompileTimeErrorCode.DUPLICATE_DEFINITION, 63, 3,
+          contextMessages: [message('/home/test/lib/test.dart', 41, 3)]),
+    ]);
+  }
+
+  test_static_setter_getter() async {
+    await assertNoErrorsInCode(r'''
+extension type E(int it) {
+  static set foo(_) {}
+  static int get foo => 0;
+}
+''');
+  }
+
+  test_static_setter_method() async {
+    await assertErrorsInCode(r'''
+extension type E(int it) {
+  static set foo(_) {}
+  static void foo() {}
+}
+''', [
+      error(CompileTimeErrorCode.DUPLICATE_DEFINITION, 64, 3,
+          contextMessages: [message('/home/test/lib/test.dart', 40, 3)]),
+    ]);
+  }
+
+  test_static_setter_setter() async {
+    await assertErrorsInCode(r'''
+extension type E(int it) {
+  static void set foo(_) {}
+  static void set foo(_) {}
+}
+''', [
+      error(CompileTimeErrorCode.DUPLICATE_DEFINITION, 73, 3,
+          contextMessages: [message('/home/test/lib/test.dart', 45, 3)]),
+    ]);
+  }
+
+  test_unitMembers_extensionType() async {
+    await assertErrorsInCode('''
+extension type E(int it) {}
+extension type E(int it) {}
+''', [
+      error(CompileTimeErrorCode.DUPLICATE_DEFINITION, 43, 1,
+          contextMessages: [message('/home/test/lib/test.dart', 15, 1)]),
+    ]);
+  }
+}
+
 @reflectiveTest
 class DuplicateDefinitionMixinTest extends PubPackageResolutionTest {
   test_instance_field_field() async {
@@ -1214,6 +1571,57 @@ mixin M {
     ]);
   }
 
+  test_instance_method_method_augment() async {
+    final a = newFile('$testPackageLibPath/a.dart', r'''
+library augment 'test.dart';
+
+augment mixin A {
+  augment void foo() {}
+}
+''');
+
+    newFile(testFile.path, r'''
+import augment 'a.dart';
+
+mixin A {
+  void foo() {}
+}
+''');
+
+    await resolveTestFile();
+    assertNoErrorsInResult();
+
+    await resolveFile2(a.path);
+    assertNoErrorsInResult();
+  }
+
+  test_instance_method_method_inAugmentation() async {
+    final a = newFile('$testPackageLibPath/a.dart', r'''
+library augment 'test.dart';
+
+augment mixin A {
+  void foo() {}
+}
+''');
+
+    newFile(testFile.path, r'''
+import augment 'a.dart';
+
+mixin A {
+  void foo() {}
+}
+''');
+
+    await resolveTestFile();
+    assertNoErrorsInResult();
+
+    await resolveFile2(a.path);
+    assertErrorsInResult([
+      error(CompileTimeErrorCode.DUPLICATE_DEFINITION, 55, 3,
+          contextMessages: [message('/home/test/lib/test.dart', 43, 3)]),
+    ]);
+  }
+
   test_instance_method_setter() async {
     await assertErrorsInCode(r'''
 mixin M {
@@ -1597,7 +2005,7 @@ f(int a, double a) {}
 
   test_switchCase_localVariable_localVariable() async {
     await assertErrorsInCode(r'''
-// @dart = 2.18
+// @dart = 2.19
 void f() {
   switch (0) {
     case 0:
@@ -1711,8 +2119,11 @@ void f<T, T>() {}
           contextMessages: [message('/home/test/lib/test.dart', 7, 1)]),
     ]);
   }
+}
 
-  test_unitMembers_class() async {
+@reflectiveTest
+class DuplicateDefinitionUnitTest extends PubPackageResolutionTest {
+  test_class() async {
     await assertErrorsInCode('''
 class A {}
 class B {}
@@ -1723,7 +2134,47 @@ class A {}
     ]);
   }
 
-  test_unitMembers_part_library() async {
+  test_class_augmentation() async {
+    final a = newFile('$testPackageLibPath/a.dart', r'''
+library augment 'test.dart';
+
+augment class A {}
+''');
+
+    newFile(testFile.path, r'''
+import augment 'a.dart';
+
+class A {}
+''');
+
+    await resolveTestFile();
+    assertNoErrorsInResult();
+
+    await resolveFile2(a.path);
+    assertNoErrorsInResult();
+  }
+
+  test_mixin_augmentation() async {
+    final a = newFile('$testPackageLibPath/a.dart', r'''
+library augment 'test.dart';
+
+augment mixin A {}
+''');
+
+    newFile(testFile.path, r'''
+import augment 'a.dart';
+
+mixin A {}
+''');
+
+    await resolveTestFile();
+    assertNoErrorsInResult();
+
+    await resolveFile2(a.path);
+    assertNoErrorsInResult();
+  }
+
+  test_part_library() async {
     var libPath = convertPath('$testPackageLibPath/lib.dart');
     var aPath = convertPath('$testPackageLibPath/a.dart');
     newFile(libPath, '''
@@ -1748,7 +2199,7 @@ class A {}
       ]);
   }
 
-  test_unitMembers_part_part() async {
+  test_part_part() async {
     var libPath = convertPath('$testPackageLibPath/lib.dart');
     var aPath = convertPath('$testPackageLibPath/a.dart');
     var bPath = convertPath('$testPackageLibPath/b.dart');
@@ -1783,7 +2234,7 @@ class A {}
       ]);
   }
 
-  test_unitMembers_typedef_interfaceType() async {
+  test_typedef_interfaceType() async {
     await assertErrorsInCode('''
 typedef A = List<int>;
 typedef A = List<int>;
